function [ ng, mu, sig ] = getHistParams( pth, hcase, Nw, nbin, opt )
%% getHistParams( pth, hcase, Nw, nbin, opt )
%
% * Parameters
%   * pth : TODO
%   * hcase : the complexity of the model
%       * hel = heterogeneous without internal nodes
%       * he  = heterogeneous with internal nodes
%       * hol, ho = homogeneous variants
%           HINT: in principle only string ending of files (can be misused
%                   by other words/strings)
%   * Nw : number of wind series
%   * nbin : TODO
%   * opt
%       opt.grid = 'rts96'; cannot be changed atm
%       opt.wb ;            leafgreen waitbar
%       opt.incr = 0;       Change to any non-zero number to load increment
%                           histograms instead of histograms of frequency
%                           itself
%       opt.nodeoff = 1;    offset for node numbering (starting at 0 =>
%                           offset = 1, starting at at 1 => offset = 0)x
%       opt.tnc = '';       turbine node config ending. To be consistent
%                           with version PGModel 2.012
%
%  * Returns
%   * ng 
%   * mu
%   * sig
%
%
% V1.0 mawolff@uos.de 10-Jul-2019

% TODO: nodeoff confusing change to startIndex (0 or 1 or ...)
 
%% pseudocode
 
 
%% input handling
if( nargin < 5 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

%% ============================== params ================================ %
if( strcmp(opt.grid, 'rts96') )
    Nk = 73;
    Nj = 33;
end

freqPerBin    = 0.005;
freqIncPerBin = 0.001;

%% =============================== load  ================================ %
sno.nodeoff = opt.nodeoff;    % offset for node numbering (default 1)
sno.tnc = opt.tnc;            % turbine node config (to be consis. with v2.012)
sno.rethst = false;
if( opt.incr ~= 0 )
    sno.incrEnd = ['-' num2str(opt.incr)];      % sprintf('%1.1f', opt.incr)
    bfile = [ pth 'histIncrFreqstochSNP96' hcase '-'];
    
else
    bfile = [ pth 'histFreqstochSNP96' hcase '-'];
end
if( ~isempty( opt.wb ) )
%     waitbar(n/Nj, opt.wb, ['node ' num2str(n) '/' num2str(Nj)]);
    waitbar(0, opt.wb, 'reading');
end
[sumhst, sumhstK, ~] = ...
    readSNPhist( bfile, Nj, Nk, Nw, nbin, sno);       % read freq hist


%% ============================== process =============================== %
maxfreq = nbin*freqPerBin;                              % max frequency in hist
binx = (-maxfreq/2+freqPerBin):freqPerBin:(maxfreq/2);  % bin x-values 
maxfreqinc = nbin*freqIncPerBin;
binxi = (-maxfreqinc/2+freqIncPerBin ...                % bin x-values for increments
    ):freqIncPerBin:(maxfreqinc/2); 

ng = zeros(Nj,Nw);     % non-gaussian parameter
mu = zeros(Nj,Nw);
sig = zeros(Nj,Nw);
for n = 1:Nj
    for w = 1:Nw
        Nc = sum( sumhstK{n,w} ); % number of counts
        if( opt.incr ~= 0 )
            normhist = sumhstK{n,w}/(Nc*freqIncPerBin);
        else
            normhist = sumhstK{n,w}/(Nc*freqPerBin);
        end
        if( isempty( normhist ) )
            continue;
        end
        pdfo.cen = true;
        pdfo.nm = 4;
        if( opt.incr ~= 0 )
            moms = pdfmom( binxi, normhist, pdfo );
        else
            moms = pdfmom( binx, normhist, pdfo );
        end
        ng(n,w) = moms(4)/(3*moms(2)^2) - 1;
        mu(n,w) = moms(1);
        sig(n,w) = sqrt(moms(2));
        
    end
    if( ~isempty( opt.wb ) )
        waitbar(n/Nj, opt.wb, ['node ' num2str(n) '/' num2str(Nj)]);
    end
end
if( ~isempty( opt.wb ) )
    close( opt.wb );
end

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'grid') )
    opt.grid = 'rts96';
 end
 if( ~isfield( opt, 'wb') )
    wbo.barcolor = 'leafgreen';
    opt.wb = cWaitbar(wbo);
 end
 if( ~isfield( opt, 'incr') )
    opt.incr = 0;
 end
 if( ~isfield( opt, 'nodeoff') )
    opt.nodeoff = 1;
 end
 if( ~isfield( opt, 'tnc') )
    opt.tnc = '';
 end
    
end
