function [ sumhst, sumhstK, hst ] = readSNPhist( bfil, Nj, N, Nw, nbin, opt )
%% [ sumhsthe, sumhstheK, hsthe ] = 
%              readSNPhist( bfil, Nj, N, Nw, nbin, opt )
%
% * Parameters
%   * bfil  : base file name (path to file and filename without number
%               ending)
%   * Nj    : number of different perturbed nodes
%   * N     : number of nodes
%   * Nw    : number of wind series
%   * nbin  : number of bins in histogram
%   * opt
%       * opt.nodeoff = 0; offset for node labels in data files
%                          (if nodeoff = 1, the nodes start from 0
%                          otherwise from 1)
%       * opt.windpre = 'w' prefix of wind data number
%                           old version: '-'
%       * opt.woff = 1;     offset for wind number as nodeoff
%       * opt.rethst = true; if the histograms should be stored and
%                            returned also
%       * opt.wb
%       * opt.IncrEnd = ''; file ending for increment files ('-0.2' ...)
%       * opt.tnc = '';  file ending for turbine configuration ('tn0' ...)
%
%  * Returns
%   * sumhst  : mean over wind series histogram summarized over all nodes
%   * sumhstK : mean over wind series histogram summarized over all nodes 
%                   except perturbed one
%   * hst     : cell matrix containing all complete histograms for all
%                   wind series and all perturbed nodes
%
% V1.0 mawolff@uos.de 19-Jun-2019
 
%% pseudocode
 
 
%% input handling
if( nargin < 6 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
sumhst  = cell(Nj,Nw); % summarized histogram of frequencies of all nodes
sumhstK = cell(Nj,Nw);%zeros(Nj,nbin); % summarized histogram without perturbed node itself
hst     = cell(Nj,Nw);    % histogram of frequencies of all nodes and all realizations

for w = 1:Nw
    for n = 1:Nj
        fil = [bfil ...
            num2str(n-opt.nodeoff) opt.windpre num2str(...
            w-opt.woff) opt.tnc opt.incrEnd];
%         disp( fil );
        if( opt.rethst )
            hst{n,w} = readFreqHist( fil, nbin );
            chst = hst{n,w};
        else
            chst = readFreqHist( fil, nbin );
        end
        if( numel(chst) == 1 )
            if( isnan(chst) )
                continue;
            end
        end
        sumhst{n,w} = sum( chst, 2)';       % sum histograms of all nodes
        sumhstK{n,w} = sum( ...             % sum histograms of all nodes except
            chst(:,[1:(n-1) (n+1):N]), 2)'; % the perturbed one
        
    end

    if( ~isempty( opt.wb ) )
        waitbar(w/Nw, opt.wb, ['reading series ' ...
            num2str(w) '/' num2str(Nw)]);
    end
end
if( ~isempty( opt.wb ) )
    close( opt.wb );
end
% for n = 1:Nj
%     sumhst(n,:) = sumhst(n,:) / Nw;
%     sumhstK(n,:) = sumhstK(n,:) / Nw;
% end
return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'nodeoff') )
    opt.nodeoff = 0;
 end
 if( ~isfield( opt, 'windpre') )    % prefix of wind number
    opt.windpre = 'w';
 end
 if( ~isfield( opt, 'woff') )    % offset of wind number
    opt.woff = 1;
 end
 if( ~isfield( opt, 'rethst') )    % if histograms should be stored and returned
    opt.rethst = true;
 end
 if( ~isfield( opt, 'wb') )    % waitbar
    opt.wb = cWaitbar();
 end
 if( ~isfield( opt, 'incrEnd') )    % file ending for increment files
    opt.incrEnd = '';
 end
 if( ~isfield( opt, 'tnc') )    % file ending for turbine configuration
    opt.tnc = '';
 end
end
