function [ surfmax ] = surfGridRT( ijtor, opt )
%% surfGridRT( ijtor, opt )
% surfGridRT( surfs, opt )
%
% Calculates the return time surface t_i for one node i maximised over all 
% return time surfaces t_ij obtained from respndongding nodes j after 
% perturbing this node (i)
% Mathematically:
% t^grid_i (theta, omega) = max_j ( t_ij( theta, omega ) )
%
%
% * Parameters
%   * ijtor : if a double matrix it is assumed to be in ijtor-format
%               (ij-nodes, theta, omega, return time) then the t_ij
%               surfaces are calculated from it.
%             if a cell matrix is given it is assmued that ijtor{i}{j}.x
%             (and y and z respectively) are already the surface of t_ij.
%   * opt: option struct
%       * opt.iseq = 1:1 : the disturbed "i" nodes
%       * opt.jseq = 1:1 : the responding "j" nodes
%
%  * Returns
%   * surfmax : cell containing all the requested T_i surfaces
%
%
% V1.0 mawolff@uos.de 19-Oct-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

if( iscell( ijtor) )
    surfs = ijtor;
    cellin = true;
else
    surfs = getSurfs(ijtor,opt);
    cellin = false;
end

surfmax = cell(numel(opt.iseq),1);

for ii = 1:numel(opt.iseq)
    i = opt.iseq(ii);
    if( cellin )
        [sx,sy] = size(surfs{i,1}.z);
    else
        [sx,sy] = size(surfs{i,1});
    end
    smaxi = zeros(sx,sy);
    for ji = 1:numel(opt.jseq)
        j = opt.jseq(ji);
        if( cellin )
            csurf = surfs{i,j}.z;
        else
            csurf = surfs{i,j};
        end
        
        for x = 1:sx
            for y = 1:sy
                smaxi(x,y) = max( smaxi(x,y), csurf(x,y));
            end
        end
    end
    surfmax{ii}=smaxi;
end


return;
 
end
 
 
function [ opt ] = readOptions( opt )

if( ~isfield( opt, 'f') )
    opt.f = 0;
end
if( ~isfield( opt, 'iseq') )
    opt.iseq = 1;
end
if( ~isfield( opt, 'jseq') )
    opt.jseq = 1;
end
end
