/*
 * CodeUtils.cpp
 *
 *  Created on: Feb 13, 2019
 *      Author: mawolff
 */

#include "CodeUtils.h"



// ================================= constructors ================================= //
	CodeUtils::CodeUtils(){
		timeInit = time(NULL);
		timeInt1 = time(NULL);
		timeInt2 = time(NULL);
		timeCPUInit = clock();
		timeCPU1 = clock();
	}

// ==================================== methods =================================== //
// --------------------------------- array methods -------------------------------- //
	/**
	 * Prints a double array on the standard outstream cout.
	 * @param d : the array to be print
	 * @param len : the array is printed from index 0 to len
	 * WARNING: no error handling if len bigger than array length
	 */
	void CodeUtils::consoleArrD(double* d, int len){
		for(int i = 0; i < len; i++){
			std::cout << d[i] <<" ";
		}
		std::cout << std::endl;
	}


//	template<typename T>
//	std::vector<T> CodeUtils::createCopyVec(std::vector<T> vec){
//		return vec;
//	}


//	TODO: TEMPLATES!!!
// ---------------------------------- read methods -------------------------------- //
	double CodeUtils::readDouble(std::ifstream& ins){
		std::string line;
		getline(ins,line);
		char* cline = Str2Char(line);
		double r = strtod(cline, &cline);
		return r;
	}

	int CodeUtils::readint(std::ifstream& ins){
		int r;
		ins >> r;
		std::string dummy;
		getline(ins,dummy);	// read dummy line to advance to next
		return r;

//		// old version
//		std::string line;
//		getline(ins,line);
//		char* cline = Str2Char(line);
//		int r = strtol(cline, &cline, 10);
// //		delete cline; // why does not work?
//		return r;
	}

	std::vector<double>* CodeUtils::readDVec(std::ifstream& ins) {
		std::string line;
		getline(ins,line);
		return CodeUtils::lineToDoubleVec(line);
	}

	void CodeUtils::readDVec(std::ifstream& ins, std::vector<double>* rvec) {
		std::string line;
		getline(ins,line);
		rvec->clear();
		std::vector<double> *h = lineToDoubleVec(line);
		rvec->assign( h->begin(), h->end() );		// copy content to rvec
		h->clear(); 								// and delete it from h
//		delete h;	// necessary?
	}


	std::vector<double> CodeUtils::readDVecFromFile(std::string file) {
		std::ifstream ins;
		ins.open(CodeUtils::Str2Char(file));
		std::vector<double>* vec = CodeUtils::readDVec(ins);
		ins.close();
		return *vec;
	}

	std::vector<std::vector<double> > CodeUtils::readExtDMat(std::string file){

		std::vector<std::vector<double> > mat; 		// initialize matrix
		std::ifstream ins;							// initialize file input stream
		ins.open(CodeUtils::Str2Char(file));		// open file input stream
//		std::cout << CodeUtils::Str2Char(file) << std::endl;

		int i = 0;									// loop counter
		if(ins.is_open()){ 							// if stream is open

			std::string line; 						// declare string
			getline(ins,line);						// read first line
			while(line.size() > 0){					// while there are numbers in the file
				std::vector<double> a;				// create a vector
				mat.push_back(a);					// and push it to matrix
				CodeUtils::lineToDoubleVec(			// convert the line into numbers
						Str2Char(line), mat.at(i));	// and insert them to the matrix
				i++;								// iterate loop counter
				getline(ins,line);					// read next line
			}
			ins.close();
		}


		return mat;
	}

	std::vector<std::vector<int> > CodeUtils::readIntMatArb(std::ifstream &ins, std::string &lastLine){
		std::vector<std::vector<int> > mat;
		std::vector<int> *iline = new std::vector<int>;
		iline->push_back(0);
		std::string line;
//		std::streampos pos;
		while(iline->size() > 0){
			iline->clear();
			delete iline;
//			pos = ins.tellg();
			getline(ins,line);
			iline = lineToIntVec(Str2Char(line));
			if( iline->size() != 0){
				mat.push_back(*iline);
			}
		}
//		if( pos != -1)
//			ins.seekg(pos, ins.beg);
		delete iline;
		lastLine = line;

		return mat;
	}

	std::vector<std::vector<double> > CodeUtils::readDMatArb(std::ifstream &ins,
			std::string &lastLine){
		std::vector<std::vector<double> > mat;
		std::vector<double> *iline = new std::vector<double>;
		iline->push_back(0);
		std::string line;
//		std::streampos pos;
		while(iline->size() > 0){
			iline->clear();
			delete iline;
//			pos = ins.tellg();
			getline(ins,line);
			iline = lineToDoubleVec(Str2Char(line));
			if( iline->size() != 0){
				mat.push_back(*iline);
			}
		}
//		if( pos != -1)
//			ins.seekg(pos, ins.beg);
		delete iline;
		lastLine = line;

		return mat;
	}

	// ------------------------------ persistence methods ----------------------------- //
	/**
	 * Appends a double array as ASCII to a file.
	 * @param arr : the array to be saved
	 * @param len : the length of the array
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::arrayDToFile(double* arr, int len, std::string fname){
	    std::ofstream   outfile(fname ,std::ios::out|std::ios::app);
	    char * buffr = new char[50];

	    for( int i = 0; i < len; i++){
	        sprintf(buffr," %1.10e",arr[i]);
	    	outfile << buffr;
	    }
	    outfile << std::endl;
	    outfile.close();

	}

	/**
	 * Appends a double array to a binary file.
	 * @param arr : the array to be saved
	 * @param len : the length of the array
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::arrayToBinFile(double* arr, int len, std::string fname){
	    std::ofstream   outfile(fname.c_str() ,
	    		std::ios::out|std::ios::app|std::ios::binary);
	    for(int i = 0; i < len; i++)
	    	outfile.write((char*) &arr[i], sizeof arr);
	    outfile.close();

	}

	/**
	 * Appends a double array to a binary file.
	 * Speed upgrade is work in progress
	 * @param arr : the array to be saved
	 * @param len : the length of the array
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::arrayToBinFileFast(double* arr, int len, std::string fname){
	    std::ofstream   outfile(fname.c_str() ,
	    		std::ios::out|std::ios::app|std::ios::binary);
	    outfile.write(reinterpret_cast<const char*>(arr),
	    		len * sizeof(arr[0])); // TODO does this work??
//	    for(int i = 0; i < len; i++)
//	    	outfile.write((char*) &arr[i], sizeof arr);
	    outfile.close();

	}


//	template <typename T, typename A>
//	void CodeUtils::vecToTextFile(std::vector<T, A> const& vec, std::ofstream& os) {
//		for( uint i = 0; i < vec.size(); i++){
//			os << vec.at(i) << " ";
//		}
//		os << std::endl;
//	}

	void CodeUtils::vecToTextFile(std::vector<int> vec, std::ofstream& os) {
			for( uint i = 0; i < vec.size(); i++){
				os << vec.at(i) << " ";
			}
			os << std::endl;
		}

	void CodeUtils::vecDToTextFile(std::vector<double> vec, std::ofstream& os) {
			for( uint i = 0; i < vec.size(); i++){
				os << vec.at(i) << " ";
			}
			os << std::endl;
		}

	/**
	 * Saves a vector to a binary file.
	 * @param vec : the vector to be saved
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::vecToBinFile(std::vector<double> vec, std::string fname){
	    std::ofstream   outfile(fname.c_str(),
	    		std::ios::out|std::ios::app|std::ios::binary);
	    for(uint i = 0; i < vec.size(); i++)
	    	outfile.write((char*) &vec.at(i), sizeof vec.at(i));
	    outfile.close();

	}

	/**
	 * Saves a vector to a binary file. Uses the complete block. So this is hopefully faster.
	 * @param vec : the vector to be saved
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::vecToBinFileFast(std::vector<double> vec, std::string fname){
	    std::ofstream   outfile(fname.c_str(),
	    		std::ios::out|std::ios::app|std::ios::binary);
	     outfile.write(reinterpret_cast<const char*>(vec.data()), vec.size() * sizeof(vec[0]));
	     outfile.close();

	}

	/**
	 * Saves a int vector to a binary file. Uses the complete block. So this is hopefully faster.
	 * @param vec : the vector to be saved
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::vecIToBinFileFast(std::vector<int> vec, std::string fname){
	    std::ofstream   outfile(fname.c_str(),
	    		std::ios::out|std::ios::app|std::ios::binary);
	     outfile.write(reinterpret_cast<const char*>(vec.data()), vec.size() * sizeof(vec[0]));
	     outfile.close();

	}


	/**
	 * Saves a single "double" to a binary file.
	 * @param d : the double to be saved
	 * @param fname : the name of the file the array should be saved in
	 */
	void CodeUtils::doubleToBinFileFast(double d, std::string fname){
	    std::ofstream   outfile(fname.c_str(),
	    		std::ios::out|std::ios::app|std::ios::binary);
	     outfile.write(reinterpret_cast<const char*>(&d),  sizeof(d));
	    outfile.close();

	}

// ------------------------------- directory methods ------------------------------ //
	/**
	 * Creates a directory with the given have in current directory.
	 * @param string dir: name of the directory to create as string
	 */
	void CodeUtils::cdir(std::string str){
		if (mkdir(str.c_str(), 0744) == -1)
			std::cerr << "Error :  " << strerror(errno) << std::endl;
		else
			std::cout << "Directory " << str << " created" << std::endl;
	}

	/**
	 * Creates a directory with the given have in the directory given
	 * by pth.
	 * @param string dir: name of the directory to create as string
	 */
	void CodeUtils::cpthdir(std::string pth, std::string str){
		if( pth.size() != 0 ){											// if additional pth is given
			if( strcmp(Str2Char(pth.substr(pth.size()-1,pth.size())),	// if pth ends with /
							"/" ) == 0 )
						str = pth + str; 								// just add pth and dir
					else{
						str = pth + "/" + str; 							// otherwise add / in between
					}
		}
		if (mkdir(str.c_str(), 0744) == -1)
			std::cerr << "Error :  " << strerror(errno) << std::endl;
		else
			std::cout << "Directory " << str << " created" << std::endl;
	}


// ------------------------------- converter methods ------------------------------ //
	/**
	 * Converts a double to a string.
	 * @param d : the double that is converted
	 */
	std::string CodeUtils::d2Str(double d){
		std::ostringstream strs;
		strs << d;
		return strs.str();
	}

	/**
		 * Converts an integer to a string.
		 * @param i : the integer that is converted
		 */
	std::string i2Str(int i){
		std::ostringstream strs;
		strs << i;
		return strs.str();
	}


	std::string CodeUtils::dvec2str(std::vector<double> dvec) {
		std::stringstream strs;
			for( uint i = 0; i < dvec.size(); i++){
				strs << dvec.at(i) << " ";
			}
			strs << std::endl;
			return strs.str();
		}

	std::string CodeUtils::ivec2str(std::vector<int> dvec) {
		std::stringstream strs;
			for( uint i = 0; i < dvec.size(); i++){
				strs << dvec.at(i) << " ";
			}
			strs << std::endl;
			return strs.str();
		}

	char* CodeUtils::Str2Char(std::string s){
		char* fil = new char[s.size() +1];	// todo this memory is possibly leaking (not freed), take
		std::copy(s.begin(), s.end(), fil);
//		strcpy(fil, s.c_str());
		fil[s.size()] = '\0';
		return fil;
	}

	std::vector<double>* CodeUtils::lineToDoubleVec(char* line){
		std::vector<double>* vout = new std::vector<double>;
		double nextD = 0;
		int strlen = 1;
		std::string strl;
		do{
			nextD = strtod(line, &line);		// stores the last index of the converted string back to line
			vout->push_back(nextD);
			strl.assign(line);
			strl = CodeUtils::trim(strl);
			strlen = strl.size();
		} while(strlen > 0);					// until no characters left to convert
		return vout;

	}


	std::vector<double>* CodeUtils::lineToDoubleVec(std::string line){
		char *c = Str2Char(line);
		std::vector<double> *ret = lineToDoubleVec(c);
		delete[] c;
		return ret;
	}


	void CodeUtils::lineToDoubleVec(char* line, std::vector<double> &vec){

		double nextD = 0;
		int strlen = 1;
		std::string strl;
		// todo trim line?
		do{
			nextD = strtod(line, &line);		// stores the last index of the converted string back to line
			vec.push_back(nextD);
			strl.assign(line);
			strl = CodeUtils::trim(strl); 		// trim such that no spaces left and loop terminates
			strlen = strl.size();
		} while(strlen > 0);					// until no characters left to convert
		return;

	}

	std::vector<int>* CodeUtils::lineToIntVec(char* line){	// todo strlen2 for other read methods
		std::vector<int>* vout = new std::vector<int>;
		int nextI = 0;
		int strlen, strlen2;	// string length and validation of conversion
		std::string strl;
		strl.assign(line);
		strl = trim(strl);
		strlen = strl.size();
		strlen2 = strlen;
		while(strlen > 0){
			nextI = strtol(line, &line, 10);	// stores the last index of the converted string back to line
			strl.assign(line);
			strl = CodeUtils::trim(strl);		// trim such that no spaces left and loop terminates
			strlen = strl.size(); 				// get string length after reading and cutting
			if( strlen2 != strlen ){ 			// if string lengths are different
				vout->push_back(nextI); 		// store read data
				strlen2 = strlen; 				// and store length of remaining string
			} else { 							// if strlen == strlen2 nothing was converted
				strlen = 0; 					// and the reading must end
			}
		}
		return vout;

	}

	/**
	 * converts a double array to a vector // todo raw type // todo unsage sizeof...
	 */
	std::vector<double> CodeUtils::darr2vec(double* arr){
		return std::vector<double>(arr, arr + sizeof arr / sizeof arr[0]);
	}

	/**
	 * converts a double array to a vector // todo raw type
	 */
	std::vector<double> CodeUtils::darr2vec(double* arr, int len){
		return std::vector<double>(arr, arr + len);
	}

	/**
	 * converts a double vector to an array
	 * this is just done by vec.data() and serves as mnemonic
	 */
	double* CodeUtils::dvec2arr(std::vector<double> vec){
		return vec.data();
	}

// ------------------------------- mathematical methods ------------------------------ //
	// TODO mathUtils
	/**
	 * Returns a random double number between fMin and fMax
	 * @param fMin : the minimum number
	 * @param fMax : the maximum number
	 * @return a random double between fMin and fMax including the boundarys
	 */
	double CodeUtils::randD(double fMin, double fMax)
	{
	    double f = (double)rand() / RAND_MAX;
	    return fMin + f * (fMax - fMin);
	}

	/**
	 * Returns a random integer number between low and high
	 * @param low : the minimum number
	 * @param high : the maximum number
	 * @return a random integer between low and high including the boundarys
	 */
	int CodeUtils::randInt(int low, int high){
		double d  = (double)rand()/ RAND_MAX;
		int r = (int)( (high-low+1)*d ) + low;	// d+1 so that there are d pots for the int "to fall in"
		return r;
	}


	/**
	 * Adds up each element of two double arrays a and b to the array res
	 * @param double[] a:
	 * @param double[] b:
	 * @param double[] res:
	 * @param int l: Number of elements in the arrays
	 */
	void CodeUtils::arrAddD(double* a, double* b, double* res, int l){

		for (int i = 0; i < l; i++){
			res[i] = a[i] + b[i];
		}
		return;
	}


	/**
	 * Adds up the absolute of all elements in the array vec until index len
	 * @param vec : the (double) array in which the elements should be added up
	 * @param len : the last index to be added
	 * @return the sum of the absolutes
	 */
	double CodeUtils::dAbsArrSum(double* vec, int len){
		double s = 0;
		for(int i = 0; i< len ; i++){
			s += fabs(vec[i]);
		}
		return s;
	}

	/**
	 * Adds up all elements in the array vec until index len
	 * @param vec : the (double) array in which the elements should be added up
	 * @param len : the last index to be added
	 * @return the sum of the elements
	 */
	double CodeUtils::dArrSum(double* vec, int len){
		double s = 0;
		for(int i = 0; i< len ; i++){
			s += vec[i];
		}
		return s;
	}

	// todo comment and const??
	double CodeUtils::dVecSum(std::vector<double> *vec){
		double s = 0;
		for(int i = 0; i< vec->size() ; i++){
			s += vec->at(i);
		}
		return s;
	}

	/**
	 * Adds up the cos of all elements in the array vec until index len
	 * @param vec : the (double) array in which the elements should be added up
	 * @param len : the last index to be added
	 * @return the sum of the cos of all elements
	 */
	double CodeUtils::dVecCosSum(double* vec, int len){
		double s = 0;
		for(int i = 0; i< len ; i++){
			s += cos(vec[i]);
		}
		return s;
	}

	/**
	 * Adds up the sin of all elements in the array vec until index len
	 * @param vec : the (double) array in which the elements should be added up
	 * @param len : the last index to be added
	 * @return the sum of the sin of all elements
	 */
	double CodeUtils::dVecSinSum(double* vec, int len){
		double s = 0;
		for(int i = 0; i< len ; i++){
			s += sin(vec[i]);
		}
		return s;
	}

	/**
	 * Adds up the absolute of all elements in the array vec until index len
	 * @param vec : the (integer) array in which the elements should be added up
	 * @param len : the last index to be added
	 * @return the sum of the absolutes
	 */
	int CodeUtils::AbsArrISum(int* vec, int len){
		int s = 0;
		for(int i = 0; i< len ; i++){
			s += abs(vec[i]);
		}
		return s;
	}

	/**
	 * Checks whether any of the values in the double* is NaN
	 * @param double* vec : the double array (pointer)
	 * @param int len : length of vec
	 * @return bool : if any of the values was NaN
	 */
	bool CodeUtils::anynan(double* vec, int len){
		for (int i = 0; i < len; i++){
			if( std::isnan(vec[i]) ){
				return true;
			}
		}
		return false;
	}

	/**
	 * Returns the arithmetic mean of all elements in the vector
	 * @param vec : vector containing doubles
	 * @return double : arithmetic mean of all elements of vec
	 */
	double CodeUtils::dvecMean(std::vector<double> vec){
		double m = 0;
		for( uint i = 0; i < vec.size(); i++){
			m += vec.at(i);
		}
		m /= vec.size();
		return m;
	}

	/**
	 * returns a logarithmic scaled sequence of n numbers
	 * between first and last given double
	 */
	double* CodeUtils::logspace(double first, double last, int n){
		double* res = new double[n];
		if( n == 1){
			res[0] = first;
			return res;
		}
		res[0] = log10f(first);
		res[n-1] = log10f(last);
		double sw = (res[n-1]-res[0])/(n-1);
		for(int i = 1; i < n; i++){
			res[i] = exp10f(res[0]+sw*i);
		}
		res[0] = exp10f(res[0]);
		return res;

	}

	double* CodeUtils::linspace(double first, double last, int n){
		double* res = new double[n];

		res[0] = first;
		if(n == 1){
			return res;
		}
		res[n-1] = last;
		double sw = (res[n-1]-res[0])/(n-1);	// There are n-1 spaces between n elements
		for( int i = 0; i < n; i++){
			res[i] = res[0] + sw*i;
		}
		return res;
	}


	/**
	 * The absolute difference of two angles can be larger than pi by value
	 * Mathematically pi is the maximum, taking the shorter way between the
	 * two angles
	 * Hence maps every angle x to the interval [-pi, pi]
	 */
	double CodeUtils::smallerAngle(double x){
		double pi = 3.14159265359;
		return sgn(x)*(fmod(fabs(x)+pi,2*pi)-pi);
	}


	/**
	 * returns the sign of the number either -1 or 1
	 */
	template <typename T> int CodeUtils::sgn(T val) {
	    return (T(0) < val) - (val < T(0));
	}

// --------------------------------- string methods ------------------------------- //
	// trim from start
	std::string& CodeUtils::ltrim(std::string &s) {
	    s.erase(s.begin(), std::find_if(s.begin(), s.end(),
	            std::not1(std::ptr_fun<int, int>(std::isspace))));
	    return s;
	}

	// trim from end
	std::string& CodeUtils::rtrim(std::string &s) {
	    s.erase(std::find_if(s.rbegin(), s.rend(),
	            std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
	    return s;
	}

	// trim from both ends
	std::string& CodeUtils::trim(std::string &s) {
	    return ltrim(rtrim(s));
	}


	bool CodeUtils::lineIs(std::string line, std::string str){
		str = trim(str);
		bool b1 = (line.size() == str.size() );
		if( !b1){
			return b1;
		}
		for( uint i = 0; i < line.size(); i++){
			if( line.at(i) != str.at(i) ){
				return false;
			}
		}
		return b1;
	}


	// --------------------------------- member methods ------------------------------- //
	// --------------------------------- timing methods ------------------------------- //
	void CodeUtils::resetTimeInterval(){
		timeInt1 = time(NULL);
	}

	double CodeUtils::takeTime(){
		timeInt2 = time(NULL);		// for more features in future
		return timeInt2-timeInt1;
	}

	double CodeUtils::getRunTime(){
		return time(NULL)-timeInit;
	}

	void CodeUtils::resetTimeCPUInterval(){
		timeCPU1 = clock();
	}

	double CodeUtils::takeTimeCPU(){
		return (double)(clock() - timeCPU1);
	}

	double CodeUtils::getRunTimeCPU(){
		return (double)(clock()-timeCPUInit);
	}

	// ==================================================== variables ===================================================== //
	// =================================================== constructors =================================================== //
	// ====================================================== methods ===================================================== //
	// --------------------------------------------------- array methods -------------------------------------------------- //
	// ------------------------------------------------- directory methods ------------------------------------------------ //
	// ------------------------------------------------ mathematical methods ---------------------------------------------- //
	// --------------------------------------------------- string methods ------------------------------------------------- //
	// --------------------------------------------------- member methods ------------------------------------------------- //
	// ------------------------------------------------------- get&set ---------------------------------------------------- //
