/*
 * CodeUtils.h
 *
 *  Created on: Feb 13, 2019
 *      Author: mawolff
 */

#ifndef CODEUTILS_H_
#define CODEUTILS_H_

#include <string>		// C++ Strings
#include "string.h"		// C methods for strings
#include <iostream>
#include <fstream>		// file stream
#include <sstream>
#include <sys/types.h>	// mkdir
#include <sys/stat.h>
#include <random>		// normal_distr
#include <stdlib.h>		// rand()
#include <vector>
#include <algorithm>	// find_if

class CodeUtils{
private:
	double timeInit,	// time whem creating this object
		timeInt1,
		timeInt2;
	clock_t
		timeCPUInit,
		timeCPU1;

	// ================================= constructors ================================= //
public:
	CodeUtils();

	// ==================================== methods =================================== //
public:
	// --------------------------------- array methods -------------------------------- //
	static void consoleArrD(double* d, int len);
//	static void setVecOrScalar(std::vector<double> *from,
//			std::vector<double> *to);
//	template<typename T>
//	static std::vector<T> createCopyVec(std::vector<T> vec);

	// ---------------------------------- read methods -------------------------------- //
	static int readint(std::ifstream& ins);
	static double readDouble(std::ifstream& ins);
	static std::vector<double>* readDVec(std::ifstream& ins);
	static void readDVec(std::ifstream& ins, std::vector<double>* rvec);
	static std::vector<std::vector<double> > readExtDMat(std::string file);
	static std::vector<double> readDVecFromFile(std::string file);

	static std::vector<std::vector<int> > readIntMatArb(std::ifstream& ins,
			std::string &lastLine);
	static std::vector<std::vector<double> > readDMatArb(std::ifstream &ins,
			std::string &lastLine);

	// ------------------------------ persistence methods ----------------------------- //
	static void arrayDToFile(double* arr, int len, std::string fname);
	static void arrayToBinFile(double* arr, int len, std::string fname);
	static void arrayToBinFileFast(double* arr, int len, std::string fname);

//	template <typename T, typename A>
//	static void vecToTextFile(std::vector<T, A> const&  vec, std::ofstream& os);
	static void vecDToTextFile(std::vector<double> vec, std::ofstream& os);
	static void vecToTextFile(std::vector<int> vec, std::ofstream& os);
	static void vecToBinFile(std::vector<double> vec, std::string fname);
	static void vecToBinFileFast(std::vector<double> vec, std::string fname);
	static void doubleToBinFileFast(double d, std::string fname);
	static void vecIToBinFileFast(std::vector<int> vec, std::string fname);	// todo raw type

	// ------------------------------- directory methods ------------------------------ //
	static void cdir(std::string dir);
	static void cpthdir(std::string pth, std::string dir);

	// ------------------------------- converter methods ------------------------------ //
	static std::string d2Str(double d);
	static std::string i2Str(int i);
	static std::string dvec2str(std::vector<double> dvec);
	static std::string ivec2str(std::vector<int> dvec);
	static char* Str2Char(std::string s);
	static std::vector<double>* lineToDoubleVec(char* line);
	static std::vector<double>* lineToDoubleVec(std::string line);
	static void lineToDoubleVec(char* line, std::vector<double> &vec);
	static std::vector<int>* lineToIntVec(char* line);
	static std::vector<double> darr2vec(double* arr);
	static std::vector<double> darr2vec(double* arr, int len);
	static double* dvec2arr(std::vector<double> vec);



	// ------------------------------ mathematical methods ---------------------------- //
	static double randD(double fMin, double fMax);
	static int randInt(int low, int high);
	static void arrAddD(double* a, double* b, double* res, int l);
	static double dAbsArrSum(double* vec, int len);
	static double dArrSum(double* vec, int len);
	static double dVecSum(std::vector<double> *vec);
	static double dVecCosSum(double* vec, int len);
	static double dVecSinSum(double* vec, int len);
	static int AbsArrISum(int* vec, int len);
	static bool anynan(double* vec, int len);
	static double dvecMean(std::vector<double> vec);
	static double* logspace(double first, double last, int n);
	static double* linspace(double first, double last, int n);
	static double smallerAngle(double x);
	template <typename T> static int sgn(T val);

	// --------------------------------- string methods ------------------------------- //
	static inline std::string& ltrim(std::string &s);	// todo eclipse bug check
	static inline std::string& rtrim(std::string &s);
	static inline std::string& trim(std::string &s);
	// inline means the code is copied when compiled such that no function call is involved

	static bool lineIs(std::string line, std::string str);

	// --------------------------------- timing methods ------------------------------- //
	void resetTimeInterval();		// reset the time interval measure
	double takeTime();				// get the time of current interval
	double getRunTime();			// get time since creating this object
	void resetTimeCPUInterval();	// reset the cpu time interval measure
	double takeTimeCPU();			// get the cpu time of current interval
	double getRunTimeCPU();			// get cpu time since creating this object


};

#endif /* CODEUTILS_H_ */


// ================================= constructors ================================= //
// ==================================== methods =================================== //
// --------------------------------- array methods -------------------------------- //
// ------------------------------ persistence methods ----------------------------- //
// ------------------------------- directory methods ------------------------------ //
// ------------------------------- converter methods ------------------------------ //
// ------------------------------ mathematical methods ---------------------------- //
// --------------------------------- string methods ------------------------------- //
// --------------------------------- timing methods ------------------------------- //
