/*
 * Function.cpp
 *
 *  Created on: Dec 3, 2015
 *      Author: mawolff
 */

#include "Function.h"

#include <math.h>
#include <iostream>
#include <stdlib.h>


 Function::Function(Param* p, PowerGrid* pg){
	 this->params = p;
	 this->pg = pg;
	 this->mt.seed(p->randSeed);
 }

 double Function::secDeriv(int i, double* om, double* th){

	 return pg->swingEq(th,om,i);

 }

 /**
  * Returns a random double number between fMin and fMax
  * @param fMin : the minimum number
  * @param fMax : the maximum number
  * @return a random double between fMin and fMax including the boundarys
  */
 double Function::dRand(double fMin, double fMax){
     double f = (double)rand() / RAND_MAX;
     return fMin + f * (fMax - fMin);
 }

 double Function::dRandNormal(double mu, double sigma){
	 return mt.randNorm(mu,sigma);
 }

 double Function::timeNoise(double t){
	 return dRandNormal(0,2);

 }

 // ---------------------------- todo ---------------------------------- //
 double Function::D1(double P){
	 return -1*(P-params->Peq.at(0));	// mu * (P - P0)
 }

 double Function::D2(double P){	// sqrt(D(2)) = sigma
	 params->psigmalog = params->psigmalog -
			 (params->psigmalog - params->interma) * params->rkStepWidth +
			 fabs(params->intermb)*params->rkStepWidSqr*timeNoise(params->t); // Intermittent noise
	 return exp(params->psigmalog);	// intermittent
	 //return params->getSigmaQuad(); // Gaussian Noise
	 // should not be as large as P0 (the prevent negative data) //why? => positive wind speed
 }


 double Function::langevinIntegration(int i){
	 return D1(pg->getPowerMech(i)) *params->rkStepWidth +
	 			D2(pg->getPowerMech(i)) *params->rkStepWidSqr * timeNoise(params->t); // warning second timestep must be sqroot
 }

 double Function::drawPoissonJumpTime(){
	 return params->inttime +
			 -1*params->invPoiRate*log(1-dRand(0.0,1.0-1e-16));
 }

 double Function::poissonProcess(){
//	 std::cout << params-> inttime << "/" << params->poissonNextJump << std::endl;
	 if(params->inttime > params->poissonNextJump){
		 	 params->poissonNextJump = drawPoissonJumpTime();
//			 std::cout << params->poissonNextJump << " ";
			 return dRandNormal(0,params->poissonStddev);
	 } else {
		 return 0;
	 }
 }

 double Function::drawTelegJumpTime(double r){
	 return params->inttime +
			 -1*r*log(1-dRand(0.0,1.0-1e-60));
 }

 double Function::telegProcess(){ 	// todo check new implementation
	 if(params->inttime > params->telegNextJump){		// todo: extend to i generators
		 if(params->Peq != params->Peq0 ){	// if the equilibrium state is higher or lower than normal

//				std::cout << "teleg Down"<<std::endl;
		 	 params->telegNextJump = drawTelegJumpTime(params->invTelegRup);		// draw next up jump time,
		 	 params->Peq.at(0) = params->Peq0.at(0);							// set the equilibrium state back
			 return -1*params->lastJumpHeight;									// and jump down
		 }
		 else{

//				std::cout << "teleg Up"<<std::endl;
		 	 params->telegNextJump = drawTelegJumpTime(params->invTelegRdown);		// draw next down jump time
//		 	 params->lastJumpHeight = dRandNormal(0,params->telegOffset.at(0));	// store the jump height
		 	 params->lastJumpHeight = params->telegOffset.at(0);				// store the jump height
		 	 params->Peq.at(0) += params->lastJumpHeight*params->fluctuating;	// set the equilibrium state higher
			 return params->lastJumpHeight;										// and jump up
		 }
	 } else {
		 return 0;
	 }
 }


// void Function::propagateP(int i){	// todo revive
//	 params->P[i] = params->P[i] +
//			 params->fluctuating*(			// if fluctuating at all
//					 langevinIntegration(i)	// standard langevin process
//					 );
//	 if(params->poisson == 1){
//		 params->P[i] = params->P[i] + params->fluctuating*poissonProcess();
//	 }
//	 if(params->telegraphic == 1){
////		 std::cout << "teleg"<<endl;
//		 params->P[i] = params->P[i] + params->fluctuating*telegProcess();
//	 }
////	 std::vector<double> Pn;
////	 Pn.push_back(P[i]);
//
////	 params->Pt.push_back(Pn);
//	 params->inttime = params->inttime + params->rkStepWidth;
////	 std::cout << params->inttime;
// }




