/*
 * Param.cpp
 *
 *  Created on: Jan 12, 2016
 *      Author: mawolff
 */

#include "Param.h" // The parameter object with information for calculating swing equation
#include <stdlib.h>
#include <fstream>
#include <math.h>
#include <tgmath.h>	// exp10
#include <string.h>
#include <algorithm>
#include <cctype>
#include <locale>
#include <functional>
#include <vector>
//#include "CodeUtils.h"	// some methods are not in Param:: and hence dont see Param.h : CodeUtils.h
//#include "vector.h"


using namespace std;


Param::Param(void){
	len = 1;					// number of dgls
	convNum = 1;				// count as conv when om end lower than this
	RKsteps = 100;				// total number of RK steps
	saveStepTraj = 98;			// save the step 98
	saveptsTraj = 2;			// how often its saved
	kMax = 8;
	draws = 500;				// how many draws in random mode
	rkStepWidth = 0.01;			// step width for RK algorithm
	filenum = "00";
	currentNet = 0;
	maxNet = 1;
	KSN = 8;
	connectProb = 0.1;
	sigmaStep = 0;
	intermb = 0;
	bstart = 0;
	intermb0 = intermb;
	blast = 1;
	fluctuating = 0;
	psigmaQuad = 0.0001;
	psigma = 0.01;
	interma = log(psigma);
	gammar = 0;
	rkStepWidSqr = 0;			// wurzel stepwidth
	version = 0;
	simtime = 0;
	graphv = false;
	refnode = 0;
	sigmaMax = 1;
	poisson = 0;
	setPoissonRate(0.1);
	poissonNextJump = rkStepWidth;
	telegNextJump = rkStepWidth;
	poissonStddev = 1;
	inttime = 0;
	telegraphic = 0;
	param = true;
	setRandSeed(time(NULL));
	randSeedStart = randSeed;
//	paramIteration = 0;
	stepwDistNode = 1;
//	stepsDistNode = (lastDistNode-firstDistNode)/stepwDistNode + 1;
	stepsDistNode = 1;
	totStepLen = new int[2];// todo: update this one, when new parameters come
	totStepLen[0] = 1;
	totStepLen[1] = stepsDistNode;
	curDraw = 1;
	setSaveOmsum(1e10);
	setSaveOrdpar(1e10);
	saveptsOmsum = RKsteps;
	saveptsOrdpar = RKsteps;
	enthresh = 0.1;
	enconv = false;
	henconv = false;
	enrt = INFINITY;
	henrt = INFINITY;
	// ======= power perturbation ===== //
	ppRKSteps = 0;		// steps with heaviside perturbed power 	// default none
	ppsteps = 1;
	deltaPstrengh = 1;	// increase of power perturbation for every step
	deltaPstep = 1;		// how many different power perturbations	// default 1 (none)
	// ====== assume no mode before user sets one ===== //
	mode_EN_THRESH = false;
	mode_RASTER_MRT = false;
	mode_NRonly = false;
	mode_RASTER = false;
	mode_MAXP = false;
	mode_MAXP_only = false;
	mode_STRAJ = false;
	mode_POWER_PERT = false;
	mode_TEST_STOCH = false;
	setModeSif(false);
	setModeIterativeNetgen(false);
	setModeRewireNetgen(false);
	setModeMeanRetTimes(false);
	setModeRasterRt(false);
	setModeStochInp(false);
	setModeSurvivall(false);
	setModeSurv(false);
	setModeStochCASC(false);
	setModeStepFun(false);
	setModeGivenSeries(false);
	setModeStochLf(false);
	// ================== mode end ================= //
	// ============================= old ============================= //
	// todo : check following if correct starting state
	edges = 1;
	psigmalog = 1;
	D1 = 1;
	D2 = 1;
	t = 0;
}

	// --------------------------------------- read methods ---------------------------------------//


	/** //todo update list
	 * See latex script for list of all parameters
	 * Reads all parameters from an ASCII-file specified by keywords (marked with +):
	 * + Mode
	 * + Number of nodes
	 * 		This should be set at the beginning
	 * + Version
	 * 		not necessary but throws warning if input is not compatible with current version
	 */
	bool Param::readParamsFromFile(string pfile){
		ifstream ins;
		ins.open(CodeUtils::Str2Char(pfile));
		if(ins.is_open()){
			std::cout << "-------------------- Param reading "
					  << " --------------------" << std::endl;
			vector<double> dv;
			vector<int> iv;
			string line = "0";
			std::string linepar = "";
			bool waspar;
			while(line.size() > 0){
				getline(ins,line);
				linepar = line;
//				cout << "reading " << line << endl;
				waspar = true;
				if( line.substr(0,2) == "//")
					continue;

//				if( lineIs(line,"Mode") ){
//					getline(ins,line);
//					mode = lineToIntVec(Str2Char(line)).at(0);
//				}

				else if( CodeUtils::lineIs(line , "Number of nodes" ) ){
					len = CodeUtils::readint(ins);
				}

				// =============================== undefined ================================ //
				else if ( CodeUtils::lineIs(line, "Single Node K") ){
					KSN = CodeUtils::readDouble(ins);
				}
				else if ( CodeUtils::lineIs(line, "Kmax" ) ){
					kMax = CodeUtils::readint(ins);
				}
				else if ( CodeUtils::lineIs(line, "Number of b") ){
					blast = CodeUtils::readint(ins);
				}
				else if ( CodeUtils::lineIs(line, "Number of gen nets") ){
					maxNet = CodeUtils::readint(ins);
				}
				else if( CodeUtils::lineIs(line, "Edges")){
					edges = CodeUtils::readint(ins);
				}
				else if( CodeUtils::lineIs(line, "Draws") ){
					draws = CodeUtils::readint(ins);
				}
				else if( CodeUtils::lineIs(line,"Save omsum every")){
					setSaveOmsum(CodeUtils::readDouble(ins));
				}
				else if( CodeUtils::lineIs(line,"Save ordpar every")){
					setSaveOrdpar(CodeUtils::readDouble(ins));
				}
//				else if( CodeUtils::lineIs(line,"Pert vec om")){
//					getline(ins,line);
//					setPertVecOm(line);
//				}

				// =============================== basin stability (?) ================================ //
				else if( CodeUtils::lineIs(line, "Convergence value") ){
//					getline(ins,line);
					convNum = CodeUtils::readDouble(ins);
				}
//				else if( CodeUtils::lineIs(line, "Start SB node") ){
////					getline(ins,line);
//					firstDistNode = CodeUtils::readDouble(ins);
//				}
//				else if( CodeUtils::lineIs(line, "End SB node") ){
////					getline(ins,line);
//					lastDistNode = CodeUtils::readDouble(ins);
//				}

				// =============================== simulation parameters ================================ //
				else if ( CodeUtils::lineIs(line, "RK steps") ){
					RKsteps = CodeUtils::readint(ins);
				}
				else if ( CodeUtils::lineIs(line, "RK stepwidth") ){
					setStepWidth(CodeUtils::readDouble(ins));
				}
				else if ( CodeUtils::lineIs(line, "Save RK every") ){
					saveStepTraj = CodeUtils::readint(ins);
					saveptsTraj = ceil(((double)RKsteps) / ((double)saveStepTraj));
				}

				// =============================== program input ================================ //
				else if( CodeUtils::lineIs(line, "Param")){
					int yn = CodeUtils::readint(ins);
					if(yn == 0){
						param = false;
					} else {
						param = true;
					}
				}
				else if( CodeUtils::lineIs(line,"Random seed")){
					setRandSeed(CodeUtils::readint(ins));
					randSeedStart = randSeed;
				}
				else if ( CodeUtils::lineIs(line, "Version") ){
					int ver = CodeUtils::readint(ins);
					if( ver!= version){
						cout << "warning! Version of input does not match program version" << endl;
					}
				}
				else if( CodeUtils::lineIs(line, "Graphviz") ){
					graphv = true;
				}
				else if ( CodeUtils::lineIs(line, "Output filename") ){
					getline(ins,line);
					filenum = line;
					filenumInit = filenum;
				}

				// =============================== Deterministic power pert ================================ //
//				else if( lineIs(line , "Power pert vec" ) ){		// todo revive this
//					getline(ins,line);
//					dv = CodeUtils::lineToDoubleVec(CodeUtils::Str2Char(line));
//					if( dv.size() == 1){ 	// if only perturbed node given not for all nodes individually
//						DeltaP.clear();
//						for( int i = 0; i < len; i++){
//							DeltaP.push_back(0); 	// none is perturbed
//						}
//						DeltaP.at(dv.at(0)) = 1;	// except of given node
//						firstDistNode = dv.at(0);
//					} else {
//						DeltaP = dv;
//						for(int i = 0; i < DeltaP.size(); i++){
//							if( DeltaP.at(i) > 0){
//								firstDistNode = i;
//								break;
//							}
//						}
//					}
//				} // the time span for every node individually
				else if( CodeUtils::lineIs(line , "Power pert span vec" ) ){	// todo no use implemented yet
//					getline(ins,line);
					CodeUtils::readDVec(ins, &DeltaPtspan);
				}
				else if( CodeUtils::lineIs(line , "Power pert init" ) ){
//					getline(ins,line);
					deltaPstrengh = CodeUtils::readDouble(ins);
				}
				else if( CodeUtils::lineIs(line , "Power pert steps" ) ){
//					getline(ins,line);
					ppsteps = CodeUtils::readint(ins);
				}
				else if( CodeUtils::lineIs(line , "Power pert step width" ) ){
//					getline(ins,line);
					deltaPstep = CodeUtils::readDouble(ins);
				}
				else if( CodeUtils::lineIs(line , "Power pert span" ) ){	// hint: needs rk step width for time unit
//					getline(ins,line);
					setPPRKSteps(CodeUtils::readDouble(ins));
				}
				// ================================ Return times ======================================== //
//				else if( lineIs(line,"Pert vec th")){
//					getline(ins,line);
//					setPertVecTh(line);
//				}

	// ================================ Energy function ===================================== //
				else if( CodeUtils::lineIs(line,"Energy thresh") ){
					enthresh = CodeUtils::readDouble(ins);
				}

	// ================================ old stochastic input ================================ //
				else if( CodeUtils::lineIs(line, "Poisson process") ){
					poisson = 1;
					// followed by rate and stddev
//					getline(ins,line);
					poissonRate = CodeUtils::readDouble(ins);
//					getline(ins,line);
					poissonStddev = CodeUtils::readDouble(ins);
				}
				else if( CodeUtils::lineIs(line, "Telegraphic process") ){
					telegraphic = 1;
					telegOffset.clear();
					// followed by rate and stddev
					telegRup = CodeUtils::readDouble(ins);
					telegRdown = CodeUtils::readDouble(ins);
					setTelegRate(telegRup,telegRdown);
					telegStddev = CodeUtils::readDouble(ins);
					telegOffset.push_back(CodeUtils::readDouble(ins));
				}
				else if( CodeUtils::lineIs(line, "Standard deviation") ){
//					getline(ins,line);
					setSigma(CodeUtils::readDouble(ins));
				}
				else if ( CodeUtils::lineIs(line, "Intermittent parameter") ){
//					getline(ins,line);
					intermb0 = CodeUtils::readDouble(ins);
					setIntermit(intermb0);
				}
				else if ( CodeUtils::lineIs(line, "Number of stddev values") ){
					sigmaMax = CodeUtils::readint(ins);
				}
				else if ( CodeUtils::lineIs(line, "Stddev stepwidth") ){
					getline(ins,line);
					sigmaStep = CodeUtils::readDouble(ins);
					if(sigmaMax > 1)
						psigma = psigma - sigmaStep;	// is increased once before used
				}

				// ================================ read modes ======================================== //
				// ------------------------------- stochastic power input ----------------------------- //
				else if( CodeUtils::lineIs(line,"Mode stoch inp") ){	// using stochastic power generally
					setModeStochInp(true); 								// necessary for following modes
				}
				else if( CodeUtils::lineIs(line,"Mode sif") ){			// stochastic interpolation of
					setModeSif(true);									// FINO data (or other empirical)
					setModeStochInp(true);
				}
				else if( CodeUtils::lineIs(line,"Mode stepfunc") ){		// using FINO data without interpolation
					setModeStepFun(true);
					setModeStochInp(true);
				}
				else if( CodeUtils::lineIs(line,"Mode given series") ){	// uses given data directly as velocity
					setModeGivenSeries(true);
					setModeStochInp(true);
				}
				else if( CodeUtils::lineIs(line,"Mode surv") ){			// measure the exceedance
					setModeSurv(true);									// mistake was thought as survivability
				}
				else if( CodeUtils::lineIs(line,"Mode surv all") ){		// measure the exceedance with
					setModeSurvivall(true);								// threshold for all nodes
				}
				else if( CodeUtils::lineIs(line,"Mode stoch casc") ){	// calculate the flow in the lines
					setModeStochCASC(true);								// when using stochastic power
				}
				else if( CodeUtils::lineIs(line,"Mode stoch lf") ){		// calculate the flow in the lines
					setModeStochLf(true);								// when using stochastic power
				}
				// ------------------------------- phase freq perturbation ----------------------------- //
				else if( CodeUtils::lineIs(line,"Mode raster rt") ){	// using return times generally
					setModeRasterRt(true);								// necessary for following modes
				}
				else if( CodeUtils::lineIs(line,"Mode RMRT") ){ 		// using multiple thresholds for
					mode_RASTER_MRT = true;								// determining the return time
				}
				else if( CodeUtils::lineIs(line,"Mode save traj") ){	// todo comment
					mode_STRAJ = true;
				}
// todo old modes
				else if( CodeUtils::lineIs(line,"Mode ERT") ){
					mode_EN_THRESH = true;
				}
				else if( CodeUtils::lineIs(line,"Mode PPERT") ){
					mode_POWER_PERT = true;
				}
				else if( CodeUtils::lineIs(line,"Mode NRonly") ){
					mode_NRonly = true;
				}
				else if( CodeUtils::lineIs(line,"Mode Raster") ){
					mode_RASTER = true;
				}
				else if( CodeUtils::lineIs(line,"Mode maxp") ){
					mode_MAXP = true;
				}
				else if( CodeUtils::lineIs(line,"Mode test stoch") ){
					mode_TEST_STOCH = true;
				}
				else if( CodeUtils::lineIs(line,"Mode iterative net") ){
					setModeIterativeNetgen(true);
				}
				else if( CodeUtils::lineIs(line,"Mode rewire net") ){
					setModeRewireNetgen(true);
				}
				else if( CodeUtils::lineIs(line,"Mode mean rt") ){
					setModeMeanRetTimes(true);
				}
				// ================================ unknown parameter ===================================== //
				else {
//					cout << "unknown parameter :" << line << endl;
					waspar = false;
				}

				if( waspar )		// if the line was a parameter read by power grid class
					std::cout << "read " << linepar << std::endl;
			} // while(line.size() > 0){

//			resetMaxPert();		// todo when enabling maxpert
//			chooseFPIC();		// let program choose initial conditions
			if(mode_MAXP && !mode_RASTER_MRT && !mode_EN_THRESH && !mode_RASTER){
				mode_MAXP_only = true;	// TODO: maxp without raster
			}
			if( isModeStochCASC()){	// if stochastic load flow mode
//				calcCflow();	// pre calculate the constant load flow 	// todo check whether here or in stochinp
				// todo do this together with creation of powerPerturbation or so...
			}

			ins.close();
			return true;
		}
		return false;
	}

	void Param::initAll(int len){	// todo ??
		DeltaP.clear();
		for( int i = 0; i < len; i++){
			DeltaP.push_back(0); 		// none is perturbed
		}
	}


	// ------------------------------------ get and set methods ------------------------------------ //

	std::string Param::getOfile(std::string opt) const {
		if( opt.compare("path") == 0){
			int ls = filenum.find_last_of('/');		// todo copy to codeutil
			if( ls != -1)
				return filenum.substr(0,ls+1);
			return "";
		}
		if( opt.compare("file") == 0){
			int ls = filenum.find_last_of('/');
			if( ls != -1)
				return filenum.substr(ls+1,filenum.size());
		}
		return filenum;
	}


	/**
	 * sets the number of Runge Kutta steps with perturbed power
	 * HINT: rkStepWidth must be set first!
	 */
	void Param::setPPRKSteps(double tdur){
		ppRKSteps = tdur/rkStepWidth;
	}

	void Param::setSaveOmsum(double so){
		this->saveStepOmsum = so;
		this->saveptsOmsum = ceil(((double)RKsteps) / ((double)this->saveStepOmsum));
	}

	void Param::setSaveOrdpar(double so){
		this->saveStepOrdPar= so;
		this->saveptsOrdpar = ceil(((double)RKsteps) / ((double)this->saveStepOrdPar));
	}

	void Param::setPoissonRate(double r){
		poissonRate = r;
		invPoiRate = 1/r;
	}

	void Param::setTelegRate(double ru, double rd){
		telegRup = ru;
		telegRdown = rd;
		invTelegRup = 1/ru;
		invTelegRdown = 1/rd;
	}

	double Param::getSigmaQuad(){
		return psigmaQuad;
	}

	void Param::setSigma(double s){
		psigma = s;
		psigmaQuad = s*s;
		if(psigma == 0){
			fluctuating = 0;
		} else{
			fluctuating = 1;
			psigmalog = log(s);
		}
	}

	void Param::setStepWidth(double sw){	// step width for random process
		rkStepWidth = sw;
		rkStepWidSqr = sqrt(sw);
	}

	void Param::setIntermit(double b){
		intermb = b;
		double bquad = intermb*intermb;
		interma =  log(psigma) - bquad*0.5;
		gammar = exp(4*bquad) + 2 *exp(3*bquad) + 3 *exp(2*bquad) - 6;
	}

	void Param::setRandSeed(uint rs){
		randSeed = rs;
	}

	uint  Param::getRandSeed(){
		return randSeed;
	}

//	void Param::setPertVecOm(std::string line){
//		pertVecOm.clear();
//		pertVecOm = CodeUtils::lineToDoubleVec(CodeUtils::Str2Char(line));
//		if(pertVecTh.size() < pertVecOm.size()){
//			pertVecTh.clear();
//			for( uint i = 0; i < pertVecOm.size(); i++){
//				pertVecTh.push_back(0);
//			}
//		}
//		setPertVecGiven(true);
//	}
//
//	void Param::setPertVecTh(std::string line){
//		pertVecTh.clear();
//		pertVecTh = CodeUtils::lineToDoubleVec(CodeUtils::Str2Char(line));
//		if(pertVecOm.size() < pertVecTh.size()){
//			pertVecOm.clear();
//			for( uint i = 0; i < pertVecTh.size(); i++){
//				pertVecOm.push_back(0);
//			}
//		}
//		setPertVecGiven(true);
//	}
//	bool Param::isPertVecGiven() const {
//		return pertVecGiven;
//	}

//	void Param::setPertVecGiven(bool pertVecGiven) {
//		this->pertVecGiven = pertVecGiven;
//	}

//	int Param::getNrICs() const {
//		return nrICs;
//	}
//
//	void Param::setNrICs(int nrICs) {
//		this->nrICs = nrICs;
//	}

	bool Param::isNRonly(){
		return mode_NRonly;
	}


	bool Param::isModeERT(){
		return mode_EN_THRESH;
	}

	bool Param::isModePPert(){
		return mode_POWER_PERT;
	}

	bool Param::isModeRMRT(){
		return mode_RASTER_MRT;
	}

	bool Param::isModeMaxp() const {
		return mode_MAXP;
	}

	void Param::setModeMaxp(bool modeMaxp) {
		mode_MAXP = modeMaxp;
	}

	bool Param::isModeMaxpOnly() const {
		return mode_MAXP_only;
	}

	void Param::setModeMaxpOnly(bool modeMaxpOnly) {
		mode_MAXP_only = modeMaxpOnly;
	}

	bool Param::isModeStraj() const {
		return mode_STRAJ;
	}

	bool Param::isModeIterativeNetgen() const {
		return mode_ITERATIVE_NETGEN;
	}

	void Param::setModeIterativeNetgen(bool modeIterativeNetgen) {
		mode_ITERATIVE_NETGEN = modeIterativeNetgen;
	}

	bool Param::isModeMeanRetTimes() const {
		return mode_MEAN_RET_TIMES;
	}

	void Param::setModeMeanRetTimes(bool modeMeanRetTimes) {
		mode_MEAN_RET_TIMES = modeMeanRetTimes;
	}

	bool Param::isModeRasterRt() const {
		return mode_RASTER_RT;
	}

	void Param::setModeRasterRt(bool modeRasterRt) {
		mode_RASTER_RT = modeRasterRt;
	}

	bool Param::isModeRewireNetgen() const {
		return mode_REWIRE_NETGEN;
	}

	void Param::setModeRewireNetgen(bool modeRewireNetgen) {
		mode_REWIRE_NETGEN = modeRewireNetgen;
	}

	bool Param::isModeSif() const {
		return mode_SIF;
	}

	void Param::setModeSif(bool modeSif) {
		mode_SIF = modeSif;
	}


	bool Param::isModeSurvivall(){
		return mode_SURVIVALL;
	}

	void Param::setModeSurvivall(bool mode){
		mode_SURVIVALL = mode;
	}

	void Param::setModeStraj(bool modeStraj) {
		mode_STRAJ = modeStraj;
	}


	void Param::setModeStochCASC(bool mode){
		mode_STOCH_CASC = mode;
	}

	bool Param::isModeStochCASC() {
		return mode_STOCH_CASC;
	}

	bool Param::isTestStoch() {
		return mode_TEST_STOCH;
	}

//	void Param::setTestStoch(bool modeTestStoch) {
//		mode_TEST_STOCH = modeTestStoch;
//	}


	double Param::getEnThresh() {
		return enthresh;
	}

	void Param::setEnThresh(double nthr) {
		enthresh = nthr;
	}

	std::vector<double>& Param::getFreqmax()  {
		return freqmax;
	}

	void Param::setFreqmax(std::vector<double> freqmax) {
		this->freqmax = freqmax;
	}

	std::vector<double>& Param::getPhasmax()  {
		return phasmax;
	}

	std::vector<double>& Param::getFreqmaxTime()  {
		return freqmaxTime;
	}

	void Param::setFreqmaxTime(std::vector<double> freqmaxTime) {
		this->freqmaxTime = freqmaxTime;
	}

	std::vector<double>& Param::getPhasmaxTime()  {
		return phasmaxTime;
	}

	void Param::setPhasmaxTime(std::vector<double> phasmaxTime) {
		this->phasmaxTime = phasmaxTime;
	}

	void Param::setPhasmax(std::vector<double> phasmax) {
		this->phasmax = phasmax;
	}


	// ------------------------------------ other methods ------------------------------------ //

//	double smallerAngle(double angle){	// todo used anywhere?
//		angle = fmod(angle+6.28318530718,6.28318530718);
//		return min(angle, 6.28318530718-angle);
//	}


	void Param::resetFlanks(){
		for(uint i = 0; i < frupflanks.size(); i++){
			frupflanks.at(i).clear();
			frdownflanks.at(i).clear();
		}
		frupflanks.clear();
		frdownflanks.clear();

		frupflanks.reserve(len);
		frdownflanks.reserve(len);
		for(int i = 0; i < len; i++){
			std::vector<double> nu;
			std::vector<double> nd;
			frupflanks.push_back(nu);
			frdownflanks.push_back(nd);
		}
	}

//	/**
//	 * check if lines are overloaded and remove them from the grid
//	 */
//	void Param::checkLines(double* voltPhases){
//
//		// TODO remove, this is wrong
//		double flowij = 0;
//
//		Param::linkline *K12, *K21;
//		int ii, ji;
//		std::complex<double> phas12, phas21;
//
//		for( int i = 0; i < K.size(); i++){
//			for( int j = 0; j < K.at(i).size(); j++){
//
//				K12 = &(K.at(i).at(j));
//				ii = K12->thisNode;
//				ji = K12->conNode;
//				K21 = findCon(ji,ii);
//
//				phas21 = complex<double>(0,voltPhases[ji]-voltPhases[ii]-K21->admitAngle);
//				phas12 = complex<double>(0,voltPhases[ii]-voltPhases[ji]-K12->admitAngle);
//
//				flowij = std::abs( K12->cflow
//						+ K21->couplingStrOnline * exp( phas21 )
//						- K12->couplingStrOnline * exp( phas12 )
//				);
//
//				if( flowij > K12->capacity ){	// if line is overloaded
//					K12->couplingStrOnline = 0;	// "remove" the line (in both directions)
//					K21->couplingStrOnline = 0;
//				}
//			}
//		}
//	}



	// ------------------------------------ persistence methods ------------------------------------ //
	bool Param::toGraphViz(){
		string fname = "GraphViz" + filenum;
		std::ofstream   outfile(fname ,ios::out|ios::app);

//		outfile << "digraph G {" << endl;		// todo
//		for( int i = 0; i < len ; i++){
//			for( uint j = 0; j < K.at(i).size(); j++){
//				if( i < K.at(i).at(j).conNode){
//					outfile << i << " -> ";
//					outfile << K.at(i).at(j).conNode << ";" <<endl;
////					outfile << K.at(i).at(j).couplingStrength;
//				}
//			}
//		}
//		outfile << "}";

		outfile.flush();
		outfile.close();
		return true;

	}

	// trim from start
	std::string& Param::ltrim(std::string &s) {
	    s.erase(s.begin(), std::find_if(s.begin(), s.end(),
	            std::not1(std::ptr_fun<int, int>(std::isspace))));
	    return s;
	}

	// trim from end
	std::string& Param::rtrim(std::string &s) {
	    s.erase(std::find_if(s.rbegin(), s.rend(),
	            std::not1(std::ptr_fun<int, int>(std::isspace))).base(), s.end());
	    return s;
	}

	// trim from both ends
	std::string& Param::trim(std::string &s) {
	    return ltrim(rtrim(s));
	}


bool Param::toFile(){			// TODO
	if( !param){				// if user don't want parameters to be printed
		return false;			// return so they won't be printed
	}
//	const int BUFFER_LEN = 100;
	string fname = getOfile("path") + "Param" + getOfile("file");	// construct name for the parameter file
	std::remove(CodeUtils::Str2Char(fname));			// remove possible old versions of the parameter file
	std::ofstream   outfile(fname ,ios::out|ios::app);	// open the stream

	outfile << "========================= Param =========================" 	<< endl;
	outfile << "---------------- Calculation Parameters -----------------" 	<< endl;	// todo: more params here
	outfile << "RungeKuttaSteps: " 		<< RKsteps 			<< endl;
	outfile << "RungeKuttaStepWidth: " 	<< rkStepWidth 		<< endl;
	outfile << "Random Seed (start): " 	<< randSeedStart 	<< endl;
	outfile << "Random Seed (end): " 	<< randSeed 		<< endl;
	outfile 												<< endl;

	//	outfile << "Newton-Raphson precision: " << nrprecision << endl;

//	outfile << "---- Version ----" << endl;

	outfile << "----------------------- Simulation ----------------------" 	<< endl;
	outfile << "Version: " 			<< version 	<< endl;
	outfile << "TimeTaken(sec): " 	<< simtime 	<< endl;
	outfile 									<< endl;

	// todo: if isMode ... corresponding ... toFile

//	outfile << "---- Simulation Parameters ----" << endl; 	// todo power grids print
//	outfile << "Damping: " << dampConst << endl;			// print out damping constant
//	outfile << "Mech Power (P0): ";							// ... old power see below
//	for( int i = 0; i < Pm0.size(); i++){
//		outfile << Pm0.at(i) << " ";
//	}
//	outfile << endl;
//	outfile << "Mech Power (P): ";
//	for( int i = 0; i < Pm.size(); i++){
//		outfile << Pm.at(i) << " ";
//	}
//	outfile << endl;
//	outfile << "Diag Power (Pd): ";
//	for( int i = 0; i < Pdiag.size(); i++){
//		outfile << Pdiag.at(i) << " ";
//	}
//	outfile << endl;
//	outfile << "Inertia (2H/omega_R): ";
//	for( int i = 0; i < len; i++){
//		outfile << 1/Hparinv.at(i) << " ";
//	}
//	outfile << endl;
//	outfile << "Steady state angles: " << endl; // todo power grids
//	char* buffr = new char[BUFFER_LEN];
//	for(int i = 0; i < len; i++){
//
//		sprintf(buffr,"%1.10e",thstable[i]);
//		outfile << buffr << " ";
//	}
//	outfile << endl;
//	delete[] buffr;
//	outfile << "Draws: " << draws << endl;
//	outfile << "Convergence value: " << convNum << endl;
//	outfile << "Perturbation delay: " << pertDelay << endl;

		// modes here todo

//	outfile << "---- Simulation Parameters (Stochastic) ----" << endl;
//	outfile << "Fluctuation: " << fluctuating << endl;
//	if(fluctuating == 1){
//		outfile << "StandardDeviationPower: " << psigma << endl;
//		outfile << "IntermittentB: " << intermb << endl;
//		outfile << "gamma: " << gammar << endl;
//		outfile << endl;
//	}
//	outfile << "Poisson process: " << poisson << endl;
//	if(poisson == 1){
//		outfile << "Poisson rate: " << poissonRate << endl;
//		outfile << "Peak height standard deviation: " << poissonStddev << endl;
//		outfile << endl;
//	}
//	outfile << "telegraphic: " << telegraphic << endl;
//	if(telegraphic == 1){
//		outfile << "Telegraphic up rate: " << telegRup << endl;
//		outfile << "Telegraphic down rate: " << telegRdown << endl;
//		outfile << "Peak height standard deviation: " << telegStddev << endl;
//		outfile << endl;
//	}
//	outfile << endl;

//	outfile << "---- Coupling Matrix ----" << endl;
//	outfile << "Connections: " << endl;

//	for(int i = 0; i < BUFFER_LEN; i++ ){
//		buffr[i] = 0;
//	}

//	sprintf(buffr,"%5s%5s%5s%11s%10s","num","from","to","K","Ygamma");
	//			outfile << buffr;
//	outfile << buffr << endl;
//	for( int i = 0; i < len ; i++){	// TODO print this in power grids
//		for( uint j = 0; j < K.at(i).size(); j++){
//			//					outfile << i << " -> ";
//			//					outfile << K.at(i).at(j).conNode << " ";
//			//					outfile << K.at(i).at(j).couplingStrength;
//			sprintf(buffr,"%5d",i);
//			outfile << buffr;
//			sprintf(buffr,"%5d",K.at(i).at(j).thisNode);
//			outfile << buffr;
//			sprintf(buffr,"%5d",K.at(i).at(j).conNode);
//			outfile << buffr;
//			sprintf(buffr," %1.4e",K.at(i).at(j).couplingStrength);
//			outfile << buffr;
//			sprintf(buffr,"%10.2f",K.at(i).at(j).angle);
//			outfile << buffr;
//			outfile << endl;
//		}
//	}
//	delete[] buffr;

	outfile << endl;

//	bool debug = false;
//	if( debug){
//		outfile << "---- Debug view ----" << endl;
//		outfile << "om0 " <<om0[0] <<endl;
//		outfile << "th0 " <<th0[0] <<endl;
//		outfile << "ome " <<ome[0] <<endl;
//		outfile << "the " <<the[0] <<endl;
//		outfile << "oms " <<oms <<endl;
//		outfile << "ths " <<ths <<endl;
//					outfile << " " << <<endl;
//	}


	outfile.flush();
	outfile.close();

	return true;
}

///**
// * Saves a vector to a binary file.
// * @param vec : the vector to be saved
// * @param fname : the name of the file the array should be saved in
// */
//void Param::vecToBinFile(std::vector<double> vec, std::string fname){
//    std::ofstream   outfile(fname.c_str(),
//    		std::ios::out|std::ios::app|std::ios::binary);
//    for(int i = 0; i < vec.size(); i++)
//    	outfile.write((char*) &vec.at(i), sizeof vec.at(i));
//    outfile.close();
//
//}

bool Param::writeMaxP() {
	CodeUtils::vecToBinFileFast(freqmax, (getOfile("path")+"frequency/freqMax"
					+getOfile("file")));
	CodeUtils::vecToBinFileFast(freqmaxTime, (getOfile("path")+"frequency/freqMaxTime"
					+getOfile("file")));
	CodeUtils::vecToBinFileFast(phasmax, (getOfile("path")+"phase/phaseMax"
					+getOfile("file")));
	CodeUtils::vecToBinFileFast(phasmaxTime, (getOfile("path")+"phase/phaseMaxTime"
					+getOfile("file")));
	return true;
}

bool Param::isModeStochInp() const {
	return mode_STOCH_INP;
}

void Param::setModeStochInp(bool modeStochInp) {
	mode_STOCH_INP = modeStochInp;
}

bool Param::isModeSurv() const {
	return mode_SURV;
}

void Param::setModeSurv(bool modeSurv) {
	mode_SURV = modeSurv;
}

bool Param::isModeStepFun() const {
	return mode_STEP_FUN;
}

void Param::setModeStepFun(bool modeStepFun) {
	mode_STEP_FUN = modeStepFun;
}

bool Param::isModeGivenSeries() const {
	return mode_GIVEN_SERIES;
}

void Param::setModeGivenSeries(bool modeGivenSeries) {
	mode_GIVEN_SERIES = modeGivenSeries;
}

bool Param::isModeStochLf() const {
	return mode_STOCH_LF;
}

void Param::setModeStochLf(bool modeStochLf) {
	mode_STOCH_LF = modeStochLf;
}
/*					while( !strcmp(Str2Char(line),"end")){	// good alternative
 * + Links
 * 		set number of nodes first; this reads twice as many lines as nodes exists
 * 		the first line are the connected nodes the second are the admittances*/

// OLD CODE

//
//else if( lineIs(line,"Vel2Pow") ){
//	v2P = CodeUtils::readDouble(ins);
//}
//	outfile << "Power (P0): ";							// ...
//	for( int i = 0; i < len; i++){
//		outfile << P0[i] << " ";
//	}
//	outfile << endl;
//	outfile << "Power (P): ";
//	for( int i = 0; i < len; i++){
//		outfile << P[i] << " ";
//	}
