/*
 * Param.h
 *
 *  Created on: Jan 11, 2016
 *      Author: mawolff
 */

#ifndef PARAM_H_
#define PARAM_H_

#include <iostream>
#include <string>
#include <sstream>
#include <vector>
#include <complex>
#include "CodeUtils.h"
//#include <tgmath.h>

/**
 * Class Power Grid Model
 * Parameter object storing all information needed to calculate the
 * trajectory of a power grid with the swing equation for different
 * analysis methods
 */
class Param{
	typedef unsigned int uint;

private:


// ============================= (raster) return times ============================= //
	bool mode_RASTER_RT;		// MAIN MODE. Return times are obtained for a grid of initial conditions. The class RasterReturnTime is used.
	bool mode_RASTER_MRT;		// raster all IC and calculate return times with multiple thresholds

	// todo refresh following sub modes
	bool mode_MEAN_RET_TIMES;	// return times are averaged for smoothing RT surfaces
	bool mode_EN_THRESH;		// raster all IC(initial conditions) and calculate return time of energy function

	bool mode_POWER_PERT;		// if power should be perturbed
	bool mode_MAXP_only;		// store max perturbation
	bool mode_MAXP;				// same but can be combined with other modes (MRT)

	bool mode_NRonly;			// calculate the fixed point only

// ============================= stochastic simulations ============================= //
	bool mode_STOCH_INP;		// MAIN MODE. If at least one node generates stochastic power. The class StochasticInput is used.
	bool mode_SIF;				// Stochastic interpolation of Fino data
	bool mode_STEP_FUN;			// use the given wind series as velocities per second and do not interpolate them
	bool mode_GIVEN_SERIES;		// use the given wind series as velocities on simulation time scale

	bool mode_SURV;				// measure exceedance independently for each node
	bool mode_SURVIVALL;		// measure exceedance for all nodes (stay below threshold)
	bool mode_STOCH_CASC;		// iterating a stochastic process and checking lines for failure
	bool mode_STOCH_LF;			// iterating a stochastic process and analyzing the line flows

	// todo currently not working
	bool mode_TEST_STOCH; 		// if this is set, only the stochastic series is calculated and print to a file

// ============================= multiple application ============================= //
	bool mode_STRAJ;			// store trajectories in raster return mode or stochastic simulation

// ============================= todo sort ============================= //
	bool mode_ITERATIVE_NETGEN;	// if a random grid should be used and created iteratively
	bool mode_REWIRE_NETGEN;	// if a random grid should be used and created by rewiring

	bool mode_RASTER;			// ??





public:

	//static constexpr double pi = 3.14159265359;		// define constants and variables

	bool graphv,
			param,			// if the parameters should be printed to a file
			writeOmsum,		// if the sum of the frequencies of all nodes should be printed into a file
			writeOrdPar,
			writeTraj,
			writeConv,
			pertVecGiven
			;
	double
			kMax,
			connectProb,
			KSN,
			gammar,
// ============================= simulation variables ============================= //
			convNum,			// convergence criteria
			rkStepWidth,		// step width of runge kutta iteration
			rkStepWidSqr,		// square root of the runge kutta step width
			simtime,			// real time process is running
			enthresh,			// threshold for the return time of the energy function
// ============================= old stochastic variables ============================= //
			psigma,
			psigmalog,
			sigmaStep,
			interma,
			intermb,
			intermb0,
			poissonRate,
			invPoiRate,
			poissonStddev,
			telegStddev,
			telegRup,
			telegRdown,
			invTelegRup,
			invTelegRdown,
			telegNextJump,
			lastJumpHeight,
			poissonNextJump,
			inttime,					//integration time of langevin process
			D1,
			D2,
			t,		// no effect yet, for time white noise later
			// ============================= power pert variables ============================= //
			deltaPstrengh,	// strength (multiplicator) of power perturbation (for all nodes)
							// use 1 for having exact the deltaP vector
			deltaPstep,		// if multiple strengths should be used this
							// is the stepwidth between them see "int ppsteps"
			deltaPts		// duration (time span) of power perturbation
				;
	int
//		paramIteration,	// the current iteration step corresponding all parameters
		version,		// to know which program version made the output
	// ============================= simulation variables ============================= //
		len,			// length of arrays, number of nodes
		RKsteps,		// how many Runge Kutta steps should be made
		saveStepTraj,	// at which points/times the trajectory should be saved
		saveStepOrdPar,	// at which points/times the order parameter should be saved
		saveStepOmsum,
		saveptsTraj,
		saveptsOmsum,
		saveptsOrdpar,
		curDraw,
		draws,
		currentNet,
		maxNet,
		edges,

//		survInterval,		// the number of RK steps measuring the survivability	// todo comment move
							// if the number is smaller than simulation length the survivability is
							// measured in multiple intervals separately
		// ============================= stochastic variables ============================= //
		sigmaMax,
		bstart,				// the beginning value of intermittence parameter and the last value
		blast,
		fluctuating,
		stepwDistNode,		// The step width for the label of the disturbed node
		stepsDistNode,		// The total number of steps for disturbed node
		refnode,			// reference node whose angle is always zero
		telegraphic,		// 1 if telegraphic Process 0 otherwise
		poisson,			// 1 if poisson process 0 otherwise (Indicator value)
		ppsteps,			// number of different power perturbation amplitudes
		ppRKSteps			// number of Runge Kutta steps of power perturbation
							// is calculated when setting deltaPts (see above)
			;

	uint randSeed;
	uint randSeedStart;

	std::vector<double> DeltaP;				// vector of power disturbance at each node
	std::vector<double> DeltaPtspan;		// time spans of power perturbation
	int* totStepLen;
	std::vector< std::vector<double> > Pt;
	std::vector< double> Peq;			// current equilibrium power when stochastic modeling
										// meaning when telegraphic noise appears the stochastic power
										// will tend to this value
	std::vector< double> Peq0;			// when telegraphic noise jumps back it jumps to this value (+Noise)
	std::vector< double> telegOffset;		// todo: what does this do? // high part of teleg noise?
//	std::vector<double> Hparinv;		// (2*H/omega_R)^-1 to save the calculation (1/2H...) every iteration
//	std::vector<double> Hpar;			// (2*H/omega_R)
//	std::vector<double> thfixg;			// guess for the fixed point
	std::vector<double> freqmax;		// stores the strongest frequency perturbation within one trajectory
	std::vector<double> freqmaxTime;	// stores the corresponding time
	std::vector<double> phasmax;		// stores the strongest phase perturbation within one trajectory
	std::vector<double> phasmaxTime;	 // stores the corresponding time
//	std::vector<double> surviv;			// survivability measure (number of time steps below freq. threshold, for every node)
	std::vector<double> maxload;		// stores the max. load on lines
	std::vector<double> maxloadp;		// stores the max. percentage load on lines
	std::vector<int> maxlij;			// stores alternating i and j the end nodes of the max loaded line

	std::vector<std::vector<double> > frupflanks;		// for reconstructing when the fluctuating frequency rises above threshold
	std::vector<std::vector<double> > frdownflanks;		// for reconstructing when the fluctuating frequency falls below threshold
	std::vector<std::vector<double> > phupflanks;		// for reconstructing when the fluctuating phase rises above threshold
	std::vector<std::vector<double> > phdownflanks;		// for reconstructing when the fluctuating phase falls below threshold

	bool enconv;			// if the energy trajectory is already under threshold
	bool henconv;			// if harmonic energy
	double** konvOut;
	double enrt;			// return time of the energy trajectory
	double henrt;			// return time of the harmonic energy trajectory

	std::string filenum;
	std::string filenumInit;
//	std::string stochFileName;


public:
	Param();

	// ---- get&set ----//
	double getKVal();
	double getSigmaQuad();
	uint getRandSeed();
	void setRandSeed(uint rs);
	std::string getOfile(std::string str) const;
	double getEnThresh();


	void setSigma(double sig);
	void setStepWidth(double sw);
	void setIntermit(double b);
	void setPoissonRate(double r);
	void setTelegRate(double ru, double rd);
	void setSaveOmsum(double so);
	void setSaveOrdpar(double sop);
//	void setPertVecOm(std::string line);
//	void setPertVecTh(std::string line);
//	void setPertVecGiven(bool pvg);
	void setEnThresh(double nthr);
	void setNRonly(bool nr);
	void setModeMaxp(bool modeMaxp);
	void setModeMaxpOnly(bool modeMaxpOnly);
	void setPPRKSteps(double tdur);


	std::vector<double>& getFreqmax() ;
	void setFreqmax(std::vector<double> //
			freqmax);
	std::vector<double>& getPhasmax() ;
	void setPhasmax(std::vector<double> // stores the strongest frequency perturbation within one trajectory
			phasmax);
	std::vector<double>& getFreqmaxTime() ;
	void setFreqmaxTime(std::vector<double> // stores the strongest frequency perturbation within one trajectory
			freqmaxTime);
	std::vector<double>& getPhasmaxTime() ;
	void setPhasmaxTime(std::vector<double> // stores the strongest phase perturbation within one trajectory
			phasmaxTime);

	// ---- getMode ----//
	bool isModeIterativeNetgen() const;
	void setModeIterativeNetgen(bool modeIterativeNetgen);
	bool isModeMeanRetTimes() const;
	void setModeMeanRetTimes(bool modeMeanRetTimes);
	bool isModeRasterRt() const;
	void setModeRasterRt(bool modeRasterRt);
	bool isModeRewireNetgen() const;
	void setModeRewireNetgen(bool modeRewireNetgen);
	bool isModeRMRT();
	bool isModeERT();
	bool isNRonly();
	bool isModeMaxp() const;
	bool isModeMaxpOnly() const;
	bool isModePPert();
	bool isTestStoch();
	bool isModeStraj() const;
	void setModeStraj(bool modeStraj);
	bool isModeSif() const;
	void setModeSif(bool modeSif);
	bool isModeSurvivall();
	void setModeSurvivall( bool modeSurv);
	bool isModeStochCASC();
	void setModeStochCASC( bool modeStochLF);
	bool isModeStochInp() const;
	void setModeStochInp(bool modeStochInp);
	bool isModeSurv() const;
	void setModeSurv(bool modeSurv);
	bool isModeStepFun() const;
	void setModeStepFun(bool modeStepFun);
	bool isModeGivenSeries() const;
	void setModeGivenSeries(bool modeGivenSeries);
	bool isModeStochLf() const;
	void setModeStochLf(bool modeStochLf);

	// ---- read&Init ---- //

	bool readParamsFromFile(std::string s);

	void initK();
	void initAll(int len);	// init all length dependent variables	// todo remove this stuff

	// ---- other methods ---- //

//	void consoleK();
//	void connectNodes(int i, int j);
//	void connectNodes(int i, int j, double admittance);
//	int connectionExists(int i, int j);
//	bool removeConnection(int i, int j);
//	bool rewire(int i1, int j1, int j2);
//	double meanNodeDegree();
//	double modfun(double x);
//	void resetSurv();
//	void countSurv(double* freq);
//	void resetSurvAll();
//	void countSurvAll(double* freq);
	void resetFlanks();

//	bool hasNextParam();
//	bool nextParam();

	std::string getUniqueFileName();

	static inline std::string& ltrim(std::string &s);
	static inline std::string& rtrim(std::string &s) ;
	static inline std::string& trim(std::string &s);

	void resetMaxPert();


	// ---- persistence ---- //
//	void vecToBinFile(std::vector<double> vec, std::string fname);
	bool toFile();
	bool toGraphViz();
	bool writeMaxP();

private:
	double
		psigmaQuad;
};




#endif /* PARAM_H_ */
