/*
 * PowerGrid.cpp
 *
 *  Created on: Feb 22, 2019
 *      Author: mawolff
 */

#include "PowerGrid.h"
//#include "TransmissionLine.h"

//#include "Param.h"


#include "CodeUtils.h"

#include <iostream>
#include <fstream>

PowerGrid::PowerGrid() {
	numNodes = 0;		// initially no nodes in the power grid
	refNode = 0;		// take first node to be reference if nothing else defined
	full = false; 		// sparse matrices are preferred
	printpf = true;		// print out power flow steps and calculations
	OUTPUT_VEC = true;	// parameter file output as vectors

	nrICs = 1;			// try at least once the Newton method
	nrPrecision = 1e-9;	// default precision for Newtons method to guarantee a stable steady state

//	constLoad = 0;		// if not given assume no constant load
}

PowerGrid::~PowerGrid() {
	nodes.clear();
	for( uint i = 0; i < lines.size(); i++){
		lines.at(i).clear();
	}
	lines.clear();
}


// ========================================================== methods ========================================================== //
// -------------------------------------------------------- create grid -------------------------------------------------------- //

/**
 * lines are generated by coupling mag or admittance mag
 * all other function just use these lines
 */
bool PowerGrid::readFromFile(std::string pfile){

	std::ifstream ins;
	ins.open(CodeUtils::Str2Char(pfile));
	if(ins.is_open()){
		std::cout << "-------------------- PowerGrid reading "
				  << " --------------------" << std::endl;
		std::vector<double> dv; 	// a double vector for temporary storing doubles
		std::vector<int> iv;		// a double vector for temporary storing integers
		bool waspar;				// indicates whether the current line was a parameter read by power grid class
		std::string line = "0";		// the file is loaded line by line into this string
		std::string linepar = "";	// storing the parameter name to inform user that parameter was read
		while(line.size() > 0){ 	// as long the file has still lines left
			getline(ins,line);
			linepar = line;
			waspar = true;			// if not negated later it is a parameter of interest
			if( line.substr(0,2) == "//")
				continue;
// --------------------------------------------------------- initialization -------------------------------------------------------- //
			else if( CodeUtils::lineIs(line , "Number of nodes" ) ){
				setNumNodes(CodeUtils::readint(ins));
			}
// ------------------------------------------------------------- nodes ------------------------------------------------------------- //
			else if( CodeUtils::lineIs(line , "Power mech") ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setPowerMechInit(dv.at(0));	// the mechanical power to this value
						nodes.at(i).setPowerMech(dv.at(0));
					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setPowerMechInit(dv.at(i));	// the power to corresponding values
						nodes.at(i).setPowerMech(dv.at(i));
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Power mech ext") ){
				getline(ins,line);
				dv = CodeUtils::readDVecFromFile(line);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setPowerMechInit(dv.at(0));	// the mechanical power to this value
						nodes.at(i).setPowerMech(dv.at(0));
					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setPowerMechInit(dv.at(i));	// the mechanical power to corresponding values
						nodes.at(i).setPowerMech(dv.at(i));
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Power diag") ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setPowerDiag(dv.at(0));		// the diagonal power terms to this value

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setPowerDiag(dv.at(i));		// the diagonal power terms to corresponding values
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Power diag ext") ){
				getline(ins,line);
				dv = CodeUtils::readDVecFromFile(line);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setPowerDiag(dv.at(0));		// the diagonal power terms to this value

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setPowerDiag(dv.at(i));		// the diagonal power terms to corresponding values
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Voltage mag") ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setVoltMagSs(dv.at(0));		// the voltage magnitude to this value

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setVoltMagSs(dv.at(i));		// the voltage magnitude to corresponding values
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Voltage mag ext") ){
				getline(ins,line);
				dv = CodeUtils::readDVecFromFile(line);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setVoltMagSs(dv.at(0));		// the voltage magnitude to this value

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setVoltMagSs(dv.at(i));		// the voltage magnitude to corresponding values
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Voltage ang") ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setVoltPhase(dv.at(0));		// the voltage phase angles to this value
						nodes.at(i).setVoltPhaseSs(dv.at(0));	// and use them as steady state angles, too

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setVoltPhaseSs(dv.at(i));	// the voltage phase angles to corresponding values
						nodes.at(i).setVoltPhase(dv.at(i));		// and use them as steady state angles, too
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Voltage ang ext") ){
				getline(ins,line);
				dv = CodeUtils::readDVecFromFile(line);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setVoltPhase(dv.at(0));		// the voltage phase angles to this value
						nodes.at(i).setVoltPhaseSs(dv.at(0));	// and use them as steady state angles, too

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setVoltPhase(dv.at(i));		// the voltage phase angles to corresponding values
						nodes.at(i).setVoltPhaseSs(dv.at(i));	// and use them as steady state angles, too
					}
				}
			}
			else if( CodeUtils::lineIs(line , "Damping" ) ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setDamping(dv.at(0));		// the damping to this value

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setDamping(dv.at(i));		// the damping to corresponding values
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Inertia") ){
				CodeUtils::readDVec(ins, &dv);
				if(dv.size() == 1){								// if only a scalar is given
					for(int i = 0; i < numNodes; i++){			// set for all nodes
						nodes.at(i).setInertia(dv.at(0));		// the inertia to this value
						nodes.at(i).setInertiaInv(1/dv.at(0));	// calculate the inverse to save calculations later

					}
				} else { 										// if a list is given
					for(int i = 0; i < numNodes; i++){ 			// set for all nodes
						nodes.at(i).setInertia(dv.at(i));		// the inertias to corresponding values
						nodes.at(i).setInertiaInv(1/dv.at(i));	// calculate the inverse to save calculations later
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Node type") ){
				CodeUtils::readDVec(ins, &dv);
				for(int i = 0; i < numNodes; i++){ 			// set for all nodes
					nodes.at(i).setType((short)dv.at(i));	// the types to corresponding values
				}
			}

			// todo sparse
// ----------------------------------------------------- full transmission lines ---------------------------------------------------- //
			else if( CodeUtils::lineIs(line, "Coupling mag") ){		// todo comments
				TransmissionLine *tl;
				for(uint i = 0; i < (uint)numNodes; i++){
					std::vector<double> cons;					// con(nection)s
					CodeUtils::readDVec(ins, &cons);			// contains the next (column) elements for K matrix
					for( uint j = 0; j < cons.size(); j++){
						if( cons.at(j) != 0 && (i!=j) ){		// if zero no connection needed. This is a sparse matrix // i != j important!!!
							tl = provideTrLine(i,j);				// get or create line between nodes i and j
							tl->setCouplingMag(cons.at(j));
							tl->setOnline(true);
						}
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Coupling mag ext") ){
				TransmissionLine *tl;
				getline(ins,line);
				std::vector<std::vector<double> > m = CodeUtils::readExtDMat(line);
				for( uint i = 0; i < m.size(); i++){
					for( uint j = 0; j < m.at(i).size(); j++){
						if( m.at(i).at(j) != 0 && (i!=j) ){			// if zero no connection needed. This is a sparse matrix // i != j important!!!
							tl = provideTrLine(i,j);				// get or create line between nodes i and j
							tl->setCouplingMag(m.at(i).at(j));
							tl->setOnline(true);
						}
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Coupling ang") ){
				TransmissionLine *tl;
				for(int i = 0; i < numNodes; i++){
					std::vector<double> cons;
					CodeUtils::readDVec(ins, &cons);	// contains the next (column) elements for gamma (admittance angle) matrix
					for( uint j = 0; j < cons.size(); j++){
						tl = getTrLine(i,j);
						tl->setCouplingAng(cons.at(j));
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Coupling ang ext") ){
				TransmissionLine *tl;
				getline(ins,line);
				std::vector<std::vector<double> > m = CodeUtils::readExtDMat(line);
				for(uint i = 0; i < m.size(); i++){
					for( uint j = 0; j < m.at(i).size(); j++){
						tl = getTrLine(i,j);
						tl->setCouplingAng(m.at(i).at(j));
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Admittance mag") ){	// todo ext
				TransmissionLine *tl;
				for(uint i = 0; i < (uint)numNodes; i++){
					std::vector<double> cons;
					CodeUtils::readDVec(ins, &cons);	// contains the next (column) elements for gamma (admittance angle) matrix
					for( uint j = 0; j < cons.size(); j++){
						if( cons.at(j) != 0 && (i!=j) ){		// if zero no connection needed. This is a sparse matrix // i != j important!!!
							tl = provideTrLine(i,j);
							tl->setAdmittanceMag(cons.at(j));
						}
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Admittance ang") ){
				TransmissionLine *tl;
				for(int i = 0; i < numNodes; i++){
					std::vector<double> cons;
					CodeUtils::readDVec(ins, &cons);
					for( uint j = 0; j < cons.size(); j++){
						tl = getTrLine(i,j);
						tl->setAdmittanceAng(cons.at(j));
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Line capacity") ){
				TransmissionLine *tl;
				for(int i = 0; i < numNodes; i++){
					std::vector<double> cons;
					CodeUtils::readDVec(ins, &cons);	// contains the next (column) elements for gamma (admittance angle) matrix
					for( uint j = 0; j < cons.size(); j++){
						tl = getTrLine(i,j);
						tl->setCapacity(cons.at(j));
					}
				}
			}
			else if( CodeUtils::lineIs(line, "Reference node") ){
				refNode = CodeUtils::readint(ins);
			}
// ----------------------------------------------------- Newton-Raphson method ---------------------------------------------------- //
			else if( CodeUtils::lineIs(line,"NRIC") ){
				setNrICs(CodeUtils::readint(ins));
			}
			else if( CodeUtils::lineIs(line, "NR precision") ){
				setNrPrecision(CodeUtils::readDouble(ins));
			}
			else {
				waspar = false;
			}

			if( waspar )		// if the line was a parameter read by power grid class
				std::cout << "read " << linepar << std::endl;
			dv.clear();
		}

		calcCflow();		// todo best here?

		return true;	// return that reading was successful or at least did not produce errors

	}

	return false;		// since program did not return success, it seems the reading failed

}

// ------------------------------------ other methods ------------------------------------ //
/**
 * return the transmission line connecting nodes i and j
 * if this line does not exist, the nullLine is returned
 * check for nullLine with TransmissionLine::isNull()
 */
TransmissionLine* PowerGrid::getTrLine(int i, int j){

	TransmissionLine *ret = TransmissionLine::getNullLine();
	for( uint jl = 0; jl < lines.at(i).size(); jl++){
		if( lines.at(i).at(jl).getToNode() == j ){
			ret = &lines.at(i).at(jl);
			break;
		}
	}
	return ret;
}

/**
 * returns the transmission line between the from node i and the to node j
 * if this line does not exist yet, it is created
 */
TransmissionLine* PowerGrid::provideTrLine(int i, int j){

	for( uint jl = 0; jl < lines.at(i).size(); jl++){
		if( lines.at(i).at(jl).getToNode() == j ){
			return &lines.at(i).at(jl);
//			break;
		}
	}
//	TransmissionLine *ret = TransmissionLine::getNullLine();
//	if( ret->isNull( ) ){
	TransmissionLine tl(i,j);
	lines.at(i).push_back(tl);
//	}
	return &lines.at(i).at(lines.at(i).size()-1);
}

Node* PowerGrid::getNode(int i ){
	return &(nodes.at(i));
}

void PowerGrid::resetLines(){

	for(uint i = 0; i < this->lines.size(); i++){
		this->lines.at(i).clear();	// delete content?
	}
	this->lines.clear();
	for(int i = 0; i < numNodes; i++){
		std::vector<TransmissionLine> tlv;
		this->lines.push_back(tlv);
	}
}

void PowerGrid::resetNodes(){

	this->nodes.clear();
	for(int i = 0; i < numNodes; i++){
		Node n(i);
		this->nodes.push_back(n);
	}
}


// ------------------------------------ get&set ------------------------------------ //

bool PowerGrid::isFull() const {
	return full;
}

void PowerGrid::setFull(bool full) {
	this->full = full;
}

int PowerGrid::getNumNodes() const {
	return numNodes;
}

/**
 * this sets numNodes. The node and transmission line vectors are
 * also reseted and adjusted to the new length
 */
void PowerGrid::setNumNodes(int numNodes) {
	this->numNodes = numNodes;
	resetLines();
	resetNodes();
}

int PowerGrid::getRefNode() const {
	return refNode;
}

void PowerGrid::setRefNode(int refNode) {
	this->refNode = refNode;
}

double PowerGrid::getCouplingMag(int i, int j){
	return getTrLine(i,j)->getCouplingMag();
}

std::vector<double>* PowerGrid::getVoltAngSS(){
	std::vector<double>* ret = new std::vector<double>;
	for(int i = 0; i < numNodes; i++){
		ret->push_back(getNode(i)->getVoltPhaseSs());
	}
	return ret;
}

int PowerGrid::getNrICs() const {
	return nrICs;
}

void PowerGrid::setNrICs(int nrICs) {
	this->nrICs = nrICs;
}

double PowerGrid::getNrPrecision() const {
	return nrPrecision;
}

void PowerGrid::setNrPrecision(double nrPrecision) {
	this->nrPrecision = nrPrecision;
}

std::vector<double>* PowerGrid::getPowerMech(){
	std::vector<double>* ret = new std::vector<double>;
	for(int i = 0; i < numNodes; i++){
		ret->push_back(getNode(i)->getPowerMech());
	}
	return ret;
}

double PowerGrid::getPowerMech(int i){
	return getNode(i)->getPowerMech();
}

void PowerGrid::setPowerMech(int i, double np){
	return getNode(i)->setPowerMech(np);
}

std::vector<Node*> PowerGrid::getGens(){
	std::vector<Node*> gens;
	for( int i = 0; i < numNodes; i++){
		if( getNode(i)->isGen() ){
			gens.push_back(getNode(i));
		}
	}
	return gens;
}

/*
 * returns an index that exclude the reference node index from a
 * regular sequence
 *
 * @param int i : regular index
 *
 * @return int j : index without reference node
 */
int PowerGrid::IWOR(int i){
	if(i < refNode){ 	// all indices lower than reference
		return i; 		// are unaffected
	}
	return i + 1; 		// all others are shifted
}

// ------------------------------------------------------ power flow -------------------------------------------------- //

/**
 * Newton Raphson iteration to find the steady state of the power grid.
 * The accuracy (the algorithm ends when the sum of the error is below it) is chosen as intern one
 * given by user or initialized as default (1e-9)
 * @return if the algorithm converged
 */
bool PowerGrid::NRPF(){
	return NRPF(nrPrecision);
}

/**
 * Newton Raphson iteration to find the steady state of the power grid.
 * @param epsilon : the accuracy (the algorithm end when the sum of the error is below epsilon)
 * @return if the algorithm converged
 */
bool PowerGrid::NRPF(double epsilon){

	if( printpf )
		std::cout << "==================== Newton Raphson Power Flow ====================" << std::endl;

	checkAllphasesZero();								// ensure the fixed point is not the zero vector

	const int MAX_NR = 10;								// maximum iterations of newton raphson method
	double limit = 1000;
	int nrtrys = 0;										// initialize counter for newton raphson tries
	double* err = new double[numNodes];					// double pointer storing the power flow mismatch for every node
	int steps;											// step counter for newton raphson. Number of steps is returned by method and printed out here
	double toterr = 0;									// total error of power flow solution

	for(int i = 0; i < numNodes;i++){					// initialize the error sum
		err[i] = PFlFunc(i);							// store the error of each node into the err array
	}

	toterr = CodeUtils::dAbsArrSum(err,numNodes);		// sum up deviation from steady state
	delete[] err;										// clear "predictor" array
	if( toterr <= epsilon ){							// if grid is already close enough to steady state
		if( printpf ){ 									// when printing is allowed, inform user that no iteration necessary
			std::cout << "The grid already fulfills"
					<< " the power flow condition."
					<< std::endl;
			std::cout << "Total error: "
					<< toterr << std::endl;
		}
		return true; 									// return that grid is in steady state
	}

	if( printpf )										// if printing is allowed inform user that the steady state is calculated now
		std::cout << "Use Newton Raphson to "
				<< "determine power flow."
				<< std::endl;
	while( nrtrys < MAX_NR ){ 							// for every iteration
		if( printpf ) 									// if print mode
			std::cout << "Try " << (nrtrys+1)			// print number of try
					  << "/" << MAX_NR << std::endl;
		steps = 0; 										//
		err = NRMethod(epsilon, steps);					// steps is changed/iterated here
		if( printpf )
			cout << "Total error: "
			<< CodeUtils::dAbsArrSum(err, numNodes)
			<< std::endl;
		if( CodeUtils::anynan(err, numNodes)) {			// if nan-phases exist the fixed point was not found
			for(int i = 0; i < numNodes ; i++){			// and a new guess has to be done
				getNode(i)->setVoltPhase( 				// therefore fill every entry initially
						0.0005*CodeUtils::randD(0,i) );	// with small random values
			}
		} else {										// otherwise the fixed point is valid
			nrtrys = MAX_NR + 1;						// and the loop can be left
		}
		nrtrys = nrtrys + 1;
	}


	if( printpf ){													// if print mode
		std::cout << "Newton Raphson iteration ended after "		// inform that Newton Raphson iteration ended
				<< std::endl;
		std::cout << steps << " / " << limit << " steps." << endl;	// display needed iteration steps
		std::cout << "The total deviation from steady state is: "   // display the deviation from steady state after iteration.
			<< CodeUtils::dAbsArrSum(err, numNodes) << std::endl;
	}
	delete[] err;													// clean up memory


	if(steps >= limit){ 											// if no steady state was found
		std::cout << "The Newton Raphson iteration"					// inform user about the failure
				<< " did not converge." << std::endl;
		return false; 												// and return not converged
	} else {														// if steady state was found
		for(int i = 0; i < numNodes; i++){							// set all nodes
			getNode(i)->setVoltPhaseSs(getNode(i)->getVoltPhase());	// steady state phases to the calculated phases
		}
	}
	return true;
}

/**
 * if all nodes phase is zero a new initial configuration is chosen
 */
void PowerGrid::checkAllphasesZero(){
	for(int i = 0; i < numNodes; i++){
		if( getNode(i)->getVoltPhase() != 0 )		// if any node has phase different from zero it is set by user
			return; 								// then dont change anything
	} 											// otherwise
	for(int i = 0; i < numNodes ; i++){			// a new guess have to be done
		getNode(i)->setVoltPhase(
				0.0005*CodeUtils::randD(0,i) );	// therefore fill every entry initially with small random values
	}
}



/**
 * the calculation of the newton raphson method itself
 */
double* PowerGrid::NRMethod(double epsilon, int& steps){

	double** Kmat = new double*[numNodes];
	double** Gmat = new double*[numNodes];
	for(int i = 0; i < numNodes; i++){
		Kmat[i] = new double[numNodes];
		Gmat[i] = new double[numNodes];
		for( int j = 0; j < numNodes; j++){
			Kmat[i][j] = 0;
			Gmat[i][j] = 0;
		}
	}
	for(uint i = 0; i < lines.size(); i++){
		for(uint j = 0; j < lines.at(i).size(); j++){
			Kmat[lines.at(i).at(j).getFromNode()][lines.at(i).at(j).getToNode()] =
					lines.at(i).at(j).getCouplingMag();
			Gmat[lines.at(i).at(j).getFromNode()][lines.at(i).at(j).getToNode()] =
					lines.at(i).at(j).getCouplingAng();
		}
	}


//	std::cout << "nr mats finished"<< std::endl;
	steps = 0;
	int MAX_NR_STEPS = 1000;
	double* err = new double[numNodes];

	for(int i = 0; i < numNodes;i++){				// initialize the error sum
		err[i] = PFlFunc(i);						// store the error of each node into the err array
	}

	getNode(refNode)->setVoltPhase(0);				// get reference bus and set its phase to zero

	Eigen::MatrixXd J(numNodes-1,numNodes-1);
	Eigen::VectorXd fn(numNodes-1);
	Eigen::VectorXd dTh(numNodes-1);
	while( CodeUtils::dAbsArrSum(err, numNodes) > epsilon
			&& steps < MAX_NR_STEPS){				// while error to big and maximal steps not exceeded
		steps ++;									// count steps
		Jij(Kmat,Gmat,J);
		for(int i = 0; i < numNodes-1; i++){
			fn(i) = PFlFunc(IWOR(i)); 				// calculate power flow for all non-reference nodes
		}
		dTh = -1*J.householderQr().solve(fn);

		for(int i = 0; i < numNodes-1; i++){
			getNode(IWOR(i))->setVoltPhase(
					nodes.at(IWOR(i)).getVoltPhase() + (double)dTh(i) );	//
			getNode(IWOR(i))->setVoltPhase( CodeUtils::smallerAngle( 		// map angle to interval [-pi, pi]
					getNode(IWOR(i))->getVoltPhase())); 					// which increases accuracy of sin() and cos()
		}
		for(int i = 0; i < numNodes; i++){
			err[i] = PFlFunc(i);
		}
		getNode(refNode)->setPowerMech(			// the mismatch in power flow in reference node
				getNode(refNode)->getPowerMech()
				- err[refNode]);				// is balanced by its power
		err[refNode] = 0;						// therefore its error becomes zero every time
		if( printpf
				&& (steps % (MAX_NR_STEPS/10) == 0) ){
			std::cout << "Step " << steps << "/" << MAX_NR_STEPS
					<< ". Total error: " << CodeUtils::dAbsArrSum(err,numNodes) << std::endl;
		}

	}

	for(int i = 0; i < numNodes; i++){	// clean up memory
		delete[] Kmat[i];
		delete[] Gmat[i];
	}
	return err;
}

/**
 * Returns the output of the power flow equation
 * For the values stored in this power grid object. Namely the nodes voltage phases,
 * mechanical and diagonal powers and the Transmission line couplings
 *
 * @param i : the node
 * @return the power flow function output
 */
double PowerGrid::PFlFunc(int i){
	double sum = 0;
	int j = 0;
	for(uint ind = 0; ind < lines.at(i).size(); ind++){
		j = lines.at(i).at(ind).getToNode();
		sum += lines.at(i).at(ind).getCouplingMag()
				*sin( getNode(i)->getVoltPhase() - getNode(j)->getVoltPhase()
						- lines.at(i).at(ind).getCouplingAng() );
	}
	return getNode(i)->getPowerMech() - getNode(i)->getPowerDiag() - sum;
}

/**
 * the result of the swing equation for node i in the grid in the
 * state (phase, frequency) for external state usage
 */
double PowerGrid::swingEq(double* phase, double* freq, int i){
	double sum = 0;
	int j = 0;
	for(uint ind = 0; ind < lines.at(i).size(); ind++){
		j = lines.at(i).at(ind).getToNode();
		sum += lines.at(i).at(ind).getCouplingMag()
							* sin( phase[i] - phase[j]
							- lines.at(i).at(ind).getCouplingAng() );
	}
	return getNode(i)->getInertiaInv() * (
			- getNode(i)->getDamping() * freq[i]
			         + getNode(i)->getPowerMech() - getNode(i)->getPowerDiag() - sum );
}

/**
 * creates the Jacobi matrix for the power flow equations
 *
 * @param J: the result is written into J
 */
void PowerGrid::Jij(double** K, double** G, Eigen::MatrixXd &J){

	Eigen::MatrixXd JN(numNodes,numNodes);

	double jsum;
	for(int i = 0; i < numNodes; i++){
		jsum = 0;
		for(int j = 0; j < numNodes; j++){
			if( i != j){
				JN(i,j) = K[i][j]*cos(getNode(i)->getVoltPhase()
						- getNode(j)->getVoltPhase()
						- G[i][j]);
				jsum += JN(i,j);
//				std::cout << J(i,j) << " ";
			}
		}
		JN(i,i) = -jsum;
//		std::cout << J(i,i);
//		std::cout << std::endl;
	}

	for( int i = 0; i < numNodes-1 ; i++){ 		// remove reference node
		for( int j = 0; j < numNodes-1; j++){
			J(i,j) = JN(IWOR(i),IWOR(j));
		}
	}
}


void PowerGrid::calcFixedPoints(Param& par){ 		// todo renew

	CodeUtils::cdir("thfix");			// create new folder if possible

	double* theta0 = new double[par.len];
	double* thetafp = new double[par.len];
//	double pi = 3.1415926535;	// todo renew
	int steps = 0;

	srand(par.getRandSeed());							// set random seed

	for(int i = 0; i < getNrICs();i++){
		for(int x = 0; x < par.len; x++){
//			theta0[x] = CodeUtils::randD(pi*par.th0[0],pi*par.the[0]);	// todo
		}
		NRMethod(nrPrecision,steps);
		CodeUtils::arrayToBinFile(theta0,par.len,par.getOfile("path")+"thfix/thfix"+par.getOfile("file"));
		CodeUtils::arrayToBinFile(thetafp,par.len,par.getOfile("path")+"thfix/thfix"+par.getOfile("file"));

	}

	delete[] theta0;
	delete[] thetafp;
}

//void PowerGrid::steadyStateFreqPhase(double* voltPhase, double* voltFreq,
//		double* voltPhase2, double* voltFreq2){
//
//}


// old code
///**
// * Returns the derivative of the power flow equation
// * @param p : the parameter object
// * @param th : the array with the theta values
// * @param i : the node
// * @return the power flow function derivative
// */
//double PowerGrid::PFDeriv(Param& p, double* th, int i){
//	double sum = 0;
//	int j;
//	for(uint ind = 0; ind < p.K.at(i).size(); ind++){
//		j = p.K.at(i).at(ind).conNode;
//		sum += p.K.at(i).at(ind).couplingStrength*cos(th[i]- th[j] - p.K.at(i).at(ind).angle);
//	}
//	return - sum;		// why negative?
//}
//
//double PowerGrid::dfidtj(Param& p, int i, int j, double* th){
//	double innerK = 0, iangle = 0;						// coupling and angle for j searched
//	for(uint k = 0; k < lines.at(i).size(); k++){
//		if((j==lines.at(i).at(k).getToNode())){				// when found
//			innerK = lines.at(i).at(k).getCouplingMag();	// set it
//			iangle = lines.at(i).at(k).getCouplingAng();
//			break;										// and end search
//		}
//	}													// otherwise its 0 (no coupling)
//	return cos(th[i]-th[j]-iangle)*innerK;
//}


// ------------------------------------------------------ graph tools -------------------------------------------------- //

/**
 * Distributes the mechanical powers -1 and 1 with equal quantity between the nodes
 */
void PowerGrid::distribEqualP(){
	if( numNodes%2 != 0){
		std::cout << "can only distribute P's if even number.";
		return;
	}

	for(int i = 0; i < numNodes; i++){
		getNode(i)->setPowerMech(1);
	}

	int countsign = 0;
	int ausw;
	while(countsign < numNodes/2){										// until half of the powers are negative
		ausw = CodeUtils::randInt(0,(numNodes-1));						// choose random node
		getNode(ausw)->setPowerMech(-1*getNode(ausw)->getPowerMech());	// changes its power sign
		countsign -= getNode(ausw)->getPowerMech(); 					// count up if the power is negative and down if it became positive again
	}

	for(int i = 0; i < numNodes; i++){
		getNode(i)->setPowerMechInit(getNode(i)->getPowerMech());		// set the initial power correct
	}

}

/**
 * marks the node 0 and recursive all nodes connected to it with a zero in the int array nodes
 */
void PowerGrid::markConnectedZero(int* nodes, int i){
	if(nodes[i] == 0)
		return;
	nodes[i] = 0;
	for(uint j = 0; j < lines.at(i).size(); j++){
		markConnectedZero(nodes,lines.at(i).at(j).getToNode());
	}
}

/**
 * better than checkAllCon
 * returns if all nodes are connected and build ONE net
 * @return false if the net is fragmented, true otherwise
 */
bool PowerGrid::oneNet(){
	int error;
	int* wayToOne = new int[numNodes];

	for( int i = 0; i < numNodes ; i++){
		wayToOne[i] = 1;				// initialize the error array
	}

	markConnectedZero(wayToOne,0);	// mark all nodes that are direct or indirect connected to node 1 as zero

	/*// prints out the number of the not connected nodes
	cout << "not connected : ";
	for(int i = 0; i < p.len; i++){
		if( wayToOne[i] == 1){
			cout << i << " ";
		}
	}
	cout << endl;*/

	error = CodeUtils::AbsArrISum(wayToOne,numNodes);	// if sum of error array is zero, all nodes are connected

	delete[] wayToOne;					// clean up memory
	return error == 0;

}

/**
 * Checks if the connection between i and j is a self-loop
 * @return true if it's a loop
 */
bool PowerGrid::isLoop(int i, int j){
	return i == j;
}

/**
 * Checks if the connection between i and j already exists
 * @return true if nodes i and j already connected
 */
bool PowerGrid::isDoubleCon(Param& p, int i, int j){
	return connectionExists(i,j) >= 0;	// returns -1 if they are not connected 	// todo without param
}

/**
 *
 */
void PowerGrid::addRemainingEdges(Param& p, int addEdges){
	int r, r2,cedge = 0;
	while( cedge < addEdges){		// add the remaining edges
		r = CodeUtils::randInt(0, numNodes-1);	// todo: fehlerabfangen, wenn verbinden nicht möglich / warum noch mal?
		r2 = CodeUtils::randInt(0,numNodes-1);
		if( !isLoop(r,r2) && !isDoubleCon(p,r,r2) ){
			connectNodes(r,r2, 8);	// todo !!!
			cedge ++;
		}
	}
}


/**
 * uses 8 as admittance use connectNodes(int, int, double admittance) for
 * choosing admittance manually
 */
//void PowerGrid::connectNodes(int i, int j){
//	// if i == j error todo
//	connectNodes(i,j,8);
//}


void PowerGrid::connectNodes(int i, int j, double coupl){
	// if i == j error todo

	TransmissionLine tl(i,j);
	tl.setCouplingMag(coupl);
	tl.setCapacity(0.25*coupl);
	lines.at(i).push_back(tl);

	TransmissionLine tl2(j,i);
	tl2.setCouplingMag(coupl);
	tl2.setCapacity(0.25*coupl);
	lines.at(j).push_back(tl2);

}

/**
 * returns the position of the edge to j in the connection-list of node i
 * returns -1 if they're not connected
 * returns the first connection. Have to be used multiple times if all connections should be found.
 * normally there exists only one connection
 */
int PowerGrid::connectionExists(int i, int j){

	for( uint l = 0; l < lines.at(i).size(); l++){
		if( lines.at(i).at(l).getToNode() == j ){	// K[i][l] == connection to node j ?
			return l; 			// return index of lines in list i
		}
	}

	return -1;
}

bool PowerGrid::removeConnection(int i, int j){
	bool out1 = false, out2 = false;

	int h = connectionExists(i,j); 						// get index of the line in the list or -1
	if( h >= 0){										// if not -1 the connection line is in the list
		lines.at(i).erase(lines.at(i).begin()+h);		// removes the line from list
		out1 = true; 									// mark that the from line is removed
	}

	h = connectionExists(j, i); 						// get index of the line in the list or -1
	if( h >= 0){										// if not -1 the connection line is in the list
		lines.at(j).erase(lines.at(j).begin()+h);		// removes the line from list
		out2 = true;									// mark that the to line is removed
	}
	return (out1 && out2); 								// returns if the removal was successful
}

bool PowerGrid::rewire(int i, int j1, int j2){

	bool out;

	// find connection and its strength and other params todo
	// remove them from the list and add them back to their new proper place
	out = removeConnection(i,j1);
	connectNodes(i, j2, 8);	// todo out = out & connectN...

	return out;
}

//double Param::meanNodeDegree(){
//	int knges = 0;
//	for (int i = 0; i < len; i++){
//		knges += K.at(i).size();
//		cout << K.at(i).size() << " ";
//	}
//	cout << endl;
//	return  (double)((double)knges / (double)len);
//}


/**
 * check if lines are overloaded and remove them from the grid. Mark the removed line
 * in the vector Kout
 */
void PowerGrid::checkLines(double* voltPhases, std::vector<int> *kout1, std::vector<int> *kout2,
		std::vector<double> *tout, double t, Param& par){

	double flowij = 0;

	TransmissionLine *K12, *K21;
	int ii, ji;
	std::complex<double> phas12;

	double maxl = 0, maxlp = 0, pfl = 0;
	int maxi, maxj;

	for( uint i = 0; i < lines.size(); i++){

		for( uint j = 0; j < lines.at(i).size(); j++){

			K12 = &(lines.at(i).at(j));
			ii = K12->getFromNode();
			ji = K12->getToNode();

			phas12 = std::complex<double>(0, voltPhases[ii] - voltPhases[ji] - K12->getAdmittanceAng());

			// todo : return getcflow * isonline !!
			flowij = K12->isOnline() * std::abs( K12->getCflow() 	// calculate the power currently flowing through line i j
					- K12->getCouplingMag()* exp( phas12 ) );	// this is set zero if line is offline

			pfl = flowij / K12->getCapacity();
			if( pfl > maxlp ){		// if flow is bigger than biggest stored in relative numbers
				maxl = flowij;		// store new max
				maxlp = pfl;		// replace previous biggest relative flow
				maxi = ii;			// store from node
				maxj = ji;			// store to node

			}

			if( flowij > K12->getCapacity() ){	// if line is overloaded
				K12->setOnline(false); 		// set it offline in both directions
				K21 = getTrLine(ji,ii); 	//
				K21->setOnline(false); 		//
				kout1->push_back(K12->getFromNode());
				kout2->push_back(K12->getToNode());
				tout->push_back(t);
			}
		}
	}

	par.maxload.push_back(maxl);		// todo object storing load flow information (param??) schlanke PG variante
	par.maxloadp.push_back(maxlp);		// or in power grids itself? => schlanke variante in übergabe,
	par.maxlij.push_back(maxi);
	par.maxlij.push_back(maxj);
}

void PowerGrid::measureFlow(double* voltPhases){

//	double flowij = 0;

	TransmissionLine *K12;//, *K21;
//	int ii, ji;
	//	std::complex<double> phas12;

	//	double maxl = 0, maxlp = 0, pfl = 0;
	//	int maxi, maxj;

	for( uint i = 0; i < lines.size(); i++){

		for( uint j = 0; j < lines.at(i).size(); j++){

			K12 = &(lines.at(i).at(j));
			K12->measureFlow(voltPhases);

		}
	}
}

std::vector<double> PowerGrid::saveFlow(double* voltPhases){

	std::vector<double> ret;

	TransmissionLine *K12;

	for( uint i = 0; i < lines.size(); i++){

		for( uint j = 0; j < lines.at(i).size(); j++){

			K12 = &(lines.at(i).at(j));
			ret.push_back(K12->measureFlow(voltPhases));

		}
	}

	return ret;
}

void PowerGrid::resetLineFlows(){
	for( uint i = 0; i < lines.size(); i++){

		for( uint j = 0; j < lines.at(i).size(); j++){

			lines.at(i).at(j).resetMaxFlow();
			lines.at(i).at(j).resetMeanFlow();

		}
	}
}

/**
 * stores four values into the vector for every transmission line
 * the from node
 * the to node
 * the average flow
 * the maximum flow
 */
void PowerGrid::getFIJ(std::vector<double> *fij, double avSteps){

	fij->clear();
	for( uint i = 0; i < lines.size(); i++){
		for( uint j = 0; j < lines.at(i).size(); j++){

			fij->push_back(lines.at(i).at(j).getFromNode());
			fij->push_back(lines.at(i).at(j).getToNode());
			fij->push_back(lines.at(i).at(j).getMeanFlow()/avSteps);
			fij->push_back(lines.at(i).at(j).getMaxFlow());

		}
	}
}

/**
 * sets powerMech equal to powerMechInit for all nodes in this grid
 */
void PowerGrid::resetPowerMech(){
	for(int i = 0; i < numNodes; i++){
		getNode(i)->setPowerMech(getNode(i)->getPowerMechInit());
	}
}

/**
 * pre calculate the constant part of power flow in every line to save time later
 */
void PowerGrid::calcCflow(){	// TODO the (much) more accurate power injection using from and to matrices!!	// to power grids

	// todo do this inside transmission line class
	TransmissionLine *K12;
	int ii;//, ji;
	std::complex<double> phas12;

	for( uint i = 0; i < lines.size(); i++){
		for( uint j = 0; j < lines.at(i).size(); j++){

			K12 = &(lines.at(i).at(j));
			ii = K12->getFromNode();
//			ji = K12->getToNode();

			phas12 = std::complex<double>(0, -K12->getAdmittanceAng());

			// todo with voltage dynamics: not voltMagSS!!
			K12->setCflow( getNode(ii)->getVoltMagSs() * getNode(ii)->getVoltMagSs() * K12->getAdmittanceMag() * exp( phas12 ) );
		}
	}
}


// ------------------------------------------------- network generation --------------------------------------------- //

/**
 * In single node model this increases K[0][0] until it reaches kMax
 * In Multi node model it generates a random symmetric K[i][j] connection matrix until the number of created nets reaches maxNet
 */
bool PowerGrid::generateNet(Param& p){		// todo necessary?

	if( p.currentNet >= p.maxNet)	// if already all networks investigated
		return false;				// return nothing will be generated or set to FP

	if( p.isModeIterativeNetgen() )
		iterativNet(p,p.edges);
	if( p.isModeRewireNetgen() )
		rewireNet(p, p.edges);
	if( ! p.isModeRewireNetgen() && ! p.isModeIterativeNetgen() ){
		NRPF(nrPrecision); // default 5e-4
	}

	p.currentNet ++;
	std::cout << p.currentNet << "/" << p.maxNet << " networks"<< std::endl;	// todo text
	return p.currentNet <= p.maxNet;
}



/**
 * takes all connections from the grid (which is a ring in most cases) and sets them between two randomly chosen nodes
 * the connections themselves are also chosen in a random order. They won't be rewired if the procedure would fragment the net
 */
void PowerGrid::rewireNet(Param& p, int edges){
	int addEdges = edges - numNodes;		// numNodes edges are included by building the ring structure (see later) the rest must be connected random
	RandomList rl(numNodes);				// a random list to prevent the correlations for the first indices

	do {
		for(int i = 0; i < numNodes; i++)
			getNode(i)->setVoltPhase(0.1);	// todo why?
		resetLines(); 						// todo comment

		for(int i = 0; i < numNodes ; i++){	// create a ring, so all nodes are connected
			connectNodes(i, (i+1)%numNodes, 8);	// at first giving p.len connections		// todo connection strength
		}
		addRemainingEdges(p,addEdges);		// add the remaining edges

		std::cout << "all connected after init: "<< oneNet() <<std::endl;


		rl.reset();							// initialize the random list
		rl.shuffle();						// and draw new realization

		int r, cnode;						// cnode is the current node i (?) under investigation // todo better name: the node is not important but the row index
		for(int i = 0; i < numNodes; i++){
			cnode = rl.getNext() - rl.getStart();							// list is start to len+start-1 not necessary 0 to len-1	// todo construct clearer random list
			for(uint j = 0; j < lines.at(cnode).size(); j++){					// for all connections of this node i
				if(cnode < lines.at(cnode).at(j).getToNode()){					// the connection is undirected so only cnode>j or j>cnode is important
					do{
						r = CodeUtils::randInt(0,numNodes-rl.getStart());					// draw the new target of the connection from i
					} while( isLoop(cnode,r) || isDoubleCon(p,cnode,r));	// until this connection is allowed
					rewire(cnode,lines.at(cnode).at(j).getToNode(),r);			// reconnect i<->j to i<->r
					if( !oneNet() ){										// if net got fragmented this way
						rewire(cnode,r,lines.at(cnode).at(j).getToNode());		// redo i<->r to i<->j // todo without param
					}
				}
			}
		}

		distribEqualP();												// set half of the nodes generators and the other half loads
	} while(!NRPF(nrPrecision) || !oneNet());				// if something went wrong and the net isn't connected anymore redo this
}

void PowerGrid::iterativNet(Param& p, int edges){
	int addEdges = edges - (numNodes - 1);
	std::vector<int> alreadyInNet;
	alreadyInNet.push_back(1);
	RandomList rl(numNodes,0);
	//cout << "okay do the do"<<endl;
	do {
		// ---- init vectors ---- //
		for(int i = 0; i < numNodes; i++)
			getNode(i)->setVoltPhase(0.1);	// todo why?
		resetLines();
		// ---- net generation ----//
		int r, cnode;
		alreadyInNet.clear();
		rl.reset();
		rl.shuffle();
		alreadyInNet.push_back(rl.getNext());	// random first node
		//cout << "dodelido for" << endl;
		for(int i = 1; i < numNodes ; i++){	// the first 99 connections	(len = 100, edges = 135)
			cnode = rl.getNext();
			r = CodeUtils::randInt(0,i-1);		// draw random node from all the nodes that still exist
			r = alreadyInNet.at(r);	// get the r-th node that is already in net
			//cout << "dododo"<<endl;
			connectNodes(cnode,r, 8);	// connect current "new (i-th)" node with it	// todo without param
			alreadyInNet.push_back(cnode);


		}
		addRemainingEdges(p, addEdges);


		distribEqualP();
		//			allCon = checkAllCon(p);
	} while(!NRPF(nrPrecision));
}

// ------------------------------------------------------ print to file -------------------------------------------------- //

std::string PowerGrid::toString() {
	std::stringstream outfile;

	std::vector<double> power, voltangss;
	std::vector<int> types;
	if( OUTPUT_VEC ){
		for(int i = 0; i < numNodes; i++){
			power.push_back(getPowerMech(i));
			voltangss.push_back(getNode(i)->getVoltPhaseSs());
			types.push_back(getNode(i)->getType());
		}
	}

	outfile << "====================== Power grid =======================" 	<< endl;
	outfile << "----------------- Power flow Parameters -----------------" 	<< endl;
	outfile << "NRPrecision: " 			<< nrPrecision		<< endl;
//	outfile << "nrIC: " 				<< nrICs 			<< endl;
	outfile 												<< endl;
	outfile << "------------------------ Nodes --------------------------" 	<< endl;
	outfile << "ReferenceNode: " 		<< refNode			<< endl;
	outfile << "Power: "			 		 				<< endl;
	outfile << CodeUtils::dvec2str(power);
	outfile << "Voltage phases SS: "			 		 	<< endl;
	outfile << CodeUtils::dvec2str(voltangss);
	outfile << "Node types: "			 		 			<< endl;
	outfile << CodeUtils::ivec2str(types);
	outfile 												<< endl;

	// todo modes: print as array here, print as node objects here, print as node objects in external files

	outfile << endl;

	return outfile.str();
}

bool PowerGrid::toFile(Param* par){

	if( !par->param){
		return false;
	}

	string fname = "Param" + par->filenum;

	std::ofstream outfile( fname, ios::out|ios::app);

	outfile << toString();

	outfile.flush();
	outfile.close();

	return true;
}


