/*
 * PowerGrid.h
 *
 *  Created on: Feb 22, 2019
 *      Author: mawolff
 */

#ifndef POWERGRID_H_
#define POWERGRID_H_

#include <vector>

#include "Param.h"
#include "RandomList.h"

#include "Node.h"
#include "TransmissionLine.h"

//#include "/home/student/m/mawolff/programs/eigen/Eigen/Eigen"
#include "/home/student/m/mawolff/simulation/eigen/Eigen/Eigen"



class PowerGrid {

// =================================== variables =================================== //
private:
	bool full;			// indicates whether this network is stored in full or sparse matrices
//	bool fixedSize;		// indicates whether a fixed number of nodes is read and stored
						// in arrays or a variable number is read and dynamically stored
	bool printpf;		// whether the steps and results of the power flow calculation should
						// be printed to console
	bool OUTPUT_VEC;	// if the output for parameter file is written as vector or as objects

	int numNodes;		// number of nodes
	int refNode;		// reference node

	int nrICs;			// maximum newton raphson initial conditions
	double nrPrecision;	// target precision of the newton-raphson iteration (to find the fixed-point)

	std::vector<Node> nodes;							// list of nodes
	std::vector< std::vector<TransmissionLine> > lines;	// list of transmission lines

//	double constLoad; 	// load power at generator nodes (or in general) that is modeled as impedance

// =========================== constructors & destructors ========================== //
public:
	PowerGrid();
	virtual ~PowerGrid();

// ==================================== methods ==================================== //
private:
	TransmissionLine* getTrLine(int i, int j);
	TransmissionLine* provideTrLine(int i, int j);
	Node* getNode(int);
	void resetLines();
	void resetNodes();
	void calcCflow();
	int IWOR(int i);	// index without reference node

// ----------------------------------- power flow ---------------------------------- //
public:
	bool NRPF();
	bool NRPF(double epsilon);

//	void steadyStateFreqPhase(double* voltPhase, double* voltFreq,
//			double* voltPhase2, double* voltFreq2);

	double swingEq(double* phase, double* freq, int i);


private:
	void checkAllphasesZero();
	double* NRMethod(double epsilon, int& steps);
	double PFlFunc(int i);
//	double PFDeriv(Param& p, double* th, int i);	// todo old code. still needed?
//	double dfidtj(Param& p, int i, int j, double* th);
	void Jij(double** K, double** G, Eigen::MatrixXd &J);
	void calcFixedPoints(Param& par);

// ------------------------------ graph tools -------------------------------------- //
private: 	// todo private and only one method like "createRandomNet"
	void distribEqualP();
	void markConnectedZero(int* nodes, int i);
	bool oneNet();
	bool isLoop(int i, int j);
	bool isDoubleCon(Param& p, int i, int j);
	void addRemainingEdges(Param& p, int addEdges);
	bool generateNet(Param& p);						// todo necessary?
	void connectNodes(int i, int j, double K);
	int connectionExists(int i, int j);
	bool removeConnection(int i, int j);
	bool rewire(int i, int j1, int j2);


// --------------------------------- calculations ---------------------------------- //
public:
	void checkLines(double* voltPhases, std::vector<int> *kout1, std::vector<int> *kout2,
			std::vector<double> *tout, double t, Param& par);
	void measureFlow(double* voltPhases);
	std::vector<double> saveFlow(double* voltPhases);
	void resetLineFlows();
	void resetPowerMech();

// ---------------------------------- create grid ---------------------------------- //
public:
	bool readFromFile(std::string file);
	void rewireNet(Param& p, int edges);
	void iterativNet(Param& p, int edges);
	// createRandom(); // todo

// ------------------------------------ get&set ------------------------------------ //
public:
	bool isFull() const;
	void setFull(bool full);
	int getNumNodes() const;
	void setNumNodes(int numNodes);
	int getRefNode() const;
	void setRefNode(int refNode);

	double getPowerMech(int i);
	void setPowerMech(int i, double nP);

	double getCouplingMag(int i, int j);
	std::vector<double>* getVoltAngSS();
	std::vector<double>* getPowerMech();
	int getNrICs() const;
	void setNrICs(int nrICs);
	double getNrPrecision() const;
	void setNrPrecision(double nrPrecision);

	std::vector<Node*> getGens();

	void getFIJ(std::vector<double>* fij, double avSteps);

// --------------------------------- print to file --------------------------------- //
	std::string toString();
	bool toFile(Param* par);

};

#endif /* POWERGRID_H_ */



// =================================== variables =================================== //
// =========================== constructors & destructors ========================== //
// ==================================== methods ==================================== //
// ------------------------------------ get&set ------------------------------------ //
