//============================================================================
// Name        : NishikawaModel.cpp
// Author      : Mawolff
// Version     :
// Copyright   : Your copyright notice
// Description : Swing equation iterated with Runge Kutta in C++, Ansi-style
//============================================================================

#include <iostream>
#include <fstream>		// file stream
#include <sstream>
#include "string.h"		// C methods for strings
#include <string>		// C++ strings
#include <iterator>
#include <sys/types.h>	// mkdir
#include <sys/stat.h>
#include <random>		// normal_distr
#include <stdlib.h>		// rand()
#include <time.h>		// init rand seed
#include <tgmath.h> 	// abs


#include "PowerGrid.h"

#include "Function.h"
#include "StochFunction.h"
#include "RungeKutta.h"
//#include "Param.h"

#include "RasterReturnTime.h"
#include "StochasticInput.h"

#include "RandomList.h"
#include "MersenneTwister.h"
#include "CodeUtils.h"

/* input name saves:
v2.008:
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochHet/local/TurbHel96MFInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochHet/local/TurbHel96MFInputPaper.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochHet/local/TurbHel96MFInputPaperFig4.txt
v2.010:
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/MTurHel96MFInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/MTurHel96MF22Input.txt
v2.011:
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF11cInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF12cInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF22cInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF11ucInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF12ucInput.txt
/home/student/m/mawolff/workspace/PBSScripter/sh2/stochLF/multiTurb/Comp96SIF22ucInput.txt
/home/student/m/mawolff/Documents/AcademicCloud/Universitaet/Dissertation/A-KapitelA-Apendix/tables/case5dynpar/he/DPHeC5TrajInput.txt
/home/student/m/mawolff/Documents/AcademicCloud/Universitaet/Dissertation/A-KapitelA-Apendix/tables/case5dynpar/he/DwDpHeC5TrajInput.txt
/home/student/m/mawolff/Documents/AcademicCloud/Universitaet/Dissertation/A-KapitelA-Apendix/tables/case5dynpar/hel/DPHelC5TrajInput.txt
/home/student/m/mawolff/Documents/AcademicCloud/Universitaet/Dissertation/A-KapitelA-Apendix/tables/case5dynpar/hel/DwDpHelC5TrajInput.txt
v2.012 (basically same as .011)
/home/student/m/mawolff/workspace/SlurmScripter/DatenUndSkripte/StochHetRedo/FrHist96HelInput.txt
 */

// todo put into param class
void writeToFile( Param& par, std::string str){
	if( !par.param){
		return;
	}

	string fname = par.getOfile("path") + "Param" + par.getOfile("file");

	std::ofstream outfile( fname, ios::out|ios::app);

	outfile << str;

	outfile.flush();
	outfile.close();

//	return true;
}

int main() {	// todo: an welcher stelle  mehrere grids?

	const int VERSION = 2012;				// version 2.012
	std::cout << "==================== Power grid models V"
			<< ((double)VERSION)/1000 <<" ====================" << std::endl;

	cout << "Moin!" << std::endl;			// greetings to user

	Param par;								// create parameter file
	par.version = VERSION;					// save version as parameter to check input consistency
	PowerGrid pg;							// create a power grid

	cout << "Type in parameter file." << std::endl;			// ask for parameter file
	std::string pfile;										// store that file path and name into string
	std::cin >> pfile;										// read param files path and name
	std::cout << "I understood: " << pfile << std::endl;	// inform user that his or her input was read
	bool fexist = par.readParamsFromFile(pfile);			// read the parameters from that file and store
															// to a bool if the file could be found
	if( ! fexist){											// if an error occured while reading the param file
		cerr << "file doesn't exist or is damaged."<<		// inform user that there was a problem
				"Program will exit." << endl;
		cout << "file doesn't exist or is damaged."<<		// inform user that there was a problem
				"Program will exit." << endl;
		return 1;											// ERRORCODE 1: no input file found
	} else {
		srand(par.getRandSeed());							// initialize random seed
		pg.readFromFile(pfile);								// read the grid from the file // todo check different modes. do this in param?
		// works as long there is only one grid
	}
	cout << "Finished reading parameters." << endl;			// when everything was read inform user	// todo reading in every class is now separate

	pg.NRPF();												// bring power grid into stationary (power flow) state

//	if( par.isTestStoch() ){		// todo check this procedure
//		stochasticPseries(par);
//		if( par.writeStoch )
//			CodeUtils::vecToBinFileFast(par.powerSeries, par.stochFileName);
//	}

	Function* f1; 	// todo put this somewhere else
//	if( par.isModeSif() ){		// check here if stochastic and define corresponding the stochfunction // todo this is no longer necessary
//		f1 = new StochFunction(&par, &pg);
//	} else {
//		f1 = new Function(&par, &pg);		// create a function object that is responsible
//											// to return correct swing equation values for
//											// for current frequency and phase
//	}
	f1 = new Function(&par, &pg);			// create a function object that is responsible
											// to return correct swing equation values for
											// for current frequency and phase	// todo the grid is now responsible. crosscheck this

	RungeKutta rk = {pg.getNumNodes(), f1};	// initialize Runge Kutta solver object

	clock_t t1 = clock();					// get current timer object to measure runtime of this program
	double t1d = time(NULL);				// get current time as double

	std::stringstream parstr;


	if(par.graphv){							// if graphviz output is requested 		// todo later
		par.toGraphViz();					// write out graphviz file
	}

	par.toFile();							// write out parameters. The user is able to check his input while program runs

	if( par.isNRonly()){					// if user only wanted to check if there are different fixed point 	// todo later
		std::cout << "mode currently not supported." << std::endl;
//		calcFixedPoints(par);				// try to determine the fixed points. Hint: This is very unlikely.
	}										// maybe there's a better way in literature

	if( par.isModeMeanRetTimes() ){ 		// if user chose monte carlo sampling of phase space	// todo later
		std::cout << "mode currently not supported." << std::endl;
//		getMeanRetTimes(par,rk);			// with return time measure
	}

	if( par.isModeRasterRt() ){				// if user choose rastering phase space with return time
		RasterReturnTime rrt(&pg);			// generate instance for analyzing return time on raster/grid
		rrt.readFromFile(pfile);			// read parameters needed for return times
		parstr << rrt.toString();			// return time parameters to string and later to file
		rrt.rasterRT(par, rk);				// of energy or frequency	// par input necessary for: modes
	}

	if( par.isModePPert()){					// iterate swing equation for perturbations in powers
		std::cout << "mode currently not supported." << std::endl;
//		swingDP(par,rk); 					// swing eq with Delta Power
	}

	if( par.isModeStochInp() ){				// iterate swing equation for perturbations in powers from time series
		StochasticInput si(&pg, &par); 		// generate instance for analyzing grid under stochastic input
		si.readFromFile(pfile);				// read parameters needed for stochastic input
		si.stochSwing(par, rk);				// start simulation with stochastic input
		parstr << si.toString();			// save stochastic parameters to string and later to file	// todo good here??
	}


	cout << "dclockt/cpu clock ticks = " << (double)(clock()-t1) << endl;	// print out the clock time
	cout << "ddoublet/real time (sec) = " << (time(NULL)-t1d) << endl;		// print out the real time in seconds
	par.simtime = (time(NULL)-t1d);											// store the taken real time into param folder

	par.toFile();															// print out the parameters again to have also runtime ones
	parstr << pg.toString(); 												// write out final form of power grid // think about better impl when mult grids
	writeToFile(par,parstr.str());

	delete f1;

	cout << "Done."<< endl;					// inform user that program is finished
	return 0;								// return that program worked fine
}

