/*
 * PowerPerturbation.cpp
 *
 *  Created on: Feb 27, 2019
 *      Author: mawolff
 */

#include "PowerPerturbation.h"

#include <tgmath.h>	// fabs, sqrt

// =================================================== constructors =================================================== //
PowerPerturbation::PowerPerturbation() {
	// TODO Auto-generated constructor stub

}

PowerPerturbation::~PowerPerturbation() {
	// TODO Auto-generated destructor stub
}

// ====================================================== methods ===================================================== //

/**
 * Stores the maximum perturbation for frequency and phase respectively
 */
void PowerPerturbation::setMaxPert(Param& par, double* freq, double* phas, double t){

	for( int i = 0; i < par.len; i++){
		if( std::fabs(freq[i]) > std::fabs(par.getFreqmax().at(i)) ){
			par.getFreqmax().at(i) = freq[i];
			par.getFreqmaxTime().at(i) = t;
		}
		if( std::fabs(phas[i]) > std::fabs(par.getPhasmax().at(i)) ){
			par.getPhasmax().at(i) = phas[i];
			par.getPhasmaxTime().at(i) = t;
		}
	}

}


void PowerPerturbation::MaxPDP(Param& par, PowerGrid& pg, RungeKutta& rk){

	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint; 							// temporary array for switching in1 and in2

	for(int i = 0; i < par.len ; i++){			// and fill every entry
		thetaIn2[i] = 0;						// of the first output array (is later switched with input)
		omegaIn2[i] = 0;						//
	}

	std::vector<double> ipm;					// vector storing dnode, rnode, dP, maxFreq, maxFreqT, maxPhase, maxPhaseT

	srand(par.getRandSeed());					// set random seed

	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = 0;						// node's initial frequency
	}
	thetaIn1 = pg.getVoltAngSS()->data(); 		// node's initial phase

//	par.resetMaxPert();	// todo revive
	for(int s = 0; s < par.ppRKSteps; s++){									// for all requested runge kutta steps

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		setMaxPert(par,omegaIn2,thetaIn2,s*par.rkStepWidth);				// store the perturbation if its bigger than former ones

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	pg.resetPowerMech();						// comment todo

	for(int s = par.ppRKSteps; s < par.RKsteps; s++){							// for all requested runge kutta steps

			rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

			setMaxPert(par,omegaIn2,thetaIn2,s*par.rkStepWidth);				// store the perturbation if its bigger than former ones

			tempPoint = omegaIn1;												// switch in and out
			omegaIn1 = omegaIn2;
			omegaIn2 = tempPoint;
			tempPoint = thetaIn1;
			thetaIn1 = thetaIn2;
			thetaIn2 = tempPoint;

		}

	for(int n = 0; n < par.len; n++){			// for every reacting node
//		ipm.push_back(par.firstDistNode);		// push back disturbed node 	// todo firstdistnode in this class. to read from param file
		ipm.push_back(n);						// push back reacting node
		ipm.push_back(par.deltaPstrengh);		// push back power disturbance
		ipm.push_back(par.getFreqmax().at(n));			// maximum frequency elongation at node n
		ipm.push_back(par.getFreqmaxTime().at(n));		// time of maximum frequency elongation at node n
		ipm.push_back(par.getPhasmax().at(n));			// maximum phase elongation at node n
		ipm.push_back(par.getPhasmaxTime().at(n));		// time of maximum phase elongation at node n
	}
	CodeUtils::vecToBinFile(ipm,par.getOfile("path")+"ipom/ipom"+par.getOfile("file"));	// write the vector to a binary file


	delete[] thetaIn1;														// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
}

void PowerPerturbation::TrajDP(Param& par, PowerGrid& pg, RungeKutta& rk){

	double** freqTraj = new double*[par.RKsteps];	// storage for the frequency trajectory
	double** phasTraj = new double*[par.RKsteps];	// storage for the phase trajectory
//	par.resetSurv();	// TODO


	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint; 							// temporary array for switching in1 and in2

	for(int i = 0; i < par.len ; i++){			// and fill every entry
		thetaIn2[i] = 0;						// of the first output array (is later switched with input)
		omegaIn2[i] = 0;						//
	}
	for(int i = 0; i < par.RKsteps; i++){		// initialize storage arrays
		freqTraj[i] = new double[par.len];		// for frequency
		phasTraj[i] = new double[par.len];		// and for phase
	}

	srand(par.getRandSeed());					// set random seed

	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = 0;						// node's initial frequency
	}
	thetaIn1 = pg.getVoltAngSS()->data(); 		// node's initial phase

	for(int s = 0; s < par.ppRKSteps; s++){									// for all requested runge kutta steps

//		rk.propagateP(par.turbineNode,s);									// depending on the underlying function the power at certain nodes // todo
																			// is varied or not

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		std::copy(&omegaIn2[0],&omegaIn2[0]+par.len,						// store current frequencies
				&freqTraj[s][0]);
		std::copy(&thetaIn2[0],&thetaIn2[0]+par.len,						// store current phases
				&phasTraj[s][0]);
//		par.countSurv(omegaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	pg.resetPowerMech();

	for(int s = par.ppRKSteps; s < par.RKsteps; s++){						// for all requested runge kutta steps

//		rk.propagateP(par.turbineNode,s);	// todo

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		std::copy(&omegaIn2[0],&omegaIn2[0]+par.len,						// store current frequencies
				&freqTraj[s][0]);
		std::copy(&thetaIn2[0],&thetaIn2[0]+par.len,						// store current phases
				&phasTraj[s][0]);
//		par.countSurv(omegaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for( int s = 0; s < par.RKsteps; s++){
		CodeUtils::arrayToBinFile(freqTraj[s],par.len,									// write stored frequency to binary file
				par.getOfile("path")+"frequency/freqTraj"
					+par.getOfile("file"));
		CodeUtils::arrayToBinFile(phasTraj[s],par.len,
				par.getOfile("path")+"phase/phasTraj"						// write stored phase to binary file
					+par.getOfile("file"));
//		vecToBinFileFast(par.surviv, par.getOfile("path")+"surv/surv"						// write stored phase to binary file
//							+par.getOfile("file"));
		delete[] freqTraj[s];												// clear frequency storage
		delete[] phasTraj[s];												// clear phase storage
	}

	delete[] thetaIn1;														// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
}


void PowerPerturbation::swingDP(Param& par, RungeKutta& rk){

	CodeUtils::cdir("frequency");
	CodeUtils::cdir("phase");
	CodeUtils::cdir("ipom");


	int pstep = 0;
	//	while( generateNet(par) ){							// for all nets which should be investigated // also NR to check FixedPoint here
	while(pstep < par.ppsteps){ 					// for all frequencies (omega) lower max frequency ome

//		for( int i = 0; i < par.len; i++){ 			// set p = P0+dp here 		// todo pwoer grid set power
//			par.Pm.at(i) = par.Pm0.at(i)
//								*( 1 +  par.DeltaP.at(i) * par.deltaPstrengh );
//		}

//		if(par.isModeStraj()){						// if a calculation of the trajectory is requested
//			CodeUtils::cdir("surv");							// check whether this folder is needed
//			TrajDP(par,rk);
//		}
//
//		if(par.isModeMaxp()){
//			MaxPDP(par,rk);
//		}


//		if(par.isModeSurvivall()){
//			CodeUtils::cdir("surv");					// folder to store surviveability results 	// todo put to stochasticinput
//			CodeUtils::cdir("flanks");					// folder to store flanks results
//			for( int tn = par.fturbnode; tn < par.lturbnode; tn++){
//				par.turbineNode = tn;
//				stochasticPseries(par);
//				Survivall(par,rk, tn);
//			}
//		}

		par.deltaPstrengh += par.deltaPstep;
		pstep++;
	}
	//	}

	/* pseudo code:
	 create files to store results
	 check in function.cpp if there is the correct power
	 for all t < t_p,end
	 iterate swing equation with p+dp
	 for all t_p,end < t < t_end
	 iterate swing equation with P0
	 save trajectory or other results */
}

/**
 * cleans up the vectors and set their size to current par.len
 */
void PowerPerturbation::resetMaxPert(){ 		// todo without param

//	getFreqmax().clear();
//	getFreqmaxTime().clear();
//	getPhasmax().clear();
//	getPhasmaxTime().clear();
//
//	getFreqmax().resize(len);
//	getFreqmaxTime().resize(len);
//	getPhasmax().resize(len);
//	getPhasmaxTime().resize(len);
//
//	for( int i = 0; i < getFreqmax().size(); i++){
//		getFreqmax().at(i) = 0;
//		getFreqmaxTime().at(i) = 0;
//		getPhasmax().at(i) = 0;
//		getPhasmaxTime().at(i) = 0;
//	}
}

// ==================================================== variables ===================================================== //
// =================================================== constructors =================================================== //
// ====================================================== methods ===================================================== //
// --------------------------------------------------- array methods -------------------------------------------------- //
// ------------------------------------------------- directory methods ------------------------------------------------ //
// ------------------------------------------------ mathematical methods ---------------------------------------------- //
// --------------------------------------------------- string methods ------------------------------------------------- //
// ------------------------------------------------------- get&set ---------------------------------------------------- //
// ------------------------------------------------------ read methods ------------------------------------------------ //
