/*
 * RasterReturnTime.h
 *
 *  Created on: Feb 25, 2019
 *      Author: mawolff
 */

#ifndef RASTERRETURNTIME_H_
#define RASTERRETURNTIME_H_

//#include "Param.h"
#include "PowerGrid.h"
#include "RungeKutta.h"

class RasterReturnTime {
// ==================================================== variables ===================================================== //
private:
	PowerGrid* pgrid;		// pointer to the grid being under investigation

	int pertDelay;			// number of (RungeKutta) iteration steps before the perturbation is applied
	int firstPertNode;		// when investigate the perturbation at different nodes start with this one
	int curPertNode;		// indicates the node currently under investigation/perturbation
	int nodeIterator;		// iterates through all nodes for whom it is requested to be perturbed
	int lastPertNode;		// last node to be investigated

	int rtmatsize;			// stores size of rtmat array
	bool fixedThreshNum;	// if number of thresholds is explicitly given

	double freqPertMin;		// minimum voltage frequency perturbation
	double freqPertMax;		// maximum voltage frequency perturbation
	double phasePertMin;	// minimum voltage phase perturbation
	double phasePertMax;	// maximum voltage phase perturbation
	double freqStepw;		// step width of the frequency perturbation
	double phaseStepw;		// step width of the phase perturbation
	double thresholdHigh;	// when all frequencies drop below this threshold the current time is taken as return time
	double thresholdLow;	// when high and low are given, multiple threshold is activated
	double threshSpread;	// for multiple return thresholds. How large is the spread of them
	int numOfThresh;		// number of multiple return thresholds
	std::vector<double>
				thresholds;	// all thresholds

	double** rtmat;			// stores the return times for all considered thresholds and nodes
	bool** trconv;			// stores if the trajectory is already under threshold(s)

// =================================================== constructors =================================================== //
public:
	RasterReturnTime(PowerGrid* pg);
	virtual ~RasterReturnTime();

// ====================================================== methods ===================================================== //
// ------------------------------------------------------ read methods ------------------------------------------------ //
	bool readFromFile(std::string file);

// ------------------------------------------------------ simulation -------------------------------------------------- //
public:
	void rasterRT(Param& par, RungeKutta& rk);		// this function selects the simulation method
private:
	void resetThresh();
	void setEnergyRet(Param& par, double* freq, double* phas, int s);
	double currentEnergy(double* freq, double* phas);
	double harmonicEnergy(double* freq, double* phas);
	double enT(double* freq, double* phas);
	double enV(double* freq, double* phas);
	bool vecConvergence(double* vec, double konvN, int len);
	void countVecConv(double* vec, int pertNode, Param* par);
	void maxPIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC);
	void MonteCarloRetTimes(Param& par, RungeKutta& rk);
	void enrtIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC);
	void mrtIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC);
	void trajIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC);

	void setMeanRet(double* rtv);
	void setRetTime(double* freq, double t, int thr);
	void setRetAllThresh(double* freq, double t);
	bool nextPertNode();
// ----------------------------------------------------- persistence -------------------------------------------------- //
public:
	std::string toString();

// ------------------------------------------------------- get&set ---------------------------------------------------- //
public:
	double getFreqPertMax() const;
	void setFreqPertMax(double freqPertMax);
	double getFreqPertMin() const;
	void setFreqPertMin(double freqPertMin);
	double getFreqStepw() const;
	void setFreqStepw(double freqStepw);
	int getNumOfThresh() const;
	void setNumOfThresh(int numOfThresh);
	double getPertDelay() const;
	void setPertDelay(double pertDelay);
	double getPhasePertMax() const;
	void setPhasePertMax(double phasePertMax);
	double getPhasePertMin() const;
	void setPhasePertMin(double phasePertMin);
	double getPhaseStepw() const;
	void setPhaseStepw(double phaseStepw);
	double getThresholdHigh() const;
	void setThresholdHigh(double threshold);
	double getThresholdLow() const;
	void setThresholdLow(double threshold);
	double getThreshSpread() const;
	void setThreshSpread(double threshSpread);

	double getThresh(int i);
	bool isFixedThreshNum() const;
	void setFixedThreshNum(bool fixedThreshNum);
};

#endif /* RASTERRETURNTIME_H_ */

// ==================================================== variables ===================================================== //
// =================================================== constructors =================================================== //
// ====================================================== methods ===================================================== //
// --------------------------------------------------- array methods -------------------------------------------------- //
// ------------------------------------------------- directory methods ------------------------------------------------ //
// ------------------------------------------------ mathematical methods ---------------------------------------------- //
// --------------------------------------------------- string methods ------------------------------------------------- //
// ------------------------------------------------------- get&set ---------------------------------------------------- //
// ------------------------------------------------------ read methods ------------------------------------------------ //
