
#include "RungeKutta.h"
#include <math.h>
#include <iostream>

// =================================================== constructors & destructors =================================================== //
	RungeKutta::RungeKutta(int vlen, Function* f){
		func = f;
		veclen = vlen;
		k1o = new double[vlen];
		k2o = new double[vlen];
		k3o = new double[vlen];
		k4o = new double[vlen];
		k1t = new double[vlen];
		k2t = new double[vlen];
		k3t = new double[vlen];
		k4t = new double[vlen];
		addveco = new double[vlen];
		addvect = new double[vlen];

	}

	RungeKutta::~RungeKutta(){
		delete[] k1o;
		delete[] k2o;
		delete[] k3o;
		delete[] k4o;
		delete[] k1t;
		delete[] k2t;
		delete[] k3t;
		delete[] k4t;
		delete[] addveco;
		delete[] addvect;
	}


// ====================================================== methods ===================================================== //
	/**
	 * @parameters
	 * xstep
	 * omstart[]
	 * thstart[]
	 * omres[]
	 * thres[]
	 */
	void RungeKutta::rk4(double xstep, double* omstart, double* thstart, double* omres, double* thres){

		int len = veclen;	// initialize number of dgls

		for(int i = 0; i < len; i++){
			k1o[i] = 1.0/2.0*xstep*valOmAt(i, omstart, thstart);
			k1t[i] = 1.0/2.0*xstep*valThAt(i, omstart, thstart);
		}

		CodeUtils::arrAddD(omstart,k1o,addveco,len);		// add omstart and k1o and put result to addveco
		CodeUtils::arrAddD(thstart,k1t,addvect,len);
		for(int i = 0; i < len; i++){
			k2o[i] = 0.5*xstep*valOmAt(i, addveco, addvect);
			k2t[i] = 0.5*xstep*valThAt(i, addveco, addvect);
		}

		CodeUtils::arrAddD(omstart, k2o, addveco,len);
		CodeUtils::arrAddD(thstart, k2t, addvect,len);
		for(int i = 0; i < len; i++){
			k3o[i] = xstep*valOmAt(i, addveco, addvect);
			k3t[i] = xstep*valThAt(i, addveco, addvect);
		}

		CodeUtils::arrAddD(omstart,k3o,addveco,len);
		CodeUtils::arrAddD(thstart,k3t,addvect,len);
		for(int i = 0; i < len; i++){
			k4o[i] = xstep*valOmAt(i, addveco, addvect);
			k4t[i] = xstep*valThAt(i, addveco, addvect);
		}

//		double pi = 3.14159265359;
		for(int i = 0; i < len; i++){
			omres[i] = omstart[i] + 2.0/6.0*k1o[i] + 2.0/3.0*k2o[i] + 1.0/3.0*k3o[i] + 1.0/6.0*k4o[i];
			thres[i] = thstart[i] + 2.0/6.0*k1t[i] + 2.0/3.0*k2t[i] + 1.0/3.0*k3t[i] + 1.0/6.0*k4t[i];
			thres[i] = CodeUtils::smallerAngle(thres[i]);
		}

		return;
	}

	/**
	 * Derivative Function of theta
	 */
	double RungeKutta::valThAt(int i, double* om, double* th){ 		// todo das sollte alles function machen
			return om[i];

	}

	/**
	 * Derivative Function of omega
	 */
	double RungeKutta::valOmAt(int i, double* om, double* th){
		return func->secDeriv(i, om, th);
	}

