/*
 * TransmissionLine.cpp
 *
 *  Created on: Feb 22, 2019
 *      Author: mawolff
 */

#include "TransmissionLine.h"

// =================================== variables =================================== //
TransmissionLine* TransmissionLine::nullLine = NULL;

// =========================== constructors & destructors ========================== //
TransmissionLine::TransmissionLine() {
	init();

}

//TransmissionLine::TransmissionLine(std::string str){ // : TransmissionLine() { // not compilable on cluster
//	init();
//	if( str.compare("null") == 0 ){
//		nulll = true;
//		fromNode = -1;
//		toNode = -1;
//	}
//}

TransmissionLine::TransmissionLine(int i, int j){ // : TransmissionLine() {	// not compilable on cluster
	init();
	online = true;
	fromNode = i;
	toNode = j;
}

void TransmissionLine::init(){
	nulll = false;
	online = false;
	fromNode = 0;
	toNode = 0;
	admittanceAng = 0;
	admittanceMag = 0;
	couplingAng = 0;
	couplingMag = 0;
	capacity = 0;
	cflow = std::complex<double>(0,0);

	meanFlow = 0;
	maxFlow = 0;
	curFlow = 0;
}

/**
 * There are currently no fields to be deleted
 */
TransmissionLine::~TransmissionLine() {

}


// ==================================== methods ==================================== //

TransmissionLine* TransmissionLine::getNullLine(){
	if( nullLine == NULL){
		nullLine = new TransmissionLine();
		nullLine->nulll = true;
		nullLine->fromNode = -1;
		nullLine->toNode = -1;
	}
	return nullLine;
}
//{
//
//		if( nullLine == NULL){
//			nullLine = new TransmissionLine();
//			nullLine->nulll = true;
//			nullLine->fromNode = -1;
//			nullLine->toNode = -1;
//		}
//		return nullLine;
//	}

double TransmissionLine::getFlow(double* voltPhases){
	std::complex<double> phas12 = std::complex<double>(0,
			voltPhases[fromNode] - voltPhases[toNode] - getAdmittanceAng());
	return std::abs( getCflow() 						// calculate power currently flowing through line
				- getCouplingMag()* exp( phas12 ) );
}

/**
 * double avSteps : the number of steps over which the flow is averaged
 */
double TransmissionLine::measureFlow(double* voltPhases){
	curFlow = getFlow(voltPhases);
	addMeanFlow(curFlow);
	setMaxFlow(curFlow);
	return curFlow;
}


// ------------------------------------ get&set ------------------------------------ //
double TransmissionLine::getAdmittanceAng() const {
	return admittanceAng;
}

void TransmissionLine::setAdmittanceAng(double admittanceAng) {
	this->admittanceAng = admittanceAng;
}

double TransmissionLine::getAdmittanceMag() const {
	return admittanceMag;
}

void TransmissionLine::setAdmittanceMag(double admittanceMag) {
	this->admittanceMag = admittanceMag;
}

double TransmissionLine::getCapacity() const {
	return capacity;
}

void TransmissionLine::setCapacity(double capacity) {
	this->capacity = capacity;
}

std::complex<double> TransmissionLine::getCflow() const {
	return cflow * (double)isOnline();
}

void TransmissionLine::setCflow(std::complex<double> cflow) {
	this->cflow = cflow;
}

double TransmissionLine::getCouplingAng() const {
	return couplingAng;
}

void TransmissionLine::setCouplingAng(double couplingAng) {
	this->couplingAng = couplingAng;
}

void TransmissionLine::setCouplingMag(double couplingStrength) {
	this->couplingMag = couplingStrength;
}

double TransmissionLine::getCouplingMag() {
	return this->couplingMag * this->isOnline();
}

int TransmissionLine::getFromNode() const {
	return fromNode;
}

void TransmissionLine::setFromNode(int fromNode) {
	this->fromNode = fromNode;
}

bool TransmissionLine::isOnline() const {
	return online;
}

void TransmissionLine::setOnline(bool online) {
	this->online = online;
}

int TransmissionLine::getToNode() const {
	return toNode;
}

void TransmissionLine::setToNode(int toNode) {
	this->toNode = toNode;
}

bool TransmissionLine::isNull(){
	return this->nulll;
}

void TransmissionLine::addMeanFlow(double flow){
	meanFlow += flow;
}

double TransmissionLine::getMeanFlow(){
	return meanFlow;
}

void TransmissionLine::resetMeanFlow(){
	meanFlow = 0;
}

bool TransmissionLine::setMaxFlow(double flow){
	if( std::fabs(flow) > maxFlow){
		maxFlow = std::fabs(flow);
		return true;
	}
	return false;
}

double TransmissionLine::getMaxFlow(){
	return maxFlow;
}

void TransmissionLine::resetMaxFlow(){
	maxFlow = 0;
}

