/*
 * TransmissionLine.h
 *
 *  Created on: Feb 22, 2019
 *      Author: mawolff
 */

#ifndef TRANSMISSIONLINE_H_
#define TRANSMISSIONLINE_H_

#include <complex>
#include <string.h>
#include <cstring>

#include "Param.h"

class TransmissionLine {

// ========================================= variables ========================================= //
// ------------------------------------ physical magnitudes ------------------------------------ //
private:
	bool nulll;						// only for null line this is true
	bool online;					// status of the line (e.g. if over heated the line gets offline)

	int fromNode;					// starting node of this line
	int toNode;						// node where line ends

	double admittanceMag;			// magnitude of total admittance of this line
	double admittanceAng;			// (complex) angle of total admittance of this line
	double couplingMag;				// |V_i||V_j||Y_ij| to save calculation steps
	double couplingAng;				// todo formula with admittance ang to save calculations
	double capacity;				// the amount of power that can flow through this line before it overheats

	std::complex<double> cflow;		// constant offset of line flow (pure mathematical expression)
									// you can guess: saves calculation time

	static TransmissionLine* nullLine;	// a null object to check missing lines

// ------------------------------------ measured magnitudes ------------------------------------ //
// think of derived class: FlowMeasureTransmissionLine or something similar

	double meanFlow; 				// the average flow in this transmission line
	double maxFlow; 				// the maximum flow in this line during simulation
	double curFlow;					// current flow in this line (auxiliary variable)

// ================================= constructors & destructors ================================ //
//private:
//	TransmissionLine(std::string);
public:
	TransmissionLine();
	TransmissionLine(int i, int j);
	void init();
	virtual ~TransmissionLine();

// ========================================== methods ========================================== //
public:
	double getFlow(double* voltPhases);
	double measureFlow(double* voltPhases);

// ------------------------------------ get&set ------------------------------------ //
public:
	bool isOnline() const;
	void setOnline(bool online);

	int getFromNode() const;
	void setFromNode(int fromNode);
	int getToNode() const;
	void setToNode(int toNode);

	double getAdmittanceAng() const;
	void setAdmittanceAng(double admittanceAng);
	double getAdmittanceMag() const;
	void setAdmittanceMag(double admittanceMag);
	double getCapacity() const;
	void setCapacity(double capacity);
	std::complex<double> getCflow() const;
	void setCflow(std::complex<double> cflow);
	double getCouplingAng() const;
	void setCouplingAng(double couplingAng);
	double getCouplingMag();							// returns coupling strength and zero if its offline
	void setCouplingMag(double couplingStrength);

	static TransmissionLine* getNullLine();
	bool isNull();

	void addMeanFlow(double flow);
	double getMeanFlow();
	void resetMeanFlow();
	bool setMaxFlow(double flow);
	double getMaxFlow();
	void resetMaxFlow();
};

#endif /* TRANSMISSIONLINE_H_ */


// =================================== variables =================================== //
// =========================== constructors & destructors ========================== //
// ==================================== methods ==================================== //
// ------------------------------------ get&set ------------------------------------ //
// --------------------------------- array methods --------------------------------- //
// ------------------------------- directory methods ------------------------------- //
