
%% electrodynamic basics
% creates plots of DC, AC and AC-, DC voltages 
%

global NO_CLEAR
if NO_CLEAR    % for testing scripts
else
    clear;
end
addpath('methods');

save_fig = false;

unir  = unicolors('dr','-1');
unilo = unicolors('lo','-1');
unio  = unicolors('o','-1');
unido = unicolors('do','-1');
dgreen = vrggrad(8,10);

alo.fontname = 'Times New Roman';   % (a)-(d) label fontname and size
alo.siz = 1.05*dissFS('fr');

%% AC 
FORMAT = 'fr';

omega_r = 50*2*pi;  % reference frequency (angular freq)

R = 1.25;
L = 2/(1*omega_r);
C = 1/(2*omega_r);
Zc = @(w,C) -1i./(w * C);
Zcr = @(w,C,R) R - 1i./(w * C);
Zi = @(w,L) 1i .* w * L;
Zr = @(w,R) R;

VA = 1;    % voltage amplitude
YLIM = [-1 1]*VA;

V  = @(VA, w, t) VA*exp(1i*w*t);    % voltage signal
Ic = @(VA, w, t) (1./Zc(w,C) .* V(VA,w,t));   % capacitive current
Ii = @(VA, w, t) (1./Zi(w,L) .* V(VA,w,t));   % inductive current
Ir = @(VA, w, t) (1./Zr(w,R) .* V(VA,w,t));   % ohmic current

te = (2*2*pi/omega_r);
secs = 2;
tps = 100;
t = 0:te/(tps*secs):te;
f = omega_r/(2*pi);
tpl = t * omega_r/(2*pi);
% tpl = (0:tps*secs)/tps;
% tpl = (tpl) .* omega_r;

fig = figure;
set( fig, 'position', [200 200 [1 0.5].*figSizeDiss(FORMAT)]);
axac = axes;

LW_STROKE_LINES = 2.5;
LW_LINES = 2;
plv = plot( tpl, real(V(VA,omega_r,t)), ...
    'color', 'k', ...
    'linewidth', LW_LINES, ...
    'displayname', '$V$');
hold on;
plir = plot( tpl, real(Ir(VA,omega_r,t)), ...
    'color', dgreen, ...
    'linewidth', LW_LINES, ...
    'displayname', '$I_{\rm R}$');
plii = plot( tpl, real(Ii(VA,omega_r,t)), ...
    'color', unido, ...
    'linewidth', LW_LINES, ...
    'displayname', '$I_{\rm L}$');
plic = plot( tpl, real(Ic(VA,omega_r,t)), ...
    'color', 'r', ...
    'linewidth', LW_LINES, ...
    'displayname', '$I_{\rm C}$');

plot( [1 1], YLIM, '-', ...
    'color', 'k', ...
    'linewidth', LW_STROKE_LINES);
plot( [1 1], YLIM, '--', ...
    'color', dgreen, ...
    'linewidth', LW_STROKE_LINES);
% plot( [1 1]*te/2+angle()/omega_r, YLIM, '--', ...
%     'color', 'r');
plot( [1 1]+angle(Zi(omega_r,L))/2/pi, YLIM, '--', ...
    'color', unido, ...
    'linewidth', LW_STROKE_LINES);
plot( [1 1]+angle(Zc(omega_r,C))/2/pi, YLIM, '--', ...
    'color', 'r', ...
    'linewidth', LW_STROKE_LINES);

set( axac, ...
    'fontsize', dissFS(FORMAT), ...
    'position', [0.1 0.22 0.8 0.7], ...
    'xtick', 0:0.25:2);

xl = dissXLabel('$t/T$',FORMAT);
yl = dissYLabel('$V/|V|, \,\, I/|V|$','hr');
tb = dissAlabel( fig, '(e)', alo );
tbpos = get( tb, 'position');
set( tb, ...
    'position', tbpos + [-0.01 -0.02 0 0],...
    'fontweight', 'bold');


leg = legend([plv plir plii plic]);
set( leg, 'interpreter', 'latex', ...
    'fontsize', dissFS(FORMAT), ...
    'position', [0.92  0.9-0.445  0.07 0.445]);


if( ~~save_fig )
    SPATH = ''; % path to save the figure in
    saveas( fig, [SPATH 'ACvoltage.eps'], 'epsc');
end



