
%% voltage and current plots 
% for dissertation
%
% V1.0 mawolff@uos.de 11-Sep-2019

addpath('methods');
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end

sav_fig = false;

V  = @(VA,w,t) VA*exp(1i*w*t);
w  = 2*pi*50;
T  = 1/50;
periods = 2;
te = periods*T;
t = 0:te/100:te;

TICK_LEN = [0.025 0.01];
YTICKU = -1:0.5:1;
YTICKD = -0.5:0.25:0.5;
Y_TICK_LU = cell(numel(YTICKU),1);
Y_TICK_LD = cell(numel(YTICKU),1);
for i = 1:numel(YTICKU)
    Y_TICK_LD{i} = num2str(YTICKD(i));
    Y_TICK_LU{i} = num2str(YTICKU(i));
end
Y_TICK_LD{end} = '';
Y_TICK_LU{1}   = '';

unir = unicolors('dr','-1');
unio = unicolors('do','-1');
unilo = unicolors('o','-1');
rosa = [1 0.6 0.55];
green = [0.25 0.8 0.2];
veffcolor = green;
pcolor = [0 0 1];

fig = figure;
set( fig, 'position', [100 300 figSizeDiss('fr')]);

% =============================== pure R  =============================== %
R = 2;
X = 0;
Z = R + 1i*X;
Y = 1./Z;
VA = 1;
Vt = V(VA,w,t);
It = Y.*Vt;
Vr = real(Vt);
Ir = real(It);
Pt = Vr.*Ir;

subplot(3,3,7);
plot( t/T, Pt, 'color', pcolor, ...
    'linewidth', 2);
hold on;
plot( [0 periods], [1 1]*mean(Pt(1:end-1)), '--', 'color', pcolor, ...
    'linewidth', 2);
axd = gca;
set( axd, 'yticklabels', Y_TICK_LD);
set( axd, 'ticklength', TICK_LEN);
set( axd, 'ylim', [-1 1]*0.5, ...
    'ygrid', 'on', ...
    'ytick', YTICKD);
dissXLabel('t/T','fr');
dissYLabel('$p(t)/|V|$','fr');

AXDPOS = get( gca, 'position');

subplot(3,3,4);
pV = plot( t/T, Vr, 'color', unir, ...
    'linewidth', 2, ...
    'displayname', '$V$');
hold on;
pI = plot( t/T, Ir, 'color', unio, ...
    'linewidth', 2, ...
    'displayname', '$I$');
% pIeff = plot( t/T, Ir, 'color', unio, ...
% %     'linewidth', 2, ...
%     'displayname', '$I_{\rm eff}$');
pVeff = plot( t/T, Vr/sqrt(2), 'color', veffcolor, ...
    'linewidth', 2, ...
    'displayname', '$\hat{V}$'); % '$V_{\rm eff}$'
set( gca, 'xticklabels', {}, 'yticklabels', Y_TICK_LU);
set( gca, 'ticklength', TICK_LEN);
set( gca, 'ylim', [-1 1], ...
    'ygrid', 'on', ...
    'ytick', YTICKU);
% dissXLabel('t/T','fr');
dissYLabel('$V/|V|,\,\, I/|V|$','fr');
AXPOS = get( gca, 'position');
set( gca, 'position', [AXPOS(1) AXDPOS(2)+AXDPOS(4) ...
    AXPOS(3) (AXPOS(2)+AXPOS(4))-(AXDPOS(2)+AXDPOS(4))]);
set( axd, 'position', [AXPOS(1) AXDPOS(2) AXPOS(3) AXDPOS(4)]);

% ----------------------------- help lines  ----------------------------- %
plot( [0 0.75], [1 1]*0.975, 'color', unir, ...
    'linewidth', 1.5);
plot( [0 0.5], [1 1]/sqrt(2), 'color', veffcolor, ...
    'linewidth', 1.5);
    
leg = legend([pV pI pVeff]);
set( leg, 'interpreter', 'latex',...
    'fontsize', dissFS('fr'),...
    'position', [0.28246 0.48061 0.070644 0.15233]);


% =============================== pure X  =============================== %
R = 0;
X = 2;
Z = R + 1i*X;
Y = 1./Z;
VA = 1;
Vt = V(VA,w,t);
It = Y.*Vt;
Vr = real(Vt);
Ir = real(It);
Pt = Vr.*Ir;

subplot(3,3,8);
plot( t/T, Pt, 'color', pcolor, ...
    'linewidth', 2);
hold on;
plot( [0 periods], [1 1]*mean(Pt(1:end-1)), '--', 'color', pcolor, ...
    'linewidth', 2);
axd = gca;
set( axd, 'yticklabels', Y_TICK_LD);
set( axd, 'ticklength', TICK_LEN);
set( axd, 'ylim', [-1 1]*0.5, ...
    'ygrid', 'on', ...
    'ytick', YTICKD);
dissXLabel('t/T','fr');
dissYLabel('$p(t)/|V|$','fr');

AXDPOS = get( gca, 'position');

subplot(3,3,5);
plot( t/T, Vr, 'color', unir, ...
    'linewidth', 2);
hold on;
plot( t/T, Ir, 'color', unio, ...
    'linewidth', 2);
set( gca, 'xticklabels', {}, 'yticklabels', Y_TICK_LU);
set( gca, 'ticklength', TICK_LEN);
set( gca, 'ylim', [-1 1], ...
    'ygrid', 'on', ...
    'ytick', YTICKU);
% dissXLabel('t/T','fr');
dissYLabel('$V/|V|,\,\, I/|V|$','fr');
AXPOS = get( gca, 'position');
set( gca, 'position', [AXPOS(1) AXDPOS(2)+AXDPOS(4) ...
    AXPOS(3) (AXPOS(2)+AXPOS(4))-(AXDPOS(2)+AXDPOS(4))]);
set( axd, 'position', [AXPOS(1) AXDPOS(2) AXPOS(3) AXDPOS(4)]);

% =============================== R and X =============================== %
R = 1;
X = 2;
Z = R + 1i*X;
Y = 1./Z;
VA = 1;
Vt = V(VA,w,t);
It = Y.*Vt;
Vr = real(Vt);
Ir = real(It);
Pt = Vr.*Ir;

SA = 0.5*VA^2 * conj(Y);    % 0.5 due to effective voltage amplitude
S = @(SA,w,t) SA*exp(-1i*2*w*t);
St = S(SA,w,t);
Sr = real(St);

subplot(3,3,9);
plot( t/T, Pt, 'color', pcolor, ...
    'linewidth', 2);
hold on;
plot( [0 periods], [1 1]*mean(Pt(1:end-1)), '--', 'color', pcolor, ...
    'linewidth', 2);
axd = gca;
set( axd, 'yticklabels', Y_TICK_LD);
set( axd, 'ticklength', TICK_LEN);
set( axd, 'ylim', [-1 1]*0.5, ...
    'ygrid', 'on', ...
    'ytick', YTICKD);
dissXLabel('t/T','fr');
dissYLabel('$p(t)/|V|$','fr');

% verifying calculations
% plot( t/T, Sr, 'b');
% plot( t/T, Sr+real(SA), 'xb');

AXDPOS = get( axd, 'position');


subplot(3,3,6);
plot( t/T, Vr, 'color', unir, ...
    'linewidth', 2);
hold on;
plot( t/T, Ir, 'color', unio, ...
    'linewidth', 2);
set( gca, 'xticklabels', {}, 'yticklabels', Y_TICK_LU);
set( gca, 'ticklength', TICK_LEN);
set( gca, 'ylim', [-1 1], ...
    'ygrid', 'on', ...
    'ytick', YTICKU);
% dissXLabel('t/T','fr');
dissYLabel('$V/|V|,\,\, I/|V|$','fr');
AXPOS = get( gca, 'position');
set( gca, 'position', [AXPOS(1) AXDPOS(2)+AXDPOS(4) ...
    AXPOS(3) (AXPOS(2)+AXPOS(4))-(AXDPOS(2)+AXDPOS(4))]);
set( axd, 'position', [AXPOS(1) AXDPOS(2) AXPOS(3) AXDPOS(4)]);

if( ~~sav_fig )
    SPATH = ''; % path to store the figure
    saveas( fig, [SPATH 'voltPower'], 'svg');
end


