
%% voltage phase and transmission loss minimal example
%
% We consider two nodes 1 and 2 connected by a transmission line with
% symmetric admittance y12 and y21.
% The transmission of power in dependance of the voltage phase shift
% between the two nodes is investigated

addpath('methods');
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
SPATH = ''; % path to store the figures

save_fig = false;
save_fig2 = false;
% ========================== toggle figure 2.8 ========================== %
fig2_8 = false;        % figure with inset in P axis and neg vals for Q axis
% fig2_8 = true;        % uncomment here to print figure 2.8
FORMAT1 = 'hr';

alo.fontname = 'Times New Roman';   % (a)-(d) label fontname and size
alo.siz = 1*dissFS(FORMAT1);

phasor = @(Vm, Va) Vm.*exp(1i*Va);    % phasor function

if( ~~fig2_8 )
    Vm1 = 1.1;                 % voltage magnitude of node 1
else
    Vm1 = 1;                 % voltage magnitude of node 1
end
Vm2 = 1;                 % voltage magnitude of node 2
phi1 = -pi:0.01:pi;      % voltage angle of node 1
phi2 = 0;                % voltage angle of node 2
V1 = phasor(Vm1,phi1);    % complex voltage of node 1
V2 = phasor(Vm2,phi2);    % complex voltage of node 2
x = 0.1;        % easy examples close to real lines
r = 0.01;
z2 = r.^2 + x.^2;
g = r./z2;
b = -x./z2;
y12 = g + 1i* b;
y21 = y12;

S12 = @(V1,V2,Y12,Y21) (abs(V1).^2 - conj(V2).*V1) ...
                        .*conj(Y12);            % power injection by node 1
S21 = @(V1,V2,Y12,Y21) (abs(V2).^2 - conj(V1).*V2) ...
                        .*conj(Y21);            % power injection by node 2
Slo = @(V1,V2,Y12,Y21) abs(V2-V1).^2 ...
                        .*conj(Y12);            % power loss in line
           
c1r = lightToDarkBlueGradient( 8, 10);  % plot color of real power injection by node 1
c2r = lightToDarkBlueGradient( 4, 10);  % plot color of real power injection by node 2
clossr = unicolors( 'dr', '-1');
clossr = (clossr+[1 0 0])/2;            % lighter red
c1i = vrggrad( 6, 10);                  % plot color of imag power injection by node 1
c2i = vrggrad( 8, 10);                  % plot color of imag power injection by node 2
clossi = vrggrad( 55, 60 );             % plot color of imag power loss in line


% ============================ active power  ============================ %
fig = figure;
FIG_SIZE = dissFigSize(FORMAT1);
set( fig, 'position', [123 56 FIG_SIZE] );
P12 = plot( phi1-phi2, real(S12(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', c1r, ...
    'displayname', '$P_{1\rightarrow 2}$');
hold on;
axp = gca;
P21 = plot( phi1-phi2, real(S21(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', c2r, ...
    'displayname', '$P_{2\rightarrow 1}$');
hold on;
Pl = plot( phi1-phi2, real(Slo(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', clossr, ...
    'displayname', '$P^{\rm loss}_{12}$');
hold on;
set( gca, 'xlim', [-1 1]*pi );

Pmax = @(V1,V2,y) abs(V1).^2 .* real(y) + abs(V1) .*abs(V2) .*abs(y);
Pmin = @(V1,V2,y) abs(V1).^2 .* real(y) - abs(V1) .*abs(V2) .*abs(y);

hlr = [-1 1]*1.2*Pmax(Vm1,Vm2,y12); % helping line range
ang  = angle(y12);       % angle of maximal transmitted power
ang2 = angle(y12)+pi;   % angle of maximal power feed-in (higher due to losses)

plot( [1 1]*ang, hlr, 'color', c1r, ...
    'linewidth', 2);
plot( [1 1]*ang2, hlr, '--', 'color', c1r, ...
    'linewidth', 2);

plot( [1 1]*-ang, hlr, 'color', c2r, ...
    'linewidth', 2);
plot( [1 1]*-ang2, hlr, '--', 'color', c2r, ...
    'linewidth', 2);

xtl = cell(1,3);
if( ~~fig2_8 )
    XTCK = [-10 -5 0]*1e-3;
    axpInset = copyobj( axp, fig );
    set( axpInset, 'xgrid', 'on', 'ygrid', 'on', ...
        'position', [0.075 0.135 0.35 0.4], ...
        'xlim', [-12 4]*1e-3, ...
        'ylim', [-1 1]*0.15, ...
        'xtick', XTCK);
    for i = 1:2
        xtl{i} = sprintf('%1.3f', XTCK(i));
    end
    xtl{end} = '0';
    xticklabels(axpInset, xtl);
end
disp( xtl );

set( axp, 'xtick', (-pi):(pi/2):pi, ... 
    'ticklabelinterpreter','latex', ...
    'ylim', hlr);
if( ~fig2_8 )
    xtl = {'$-\pi$', '$-\pi/2$', '$0$', '$\pi/2$', '$\pi$'};
else
    xtl = {'', '', '$0$', '$\pi/2$', '$\pi$'};
end
xticklabels(xtl);
set( gca, 'fontsize', dissFS( 'hr' ) );

dissXLabel('$\varphi_1 - \varphi_2$', FORMAT1);
set( gca, 'xgrid', 'on', 'ygrid', 'on');

leg = legend([P12 P21 Pl]);
set( leg, 'interpreter', 'latex', ...
    'fontsize', dissFS(FORMAT1));
set( leg, 'position', ...
    [0.81061 0.23889 0.17798 0.33667]);

tb = dissAlabel( fig, '(a)', alo );
tbpos = get( tb, 'position');
set( tb, ...
    'position', tbpos + [0.01 -0.02 0 0] );

if( ~~save_fig )
    pause(1);
    saveas( fig, [SPATH 'Ptransmission.eps'], 'epsc');
end
if( ~~save_fig2 )
    pause(1);
    saveas( fig, [SPATH 'Ptransmission2.eps'], 'epsc');
end

% =========================== reactive power  =========================== %
fig = figure;
set( fig, 'position', [123+FIG_SIZE(1) 56 FIG_SIZE] );
P12 = plot( phi1-phi2, imag(S12(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', c1r, ...
    'displayname', '$Q_{1\rightarrow 2}$');
hold on;
P21 = plot( phi1-phi2, imag(S21(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', c2r, ...
    'displayname', '$Q_{2\rightarrow 1}$');
hold on;
Pl = plot( phi1-phi2, imag(Slo(V1,V2,y12,y21)), ...
    'linewidth', 2, ...
    'color', clossr, ...
    'displayname', '$Q^{\rm loss}_{12}$');
hold on;
if( ~fig2_8 )
    set( gca, 'xlim', [-1 1]*pi );
else
    set( gca, 'xlim', [-1 1]*0.75*pi );
end

Qmax = @(V1,V2,y) abs(V1).^2 .* -imag(y) + abs(V1) .*abs(V2) .*abs(y);
% Qmin = @(V1,V2,y) abs(V1).^2 .* -imag(y) - abs(V1) .*abs(V2) .*abs(y);
Qmin = @(V2,V1,y) abs(V1).^2 .* -imag(y) - abs(V1) .*abs(V2) .*abs(y);

hlr = [Qmin(Vm1,Vm2,y12) Qmax(Vm1,Vm2,y12)]*1.25; % helping line range
ang  =  pi/2+angle(y12);   % angle of maximal transmitted power
ang2 = -pi/2+angle(y12);   % angle of maximal power feed-in (higher due to losses)

hlr = hlr-[3 20]*fig2_8; % modify ylim and vertical lines for fig2

if( ~fig2_8 )
plot( [1 1]*ang, hlr, 'color', c1r, ...
    'linewidth', 2);
plot( [1 1]*ang2, hlr, '--', 'color', c1r, ...
    'linewidth', 2);

plot( [1 1]*-ang, hlr, 'color', c2r, ...
    'linewidth', 2);
plot( [1 1]*-ang2, hlr, '--', 'color', c2r, ...
    'linewidth', 2);
end

set( gca, 'xtick', (-pi):(pi/2):pi, ... 
    'ticklabelinterpreter','latex', ...
    'ylim', hlr);
xtl = {'$-\pi$', '$-\pi/2$', '$0$', '$\pi/2$', '$\pi$'};
xticklabels(xtl);
% set( gca, 'fontsize', 0.75*dissFS( 'hr' ) );
set( gca, 'fontsize', dissFS( 'hr' ) );

dissXLabel('$\varphi_1 - \varphi_2$', FORMAT1);
set( gca, 'xgrid', 'on', 'ygrid', 'on');

leg = legend([P12 P21 Pl]);
set( leg, 'interpreter', 'latex', ...
    'fontsize', dissFS(FORMAT1));
set( leg, 'position', ...
    [0.81061 0.23889 0.17798 0.33667]);

tb = dissAlabel( fig, '(b)', alo );
set( tb, ...
    'position', tbpos + [0.01 -0.02 0 0] );

if( ~~save_fig )
    pause(1);
    saveas( fig, [SPATH 'Qtransmission.eps'], 'epsc');
end
if( ~~save_fig2 )
    pause(1);
    saveas( fig, [SPATH 'Qtransmission2.eps'], 'epsc');
end



