
%% Figure 2.18 example graph for internal nodes and kron reduction
%
% V2.0 mawolff@uos.de 05-Aug-2020


close all;
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
SPATH = ''; % path to store the figures in
dpth = 'data/';

%% ============================== Settings  ============================== %
save_fig = false;
modgraph = false;
savemat = false;

labo.siz = dissFS( 'hs' );%*0.8;
FigSize = dissFigSize('hs')*0.5;
fsIntLabels = labo.siz*0.6;         % font size of terminal node labels (inside black circles)
nopt.bcScale = 20;

nopt.genscale = [];    
nopt.loadscale = [];
nopt.numbering = 'seq';
nopt.tboxlabel = true;
nopt.gencolor = [0.6902    0.0627    0.2078]*1.4; % gencolor etwas heller
nopt.redpass = false;
nopt.tbfs = labo.siz;

TB_POS = [0.05 0.940 0.025 0.025];

XLim = [-2 1.5];
YLim = [-2 1.5];

%% ======================== Read and process data ======================== %
matread = false;
try 
    file = [dpth 'fig2-18-terminalInternalGraph.mat'];
    load( file );
    disp( 'data read from mat file.' );
    matread = true;
    
    xgi = xgs(1:5);
    xgt = xgs(6:10);
    ygi = ygs(1:5);
    ygt = ygs(6:10);
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be found.']);  
        otherwise 
            rethrow( ME ); 
    end 
end

if( ~matread )
    mpc5 = loadcase( case5 );
    
    mpc5 = gatherGen( mpc5 );
    y0 = makeYbus( mpc5 );
    
    rso.dynmodel = 'sm';
    [sm5, par5] = raw2swing( mpc5, rso );   % nur xd formel klauen ...
    
    yint = -1i./par5.xd;                    % get internal admittances
    yi = diag(yint);
    
    Y = [  yi -yi;
        -yi yi+y0 ];
    
    Yred = yi - yi/(yi+y0) * yi;
end

mpc5 = loadcase( case5 );

% --------------------------- simplify graph  --------------------------- %
% simplify graph to make the Kron reduction more instructive
% gather generators and
% make nodes either generator or load nodes and never both
mpc5 = gatherGen( mpc5 );                       % gather generators

mpc5.bus(4,3) = mpc5.bus(4,3) - mpc5.gen(3,2);  % make bus 4 effective load
mpc5.gen(3,:) = [];                             % and delete its generator
mpc5.gen(2,2) = mpc5.gen(2,2) - mpc5.bus(3,3);  % make bus 3 effective generator
mpc5.bus(3,3) = 0;                              % delete its load 

%% =============================== Figures =============================== %
% --------------------------- Terminal graph  --------------------------- %
gro.model = 'pf';
nopt.model = gro.model;
busGraph = mpc2Graph( mpc5, gro );

figt = figure;  % figure for terminal grid graph
set( figt, 'position', [100 400 FigSize] );

busgplot = plot( busGraph );

ax = gca;
set( ax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');
set( busgplot, 'xdata', xgt, 'ydata', ygt );

highlightEdges( busgplot, busGraph );
nopt.dualbus = 'both';
tbl = highlightPGNodes( busgplot, mpc5, nopt );

if( matread )
    for i = 1:5
        set( tbl.ntb(i), 'position', labelPosTerm(i,:) );
    end
end


tb = dissAlabel( figt, '(a)', labo );
set( tb, 'position', TB_POS );

% --------------------------- Internal graph  --------------------------- %
gro.model = 'gsm';
nopt.model = gro.model;
gsmGraph = mpc2Graph( mpc5, gro );

figint = figure;  % figure for internal grid graph
set( figint, 'position', [500 400 FigSize] );

gsmgplot = plot( gsmGraph );

ax = gca;
set( ax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');

set( gsmgplot, 'xdata', [xgt xgi], 'ydata', [ygt ygi] );

highlightEdges( gsmgplot, gsmGraph );
tbl2 = highlightPGNodes( gsmgplot, mpc5, nopt );

itLabel = [6:10 1:5];   % first internal then terminal labelling
fontsizes = [ ones(1,5)*fsIntLabels ... % terminal nodes have smaller 
    ones(1,5)*labo.siz ];               % font size to fit inside circle
if( matread )
    for i = 1:10
        set( tbl2.ntb(i), 'position', labelPosInt(i,:), ...
            'String', num2str(itLabel(i)), ...
            'fontsize', fontsizes(i) );
    end
end


tb = dissAlabel( figint, '(b)', labo );
set( tb, 'position', TB_POS );

% ------------------------- Kron reduced graph  ------------------------- %
redGraph = ybus2Graph( Yred );

figred = figure;  % figure for reduced graph between internal nodes only
set( figred, 'position', [900 400 FigSize] );
redPlot = plot( redGraph );

ax = gca;
set( ax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');

redPlot.XData = xgi;
redPlot.YData = ygi;

nopt.model = 'pf';
highlightEdges( redPlot, redGraph );
tbl3 = highlightPGNodes( redPlot, mpc5, nopt );

if( matread )
    for i = 1:5
        set( tbl3.ntb(i), 'position', labelPosKron(i,:) );
    end
end

tb = dissAlabel( figred, '(c)', labo );
set( tb, 'position', TB_POS );


%% ============================ Save figures  ============================ %
if( ~~save_fig )
    saveas( figt, [SPATH 'gridTerminal.eps'], 'epsc' );
    saveas( figint, [SPATH 'gridInternal.eps'], 'epsc' );
    saveas( figred, [SPATH 'gridKronRedu.eps'], 'epsc' );

end

%%
labelPosTerm = zeros(5,4);
labelPosInt  = zeros(10,4);
labelPosKron = zeros(5,4);

for i = 1:5
    tbh = get( tbl.ntb(i) );
    labelPosTerm(i,:) = tbh.Position;
end
for i = 1:10
    tbh = get( tbl2.ntb(i) );
    labelPosInt(i,:) = tbh.Position;
end
for i = 1:5
%     tbh = get( tbl3.ntb(i) );
%     labelPosKron(i,:) = tbh.Position;
    labelPosKron(i,:) = labelPosInt(i+5,:);
end

%%
if( ~~savemat )
    save( file, ...
        'par5', ...    % data tables
        'y0', 'Y', 'Yred', ...          % admittances matrices
        'xgs', 'ygs', ...               % node coordinates x, y
        'labelPosTerm', ...             % tbox label coordinates x,y,w,h
        'labelPosInt', ...
        'labelPosKron');                     
end
    
