%% per gen Figures

addpath('methods');
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
% Path to pg_dyn_models 
% [Nishikawa and Motter, New J. Phys. 17, 015012 (2015) ]
% needed:
addpath('C:\Users\Anwender\Documents\MATLAB\pg_dyn_models');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
SPATH = '';
dpth = 'data/';

% ============================== settings  ============================== %
MARKERSIZE = 4;
LINEWIDTH  = 2;
FORMAT = 'hs';
LEG_FS = dissFS('hs')-5;

save_fig = false;

eqNum = {'90'; '93'; '91'; '96'};  % numbers of equations in latex document

plot_xd2 = false;

close all;

% ============================== read data ============================== %
load( [dpth 'genDynDataNumeric.mat'], 'dynDataNumeric' );

% row 10 is synchronous condensor; not considered here
PG   = dynDataNumeric(1:9,1); % power generation at each generator unit in MW
MVAB = dynDataNumeric(1:9,2); % generator power (MVA) base
XD   = dynDataNumeric(1:9,3); % generator direct axis reactance (X_d)
XT   = dynDataNumeric(1:9,4); % transformer reactance (X_T)
INr  = dynDataNumeric(1:9,5); % relative intertia at each generator in MJ/MW

Xd = XD + XT;           % synchrnous reactance
Inertia = INr .* PG;    % total inertia at each generator

MVABn = 100;            % power (MVA) base of network (power grid)

xdformular = @(P) 92.8* (P).^(-1.3);    % Nishikawa's formula for x_d

mpc50 = test_system_50gen;          % get data of 50 gen case
xdstroke50 = mpc50.gen_dyn(:,1);    % x'_d of 50 gen case
inertia50  = mpc50.gen_dyn(:,2);    % intertia of 50 gen case 
PG50        = mpc50.gen(:,2);       % power generation at nodes of 50 gen case

xd50gnum = [9 20 21 22 25 26]';
Pxd50 = mpc50.gen(xd50gnum, 2);
xd50 = [ 0.09842; 0.1016; 0.1144; 0.17165; 0.09842; 0.1016 ];

unir = unicolors( 'dr', '-1' );
unio = unicolors( 'do', '-1' );

LLao = ['Eq. (2.' eqNum{1} ')'];  % '$4P^{\rm (g)}$'    ao: (a) orange
LLar = ['Eq. (2.' eqNum{2} ')'];  % '$3P^{\rm (g)}$'    ar: (a) red
LLbo = ['Eq. (2.' eqNum{3} ')'];  % [  '$-1.3P^{\rm (g)} + log_{10}(92.8)$' '']
LLbr = ['Eq. (2.' eqNum{4} ')'];  % [num2str(pf(1),3) '$P^{\rm (g)}$' ...
%                       sprintf('%+1.3f',pf(2)) ]

% ========================== figure H per gen  ========================== %
fig = figure;
set( fig, 'position', [ 222 200 dissFigSize( FORMAT )] );

x = PG/MVABn;
y = Inertia/MVABn;
xlin = [min(x) max(x)];
pf = polyfit(x,y,1);
disp( pf );
x50 = PG50/MVABn;
x50lin = [min(x50) max(x50)];
y50 = inertia50;

% ----------------------------- 50gen case  ----------------------------- %
plot( x50, y50, 'o', ...
    'color', unio, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', '50gen');
hold on;
plot( x50lin, 4*(x50lin), '-', ...
    'color', unio, ...
    'linewidth', LINEWIDTH, ...
    'displayname', LLao );

% ------------------------------- RTS-96  ------------------------------- %
plot( x, y, 'o', ...
    'color', unir, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', 'RTS-96');
hold on;
plot( xlin, 3*xlin, '-', ...
    'color', unir, ...
    'linewidth', LINEWIDTH, ...
    'displayname', LLar );

% plot( xlin, polyval(pf,xlin), '-', ...
%     'color', 'g', ...
%     'linewidth', LINEWIDTH);

% ------------------------- labels adjustments  ------------------------- %
ax = gca;
set( ax, 'xlim', [0 4.5], ...
    'ylim', [0 40], ...
    'fontsize', dissFS(FORMAT) );
dissXLabel('$P^{\rm (g)}$ [100 MW]',FORMAT);
dissYLabel('$H/S_{\rm b}$ [s]',FORMAT);    %[MJ/MVA]

% -------------------------------- inset -------------------------------- %
axoffset = [0.02 0.02 0 0]; % to have (a) and (b) on same height and both 
% x- and y-labels are okay

axInset = copyobj( ax, fig );
axInsetPos = [0.30667 0.59167 0.49619 0.31166]+axoffset;
set( axInset, ... 'xgrid', 'on', 'ygrid', 'on', ...
    'position', axInsetPos , ...
    'xlim', [0 600], ...
    'ylim', [0 2500], ...
    'fontsize', 12);

% ------------------------------- legend  ------------------------------- %
leg = legend(axInset,'show');
set( leg, 'interpreter', 'latex', ...
    'fontsize', LEG_FS, ...
    'position', [0.69023 0.61158 0.20103 0.16917]+axoffset);

% ----------------------------- axis stuff  ----------------------------- %
tbo.siz = dissFS('hs');
tba = dissAlabel( fig, '(a)', tbo );

axpos = get( ax, 'position' );
axpos = axpos+axoffset;
set( ax, 'position', axpos );


% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
%     set(fig,'Units','Inches');
%     pos = get(fig,'Position');
%     set(fig,'PaperPositionMode','Auto',...
%         'PaperUnits','Inches',...
%         'PaperSize',[pos(3), pos(4)])
    saveas( fig, [SPATH 'HVsP.eps'], 'epsc' ); 
end


% ========================== figure xd per gen ========================== %
fig = figure;
set( fig, 'position', [ 222 222 dissFigSize( FORMAT )] );

lox = log10( x );
loy = log10( Xd.*MVABn./MVAB );
pf = polyfit( lox, loy, 1);
disp(pf);
x50 = log10(PG50/MVABn);
y50 = log10(xdstroke50);
loyn = log10(xdformular(x50lin*MVABn));
loxn = log10( x50lin );

x250 = log10(Pxd50/MVABn);
y250 = log10(xd50);
pf2 = polyfit( x250, y250, 1);
xlin2 = [xlin(1) max(10.^x250)];

% ----------------------------- 50gen case  ----------------------------- %
loglog( 10.^(x50), 10.^(y50), 'o', ...
    'color', unio, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', '50gen' );
hold on;
loglog( 10.^(loxn), 10.^(loyn), '-', ...
    'color', unio, ...
    'linewidth', LINEWIDTH, ...
    'displayname', LLbo);

if( ~~plot_xd2 )    % plot x_d2? (sync reactance? look up)
loglog( 10.^(x250), 10.^(y250), 'o', ...
    'color', 'g', ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', '50gen xd' );

loglog( xlin2, 10.^( -0.8*log10(xlin2) + pf(2)+0.2 ), '-', ...
    'color', 'g', ...
    'linewidth', LINEWIDTH, ...
    'displayname', LLbr );
end

% ------------------------------- RTS-96  ------------------------------- %
loglog( 10.^(lox), 10.^(loy), 'o', ...
    'color', unir, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', 'RTS-96' );
hold on;
% loglog( xlin, 10.^(polyval(pf,log10(xlin))), '-', ...
loglog( xlin, 10.^( -1*log10(xlin) + pf(2) ), '-', ...
    'color', unir, ...
    'linewidth', LINEWIDTH, ...
    'displayname', LLbr );

% ------------------------- labels adjustments  ------------------------- %
ax2 = gca;
set( ax2, 'xlim', 10.^[-1 1], ...
    'ylim', 10.^[-1.5 2], ...
    'fontsize', dissFS(FORMAT) );
dissXLabel('$P^{\rm (g)}$ [100 MW]',FORMAT);
dissYLabel('$x_d$ [pu]',FORMAT);

% -------------------------------- inset -------------------------------- %
ax2Inset = copyobj( ax2, fig );
ax2iPos = [0.39857 0.52667 0.48547 0.38166]+axoffset;
set( ax2Inset, ... 'xgrid', 'on', 'ygrid', 'on', ...
    'position', ax2iPos , ...
    'xlim', 10.^[-1 3], ...
    'ylim', 10.^[-4.5 1.5], ...
    'fontsize', 12 );

% ------------------------------- legend  ------------------------------- %
leg2 = legend(ax2Inset,'show');
set( leg2, 'interpreter', 'latex', ...
    'fontsize', LEG_FS, ...
    'position', [0.69116 0.74251 0.20103 0.16917]+axoffset);

% ----------------------------- axis stuff  ----------------------------- %
tbb = dissAlabel( fig, '(b)', tbo );

set( ax2, 'position', axpos );

% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
%     set(fig,'Units','Inches');
%     pos = get(fig,'Position');
%     set(fig,'PaperPositionMode','Auto',...
%         'PaperUnits','Inches',...
%         'PaperSize',[pos(3), pos(4)])
    saveas( fig, [SPATH 'xdVsP.eps'], 'epsc' ); 
end



% WICHTIG: da die tatsachlichen xd-werte mit 1/MVAbase gehen, andert sich
% die kurve komplett, H geht linear mit MVAbase, daher im prinzip egal