
%% RTS graph plots for Dissertation
%
%

addpath('methods');
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));

%% ============================= settings  ============================== %
move_nodes = false;
save_fig = false;
newTBLpos = false;

FORMAT = 'fr';
dpth = 'data/';
spth = '';


%% ========================== power flow view =========================== %
% ------------------------------ load data ------------------------------ %
l = load( 'data/RTS96' );           % load RTS-96 data in MPC format
mpc = l.mpc;
mpc = runpf( mpc );                 % make sure that the power-flow is solved for that case
mpc.branch = sortrows( mpc.branch, [1 2]);  % same sorting as matlab graph struct

% ----------------------------- processing  ----------------------------- %
file = [dpth 'fig-2-20-rts96geographic'];
[xn, yn] = loadGraphXY( file );                     % load coordinates

gro.dir = true; % make directed graph
pgGraph = mpc2Graph( mpc, gro );    % 

% ----------------------------- graph plot  ----------------------------- %
fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT)]);
pfax = axes;
set( pfax, 'position', [0.025 0.025 0.95 0.95] );


pgGraphPlot = plot( pgGraph );
set( pgGraphPlot, 'xdata', xn );
set( pgGraphPlot, 'ydata', yn );

%                           mark voltage levels                           %
x_vl1 = 290;
x_vl2 = 1369;
hold on;
plot( [1 1]*x_vl1, [   0 1300], 'k--');
plot( [1 1]*x_vl2, [   0 1300], 'k--');
set( pfax, 'xlim', [0 1700], 'ylim', [0 1300] );
tbh = 1300*0.95;
fv1 = val2FigCoords( [0.45*x_vl1  tbh], pfax );
fv2 = val2FigCoords( [800 tbh], pfax );
fv3 = val2FigCoords( [0.45*x_vl1+x_vl2 tbh], pfax );
tb1381 = tbox( [fv1 0.03 0.05], '138kV', dissFS(FORMAT) );
tb230  = tbox( [fv2 0.03 0.05], '230kV', dissFS(FORMAT) );
tb1382 = tbox( [fv3 0.03 0.05], '138kV', dissFS(FORMAT) );


[~,uidx,~,dl] = findDlines( mpc );
N = numel(mpc.branch(:,1))-dl;
col_sing = [1 1 1]*0.6;
col_mult = [0.6 0.6 1];
exo.dim = N;
colors = expand( col_sing, exo );
exo.dim = numel(uidx);
mcol = expand( col_mult, exo );
colors( uidx, : ) = mcol;

edgeo.arrowsize = 0.1;
edgeo.color = colors;
highlightEdges( pgGraphPlot, pgGraph, edgeo );

%
clear nodeo;
nodeo.numbering = 'seq';
% nodeo.numbering = 'gl';
nodeo.tboxlabel = true;
nodeo.dualbus = 'both';
nodeo.tbshift = [-0.0165 0.0165];
% nodeo.dualbus = 'effective';
nodeo.model = 'pf';
nodeprop = highlightPGNodes( pgGraphPlot, mpc, nodeo );

file = [dpth 'fig-2-20-textpos.mat'];
load( file );
nntb = numel( nodeprop.ntb );
% ntbpos = zeros(nntb,4);
for i = 1:nntb
    set( nodeprop.ntb(i), 'position', ntbpos(i,:) );
%     ntbpos(i,:) = get( nodeprop.ntb(i), 'position' );
end
% save( file, 'ntbpos' );
%
whit = [1 1 1];
set( pfax, 'xtick', [], 'ytick', [], ...
    'xcolor', whit, 'ycolor', whit , 'zcolor', whit );

if( ~~save_fig )
    pause(1);
    saveas( fig, [spth 'RTS96-pfl.eps'], 'epsc');
end

%%
move_nodes = false;
if( ~~move_nodes )
   
    % [xnn, ynn] = editGdots( xn, yn, pgGraphPlot );
    
    saveGraphXY( xnn, ynn, [dpath 'fig-2-20-rts96PFVlevel.mat'] );

end

%% =========================== full SM model ============================ %
% ------------------------------ load data ------------------------------ %
l = load( 'data/rts96fsm.mat' );         % load RTS-96 data in MPC format
mpc = l.mpc;
mpc = runpf( mpc );                         % make sure that the power-flow is solved for that case
mpc.branch = sortrows( mpc.branch, [1 2]);  % same sorting as matlab graph struct

% ----------------------------- processing  ----------------------------- %
file = [dpth 'fig-A-10-rts96FSM'];
[xgeo, ygeo] = loadGraphXY( file );                     % load coordinates

Nload = sum( mpc.bus(:,3) > 0 );
Ngen  = size( mpc.gen, 1);

% ----------------------------- graph plot  ----------------------------- %
fig = figure;
set( fig, 'position', [50 50 dissFigSize(FORMAT)*1.5]);
fsmAx = axes;
% set( fsmAx, 'position', [0.025 0.025 0.95 0.95] );
set( fsmAx, 'position', [0.015 0.015 0.97 0.97] );


gro.dir = true;                     % make directed graph
gro.model = 'fsm';
pgGraph = mpc2Graph( mpc, gro );    % 

pgGraphPlot = plot( pgGraph );
set( pgGraphPlot, 'xdata', xgeo );
set( pgGraphPlot, 'ydata', ygeo );
set( fsmAx, 'xlim', [-15 1720], 'ylim', [-10 1315] );
[~,uidx,~,dl] = findDlines( mpc );
N = numel(mpc.branch(:,1))-dl;
col_int = [1 0 1];
col_sing = [1 1 1]*0.6;
col_mult = [0.6 0.6 1];
exo.dim = N;
colors = expand( col_sing, exo );
exo.dim = numel(uidx);
mcol = expand( col_mult, exo );
colors( uidx, : ) = mcol;
exo.dim = Ngen+Nload;
intcol = expand( col_int, exo );
colors = [colors; intcol];

edgeo.arrowsize = 0.001;
edgeo.color = colors;
highlightEdges( pgGraphPlot, pgGraph, edgeo );


clear nodeo;
nodeo.numbering = 'seq';
% nodeo.tboxlabel = false;
nodeo.tboxlabel = true;
nodeo.model = 'fsm';
nodeo.tbshift = [-0.015 0.015];
nodeprop = highlightPGNodes( pgGraphPlot, mpc, nodeo );

whit = [1 1 1];
set( fsmAx, 'xtick', [], 'ytick', [], ...
    'xcolor', whit, 'ycolor', whit );

file = [dpth 'fig-A-10-textpos.mat'];
load( file );
nntb = numel( nodeprop.ntb );
for i = 1:nntb
    set( nodeprop.ntb(i), 'position', ntbpos(i,:), ... 
        'fontsize', 17);
end
for i = 1:size(mpc.bus,1)
    set( nodeprop.ntb(i), 'fontsize', 9 );
    set( nodeprop.ntb(i), 'position', ...
        [(val2FigCoords([xgeo(i) ygeo(i)], fsmAx) ... 
            -[45 62]*1e-4) 0.01 0.01]  );
end


pause(1);
% %%
% savefig = true;
if( ~~save_fig )
    saveas( fig, [spth 'RTS96-FSM.eps'], 'epsc');
end

%% get text box positions
file = [dpth 'fig-A-10-textpos.mat'];
% newTBLpos = true;
if( ~~newTBLpos )
    nntb = numel( nodeprop.ntb );
    ntbpos = zeros(nntb,4);
    for i = 1:nntb
        ntbpos(i,:) = get( nodeprop.ntb(i), 'position' );
    end
    
    save( file, 'ntbpos' ); % save new text box positions
end

%% =========================== gath SM model ============================ %
% ========================== (or dynamic view) ========================== %
% ------------------------------ load data ------------------------------ %
l = load( 'data/rts96fsm.mat' );         % load RTS-96 data in MPC format
mpc = l.mpc;
mpc = runpf( mpc );                         % make sure that the power-flow is solved for that case
mpc.branch = sortrows( mpc.branch, [1 2]);  % same sorting as matlab graph struct
mpc = gatherGen( mpc );

% ----------------------------- processing  ----------------------------- %
file = [dpth 'fig-2-21-rts96GSM'];
[xgeo, ygeo] = loadGraphXY( file );                     % load coordinates


% ----------------------------- graph plot  ----------------------------- %
fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT)]);
fsmAx = axes;
set( fsmAx, 'position', [0.025 0.025 0.95 0.95] );


gro.dir = true;                     % make directed graph
gro.model = 'gsm';
gro.lscale = @(x) x + 10;
pgGraph = mpc2Graph( mpc, gro );    % 

pgGraphPlot = plot( pgGraph );
set( pgGraphPlot, 'xdata', xgeo );
set( pgGraphPlot, 'ydata', ygeo );
set( fsmAx, 'xlim', [0 1700], 'ylim', [0 1300] );

[gbus, lbus] = getglbus( mpc );
Nload = numel( lbus );
Ngen  = numel( gbus );
[~,uidx,~,dl] = findDlines( mpc );
N = numel(mpc.branch(:,1))-dl;
col_int = [1 0 1];
col_sing = [1 1 1]*0.6;
col_mult = [0.6 0.6 1];
exo.dim = N;
colors = expand( col_sing, exo );
exo.dim = numel(uidx);
mcol = expand( col_mult, exo );
colors( uidx, : ) = mcol;
exo.dim = Ngen+Nload;
intcol = expand( col_int, exo );
colors = [colors; intcol];

edgeo.arrowsize = 0.001;
edgeo.color = colors;
highlightEdges( pgGraphPlot, pgGraph, edgeo );


clear nodeo;
nodeo.numbering = 'seq';
% nodeo.tboxlabel = false;
nodeo.tboxlabel = true;
nodeo.model = gro.model;
nodeo.tbshift = [-0.015 0.015];
nodeprop = highlightPGNodes( pgGraphPlot, mpc, nodeo );

whit = [1 1 1];
set( fsmAx, 'xtick', [], 'ytick', [], ...
    'xcolor', whit, 'ycolor', whit );

file = [dpth 'fig-2-21-textpos.mat'];
load( file );
nntb = numel( nodeprop.ntb );
for i = 1:nntb
    set( nodeprop.ntb(i), 'position', ntbpos(i,:) );
end
for i = 1:size(mpc.bus,1)
    set( nodeprop.ntb(i), 'fontsize', 9 );
    set( nodeprop.ntb(i), 'position', ...
        [(val2FigCoords([xgeo(i) ygeo(i)], fsmAx) ... 
            -[45 62]*1e-4) 0.01 0.01]  );
end


pause(1);
% %%
% save_fig = true;
if( ~~save_fig )
    saveas( fig, [spth 'RTS96-GSM.eps'], 'epsc');
end

%% get text box positions
file = [dpth 'fig-2-21-textpos.mat'];
% newTBLpos = true;
if( ~~newTBLpos )
    nntb = numel( nodeprop.ntb );
    ntbpos = zeros(nntb,4);
    for i = 1:nntb
        ntbpos(i,:) = get( nodeprop.ntb(i), 'position' );
    end
    
    save( file, 'ntbpos' ); % save new text box positions
end












