
%% Figure 3.1 stability basin single node model
% stability basin plot for dissertation
%
%

close all;
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
spth = '';

% ============================== settings  ============================== %
update_mat = false;
savefig = false;

FORMAT = 'hs';

LC_col = [0 0 0];                       % color for limit cycle
mapColors = [ unicolors( 'do', '-1');   % color for fixed point basin
            unicolors('dr', '-1')];     % color for limit cycle basin
lcplot = true;
z_top = 1.2;
XLIM = [-pi pi-0.1];
YLIM = [-15 15];
ticklen = [0.75 0.25]*0.04;    % tick length on x-axis for minor and major ticks
xticks = [-pi -pi/2 0 pi/2 pi-0.1];
XTL = {'$-\pi$';'$-\pi/2$';'0'; '$\pi/2$'; '$\pi$'};
% ============================== load data ============================== %
dpth = 'data/';
S = load([dpth 'fig3-1-stabilityBasin.mat']);
X = S.X;
Y = S.Y;
Z = S.Z;
traj_fp = S.traj_fp;
traj_lc = S.traj_lc;
if( ~update_mat )
    clear S;
end

% ---------------------------- process data  ---------------------------- %
Z = min(ones(size(Z)), Z);  % make the stability binary
Z(Z < 0.5) = 0;             % 

K = 8;      % coupling strength
D = 0.1;    % damping ratio
P = 1;      % power
lcafreq = P/D;    % angular frequency of limit cycle
limcy = @(t) P/D + D*K/P * cos(P/D * t);  % limit cycle formula
t = timeseriesFromFreq( lcafreq, [-0.5 0.5], 200); % time instants for one period
phaset = -pi:2*pi/numel(t):pi; % parametrization of phase angle with time
phaset = phaset(1:numel(t));

% ================================ plot  ================================ %
fig = figure;
set( fig, 'position', [123 222 dissFigSize(FORMAT)]);

% --------------------------- stability basin --------------------------- %
sur = surf( X, Y, Z);
hold on;
view(0,90);
set( sur, 'EdgeAlpha', 0);
ax = gca;
colmap = colormap(mapColors);   % colormap for surface plot

set( ax, 'XLim', XLIM);
set( ax, 'YLim', YLIM);
set( ax, 'fontsize', dissFS(FORMAT)*0.9 ,...
    'ticklength', ticklen ,...
    'ticklabelinterpreter','latex');
xl = dissXLabel('$ \Delta \varphi = \varphi - \varphi^{\rm FP} $',FORMAT);
yl = dissYLabel('$ \Delta \omega = \omega - \omega_{\rm r} $',FORMAT);

% -------------------------------- ticks -------------------------------- %
set(gca, 'XTick', xticks);
xticklabels( XTL );
set( ax, 'Layer', 'top'); % ticks over surface plot
% grid off
for i = 1:numel( xticks )
    plot3( [1 1]*xticks(i), [0 0.03*diff(YLIM)]+YLIM(1), [1 1]*z_top, ...
        'color', 'k', 'linewidth', 1.5 );
end
yticks = get( ax, 'ytick' );
for i = 1:numel( yticks )
    plot3( [0 0.03*diff(XLIM)]+XLIM(1), [1 1]*yticks(i), [1 1]*z_top,...
        'color', 'k', 'linewidth', 1.5 );
end


% ----------------------------- limit cycle ----------------------------- %
if( ~~lcplot )
    
    limc = plot3(phaset,limcy(t),1.1*ones(size(t)), ...
        'color', LC_col);    % plot limit cycle
    set(limc, 'LineWidth',2);
    
end


% ----------------------- trajectory return to fp ----------------------- %
pert_fp = [0.5 2];
pert_lc = [-2 5];
z_top_2 = z_top-0.1;

phiSS = 0.1253;

plot3( pert_fp(1), pert_fp(2), z_top_2, ... 
    'marker', 'o', ...
    'markersize', 3, ...
    'linewidth', 3, ...
    'linestyle', 'none', ...
    'color', lightToDarkBlueGradient(1,10) );

plot3( 0,0,z_top_2, 'ok', ...   % fixed point
    'linewidth', 2, ...
    'markersize', 2);

tro.fig = fig;
tro.ax = gca;
tro.cgrad = 'blue';
tro.z = 0.1;
tro.linewidth = 1.5;
trajPlot( traj_fp(:,1)-phiSS, traj_fp(:,2), 40, tro );
% the trajectories were obtained from the SingleNodeModel.m script with
% parameters
% H = 1;
% D = 0.2;  % instead of 0.1 to have smaller rings in trajectory
% P = 1;
% K = 8;
% te = 35;

% ------------------------ trajectory turn to lc ------------------------ %
plot3( pert_lc(1), pert_lc(2), z_top_2, ...
    'marker', 'o', ...
    'markersize', 3, ...
    'linewidth', 3, ...
    'linestyle', 'none', ...
    'color', 'g' );

tro.fig = fig;
tro.ax = gca;
tro.cgrad = 'green';
tro.z = z_top_2;
tro.linewidth = 1;
trajPlot( traj_lc(:,1)-phiSS, traj_lc(:,2), 40, tro );
% the trajectories were obtained from the SingleNodeModel.m script with
% parameters
% H = 1;
% D = 0.1;
% P = 1;
% K = 8;
% te = 20;

% ================================ save  ================================ %
if( ~~savefig )
    saveas( fig, [spth 'stabBasinSNM.png'] );
end

%% -------------------------- update mat file --------------------------- %
if( ~~update_mat )
    Z = S.Z; 
    save([dpth 'fig3-1-stabilityBasin.mat'], 'X', 'Y', 'Z', ...
        'traj_fp', 'traj_lc');
end

