
%% Basin stability in dependence of coupling strength K
%
%
% V1.0 mawolff@uos.de 09-Jul-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================== settings  ============================== %
save_fig = false;
spth = '';

MARKERSIZE = 2.5;
LINEWIDTH  = 1.5;
FORMAT = 'hr';
LEG_FS = dissFS(FORMAT)-5;

close all;

% ============================== read data ============================== %
pth = 'data/';
load( [pth 'fig3-2-SvsK.mat'], 'bs6', 'bs15', 'rt6', 'rt15', ...
    'bs6P6');
K = 0:79;

unir = unicolors( 'dr', '-1' );
unio = unicolors( 'do', '-1' );

% =========================== figure S vs K  ============================ %
fig = figure;
set( fig, 'position', [ 222 200 dissFigSize( FORMAT )] );


ra1 = 1:62;
ra2 = 63:80;
plot( K(ra1), bs15(ra1), 'o', ...
    'color', unio, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', 'R -15 15');
hold on;
plot( K(ra2), bs15(ra2), 'o', ...
    'color', unio, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE*2, ...
    'displayname', '');
plot(  K, bs6, 'o', ...
    'color', unir, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', 'R -2pi 2pi');

% ------------------------- labels adjustments  ------------------------- %
ax = gca;
set( ax, 'xlim', [0 80], ...
    'ylim', [0 1.1], ...
    'fontsize', dissFS(FORMAT), ...
    'xgrid', 'on', 'ygrid', 'on');
dissXLabel('$K$',FORMAT);
dissYLabel('$\mathcal{S}$',FORMAT);

% ----------------------------- axis stuff  ----------------------------- %
tbo.siz = dissFS(FORMAT);
tba = dissAlabel( fig, '(a)', tbo );
set( tba, 'position', get(tba,'position') - [-0.01 0.02 0 0] );

% ========================= figure S vs K (b)  ========================== %
figB = figure;
set( figB, 'position', [ 222 200 dissFigSize( FORMAT )] );

bs6P6(1:7) = 0;
plot(  K, bs6P6, 'o', ...
    'color', unir, ...
    'linewidth',  MARKERSIZE, ...
    'markersize', MARKERSIZE, ...
    'displayname', 'R -2pi 2pi');

% ------------------------- labels adjustments  ------------------------- %
ax = gca;
set( ax, 'xlim', [0 80], ...
    'ylim', [0 1.1], ...
    'fontsize', dissFS(FORMAT), ...
    'xgrid', 'on', 'ygrid', 'on' );
dissXLabel('$K$',FORMAT);
dissYLabel('$\mathcal{S}$',FORMAT);

% ----------------------------- axis stuff  ----------------------------- %
tbo.siz = dissFS(FORMAT);
tba = dissAlabel( figB, '(b)', tbo );
set( tba, 'position', get(tba,'position') - [-0.01 0.02 0 0] );

% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    saveas( fig, [spth 'SvsK.eps'], 'epsc' ); 
    saveas( figB, [spth  'SvsK-B.eps'], 'epsc' ); 
end
