
%% figure 3-03 for Dissertation
% plot frequency trajectory and demonstrate the multiple threshold approach
% (a) reads and plots the frequency trajectory for RTS 96 grid
% for an arbitrary node.
% (b) demonstrates the effect of unsmooth return times surfaces
%
% V2.0                  16-Jun-2020
% V1.0 mawolff@uos.de   15-Apr-2018

close all;
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

dpth = 'data/';
spth = '';

%% settings
save_fig = false;

FORMAT = 'hs';

xoff = 0.2;
yoff = 0.15;
figsx = 0.75;
figsy = 0.8;
% font = 'LM Roman';          % set font style
font = dissFont;
lfs = 1.2*dissFS(FORMAT);   % label font size
tbfs = 1.1*dissFS(FORMAT);

alabel_xoff = 0.07;


TICK_LEN = [0.03 0.015]*0.66;

jnode = 9;
knode = 3;

%% load data
file = [dpth 'fig3-3-RTconcept.mat'];
load(file);     % provides 'afreq', 'bfreq', 't'

%% ========================== plot freq traj  =========================== %
fig1 = figure;
set( fig1, 'position', [123 56 dissFigSize(FORMAT) ] );

ylim = 10.^[-4 2];
axa = axes;
pl = semilogy( t, afreq, 'color', unicolors('dr', '-1') );
hold on;
set( pl, 'linewidth', 2);
set( axa, 'fontsize', dissFS(FORMAT),...
    'xtick', 0:10,...
    'ytick', 10.^(-4:1), ...
    'xlim', [0 3],...
    'ylim', ylim,...
    'ticklength', [0.05 0.025],...
    'xminortick', 'on',...
    'yminortick', 'off', ...
    'position', [xoff yoff figsx figsy]);
xlh1 = dissXLabel('$t$', FORMAT);
set( xlh1, 'fontsize', lfs );
ylh1 = dissYLabel(['$|\omega_{' num2str(knode) '}| $'], FORMAT);
rotYlabel( ylh1 );
set( ylh1, 'fontsize', lfs );

% apos = get(gca,'position');
set(ylh1, 'Position',[-0.6 0.1 0]);
axa.XAxis.MinorTickValues = 0.25:0.25:10;

leftDotted  = 1.5825;
rightDotted = 1.8950;

plot( ones(2,1)*leftDotted, [ylim(1) 10^(-0.2)] , 'k--');
plot( ones(2,1)*rightDotted, [ylim(1) 10^(-0.2)] , 'k--');

ax1 = gca;

axpos = get( gca, 'position');
aopt.siz = dissFS( FORMAT );
tba = dissAlabel( fig1, '(a)', aopt );
posal = get( tba, 'position');
auxtb = tbox( [0.005 0.995 0.005 0.005], ' ', 12 );
set( tba, 'position', posal+[alabel_xoff 0 0 0] );

% -------------------------------- inset -------------------------------- %
inset = axes;
set( inset, 'position', [0.45 0.72 0.45 0.21]);

ylim = 10.^[-12 1.2];
YTICK_LABELS = cell(13,1);
for k = -12:1
    if( k == -12 || k== -5 || k == 1 )
        YTICK_LABELS{k+13} = ['10^{' num2str(k) '}'];
    else
        YTICK_LABELS{k+13} = '';
    end
end
pl = semilogy( t, afreq, 'color', unicolors('dr', '-1') );
hold on;
set( gca, 'xlim', [0 25],...
    'ylim', ylim,...
    'ytick',10.^(-12:1),...
    'yticklabels', YTICK_LABELS,...
    'yminortick', 'off',...
    'ticklength', TICK_LEN, ...
    'fontsize', 0.6*dissFS(FORMAT) );

% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    saveas( gcf, [spth 'freqTrajRT-a.eps'],'epsc');
end


%% ====================== compare consecutive trajs ====================== %
fig2 = figure;
FIG_SIZE = dissFigSize(FORMAT);
set( fig2, 'position', [123+FIG_SIZE(1) 56 FIG_SIZE] );

% ------------------------------- params  ------------------------------- %
tra = findRange( 1.4, 1.9, t );
ylimlog = [-0.8 -0.5];
tcleave = t( tra );

yfreq  = bfreq(tra)-0.01;
tyfreq = tcleave-0.035;

threshup = 0.205;
threshlow = 0.195;

% -------------------------------- plot  -------------------------------- %
axb = axes;
pl = semilogy( tcleave, afreq(tra) , 'color', unicolors('dr', '-1'));
hold on;
set( pl, 'linewidth', 2);
pl = semilogy( tyfreq, yfreq , 'color', unicolors('do', '-1') );
set( pl, 'linewidth', 2);
set( axb, 'fontsize', dissFS(FORMAT),...
    'xtick', [],...
    'ytick', threshup, ...
    'position', [xoff yoff figsx figsy],...
    'xlim', [t(tra(1)) t(tra(end))],...
    'ylim', 10.^ylimlog);
set( gca, 'TickLabelInterpreter', 'latex', ...
    'fontsize', 24);
yticklabels( '$\epsilon$' );
xlh2 = dissXLabel('$t$', FORMAT);
set( xlh2, 'fontsize', lfs, ...
    'position', get( xlh2, 'position' )+[0 -0.008 0] );
ylh2 = dissYLabel(['$|\omega| $'], FORMAT);
set( ylh2, 'fontsize', lfs );
rotYlabel( ylh2 );
set(ylh2, 'Position',[t(tra(1))-0.05 10^mean(ylimlog) 0]);

plot( [t(tra(1)) t(tra(end))], ones(2,1)*threshup, 'k');

rau = aproxZeros( afreq(tra)-threshup, t(tra) );    % get intersections with thresholds
rbu = aproxZeros( yfreq-threshup, tyfreq );

p = plot( [rau(end) rbu(end-2) rbu(end) ],...
    ones(3,1)*threshup, 'sk');
set( p, 'markersize', 12,...
    'linewidth', 1.5 );
p = plot( [rau(end) rbu(end) ],...
    ones(2,1)*threshup, 'xk');
set( p, 'markersize', 12,...
    'linewidth', 1.5 );
% fatMarkers( p );

axpos = get( gca, 'position');
aopt.siz = dissFS(FORMAT);
blh = dissAlabel( fig2, '(b)', aopt );
posbl = get( blh, 'position');
tbox( [0.005 0.995 0.005 0.005], ' ', 12 );
set( blh, 'position', posbl+[alabel_xoff 0 0 0] );

TA1_POS = [0.41523 0.72842 0.41523 0.72842]+[0 0 -0.072482 -0.25684];
TA2_POS = [0.73944 0.5476 0.73944 0.5476]+[0 0 0.0056 -0.0758];

ta1 = textarrow( TA1_POS, ...
    '$\tilde{t}_{jk}(\Delta\varphi,\Delta\omega;\epsilon)$', tbfs);
set( ta1, 'interpreter', 'latex', ...
    'TextMargin', 50);
ta2 = textarrow( TA2_POS, ...
    '$\tilde{t}_{jk}(\Delta\varphi'',\Delta\omega'';\epsilon)$', tbfs);
set( ta2, 'interpreter', 'latex');

% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    saveas( fig2, [spth 'freqTrajRT-b.eps'],'epsc');
end














