
%% return time surface matrix and mean and max RT for case 5
%
%

addpath('methods');
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
% ============================== settings  ============================== %
save_fig = false;
dpth = 'data/';
spth = '';

FORMAT = 'fs';
fontsiz = dissFS(FORMAT);
fontname = 'LM Roman 12';
EDGE_ALPHA = 0;

xOffset = 1/14;
yOffset = 3/56;
xSpace  = 1/56;
ySpace  = 3/56;
axSizeX = 1/7;
axSizeY = 1/7;

phi0 = [0.2368    0.1395    0.0177    0.2193   -0.1928];
plotTicklen = [pi/4 pi/8];
cbTicklen   = 0.1;  % ticklength of colorbar
LC_COLOR = [1 1 1];

% ============================== load data ============================== %
S = load( [dpth 'fig3-05-rtsurfsCase5.mat'] );
surfs = S.surfs;
tjgrid = S.tjgrid;
tknode = S.tknode;
clear S;

% ============================= processing  ============================= %
axsize = [ axSizeX axSizeY ];
axHandles = zeros(6,6);         % array with all axes handles

maxRTgrid = getAllMaxRT( tjgrid );
maxRTnode = getAllMaxRT( tknode );
maxRTever = max( [ maxRTgrid; maxRTnode ] );

% xlim = [min(min(surfs{1}.x)) max(max(surfs{1}.x))];
% ylim = [min(min(surfs{1}.y)) max(max(surfs{1}.y))];
xlim = [-pi max(max(surfs{1}.x-phi0(1)))];
ylim = [-6.2 6.2];
clim = [0 8.5];

LC_VALUE = floor(clim(2) + 1);

X_TICK = sort( [-pi:(pi/2):pi-1 3] );
X_TICK_LABEL1 = {'$-\pi$';'$-\frac{\pi}{2}$';'$0$';'$\frac{\pi}{2}$';'';};
X_TICK_LABEL = {''; '$-\frac{\pi}{2}$'; '$0$'; '$\frac{\pi}{2}$';'';};
X_TICK_LABEL5 = {'';'$-\frac{\pi}{2}$';'$0$';'$\frac{\pi}{2}$';'$\pi$';};
X_TICK_LABEL_ALL = {'$-\pi$';'$-\frac{\pi}{2}$';'$0$'; ...
    '$\frac{\pi}{2}$';'$\pi$';};
% $^{\pi}/_{-\pi}$

Y_TICK = -6.2:3.1:6.2;
Y_TICK_LABEL1 = {'$-2\pi$';'$-\pi$';'$0$';'$\pi$';'';};
Y_TICK_LABEL = {'';'$-\pi$';'$0$';'$\pi$';'';};
Y_TICK_LABEL5 = {'';'$-\pi$';'$0$';'$\pi$';'$2\pi$';};
Y_TICK_LABEL_ALL = {'$-2\pi$';'$-\pi$';'$0$';'$\pi$';'$2\pi$';};

TGRID = ['$\uparrow$                          ';
         '$t_j^{\,\rm\scriptscriptstyle grid}$'];
TNODE = '$\leftarrow t_k^{\,\rm\scriptscriptstyle node}$';

% =============================== figure  =============================== %
fig = figure;
set( fig, 'position', [123 45 dissFigSize(FORMAT)]);

% ---------------------------- plot RT surfs ---------------------------- %
for i = 1:5
    for j = 1:5
        axHandles(i,j) = axes;                      % new current axes
        curaxx = xOffset + (j-1)*axSizeX;           % x pos of cur axes
        curaxy = yOffset + ySpace + (6-i)*axSizeY;  % y pos of cur axes
        su = surfs{i,j};                            % current surface
        su.z(isinf(su.z)) = LC_VALUE;
        
        curSplot = surf( su.x-phi0(i), su.y, su.z );    % cur surface plot
        view([0 90]);                               % 2D plot
        set( curSplot, 'edgealpha', EDGE_ALPHA );   % no edges
        set( axHandles(i,j), ...
            'xlim', xlim, ...                       % use x limit
            'ylim', ylim, ...                       % use y limit
            'clim', clim, ...                       % use color limit
            'position', [curaxx curaxy axsize], ... % use determined position
            'fontsize', fontsiz ...
            ,'ticklabelinterpreter','latex'...
            ,'xtick', X_TICK ...
            ,'ytick', Y_TICK ...
            ,'Layer', 'top' ...
            );
        
        if( j ~= 1 )
            plotTickOpt.ytick = [];
            set( axHandles(i,j), 'yticklabels', {} );
        else
            plotTickOpt.ytick = Y_TICK;
            if( i == 1 )
                set( axHandles(i,j), 'yticklabels', Y_TICK_LABEL5 );
            elseif( i == 5 )
                set( axHandles(i,j), 'yticklabels', Y_TICK_LABEL1 );
            else
                set( axHandles(i,j), 'yticklabels', Y_TICK_LABEL );
            end
            ylh = dissYLabel( ['$j = ' num2str(i) '$'], FORMAT );
            set( ylh, 'position', [-5 0 0]);
        end
        
        if( i ~= 5 )
            plotTickOpt.xtick = [];
            set( axHandles(i,j), 'xticklabels', {});
            if( i == 1 )
                xlh = dissXLabel( ['$k = ' num2str(j) '$'], FORMAT );
                set( xlh, 'position', [0 3*pi 0] );
            end
        else
            plotTickOpt.xtick = X_TICK;
            if( j == 1 )
                set( axHandles(i,j), 'xticklabels', X_TICK_LABEL1 );
            elseif( j == 5 )
                set( axHandles(i,j), 'xticklabels', X_TICK_LABEL5 );
            else
                set( axHandles(i,j), 'xticklabels', X_TICK_LABEL );
            end
        end
        
        hold on;
        plotTicks( axHandles(i,j), 100, plotTicklen, plotTickOpt );
        % title( ['i' num2str(i) ', j' num2str(j)] );
    end
end


% ------------------------- plot grid RT surfs  ------------------------- %
plotTickOpt.ypos = 'right';
for i = 1:5
    axHandles(i,6) = axes;                      % new current axes
    curaxx = xOffset + xSpace + 5*axSizeX;      % x pos of cur axes
    curaxy = yOffset + ySpace + (6-i)*axSizeY;  % y pos of cur axes
    su.z = tjgrid{i};                         % current surface
    su.x = surfs{i,1}.x;
    su.y = surfs{i,1}.y;
    su.z(isinf(su.z)) = LC_VALUE;
    
    curSplot = surf( su.x-phi0(i), su.y, su.z );    % cur surface plot
    view([0 90]);                               % 2D plot
    set( curSplot, 'edgealpha', EDGE_ALPHA );   % no edges
    set( axHandles(i,6), ...
        'xlim', xlim, ...                       % use x limit
        'ylim', ylim, ...                       % use y limit
        'clim', clim, ...                       % use color limit
        'position', [curaxx curaxy axsize], ... % use determined position
        'fontsize', fontsiz ...
        ,'ticklabelinterpreter','latex'...
        ,'xtick', X_TICK ...
        ,'ytick', Y_TICK ...
        ,'Layer', 'top' ...
        );
    
    if( i == 1 )
        set( axHandles(i,6), 'yticklabels', Y_TICK_LABEL5 );
    elseif( i == 5 )
        set( axHandles(i,6), 'yticklabels', Y_TICK_LABEL1 );
    else
        set( axHandles(i,6), 'yticklabels', Y_TICK_LABEL );
    end
    
    plotTickOpt.ytick = Y_TICK;
    if( i ~= 5 )
        set( axHandles(i,6), 'xticklabels', {});
        plotTickOpt.xtick = [];
    else
        plotTickOpt.xtick = X_TICK;
        set( axHandles(i,6), 'xticklabels', X_TICK_LABEL5);
    end
    
    set( axHandles(i,6), 'yAxisLocation', 'right' );
    
    hold on;
    plotTicks( axHandles(i,6), 100, plotTicklen, plotTickOpt );
end

% ------------------------- plot node RT surfs  ------------------------- %
plotTickOpt.ypos = 'left';
for j = 1:5
    axHandles(6,j) = axes;                      % new current axes
    curaxx = xOffset + (j-1)*axSizeX;           % x pos of cur axes
    curaxy = yOffset;                           % y pos of cur axes
    su.z = tknode{j};                           % current surface
    su.x = surfs{1,j}.x;
    su.y = surfs{1,j}.y;
    su.z(isinf(su.z)) = LC_VALUE;
    
    curSplot = surf( su.x-phi0(1), su.y, su.z );    % cur surface plot
    view([0 90]);                               % 2D plot
    set( curSplot, 'edgealpha', EDGE_ALPHA );   % no edges
    set( axHandles(6,j), ...
        'xlim', xlim, ...                       % use x limit
        'ylim', ylim, ...                       % use y limit
        'clim', clim, ...                       % use color limit
        'position', [curaxx curaxy axsize], ... % use determined position
        'fontsize', fontsiz ...
        ,'ticklabelinterpreter','latex'...
        ,'xtick', X_TICK ...
        ,'ytick', Y_TICK ...
        ,'Layer', 'top' ...
        );
    
    if( j == 1 )
        set( axHandles(6,j), 'xticklabels', X_TICK_LABEL1 );
    elseif( j == 5 )
        set( axHandles(6,j), 'xticklabels', X_TICK_LABEL5 );
    else
        set( axHandles(6,j), 'xticklabels', X_TICK_LABEL );
    end
    
    plotTickOpt.xtick = X_TICK;
    if( j ~= 1 )
        plotTickOpt.ytick = [];
        set( axHandles(6,j), 'yticklabels', {});
    else
        plotTickOpt.ytick = Y_TICK;
        set( axHandles(6,j), 'yticklabels', Y_TICK_LABEL_ALL );
    end
    
    hold on;
    plotTicks( axHandles(6,j), 100, plotTicklen, plotTickOpt );
end

tboxTnode = tbox( [0.795 0.06 0.075 0.0435], TNODE, fontsiz+4 );
set( tboxTnode, 'interpreter', 'latex' );
tboxTgrid = tbox( [0.82 0.116 0.05 0.072], TGRID, fontsiz+4 ); 
set( tboxTgrid, 'interpreter', 'latex' );

% ------------------------------ colorbar  ------------------------------ %
cm = colormap('parula');                  % get standard color map
% cmadd1 = [0.5:0.05:0.2 0];
% cmadd = zeros(3,numel(cmadd1));
cmadd = LC_COLOR;
cm = [ cm; cmadd];
colormap(cm);

cb = colorbar;
xColorbar = (axSizeX/5);
CB_TICKS = [0:2:8 LC_VALUE+0.5];

CB_TICK_L = cell( numel(CB_TICKS),1);
for i = 1:numel(CB_TICKS)
    CB_TICK_L(i) = {num2str(CB_TICKS(i))};
end
CB_TICK_L(end) = {'LC'};

set( cb, 'position', [(xOffset+6*axSizeX-xColorbar+xSpace) yOffset ...
    xColorbar axSizeY], ...
    'ticks', CB_TICKS, ...
    'ticklabels', CB_TICK_L, ...
    'tickLength', cbTicklen, ...
    'fontname', fontname, ...
    'limits', [clim(1) (LC_VALUE+0.5)] ...
    );

% ============================== save fig  ============================== %

if( ~~save_fig )
    saveas( fig, [spth 'RTgridNode.png']);
end



