
%% figure 3.7 Dissertation
% shows the parameters needed to calculate the swing equation for RTS-96
% grid
% 
%
% V1.0 mawolff@uos.de 11-Sep-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
% Path to pg_dyn_models 
% [Nishikawa and Motter, New J. Phys. 17, 015012 (2015) ]
% needed:
addpath('C:\Users\Anwender\Documents\MATLAB\pg_dyn_models');

%% ============================= settings  ============================== %
save_fig = false;
dpth = 'data/';
spth = '';

K_THRESH = 0.05;

FORMAT = 'fr';
fs = dissFS(FORMAT);
tbfs = fs;
FS_LABEL = fs;
font = 'Times New Roman';          % set font style

EDGE_COL = 'none';
% EDGE_COL = [1 1 1]*0.5;

% ---------------------------- axis settings ---------------------------- %
XOFF = 0.075;
YOFF = 0.18;
FIGSY = 0.36;
FIGSX = 0.25;
XSPACE = XOFF;

% YSPACE = 0.12;
% ---------------------------- plot settings ---------------------------- %
MARKERSIZE_HET = 5;
MARKERSIZE_HOM = 3;
TICK_LEN = [0.015 0.008];
LCLINEW = 1;
LINEW = 1;

XLIM_UP = [0 73];

lstyle = 'none';
% paper
% COLOR_HET = [ 0 0 0 ];          % black
% COLOR_HOM = [ 1 0.7 0.2 ];      % orange
% COLOR_HOM = [ 0.3 0.7 0.2 ];  % "olive" green

COLOR_HET = 'none';
COLOR_HOM = 'none';
% uos style
% COLOR_HET_FACE = unicolors('dr');   % for histogram
% COLOR_HET_FACE1 = unicolors('dr');  % for dots
% COLOR_HOM_FACE = unicolors('do');

% diss style
COLOR_HET_FACE = [0.2 0.2 1];   % for histogram
COLOR_HET_FACE1 = [0.2 0.2 1];  % for dots
COLOR_HOM_FACE = [0.3 0.7 0.2];

%% ----------------------------- get data  ------------------------------ %
l = load( [dpth 'rts96hom-R'] );
mpchom    = l.mpc;
l = load( [dpth 'RTS96homloc'] );
mpchomloc = l.mpc;
l = load( [dpth 'RTS96'] );
mpchet    = l.mpc;
l = load( [dpth 'RTS96hetloc'] );
mpchetloc = l.mpc;
smhom     = SM_model( mpchom );
smhomloc  = SM_model( mpchomloc );
smhet     = SM_model( mpchet );
smhetloc  = SM_model( mpchetloc );

PD = 3;
PG = 2;
[gb, lb] = getglbus( mpchet, '-idx');
Phet    = [ mpchet.gen(:,PG)-mpchet.bus(gb,PD);
           -mpchet.bus(lb,PD)]/mpchet.baseMVA;
% Phetloc = mpc.gen(:,PG); - mpchetloc.bus(lb,PD);
Phom    = [ mpchom.gen(:,PG)-mpchom.bus(gb,PD);
           -mpchom.bus(lb,PD)]/mpchom.baseMVA;

%%

close all;

fig = figure;
set( fig, 'position', [123 456 dissFigSize(FORMAT)-[0 150] ]);
% delete( gca );

% ----------------------------- Power plot  ----------------------------- %
axPower = axes;
phe = plot( Phet, 'o' ); hold on;
set( phe, 'markersize', MARKERSIZE_HET, ...
    'markerfacecolor', COLOR_HET_FACE1, ...
    'color', COLOR_HET, ...
    'displayname', 'Het', ...
    'linestyle', lstyle, ...
    'linewidth', LINEW);
phe = plot( Phom, 'o' ); hold on;
set( phe, 'markersize', MARKERSIZE_HOM, ...
    'markerfacecolor', COLOR_HOM_FACE, ...
    'color', COLOR_HOM, ...
    'displayname', 'Hom', ...
    'linestyle', lstyle, ...
    'linewidth', LINEW);
plot( [1 1]*33.5, [-3 7], 'k--' );

set( axPower, 'position', [XOFF ...
    YOFF FIGSX FIGSY], ...
    'xlim', XLIM_UP, ...
    'ylim', [-3 7], ...
    'xminortick', 'on', ...
    'yminortick', 'on', ...
    'xgrid', 'on', 'ygrid', 'on', ...
    'xminorgrid', 'off' , ...
    'yminorgrid', 'on' , ...
    'ytick', -3:3:6, ...
    'xtick', 10:10:70, ...
    'fontsize', fs, ...
    'ticklength', TICK_LEN*2);
axPower.XAxis.MinorTickValues = 1:73;
axPower.YAxis.MinorTickValues = -3:6;

ylhp = ylabel('$P_j^{\rm (m)}$', ...
    'fontsize', FS_LABEL, ...
    'interpreter', 'latex');
set(ylhp, 'Rotation',0);
set(ylhp, 'Position',[-14 0.55 0]);
xlabel('$j$', ...
    'fontsize', FS_LABEL, ...
    'interpreter', 'latex');

tbgen = tbox(  [0.121 0.458 0.0700 0.06], 'generators', 10 );
tbload = tbox( [0.192 0.458 0.0325 0.06], 'loads', 10 );

% ---------------------------- Inertia plot  ---------------------------- %

axInertia = axes;
phe = plot( 2*smhet.H/(2*pi*60), 'o' ); hold on;
set( phe, 'markersize', MARKERSIZE_HET, ...
    'markerfacecolor', COLOR_HET_FACE,...
    'color', COLOR_HET,...
    'displayname', 'Het',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
phom = plot( 2*smhom.H/(2*pi*60), 'o' ); hold on;
set( phom, 'markersize', MARKERSIZE_HOM, ...
    'markerfacecolor', COLOR_HOM_FACE,...
    'color', COLOR_HOM,...
    'displayname', 'Hom',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
pheLeg = plot( NaN, NaN, 'o', ...
    'markersize', MARKERSIZE_HET, ...
    'markerfacecolor', COLOR_HET_FACE,...
    'color', COLOR_HET,...
    'displayname', 'Het',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
phoLeg = plot( NaN, NaN, 'o', ...
    'markersize', MARKERSIZE_HET, ...
    'markerfacecolor', COLOR_HOM_FACE,...
    'color', COLOR_HOM,...
    'displayname', 'Hom',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
plot( [1 1]*33.5, [-0.01 0.16], 'k--' );

set( axInertia, 'position', [XOFF ...
    YOFF+FIGSY FIGSX FIGSY],...
    'xlim', XLIM_UP,...
    'ylim', [-0.01 0.16],...
    'xminortick', 'on',...
    'yminortick', 'on',...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminorgrid', 'off' ,...
    'ytick', 0:0.04:0.17, ...
    'xtick', 0:10:80, ...
    'xticklabels', {}, ...
    'yticklabels', {'0','4','8','12','16'},...
    'fontsize', fs,...
    'ticklength', TICK_LEN*2);
axInertia.XAxis.MinorTickValues = 1:73;
axInertia.YAxis.MinorTickValues = 0:0.01:0.16;

ylabel('$H_j$', ...
    'fontsize', FS_LABEL, ...
    'interpreter', 'latex');
ylh = get(gca,'ylabel');
set(ylh, 'Rotation',0);
set(ylh, 'Position',[-14  0.0606    0.0000]);
leg = legend([pheLeg phoLeg]);


tbhj = tbox( [0.065371 0.93062 0.01 0.01] ,...
    '$\times 10^{-2}$', tbfs );
set( tbhj, 'fontname', font,...
    'interpreter', 'latex',...
    'horizontalAlignment', 'left');
% ---------------------------- Coupling plot ---------------------------- %


pa = [XOFF+FIGSX+XSPACE ...   % plot area
    YOFF 2*FIGSX 2*FIGSY];
% 1 | 2 
% -------
% 3 | 4
p = zeros(4,4);
p(1,:) = [pa(1) pa(2)+pa(4)/2 pa(3)/2 pa(4)/2];
p(2,:) = [pa(1)+pa(3)/2 pa(2)+pa(4)/2 pa(3)/2 pa(4)/2];
p(3,:) = [pa(1) pa(2) pa(3)/2 pa(4)/2];
p(4,:) = [pa(1)+pa(3)/2 pa(2) pa(3)/2 pa(4)/2];

K = cell(4,2);
K{1,1} = -6:0.1:2;
K{2,1} = -6:0.1:2;
K{3,1} = -6:0.1:3;
K{4,1} = -6:0.1:3;
Khet = shape('h', nodiag(log10(abs(smhet.K))) );
Khom = shape('h', nodiag(log10(abs(smhom.K))) );
Khetloc = shape('h', nodiag(log10(abs(smhetloc.K))) );
Khomloc = shape('h', nodiag(log10(abs(smhomloc.K))) );
[K{1,2}, ~] = hist( Khet(Khet~= 0), K{1,1} );
[K{2,2}, K{2,1}] = hist( Khom(Khom~= 0), K{1,1} );
[K{3,2}, K{3,1}] = hist( Khetloc(Khetloc ~= 0), K{3,1}  );
[K{4,2}, K{4,1}] = hist( Khomloc(Khomloc ~= 0), K{4,1}  );

tbt = {'Het+IN','Hom+IN','Het-IN','Hom-IN'};
hi = zeros(4,1);
axd = zeros(4,1);
for i = 1:4
    axd(i) = axes;
    
    hi(i) = bar( K{i,1}, K{i,2} );
    hold on;
    set( hi(i), 'edgecolor', EDGE_COL );
    if( i == 1 )
        plot( [1 1]*log10(K_THRESH), [0 200], ':', ...
            'color', unicolors('do'), ...   
            'linewidth', 3);
    end
    
    set( axd(i), 'position', p(i,:) ,...
        'xlim', [-5 3],...
        'xminortick', 'on',...
        'yminortick', 'on',...
        'xgrid', 'on', 'ygrid', 'on',...
        'xminorgrid', 'on' ,...
        'xtick', -4:2:2 , ...
        'xticklabels', {'10^{-4}','10^{-2}','10^{0}','10^{2}'},...
...%         'ytick', 0:10:30, ...
        'fontsize', fs,...
        'ticklength', TICK_LEN*2);
    ax = get(axd(i));
    ltick = log10(1:9);
    ax.XAxis.MinorTickValues = [ltick-5 ltick-4 ltick-3 ltick-2 ...
        ltick-1 ltick ltick+1 ltick+2 ];
    
    ylabel('counts', 'fontname', font);
    if( i < 3 )
        axcoord = val2FigCoords([1 5.5]);
        tbc = [axcoord(1) FIGSY*2+YOFF/2 0.01 0.01];
        tbh = tbox( tbc,tbt{i}, tbfs);
    else
        axcoord = val2FigCoords([-3.5 5.5]);
        tbc = [axcoord(1) FIGSY+(YOFF)/2 0.01 0.01];
        tbh = tbox( tbc,tbt{i}, tbfs);
    end
%     axup.XAxis.MinorTickValues = 1:73;
%     axup.YAxis.MinorTickValues = 0:5:30;
end

xl = get( axd(3), 'xlabel');
set(xl , 'String','$K_{jk}$', ...
    'fontsize', FS_LABEL, ...
    'interpreter', 'latex');
xl = get( axd(4), 'xlabel');
set(xl , 'String','$K_{jk}$', ...
    'fontsize', FS_LABEL, ...
    'interpreter', 'latex');

set( axd(2), 'yAxisLocation', 'right' );
set( axd(4), 'yAxisLocation', 'right' );

yt1 = get( axd(1), 'ytick');
set( axd(1), 'ytick', yt1(2:end));
yt2 = get( axd(2), 'ytick');
set( axd(2), 'ytick', yt2(2:end));

set( hi(2), 'facecolor', COLOR_HOM_FACE );
set( hi(4), 'facecolor', COLOR_HOM_FACE );
set( hi(1), 'facecolor', COLOR_HET_FACE );
set( hi(3), 'facecolor', COLOR_HET_FACE );

% %%
alabw = 0.025;
ao.siz = fs;
cl = dissAlabel( fig, '(c)', ao );
clpos = get( cl,'position');
set( cl, 'position', [clpos(1:2) alabw clpos(4)]);
dl = dissAlabel( fig, '(d)', ao );
dlpos = get( cl,'position');
set( dl, 'position', [clpos(1:2) alabw clpos(4)]+[FIGSX+XSPACE+0.002 0 0 0]);

if( ~~save_fig )
    saveas( fig, ['figures/' 'ParamRTS4models.eps'], ...
        'epsc');
end

