

%% basinStabHet RTS-96
% plots the basin stability of every node in the RTS96 test grid.
% This is done for the models Het+IN and Het-IN.
%
% 16-Jun-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

%% ============================ preferences ============================= %

save_fig = false;             % if the figure should be saved to disk automatically
SPATH = '';                 % path to save the figures in

FORMAT = 'hr';
% ealpha = 0.1;               % alpha of edges in surf plot
% font = 'LM Roman';          % set font style
ticklenx = [1 0.5]*0.04;    % tick length on x-axis for minor and major ticks
tickleny = [1 0.5]*0.05;    % tick length on y-axis for minor and major ticks

% minc = min(min(sbhet));   %
minc = 0.65;                % minimum of colorbar

facecolr = 0.75*[1 1 1];  % color of bars

% fs = dissFS(FORMAT);                    % fontsize of axis
% tbfs = dissFS(FORMAT);                  % textbox fontsize
% cm = colormapGen(80,'');    % generate colormap
% cm = flip(cm);              % invert colormap

%% ======================= load and process data ======================== %
% clear;
dpth = 'data/';
load( [dpth 'fig3-4-basinStab.mat'], 'sbhe', 'sbhel' );

%% ============================ koordinaten ============================= %

xoffs = 0.2;
xfig  = 0.70;
% xbetw = 0.11;
xcbar = 0.03;

yoffs = 0.25;
yfig  = 0.70;

% ============================== figure 1  ============================== %

fig1 = figure;
set( fig1, 'position', [123 56 dissFigSize(FORMAT)] + [0 0 0 75]);
% subplot(1,2,1);

axes;
ax2 = gca;
set(ax2,'XTick',[]) %remove the ticks
set(ax2, ...
    'xlim', [0 73], ...
    'ylim', [0 0.3],...
    'fontsize', dissFS(FORMAT), ...
    'position', [xoffs yoffs xfig yfig-0.1],...
    'xtick',0:10:70,...
    'xminortick', 'on',...
    'ticklength', ticklenx,...
    'ytick',[],...
    'tickdir','out',...
    'fontname', dissFont,...
    'box', 'on') %remove the ticks
xlh1 = dissXLabel('$j$', FORMAT);
% set(xlh1, 'Position',[35 -0.07 0]);
ax2.XAxis.MinorTickValues = 1:73;

axes;
axmain = gca;
bhe = bar( 1-sbhe );

set( bhe, 'facecolor', facecolr );

set( gca, ...
    'xlim', [0 73], ...
    'ylim', [0 0.3],...
    'fontsize', dissFS(FORMAT), ...
    ...'facecolor', facecolr, ...
    'position', [xoffs yoffs xfig yfig],...
    'xtick',[],...
    'ytick',0:0.1:0.3,...
    'ticklength', tickleny,...
    'tickdir','in',...
    'yminortick', 'on',...
    'fontname', dissFont);
axmain.YAxis.MinorTickValues = 0:0.01:0.3;

ylh1 = dissYLabel('$(1-S_j)$',FORMAT);
% rotYlabel( ylh1 );
% set(ylh1, 'Position',[-13 0.225 0]);

aopt.siz = dissFS(FORMAT);
tb = dissAlabel( fig1, '(a)', aopt );
tbpos = get( tb, 'position');
set( tb, ...
    'position', tbpos + [0.01 -0.02 0 0] );
fval = val2FigCoords([26 0.25]);
tbhe = tbox( [fval 0.15 0.15], 'Het+IN', dissFS(FORMAT) );
set( tbhe, 'fontname', dissFont );

if( ~~save_fig )
    pause(1);
    saveas(fig1, [SPATH 'basinstabRTS-a.eps'], 'epsc');
end


%% ============================= figure 2  ============================== %

% subplot(1,2,2);
fig2 = figure;
set( fig2, 'position', [123 456 dissFigSize(FORMAT)] + [0 0 0 75]);

axes;
ax2 = gca;
% set(ax2, 'XAxisLocation','top','linewidth',3) %cover the top box manually
set(ax2,'XTick',[]) %remove the ticks
set(ax2, ...
    'xlim', [0 73], ...
    'ylim', [0 0.3],...
    'fontsize', dissFS(FORMAT), ...
    'position', [xoffs yoffs xfig yfig-0.1],...
    'xtick',0:10:70,...
    'xminortick', 'on',...
    'ticklength', ticklenx,...
    'ytick',[],...
    'tickdir','out',...
    'fontname', dissFont,...
    'box', 'on') %remove the ticks
xlh2 = dissXLabel('$j$', FORMAT);
ax2.XAxis.MinorTickValues = 1:73;

axes;
axmain = gca;
bhel = bar( 1-sbhel );

set( bhel, 'facecolor', facecolr );

set( gca, ...
    'xlim', [0 73], ...
    'ylim', [0 0.3],...
    'fontsize', dissFS(FORMAT), ...
    'position', [xoffs yoffs xfig yfig],...
    'xtick',[],...
    'ytick',0:0.1:0.3,...
    'ticklength', tickleny,...
    'tickdir','in',...
    'yminortick', 'on',...
    'fontname', dissFont);
axmain.YAxis.MinorTickValues = 0:0.01:0.3;

ylh2 = dissYLabel('$(1-S_j)$', FORMAT);
% rotYlabel( ylh1 );
% set(ylh1, 'Position',[-13 0.225 0]);

aopt.siz = dissFS(FORMAT);
tb = dissAlabel( fig2, '(b)', aopt );
tbpos = get( tb, 'position');
set( tb, ...
    'position', tbpos + [0.01 -0.02 0 0] );
fval = val2FigCoords([26 0.245]);
tbhel = tbox( [fval 0.15 0.15], 'Het$-$IN', dissFS(FORMAT) );
% 0.43 fval(2)
set( tbhel, 'fontname', dissFont, 'interpreter', 'latex' );

if( ~~save_fig )
    pause(1);
    saveas(fig2, [SPATH 'basinstabRTS-b.eps'], 'epsc');
end

