
%% creates a beautiful plot of the grid return times (WIP)
% for every perturbation of a given perturbed node j. This is done for 
% every hcase (het, hom, ...)
%
%
%
% V1.0 mawolff@uos.de 14-Jul-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================== settings  ============================== %
save_fig = false;
SPATH = '';         % path to store the figures in

FORMAT = 'fs';

% ---------------------- constants  ---------------------- %
% N = 73;                         % number of nodes in the grid
fsax = 22;                      % font size for axes
fscb = 22;                      % font size for colorbar
fscbh = 26;                     % font size colorbar header
fslabs = 24;                    % font size x,y labels
figs = (0.7/2);                 % figure size for return time surfaces
X_OFFSET = 0.1;                % offset of x values from left border
Y_OFFSET = 0.12;                % offset of y values from down border

CB_WIDTH = 0.035;               % how wide the color bar is
TICK_WIDTH = 1;                 % width of the tick strokes
TICK_LEN = [1 0.5]*0.04;        % length of the tick strokes
MF_POS = [ 10 50 ...
    dissFigSize(FORMAT)];     % position and size of the surface frame in the end

LCVAL = 15;                     % limit cycle value
LCVALLOG = 5;

CB_LAST_TICK = 'LC';

jnode = 9;

X_LABEL = '$\Delta\varphi$';

%% =============================== load  ================================ %
try 
    file = ['data/' 'fig-3-7-tjgridX4.mat'];
    S = load(file);
    Tj = S.Tj;
    sx = S.sx;
    sy = S.sy;
    clear S;
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be find.']);  
        otherwise 
            rethrow( ME ); 
    end 
end 

% ============================= processing  ============================= %
log_plot = false;
if( log_plot )
    clim = [0 1.2];             % define the range of values corresponding
                                % to the colors
    CB_TICKS = [0 log10(2:12) clim(end)];%floor(10^(clim(end)))];
else
    clim = [1 12];
    CB_TICKS = 0:2:12;
end

XTICK = (-1:0.5:1)*pi;
YTICK = (-2:1:2)*pi;
YMTICK = (-2:0.5:2)*pi;
XTICKL = {'$-\pi$';'$-\pi/2$';'0';'$\pi/2$';'$\pi$'};
YTICKL = {'$-2\pi$';'$-\pi$';'0';'$\pi$';'$2\pi$'};
EMPTYL = {'';'';'';'';''};

xspace = 0.075;
yspace = 0.075;


tbopt.alignx = 'center';        % options for the titles alignment

% plotjl = false;                 % plot j labels in the surface
% -------------------- option strucs  -------------------- %
% opt.regex = 'heijtorEN.*';      % regular expr to find the data files
% opt.iseq = [1 2 8 9];           % which nodes are observed
% opt.jseq = 1;                   % dummy for energy return times (there is no j)
% opt.plotxy = [2 2];             % help figures have 2 times 2 plots
% opt.mat = true;                 % use mat files if findable


% =============================== figures =============================== %
mainFig = figure; hold on;                      % create main figure
% set( mainFig, 'position', [100 100 500 500]);   % set its position
set( mainFig, 'position', MF_POS );   % set its position

delete(gca);
cm = colormap('jet');                              % get standard color map
cmadd1 = [0.5:0.05:0.2 0];
cmadd = zeros(3,numel(cmadd1));
cmadd(1,:) = cmadd1;
cm = [ cm; cmadd'];

% ------------------------------- het+in -------------------------------- %
zd = Tj.het{jnode};         % get the z-surf
if(log_plot)                 % if logarithmic scale requested
    zd = log10( zd);        % log10 of z-surf
    zd(zd>1) = LCVALLOG;           % set return times larger than 10s to lim cycle value
else
    zd( zd>19) = LCVAL;
end

figure(mainFig);                % select figure
axes;                           % new axis
su = surf(sx.hetpin,sy.hetpin,zd); hold on;   % plot the surf
set(su, 'edgealpha', 0, ...     % remove the edges (lines between faces)
    'edgecolor', 'none');       %
view([0 90]);                   % view xy-plane
set( gca, 'xlim', [-pi pi],...  % set x-limits
    'ylim', [-2*pi 2*pi],...    % set y-limits
    'clim',clim,...             % set limits for color
    'position', [X_OFFSET Y_OFFSET+figs+yspace figs figs],...   % set axis to right position
    'xtick', XTICK ,...         %
    'ytick', YTICK,...
    'yminortick', 'on',...
    'fontsize',fsax);
colormap(cm);

ax = gca;
ax.YAxis.MinorTickValues = YMTICK;
xticklabels(EMPTYL);        % 

set( gca, 'TickLabelInterpreter', 'latex');
yticklabels(YTICKL);        % needs matlab 2016/17
set(gca,'TickDir','out',...
    'LineWidth',TICK_WIDTH,...
    'TickLength',TICK_LEN);

dissYLabel('$\Delta\omega$', FORMAT );
% ylabel('$\Delta\omega$',...
%     'interpreter','latex');
ylh = get(gca,'ylabel');
xlim = get(gca,'xlim');
ylim = get(gca,'ylim');
set(ylh, 'Rotation',0);
set(ylh, 'Position',[xlim(1)-pi/3 ...
    (ylim(2)+ylim(1))/2+0.5 0]);
set( ylh, 'fontsize', fslabs);

% ------------------------------- hetloc  ------------------------------- %
i = 2;

zd = Tj.hetl{jnode};
if(log_plot)                 % if logarithmic scale requested
    zd = log10( zd);        % log10 of z-surf
    zd(zd>1) = LCVALLOG;           % set return times larger than 10s to lim cycle value
else
    zd( zd>19) = LCVAL;
end

figure(mainFig);
axes;	
su = surf(sx.hetmin,sy.hetmin,zd); hold on;
set(su, 'edgealpha', 0, 'edgecolor', 'none');
set( gca, 'xlim', [-pi pi],...
    'ylim', [-2*pi 2*pi],...
    'clim',clim,...
    'position',[X_OFFSET Y_OFFSET figs figs],...
    'xtick',XTICK,...
    'ytick',YTICK,...
    'yminortick', 'on',...
    'fontsize',fsax);
view([0 90]);

set( gca, 'TickLabelInterpreter', 'latex');
yticklabels(YTICKL); 
xticklabels(XTICKL); 
set(gca,'TickDir','out',...
    'LineWidth',TICK_WIDTH,'TickLength',TICK_LEN);

ax = gca;
ax.YAxis.MinorTickValues = YMTICK;

dissYLabel('$\Delta\omega$', FORMAT );
ylh = get(gca,'ylabel');
xlim = get(gca,'xlim');
ylim = get(gca,'ylim');
set(ylh, 'Rotation',0);
set(ylh, 'Position',[xlim(1)-pi/3 ...
    (ylim(2)+ylim(1))/2+0.5 0]);
set( ylh, 'fontsize', fslabs);

xlh = dissXLabel(X_LABEL, FORMAT);
set( xlh, 'fontsize', fslabs);


% ------------------------------- homloc  ------------------------------- %
zd = Tj.homl{jnode};

if( ~~log_plot)                 % if logarithmic scale requested
    zd = log10( zd);            % log10 of z-surf
    zd(zd>1) = LCVALLOG;        % set return times larger than 10s to lim cycle value
else
    zd( zd>19) = LCVAL;
end

figure(mainFig);        %
axes;	
su = surf(sx.hommin,sy.hommin,zd); hold on;
set(su, 'edgealpha', 0, 'edgecolor', 'none');
set( gca, 'xlim', [-pi pi],...
    'ylim', [-2*pi 2*pi],...
    'clim',clim,...
    'position',[X_OFFSET+figs+xspace Y_OFFSET figs figs],...
    'xtick',XTICK,...
    'ytick',YTICK,...
    'yminortick', 'on',...
    'fontsize',fsax);
view([0 90]);

set( gca, 'TickLabelInterpreter', 'latex');
xticklabels(XTICKL); 
ax = gca;
ax.YAxis.MinorTickValues = YMTICK;
yticklabels(EMPTYL);        % 
set(gca,'TickDir','out',...
    'LineWidth',TICK_WIDTH,'TickLength',TICK_LEN );

xlh = dissXLabel(X_LABEL, FORMAT);
set( xlh, 'fontsize', fslabs);
% xlabel(X_LABEL,...
%     'interpreter','latex');



% --------------------------------- hom --------------------------------- %
   
i = 2;
zd = Tj.hom{jnode};

if(log_plot)                 % if logarithmic scale requested
    zd = log10( zd);        % log10 of z-surf
    zd(zd>1) = LCVALLOG;           % set return times larger than 10s to lim cycle value
else
    zd( zd>19) = LCVAL;
end

figure(mainFig);
axes; 	
su = surf(sx.hompin,sy.hompin,zd); hold on;
set(su, 'edgealpha', 0, 'edgecolor', 'none');
set( gca, 'xlim', [-pi pi],...
    'ylim', [-2*pi 2*pi],...
    'clim',clim,...
    'position',[X_OFFSET+xspace Y_OFFSET+yspace 0 0]+figs,...
    'xtick',XTICK,...
    'ytick',YTICK,...
    'yminortick', 'on',...
    'fontsize',fsax);
view([0 90]);

set( gca, 'TickLabelInterpreter', 'latex',...
    'TickDir','out',...
    'LineWidth',TICK_WIDTH,'TickLength',TICK_LEN);
ax = gca;
ax.YAxis.MinorTickValues = YMTICK;
xticklabels( EMPTYL);        % 
yticklabels( EMPTYL );
    

% ------------------------ titles ------------------------ %
figure( mainFig);

middleLeftX = X_OFFSET+ 0.25*figs;
middleLeftY = Y_OFFSET+figs*2+yspace;
middleLeftW = 0.5*figs;
middleLeftH = 0.05;
tbpos = [middleLeftX middleLeftY middleLeftW middleLeftH];
tb = tbox( tbpos, 'Het+IN',20,'',[1 1 1], tbopt);
set( tb, 'fontname', 'LM Roman 12');

middleLeftDX = X_OFFSET+ 0.25*figs;
middleLeftDY = Y_OFFSET+figs;
middleLeftDW = 0.5*figs;
middleLeftDH = 0.05;
tbpos = [middleLeftDX middleLeftDY middleLeftDW middleLeftDH];
tb = tbox( tbpos, 'Het-IN',20,'',[1 1 1], tbopt);
set( tb, 'fontname', 'LM Roman 12');

middleRightX = X_OFFSET+1.25*figs+xspace;
middleRightY = Y_OFFSET+figs*2+yspace;
middleRightW = 0.5*figs;
middleRightH = 0.05;
tbpos = [middleRightX middleRightY middleRightW middleRightH];
tb = tbox( tbpos, 'Hom+IN',20,'',[1 1 1], tbopt);
set( tb, 'fontname', 'LM Roman 12');

middleRightDX = X_OFFSET+1.25*figs+xspace;
middleRightDY = Y_OFFSET+figs;
middleRightDW = 0.5*figs;
middleRightDH = 0.05;
tbpos = [middleRightDX middleRightDY middleRightDW middleRightDH];
tb = tbox( tbpos, 'Hom-IN',20,'',[1 1 1], tbopt);
set( tb, 'fontname', 'LM Roman 12');

% ---------------------- colorbar  ---------------------- %
cb = colorbar;
spacy = yspace;
cbx = X_OFFSET+(2*figs+1.5*xspace);
CB_TICK_L = cell( numel(CB_TICKS),1);
for i = 1:(numel(CB_TICKS)-1)
    if( log_plot )
        CB_TICK_L(i) = {num2str(10^CB_TICKS(i))};
    else
        CB_TICK_L(i) = {num2str(CB_TICKS(i))};
    end
end
CB_TICK_L(numel(CB_TICK_L)) = {CB_LAST_TICK};
% if( ~PLOT_HETLOC && ~PLOT_HOMLOC )
%     set( cb, 'position', [ cbx Y_OFFSET+2*figs+spacy...
%         CB_WIDTH 2*figs],...
%     'fontsize', fscb, 'fontname', 'LM Roman 12', 'ticks', CB_TICKS,...
%     'ticklabels', CB_TICK_L);
% else
set( cb, 'position', [ cbx Y_OFFSET CB_WIDTH 2*figs+spacy],...
    'fontsize', fscb, 'fontname', 'LM Roman 12', 'ticks', CB_TICKS,...
    'ticklabels', CB_TICK_L);
% end

tb = tbox([ cbx Y_OFFSET+2*figs+spacy+0.015 0.05 0.05],...
    '$t_{j}^{\rm grid}$',fscbh);
set(tb, 'interpreter', 'latex');

% set( mainFig, 'position',MF_POS);

if( ~~save_fig )
    saveas( gcf, [SPATH 'tjgridX4.png']);
end




