

%% T_j max and bar t_j mean  vs j respectively
% this is the dissertation version plotting maximum grid return times 
% T_j^grid as well as average grid return times bar t_j^grid 
%
% V1.0 mawolff@uos.de 15-Jul-2020


global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

%% ============================== load data ============================== %

S = load( 'data/fig-3-08-gridNodeRTallJ.mat' );
tbjgrid = S.tbjgrid;    % read out grid return times
tbjlim = S.tbjlim;      % read out if limit cycles occured
tbknode = S.tbknode;    % read out node return times
tbknlim = S.tbknlim;    % read out if limit cycles occured

clear S;

%% ============================= settings  ============================== %
save_fig = false;

% ---------------------------- axis settings ---------------------------- %
XOFF = 0.075;
XOFFLOW = 0.1;
YOFF = 0.15;
XSPACE = 0.1;
YSPACE = 0.12;
FIGSX_UP = 0.85;
FIGSX_LOW = 0.35;
FIGSY = 0.22;

LEG_POS = [0.7752    0.8272    0.0956    0.1543];
LEG_FS = 20;

TB_XOFF = 0.0175;

% ---------------------------- plot settings ---------------------------- %
MARKERSIZE = 10;
TICK_LEN = [0.015 0.008];
LCLINEW = 1;
LINEW = 2;

% font = 'LM Roman';
font = 'Times New Roman';
fs = 24;

lstyle = 'none';
c = [ [0 0 0];  % color het+IN
    [0 0 1];    % color hom+IN
    [1 0 0];    % color het-IN
    [0 1 0]];   % color hom-IN

% ================================ plots ================================ %
close all;

jnode = 1:73;

fig = figure;
set( fig, 'position',[10 100 1350 835]);    % 1670

% -------------------------- plot bar t_j^grid -------------------------- %
axtg = axes;
p1 = plot(jnode(~tbjlim.het), tbjgrid.het(~tbjlim.het) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~tbjlim.hom), tbjgrid.hom(~tbjlim.hom) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~tbjlim.hetloc), tbjgrid.hetloc(~tbjlim.hetloc) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~tbjlim.homloc), tbjgrid.homloc(~tbjlim.homloc) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(tbjlim.het), tbjgrid.het(tbjlim.het) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(tbjlim.hom), tbjgrid.hom(tbjlim.hom) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(tbjlim.hetloc), tbjgrid.hetloc(tbjlim.hetloc) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(tbjlim.homloc), tbjgrid.homloc(tbjlim.homloc) , 'v',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_j^{\,\,\rm grid}$', ...
    'fontsize', fs, ...
    'interpreter', 'latex');
leg = legend([p1 p2 p3 p4]);
set( leg, 'fontsize', LEG_FS,...
    'position', LEG_POS);

set( axtg, 'position', [XOFF YOFF+FIGSY*2+YSPACE FIGSX_UP FIGSY],...
    'xlim', [0 73],...
    'ylim', [0 12],...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', [],...
    'ytick', 4:4:20, ...
    'fontsize', fs,...
    'ticklength', TICK_LEN);
axtg.XAxis.MinorTickValues = 0:73;
axtg.YAxis.MinorTickValues = 1:12;

tbo.siz = dissFS('fr');
tba = dissAlabel( fig, '(a)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[TB_XOFF -0.02]) 0.01 0.02] );


% -------------------------- plot bar t_j^node -------------------------- %
axtn = axes;

p2 = plot(jnode(~tbknlim.hom), tbknode.hom(~tbknlim.hom) , 's'); hold on;
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p4 = plot(jnode(~tbknlim.homloc), tbknode.homloc(~tbknlim.homloc) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(tbknlim.het), tbknode.het(tbknlim.het) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', 'none',...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(tbknlim.hetloc), tbknode.hetloc(tbknlim.hetloc) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', 'none',...
    'displayname','',...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
set( axtn, 'position', [XOFF YOFF+FIGSY+YSPACE FIGSX_UP FIGSY],...
    'xlim', [0 73],...
    'ylim', [0 10],...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'ytick', 2:2:8, ...
    'fontsize', fs,...
    'ticklength', TICK_LEN,...
    'box','on');
axtn.XAxis.MinorTickValues = 0:73;
axtn.YAxis.MinorTickValues = 1:10;

ylnode = ylabel('$\bar{t}_j^{\,\,\rm node}$', ...
    'fontsize', fs, ...
    'interpreter', 'latex');

tbb = dissAlabel( fig, '(b)', tbo );
tbapos = get( tba, 'position' );
set( tbb, 'position', tbapos-[0 FIGSY 0 0] );

xl = xlabel('$j$', ...
    'fontsize', fs, ...
    'interpreter', 'latex');
set( xl, 'position', [35.0687   -1.4890   -1.0000])

ylgp = get( ylgrid, 'position');
ylnp = get( ylnode, 'position');

set( ylnode, 'position', [-3 ylnp(2) ylnp(3)]);
set( ylgrid, 'position', [-3 ylnp(2) ylnp(3)]);

% ======== Correlations of return times in the Het +- IN models  ======== %
% ------------------------------ tb j grid ------------------------------ %

hlim = or( tbjlim.het, tbjlim.hetloc );
helim = tbjlim.het;
axes;

p = plot(tbjgrid.het(~hlim), tbjgrid.hetloc(~hlim), 'o'); hold on;
set( p, 'linewidth', LINEW,...
    'color', 'k');
p = plot(tbjgrid.het(hlim), tbjgrid.hetloc(hlim), 'o'); hold on;
set( p, 'linewidth', LINEW,...
    'color', 'k',...
    'markerfacecolor','k');

x = [1.5 9];
pf = polyfit( tbjgrid.het, tbjgrid.hetloc, 1);
p2 = plot( x, polyval(pf,x), 'k');
R = PCC(tbjgrid.het, tbjgrid.hetloc);
disp( ['PCC tbj grid = ' num2str(R) ]);

xlabel('$\bar{t}_j^{\,\,\rm grid}$  (Het+IN)', ...
    'fontsize', fs, ...
    'interpreter', 'latex');
ylabel('$\bar{t}_j^{\,\,\rm grid}$  (Het-IN)', ...
    'fontsize', fs, ...
    'interpreter', 'latex');

xtckl = {'2','','4','','6','','8',''};
ax = gca;
set( ax, 'position', [ XOFFLOW YOFF FIGSX_LOW FIGSY],...
    'fontsize', fs,...
    'ticklength', TICK_LEN*1.5,...
    'xminortick', 'on',...
    'yminortick', 'on',...
    'xtick', 2:9,...
    'xticklabels', xtckl,...
    'xlim', x,...
    'ylim', [2 7]);
ax.XAxis.MinorTickValues = x(1):0.1:x(2);
ax.YAxis.MinorTickValues = 2:0.5:7;

tbc = dissAlabel( fig, '(c)', tbo );
set( tbc, 'position', tbapos+[0 -2*FIGSY-YSPACE 0 0] );


% ------------------------------ tb k node ------------------------------ %

axes;

p = plot(tbknode.het, tbknode.hetloc, 'o'); hold on;
set( p, 'linewidth', LINEW,...
    'color', 'k');

xtckl = {'2','','4','','6','','8',''};
x = [2.5 9.5];
pf = polyfit( tbknode.het, tbknode.hetloc, 1);
p2 = plot( x, polyval(pf,x), 'k');
R = PCC(tbknode.het, tbknode.hetloc);
disp( ['PCC tbk node = ' num2str(R)] );

xlabel('$\bar{t}_j^{\,\,\rm node}$  (Het+IN)', ...
    'fontsize', fs, ...
    'interpreter', 'latex');
ylabel('$\bar{t}_j^{\,\,\rm node}$  (Het-IN)', ...
    'fontsize', fs, ...
    'interpreter', 'latex');

ax = gca;
set( ax, 'position', [ XOFFLOW+FIGSX_LOW+XSPACE YOFF FIGSX_LOW FIGSY],...
    'fontsize', fs,...
    'ticklength', TICK_LEN*1.5,...
    'xminortick', 'on',...
    'yminortick', 'on',...
    'xtick', 2:9,...
    'xticklabels', xtckl,...
    'xlim', x,...
    'ylim', [3.5 7]);
ax.XAxis.MinorTickValues = x(1):0.1:x(2);
ax.YAxis.MinorTickValues = 2:0.5:7;


tbd = dissAlabel( fig, '(d)', tbo );
tbcpos = get( tbc, 'position' );
set( tbd, 'position', tbcpos+[FIGSX_LOW+XOFFLOW 0 0 0] );


%% =============================== save  ================================ %
if( ~~save_fig ) 
    pth = 'figures/';
    saveas( fig, [pth 'tbjgrid.eps'], 'epsc');
end





