
%% topology correlations RTS 96
% for the RTS96 test grid topogogical properties were correlated with
% mean grid return times. This is done for the model
% levels heterogeneous+IN and het-IN.
%
% V1.0 mawolff@uos.de 15-Jul-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ======================== load and process data ======================== %
lfile = 'data/fig-3-9-HKtSM.mat';
S = load( lfile );

smho  = S.smho;
smhol = S.smhol;
smhe  = S.smhe;
smhel = S.smhel;

Kjho  = S.Kjho;
Kjhol = S.Kjhol;
Kjhe  = S.Kjhe;
Kjhel = S.Kjhel;

tbjgrid = S.tbjgrid;
tbknode = S.tbknode;

clear S;

%% ============================ preferences ============================= %
save_fig = false;           % if the figure should be saved to disk automatically
disp_PCC = false;           % if the pierson correl. coeff. are displayed

swcolor = true; % true = black and white, false = red (colored)

FORMAT  = 'hs';
fs      = 1.0*dissFS(FORMAT);   % fontsize of axis
tbfs    = fs;                   % textbox fontsize
ealpha  = 0.1;                  % alpha of edges in surf plot
% font    = 'LM Roman 10';      % set font style
font    = 'Times New Roman';    % set font style
ticklenx = [1 0.5]*0.04;        % tick length on x-axis for minor and major ticks
tickleny = [1 0.5]*0.05;        % tick length on y-axis for minor and major ticks

if( ~swcolor )
    mcolor = unicolors('dr');           % marker color
else
    mcolor = 0.45*[1 1 1];           % marker color black and white
end

pfxpIN = 10.^(-2:4);        % x values for the polyfit Het+IN
pfxmIN = 10.^(0:4);         % x values for the polyfit Het-IN

% ============================= koordinaten ============================= %
xoffs = 0.18;
xfig  = 0.38*2;
xbetw = 0.11;

yoffs = 0.17;
yfig  = 0.78;

tboxw = 0.05;    % width of textboxes
tboxh = 0.05;    % height of textboxes
tbnoff = 0.125; % negative y offset of textboxes

% ================================ plots ================================ %
% ------------------------------- Het-IN  ------------------------------- %
fig = figure;
% set( fig, 'position', [123 456 (dissFigSize(FORMAT)-[0 50] )] );
set( fig, 'position', [123 56 (dissFigSize(FORMAT)-[0 0] )] );

ax1 = axes;

plhel = semilogx(  Kjhel .* smhel.H, (tbjgrid.hetloc) );
set( plhel, 'marker', 'o', ...
    'markersize', 6, ...
    'markerfacecolor', mcolor,...
    'color', mcolor,...
    'linestyle', 'none', 'linewidth', 2);
dissXLabel('$H_j K_{j} $', FORMAT );
ylh1 = dissYLabel('$\bar{t}_j^{\rm{\  grid}}$', FORMAT );
hold on;

pfmIN =  polyfit( log10(Kjhel .* smhel.H), (tbjgrid.hetloc), 1);
semilogx( pfxmIN, polyval( pfmIN, log10(pfxmIN) ), 'k' );

set(ylh1, 'Rotation',0);
% ylhp = get(ylh,'position');
set(ylh1, 'Position',[0.25 5 0]);

set( ax1, ...
    'fontsize', fs, ...
    'xlim', [1e0 1e4], ...
    'ylim', [0 10], ...
    'position', [xoffs yoffs xfig yfig], ...
    'ticklength', tickleny, ...
    'tickdir','in', ...
    'fontname', font);

% --------------------------------- PCC --------------------------------- %
if( ~~disp_PCC )
    dispsn( 'PCC for Het-IN ', PCC( log10(Kjhel .* smhel.H), (tbjgrid.hetloc) ) );
end



% ------------------------------ textboxes ------------------------------ %
tb1 = tbox( [   xoffs+xfig/2-tboxw/2 yoffs+yfig-tbnoff ...   
    tboxw tboxh], 'Het$-$IN', tbfs );
set( tb1, 'fontname', font, 'interpreter', 'latex' );

tbo.siz = fs;
tbla = dissAlabel( fig, '(a)', tbo );
% tbposa = get( tbla, 'position');
% set( tbla, 'position', ...
%      [tbposa(1:2) 0.025 0.04] );


% ------------------------------- Het+IN  ------------------------------- %
fig2 = figure;
set( fig2, 'position', [123 56 (dissFigSize(FORMAT)-[0 0] )] );
ax2 = axes;

plhe = semilogx(  Kjhe .* smhe.H, (tbjgrid.het) );
set( plhe, 'marker', 'o', ...
    'markerfacecolor', mcolor,...
    'color', mcolor, ...
    'markersize', 6, ...
    'linestyle', 'none', 'linewidth', 2);
hold on;

dissXLabel('$H_j K_{j} $', FORMAT );
ylh2 = dissYLabel('$\bar{t}_j^{\rm{\  grid}}$', FORMAT );
set(ylh2, 'Rotation',0);
ylhp = get(ylh2,'position');
set(ylh2, 'Position',[0.0015 5 0]);

smallt = find( tbjgrid.het < 8 );       % return times considered in both regressions

pfpIN =  polyfit( log10(Kjhe .* smhe.H), (tbjgrid.het), 1); % regression of all times
semilogx( pfxpIN, polyval( pfpIN, log10(pfxpIN) ), 'k--' ); % plot regression

pfpINsm =  polyfit( log10(Kjhe(smallt) .* smhe.H(smallt)), ...  % regression of small
    (tbjgrid.het(smallt)), 1);                                  % times only
semilogx( pfxpIN, polyval( pfpINsm, log10(pfxpIN) ), 'k' ); % plot regression

set( ax2, ...
    'fontsize', fs, ...
    'xlim', [1e-2 1e4],...
    'ylim', [0 10], ...
    'position', [xoffs yoffs xfig yfig], ...
    'ticklength', tickleny,...
    'fontname', font);

% --------------------------------- PCC --------------------------------- %
if( ~~disp_PCC )
    dispsn( 'PCC for Het+IN ', PCC( log10(Kjhe .* smhe.H), ...
        (tbjgrid.het) ) );
    dispsn( 'PCC for Het+IN without biggest times ', ...
        PCC( log10( Kjhe(smallt) .* smhe.H(smallt) ), ...
        (tbjgrid.het(smallt) ) ));
end

% ------------------------------ textboxes ------------------------------ %
tb2 = tbox( [ xoffs+xfig*0.5+0*xbetw-tboxw/2 yoffs+yfig-tbnoff ...   
    tboxw tboxh], 'Het+IN', tbfs );
set( tb2, 'fontname', font, 'interpreter', 'latex' );

tblb = dissAlabel( fig2, '(b)', tbo );
% tbposa = get( tbla, 'position');
% set( tblb, 'position', ...
%      tbposa + [0.525 0 0 0] );


% ============================== save fig  ============================== %
if( ~~save_fig )
    pth = 'figures/';
    if( ~~swcolor )
        saveas(fig, [pth 'hktgrid-a-SW.eps'], 'epsc');
        saveas(fig2, [pth 'hktgrid-b-SW.eps'], 'epsc');
    else
        saveas(fig, [pth 'hktgrid-a.eps'], 'epsc');
        saveas(fig2, [pth 'hktgrid-b.eps'], 'epsc');
    end
end
