

%% t_grid vs. t_node in RTS-96
% figure for Dissertation
%
% V1.0 mawolff@uos.de 15-Jul-2020

%%
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

S = load( 'data/fig3-10-tgridtnode.mat' );
tbjg = S.tbjg;
tbjglim = S.tbjglim;
tbkn = S.tbkn;
tbknlim = S.tbknlim;
clear S;

%% ============================= settings  ============================== %
save_fig = false;

swcolor = true;

% ---------------------------- plot settings ---------------------------- %
FORMAT = 'hs';
fontnam = 'Times New Roman';
fs = dissFS(FORMAT);

MARKERSIZE = 6;
if( ~swcolor )
    MARKER_COLOR = unicolors('dr');     % marker color
else
    MARKER_COLOR = 0.45*[1 1 1];        % marker color black and white
end
LINEW = 1;

TICK_LEN = [0.025 0.012];

% ---------------------------- axis settings ---------------------------- %
XOFF = 0.15;
YOFF = 0.2;
SIZE_AX_X = 0.75;
SIZE_AX_Y = 0.75;

%% =============================== plot  ================================ %
close all;

fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT)]);

ax = axes;
p = plot( tbjg.het , tbkn.het , 'o') ; hold on;

set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', MARKER_COLOR, ...
    'color', MARKER_COLOR, ...
    'linewidth', LINEW);

pf = polyfit( tbjg.het , tbkn.het , 1);
x = [0 10];
p2 = plot( x, polyval(pf,x), 'k');

set( ax, 'fontsize', fs, ...
    'position', [XOFF YOFF SIZE_AX_X SIZE_AX_Y], ...
    'ticklength', TICK_LEN, ...
    'xtick', 0:2:10, ...
    'ytick', 0:2:10, ...
    'xgrid', 'on', 'ygrid', 'on', ...
    'fontname', fontnam );

ylh = dissYLabel('$\bar{t}_j^{\,\,\rm{ node}}$', FORMAT );
xlh = dissXLabel('$\bar{t}_j^{\,\,\rm{ grid}}$', FORMAT );
set( xlh, 'fontsize', fs*1.2 );
set( ylh, 'fontsize', fs*1.2 );


if( ~~save_fig ) 
    spth = 'figures/';
    if( ~~swcolor )
        saveas( fig, [spth 'tgridtnode-SW.eps'], 'epsc');
    else
        saveas( fig, [spth 'tgridtnode.eps'], 'epsc');
    end
end

