
%% dissertation fig 3.14 and 3.15 - c39
% case39 RMRT analysis results
%
% V1.0 mawolff@uos.de 06-Dec-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));

% ============================== settings  ============================== %
save_fig = false;
save_mat = false;

FORMAT   = 'fr';
fontsiz  = dissFS(FORMAT);
fontname = 'Times New Roman';

figs = dissFigSize(FORMAT);
figw = figs(1);
figh = figs(2)-150;

% ----------------------------- coordinates ----------------------------- %
X_OFFSET = 0.05;                % offset of x values from left border
Y_OFFSET = 0.05;                % offset of y values from down border

X_SPACE = 0.025;
X_WIDTH = (0.85 - X_SPACE)/2;
Y_WIDTH = 0.9;

xshift = X_WIDTH+X_SPACE;


LEG_POS = [0.44132 0.84444 0.1275 0.13867];
% LEG_POS = [0.26 0.846 0.1275 0.14];
% LEG_POS = [0.39194 0.69622 0.1275 0.27];
LEG_FS = fontsiz;

TB_XOFF = 0.0175;

% ---------------------- constants  ---------------------- %
Nn = 39;        % number of nodes
% N = 73;                         % number of nodes in the grid
% fsax = 22;                      % font size for axes
% fscb = 22;                      % font size for colorbar
% fscbh = 26;                     % font size colorbar header
% figs = (0.7/2);                 % figure size for return time surfaces

% CB_WIDTH = 0.035;               % how wide the color bar is
TICK_WIDTH = 1;                 % width of the tick strokes
TICK_LEN = [1 0.5]*0.04;        % length of the tick strokes
% MF_POS = [ 10 50 ...
%     dissFigSize(FORMAT)];     % position and size of the surface frame in the end
% ---------------------------- plot settings ---------------------------- %
MARKERSIZE = 10;
TICK_LEN = [0.015 0.008];
LCLINEW = 1;
LINEW = 2;

lstyle = 'none';
c = [ [0 0 0];  % color het+IN
    [0 0 1];    % color hom+IN
    [1 0 0];    % color het-IN
    [0 1 0]];   % color hom-IN

% FLAB = '$\mathcal{F}_j = (1-\mathcal{S}_j)$';
FLAB = '$\mathcal{F}_j$';
TGRID = '$\bar t_j^{\,\rm\scriptscriptstyle grid}$';

%% ============================= load data ============================== %

matName = 'fig-3-RtBsCase39+tk';
read_mat = false;
try 
    file = ['data/' matName '.mat'];
    L = load(file);
    tjgridHe        = L.tjgridHe;
    tjgridHel       = L.tjgridHel;
    tjgridHeSeq     = L.tjgridHeSeq;
    tjgridHelSeq    = L.tjgridHelSeq;
    gridfailHe      = L.gridfailHe;
    gridfailHel     = L.gridfailHel;
    gridfailHeSeq   = L.gridfailHeSeq;
    gridfailHelSeq  = L.gridfailHelSeq;
    tjgridHo        = L.tjgridHo;
    tjgridHol       = L.tjgridHol;
    tjgridHoSeq     = L.tjgridHoSeq;
    tjgridHolSeq    = L.tjgridHolSeq;
    gridfailHo      = L.gridfailHo;
    gridfailHol     = L.gridfailHol;
    gridfailHoSeq   = L.gridfailHoSeq;
    gridfailHolSeq  = L.gridfailHolSeq;
    
    tknodeHe        = L.tknodeHe;
    tknodeHel       = L.tknodeHel;
    tknodeHo        = L.tknodeHo;
    tknodeHol       = L.tknodeHol;
    
    xg = L.xg;
    yg = L.yg;
    Yhom = L.Yhom;
    tblposs = L.tblposs;
    tblpossRT = L.tblpossRT;
    clear L;
    read_mat = true;
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be find.']);  
        otherwise 
            rethrow( ME ); 
    end 
end

mpc = loadcase( case39 );
if( ~read_mat )
    disp('need to read from external');
    filen = 'rtsurfsCase39.mat';
    dpth = 'Dissertation/case39/';
    
% ========================== load graph coords ========================== %
    gpth = loadGraph('defpath');
    [xg,yg] = loadGraphXY( [gpth 'c39-PF.mat'] );
    
% ================================= Het+IN ============================== %
    fileHe = [dpth 'rrt03/' filen];
    load( fileHe, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHe = getAllMeanRT( tjgrid );
    tjgridHeSeq = glM2seqM( tjgridHe, mpc );
    tknodeHe = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHe = 1-bs(:,1);
    gridfailHeSeq = glM2seqM( gridfailHe, mpc );
    
% =============================== Het-IN ================================ %
    fileHel = [dpth 'rrt01/' filen];
    load( fileHel, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHel = getAllMeanRT( tjgrid );
    tjgridHelSeq = glM2seqM( tjgridHel, mpc );
    tknodeHel = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHel = 1-bs(:,1);
    gridfailHelSeq = glM2seqM( gridfailHel, mpc );
    
% =============================== Hom+IN ================================ %
    fileHo = [dpth 'rrt04/' filen];
    load( fileHo, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHo = getAllMeanRT( tjgrid );
    tjgridHoSeq = glM2seqM( tjgridHo, mpc );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHo = 1-bs(:,1);
    gridfailHoSeq = glM2seqM( gridfailHo, mpc );
    tknodeHo = getAllMeanRT( tknode );
    
% =============================== Hom-IN ================================ %
    fileHol = [dpth 'rrt02/' filen];
    load( fileHol, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHol = getAllMeanRT( tjgrid );
    tjgridHolSeq = glM2seqM( tjgridHol, mpc );
    tknodeHol = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHol = 1-bs(:,1);
    gridfailHolSeq = glM2seqM( gridfailHol, mpc );
    
% ============================== save mat  ============================== %
    if( ~~save_mat )
        file = ['data/' matName '.mat'];
        save( file, ...
            'tjgridHe'  , 'tjgridHel'  , 'tjgridHeSeq'  , 'tjgridHelSeq'  , ...
            'gridfailHe', 'gridfailHel', 'gridfailHeSeq', 'gridfailHelSeq', ...
            'tjgridHo'  , 'tjgridHol'  , 'tjgridHoSeq'  , 'tjgridHolSeq'  , ...
            'gridfailHo', 'gridfailHol', 'gridfailHoSeq', 'gridfailHolSeq', ...
            'tknodeHe'  , 'tknodeHel'  , 'tknodeHo'  , 'tknodeHol'  ,...
            'xg', 'yg', 'Yhom', 'tblposs', 'tblpossRT' ...
            );
    end
end


limche = logical( gridfailHe );
limchel = logical( gridfailHel );
limcho = logical( gridfailHo );
limchol = logical( gridfailHol );

%% ============================== figures =============================== %

% -------------------------- create het graph  -------------------------- %
graphHet = mpc2Graph( mpc );

% -------------------------- create hom graph  -------------------------- %
% hoo.method = 'rxb';
mpc_hom = homMPC( mpc );
graphHom = ybus2Graph( Yhom );
% Pmlhom = 1.7878;
% Pmghom = 5.1847;

loadscalef = @(x) (6 + abs(x)/1200*32);
genscalef = @(x) (6 + abs(x)/1200*32);


%% =========================== basin stability =========================== %
figBS = figure;
set( figBS, 'position', [123 56 figw figh]);

% ------------------------------- Het+IN  ------------------------------- %
axBSHet = axes;
set( axBSHet, 'position', [X_OFFSET Y_OFFSET X_WIDTH Y_WIDTH] );

grplBSHet = plot( graphHet );
set( grplBSHet, 'xdata', xg );
set( grplBSHet, 'ydata', yg );

set( axBSHet, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplBSHet, mpc, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:)-[xshift 0 0 0];
end

highlightEdges( grplBSHet, graphHet );

maxBS = 0.35;
stepBS = 0.05;
grplBSHet.NodeColor = vrggrad( maxBS-gridfailHeSeq, maxBS );
% grpl39.NodeColor = redToGreenGradient( maxBS-gridfailseq, maxBS );

% %%

set( axBSHet, 'clim', [0 (maxBS+stepBS)], 'visible', 'off' );
cm = vrggrad( 0:stepBS:maxBS, maxBS );
colormap( flip(cm) );
% colorbar;

alo.siz = fontsiz;
alab = dissAlabel( figBS, '(a)', alo );
% set( 

tbBShet = tbox( [(X_OFFSET+X_WIDTH/2) Y_WIDTH 0.05 0.05], ...
    'Het+IN', fontsiz );
set( tbBShet, 'fontname', fontname );

% ------------------------------- Hom+IN  ------------------------------- %
axBSHom = axes;
set( axBSHom, 'position', ...
    [X_OFFSET+X_WIDTH+X_SPACE Y_OFFSET X_WIDTH Y_WIDTH]);

grplBSHom = plot( graphHom );
set( grplBSHom, 'xdata', xg );
set( grplBSHom, 'ydata', yg );

set( axBSHom, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplBSHom, mpc_hom, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:);
end


heo.wfactor = 2.5;
highlightEdges( grplBSHom, graphHom, heo );

% maxBS = 0.25;
% stepBS = 0.05;
grplBSHom.NodeColor = vrggrad( maxBS-gridfailHoSeq, maxBS );
% grpl39.NodeColor = redToGreenGradient( maxBS-gridfailseq, maxBS );

% %%

set( axBSHom, 'clim', [0 (maxBS+stepBS)], 'visible', 'off' );
cm = vrggrad( 0:stepBS:maxBS, maxBS );
colormap( flip(cm) );

% ------------------------------ colorbar  ------------------------------ %
cb = colorbar;
set( cb, 'position', [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    Y_OFFSET 0.025 Y_WIDTH*0.9], ...
    'fontsize', fontsiz);

tbcb = tbox( [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    1.25*Y_OFFSET+Y_WIDTH*0.9 0.025 0.1*Y_WIDTH], FLAB, fontsiz );
set( tbcb, 'interpreter', 'latex' );


blo.siz = fontsiz;
blab = dissAlabel( figBS, '(b)', blo );
blpos = get( blab, 'position' );
set( blab, 'position', blpos + [(X_WIDTH+2*X_SPACE) 0 0 0] );

tbBShom = tbox( [(X_OFFSET+X_WIDTH/2+X_WIDTH+X_SPACE) Y_WIDTH 0.05 0.05], ...
    'Hom+IN', fontsiz );
set( tbBShom, 'fontname', fontname );

if( ~~save_fig )
    pth = 'figures/';
    saveas( figBS, [pth 'case39BS.eps'], 'epsc' );
end

%% move texboxlabels and save their position

% tblpossRT = zeros(Nn,4);
% 
% for i = 1:Nn
%     tblpossRT(i,:) = hlret.ntb(i).Position;
% end


%% ========================== return times Het  ========================== %
fig2 = figure;
set( fig2, 'position', [123 456 figw figh]);

% ------------------------------- Het-IN  ------------------------------- %
axRTHet = axes;
set( axRTHet, 'position', [X_OFFSET Y_OFFSET X_WIDTH Y_WIDTH] );

grplBSHet = plot( graphHet );
set( grplBSHet, 'xdata', xg );
set( grplBSHet, 'ydata', yg );

set( axRTHet, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplBSHet, mpc, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:)-[xshift 0 0 0];
end

highlightEdges( grplBSHet, graphHet );

maxRT = 11;
stepRT = 1;
grplBSHet.NodeColor = vrggrad( maxRT-tjgridHelSeq, maxRT );
% grpl39.NodeColor = redToGreenGradient( maxRT-gridfailseq, maxRT );

% %%

set( axRTHet, 'clim', [0 (maxRT+stepRT)], 'visible', 'off' );
cm = vrggrad( 0:stepRT:maxRT, maxRT );
colormap( flip(cm) );
% colorbar;

alo.siz = fontsiz;
alab = dissAlabel( fig2, '(c)', alo );
% set( 

tbRThel = tbox( [(X_OFFSET+X_WIDTH/2) Y_WIDTH 0.05 0.05], ...
    'Het-IN', fontsiz );
set( tbRThel, 'fontname', fontname );

% ------------------------------- Het+IN  ------------------------------- %
axBSHom = axes;
set( axBSHom, 'position', ...
    [X_OFFSET+X_WIDTH+X_SPACE Y_OFFSET X_WIDTH Y_WIDTH]);

grplBSHom = plot( graphHet );
set( grplBSHom, 'xdata', xg );
set( grplBSHom, 'ydata', yg );

set( axBSHom, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplBSHom, mpc, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:);
end

highlightEdges( grplBSHom, graphHet );

grplBSHom.NodeColor = vrggrad( maxRT-tjgridHeSeq, maxRT );
% grpl39.NodeColor = redToGreenGradient( maxRT-gridfailseq, maxRT );

% %%

set( axBSHom, 'clim', [0 (maxRT+stepRT)], 'visible', 'off' );
cm = vrggrad( 0:stepRT:maxRT, maxRT );
colormap( flip(cm) );

% ------------------------------ colorbar  ------------------------------ %
cb = colorbar;
set( cb, 'position', [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    Y_OFFSET 0.025 Y_WIDTH*0.9], ...
    'fontsize', fontsiz);
tbcb = tbox( [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    1.25*Y_OFFSET+Y_WIDTH*0.9 0.025 0.1*Y_WIDTH], TGRID, 1.2*fontsiz );
set( tbcb, 'interpreter', 'latex' );


blo.siz = fontsiz;
dlab = dissAlabel( fig2, '(d)', blo );
blpos = get( dlab, 'position' );
set( dlab, 'position', blpos + [(X_WIDTH+2*X_SPACE) 0 0 0] );

tbRThe = tbox( [(X_OFFSET+X_WIDTH/2+X_WIDTH+X_SPACE) Y_WIDTH 0.05 0.05], ...
    'Het+IN', fontsiz );
set( tbRThe, 'fontname', fontname );

if( ~~save_fig )
    pth = 'figures/';
    saveas( fig2, [pth 'case39RThet.eps'], 'epsc' );
end

%% ========================== return times Hom  ========================== %
fig3 = figure;
set( fig3, 'position', [123 456 figw figh]);

% ------------------------------- Hom-IN  ------------------------------- %
axRTHom = axes;
set( axRTHom, 'position', [X_OFFSET Y_OFFSET X_WIDTH Y_WIDTH] );

grplRTHoml = plot( graphHom );
set( grplRTHoml, 'xdata', xg );
set( grplRTHoml, 'ydata', yg );

set( axRTHom, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplRTHoml, mpc_hom, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:)-[xshift 0 0 0];
end

hleo.wfactor = 2.5; 
highlightEdges( grplRTHoml, graphHom, hleo );

maxRT = 11;
stepRT = 1;
grplRTHoml.NodeColor = vrggrad( maxRT-tjgridHolSeq, maxRT );
% grpl39.NodeColor = redToGreenGradient( maxRT-gridfailseq, maxRT );

% %%

set( axRTHom, 'clim', [0 (maxRT+stepRT)], 'visible', 'off' );
cm = vrggrad( 0:stepRT:maxRT, maxRT );
colormap( flip(cm) );
% colorbar;

alo.siz = fontsiz;
alab = dissAlabel( fig3, '(e)', alo );
% set( 

tbRThol = tbox( [(X_OFFSET+X_WIDTH/2) Y_WIDTH 0.05 0.05], ...
    'Hom-IN', fontsiz );
set( tbRThol, 'fontname', fontname );

% ------------------------------- Hom+IN  ------------------------------- %
axBSHom = axes;
set( axBSHom, 'position', ...
    [X_OFFSET+X_WIDTH+X_SPACE Y_OFFSET X_WIDTH Y_WIDTH]);

grplRTHoml = plot( graphHom );
set( grplRTHoml, 'xdata', xg );
set( grplRTHoml, 'ydata', yg );

set( axBSHom, 'xlim', [0 100], 'ylim', [0 100], ...
    'xtick', [], 'ytick', [] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.loadscalef = loadscalef;
nodeso.genscalef = genscalef;
nodeso.dualbus = 'effective';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplRTHoml, mpc_hom, nodeso );
delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblpossRT(i,:);
end


hleo.wfactor = 2.5; 
highlightEdges( grplRTHoml, graphHom, hleo );

grplRTHoml.NodeColor = vrggrad( maxRT-tjgridHoSeq, maxRT );
% grpl39.NodeColor = redToGreenGradient( maxRT-gridfailseq, maxRT );

% %%

set( axBSHom, 'clim', [0 (maxRT+stepRT)], 'visible', 'off' );
cm = vrggrad( 0:stepRT:maxRT, maxRT );
colormap( flip(cm) );

% ------------------------------ colorbar  ------------------------------ %
cb = colorbar;
set( cb, 'position', [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    Y_OFFSET 0.025 Y_WIDTH*0.9], ...
    'fontsize', fontsiz);
tbcb = tbox( [(X_OFFSET + 2*(X_WIDTH+X_SPACE)) ...
    1.25*Y_OFFSET+Y_WIDTH*0.9 0.025 0.1*Y_WIDTH], TGRID, 1.2*fontsiz );
set( tbcb, 'interpreter', 'latex' );

blo.siz = fontsiz;
flab = dissAlabel( fig3, '(f)', blo );
blpos = get( flab, 'position' );
set( flab, 'position', blpos + [(X_WIDTH+2*X_SPACE) 0 0 0] );

tbRTho = tbox( [(X_OFFSET+X_WIDTH/2+X_WIDTH+X_SPACE) Y_WIDTH 0.05 0.05], ...
    'Hom+IN', fontsiz );
set( tbRTho, 'fontname', fontname );

if( ~~save_fig )
    pth = 'figures/';
    saveas( fig3, [pth 'case39RThom.eps'], 'epsc' );
end


%% ====================== buses power flow level  ======================= %
figPFG = figure;
set( figPFG, 'position', [123 456 dissFigSize('hs')]);
axPFG = axes;
% set( 

% graphHet = mpc2Graph( mpc );

grplHet = plot( graphHet );
set( grplHet, 'xdata', xg );
set( grplHet, 'ydata', yg );

set( axPFG, 'xlim', [0 100], 'ylim', [0 100] );

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.dualbus = 'both';    
nodeso.model = 'pf';
nodeso.tboxlabel = true;
nodeso.tbshift = [0.01 -0.03];
hlret = highlightPGNodes( grplHet, mpc, nodeso );
% delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = (tblposs(i,:)-[0.5 0 0 0]).*[2 1 1 1]+[0.1 0 0 0];
end

highlightEdges( grplHet, graphHet );

set( axPFG, 'visible', 'off' );

if( ~~save_fig )
    pth = 'figures/';
    saveas( figPFG, [pth 'case39busGraph.eps'], 'epsc' );
end

%%

% ----------------------------- coordinates ----------------------------- %
XOFF = 0.075;
YOFF = 0.15;
XSPACE = 0.075;
% YSPACE = 0.12;
FIGSX = (1-XSPACE-2*XOFF);
FIGSY = 1-YOFF-0.075;

% ----

Y_LIM_RT = [0 12];

jnode = 1:Nn;

tknodeHeSeq  = glM2seqM(tknodeHe, mpc);
tknodeHelSeq = glM2seqM(tknodeHel, mpc);
tknodeHoSeq  = glM2seqM(tknodeHo, mpc);
tknodeHolSeq = glM2seqM(tknodeHol, mpc);

% limche  = logical( gridfailHeSeq );
% limchel = logical( gridfailHelSeq );
% limcho  = logical( gridfailHoSeq );
% limchol = logical( gridfailHolSeq );
limche  = logical( gridfailHe );
limchel = logical( gridfailHel );
limcho  = logical( gridfailHo );
limchol = logical( gridfailHol );

limchel = gridfailHel > 0.02;

fig = figure;
set( fig, 'position',[123 345 figw figh]);    %

% -------------------------- plot bar t_j^grid -------------------------- %
axtg = axes;
p1 = plot(jnode(~limche), tjgridHe(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tjgridHo(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tjgridHel(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tjgridHol(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tjgridHe(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tjgridHo(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tjgridHel(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tjgridHol(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_j^{\,\,\rm grid}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
% leg1 = legend([p1 p2]);
% set( leg1, 'fontsize', LEG_FS,...
%     'position', LEG_POS, ...
%     'fontname', fontname, ...
%     'edgecolor', 'none');

set( axtg, 'position', [(XOFF+XSPACE+FIGSX*0.4) (YOFF+FIGSY*0.5) ...
    FIGSX*0.6 (FIGSY*0.5)],...
    'xlim', [0 Nn+1],...
    'ylim', Y_LIM_RT,...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:5:50,...
    'xticklabels', {},...
    ...'ytick', 4:4:20, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtg.XAxis.MinorTickValues = 0:73;
axtg.YAxis.MinorTickValues = 1:12;

tbo.siz = dissFS('fr');
tba = dissAlabel( fig, '(a)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[0.01 -0.01]) 0.01 0.02] );

% -------------------------- plot bar t_k^node -------------------------- %
axtn = axes;
p1 = plot(jnode(~limche), tknodeHe(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tknodeHo(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tknodeHel(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tknodeHol(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tknodeHe(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tknodeHo(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tknodeHel(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tknodeHol(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_k^{\,\,\rm node}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
leg34 = legend([p1 p2 p3 p4]);
set( leg34, 'fontsize', 0.75*LEG_FS,...
...%     'position', LEG_POS+[LEG_POS(3) 0 0 0], ...
    'position', [0.85144 0.78186 0.10583 0.20778], ...
    'fontname', fontname, ...
    'edgecolor', 'none');

set( axtn, 'position', [(XOFF+XSPACE+FIGSX*0.4) YOFF ...
    FIGSX*0.6 FIGSY*0.5],...
    'xlim', [0 Nn+1],...
    'ylim', Y_LIM_RT,...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:5:60,...
    ...'ytick', 4:4:20, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtn.XAxis.MinorTickValues = 0:73;
axtn.YAxis.MinorTickValues = 1:12;

% tbo.siz = dissFS('fr');
% tba = dissAlabel( fig, '(a)', tbo );
% tbapos = get( tba, 'position' );
% set( tba, 'position', [(tbapos(1:2)+[0.01 -0.01]) 0.01 0.02] );


% %% graph plot
% ============================= graph plot  ============================= %
mpc = ext2int( mpc );

gpth = loadGraph('defpath');
[xn, yn] = loadGraphXY( [gpth 'c39-PF.mat'] );


gr39 = mpc2Graph( mpc );

% figure;
axgr = axes;
grpl39 = plot( gr39 );
set( grpl39, 'xdata', xn );
set( grpl39, 'ydata', yn );

set( axgr, 'xlim', [0 100], 'ylim', [0 100], ...
    'visible', 'off', ...
    'position', [(0.5*XOFF) YOFF*0.8 ...
    FIGSX*0.4 (FIGSY+YOFF*0.3)]);

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.tboxlabel = true;
nodeso.dualbus = 'both';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
hlret = highlightPGNodes( grpl39, mpc, nodeso );
% delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblposs(i,:);%-[(XSPACE+FIGSX*0.6-0.1) 0 0 0];
end

highlightEdges( grpl39, gr39 );

xg = get( grpl39, 'xdata' );
yg = get( grpl39, 'ydata' );

tbo.siz = dissFS('fr');
tbb = dissAlabel( fig, '(b)', tbo );
tbbpos = get( tbb, 'position' );
set( tbb, 'position', [(tbbpos(1:2)+[(XOFF+FIGSX*0.4) 0]) 0.01 0.02] );

if( ~~save_fig )
    pth = 'figures/';
    saveas( fig, [pth 'case39busGraphRT.eps'], 'epsc' );
end

%%
tblposs = zeros(Nn,4);
for i = 1:Nn
    tblposs(i,:) = hlret.ntb(i).Position;
end

% ============================== save mat  ============================== %
% save_mat = true;
save_mat = false;
if( ~~save_mat )
    matName = 'fig-3-RtBsCase39+tk';
    file = ['data/' matName '.mat'];
    save( file, ...
        'tjgridHe'  , 'tjgridHel'  , 'tjgridHeSeq'  , 'tjgridHelSeq'  , ...
        'gridfailHe', 'gridfailHel', 'gridfailHeSeq', 'gridfailHelSeq', ...
        'tjgridHo'  , 'tjgridHol'  , 'tjgridHoSeq'  , 'tjgridHolSeq'  , ...
        'gridfailHo', 'gridfailHol', 'gridfailHoSeq', 'gridfailHolSeq', ...
        'tknodeHe'  , 'tknodeHel'  , 'tknodeHo'  , 'tknodeHol'  ,...
        'xg', 'yg', 'Yhom', 'tblposs' ...
        );
end

%% ================= tjgrid and tknode for all models  ================== %
jnode = 1:Nn;

Y_LIM_RT = [0 12];

% ----------------------------- coordinates ----------------------------- %
XOFF = 0.075;
YOFF = 0.15;
XSPACE = 0.075;
% YSPACE = 0.12;
FIGSX = (1-XSPACE-2*XOFF)/2;
FIGSY = 1-YOFF-0.075;

LEG1POS = [0.09 0.7373 0.1275 0.23148];

tknodeHeSeq  = glM2seqM(tknodeHe, mpc);
tknodeHelSeq = glM2seqM(tknodeHel, mpc);
tknodeHoSeq  = glM2seqM(tknodeHo, mpc);
tknodeHolSeq = glM2seqM(tknodeHol, mpc);

limche  = logical( gridfailHeSeq );
limchel = logical( gridfailHelSeq );
limcho  = logical( gridfailHoSeq );
limchol = logical( gridfailHolSeq );

limchel = gridfailHelSeq > 0.02;

figRTl = figure;
set( figRTl, 'position',[123 345 figw 0.6*figh]);    %

% -------------------------- plot bar t_j^grid -------------------------- %
axtg = axes;
p1 = plot(jnode(~limche), tjgridHeSeq(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tjgridHoSeq(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tjgridHelSeq(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tjgridHolSeq(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tjgridHeSeq(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tjgridHoSeq(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tjgridHelSeq(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tjgridHolSeq(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_j^{\,\,\rm grid}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
leg1 = legend([p1 p2]);
set( leg1, 'fontsize', LEG_FS,...
    'position', LEG1POS, ...
    'fontname', fontname, ...
    'edgecolor', 'none');

set( axtg, 'position', [XOFF YOFF FIGSX FIGSY],...
    'xlim', [0 Nn+1],...
    'ylim', Y_LIM_RT,...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:5:40,...
    ...'ytick', 4:4:20, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtg.XAxis.MinorTickValues = 0:73;
axtg.YAxis.MinorTickValues = 1:12;

tbo.siz = dissFS('fr');
tba = dissAlabel( figRTl, '(g)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[0.01 -0.01]) 0.01 0.02] );

% -------------------------- plot bar t_k^node -------------------------- %
axtn = axes;
p1 = plot(jnode(~limche), tknodeHeSeq(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tknodeHoSeq(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tknodeHelSeq(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tknodeHolSeq(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tknodeHeSeq(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tknodeHoSeq(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tknodeHelSeq(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tknodeHolSeq(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

tbo.siz = dissFS('fr');
tba = dissAlabel( figRTl, '(h)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[0.01+FIGSX+XSPACE*1.2 -0.01]) 0.01 0.02] );

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_k^{\,\,\rm node}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
leg34 = legend([p3 p4]);
set( leg34, 'fontsize', LEG_FS,...
    'position', LEG1POS+[FIGSX+XSPACE 0 0 0], ...
    'fontname', fontname, ...
    'edgecolor', 'none');

set( axtn, 'position', [XOFF+XSPACE+FIGSX YOFF FIGSX FIGSY],...
    'xlim', [0 Nn+1],...
    'ylim', Y_LIM_RT,...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:5:40,...
    ...'ytick', 4:4:20, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtn.XAxis.MinorTickValues = 0:73;
axtn.YAxis.MinorTickValues = 1:12;

if( ~~save_fig )
    pth = 'figures/';
    saveas( figRTl, [pth 'case39RTlist.eps'], 'epsc' );
end





