
%% dissertation fig 3.16 - c57
% case57 RMRT analysis results
%
% V1.0 mawolff@uos.de 06-Dec-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));

% ============================== settings  ============================== %
save_fig = false;
save_mat = false;

FORMAT = 'fr';
fontsiz = dissFS(FORMAT);
fontname = 'Times New Roman';

figs = dissFigSize(FORMAT);
figw = figs(1);
figh = figs(2)-150;

% ----------------------------- coordinates ----------------------------- %
XOFF = 0.075;
YOFF = 0.1;
XSPACE = 0.01;
% YSPACE = 0.12;
FIGSX = (1-XSPACE-XOFF*1.2);
FIGSY = 0.85;

XGROFF = -0.03;

LEG_POS = [0.7 0.85278 0.1275 0.14];
LEG_FS = fontsiz;

TB_XOFF = 0.0175;

% ---------------------- constants  ---------------------- %
Nn = 57;        % number of nodes
% N = 73;                         % number of nodes in the grid
% fsax = 22;                      % font size for axes
% fscb = 22;                      % font size for colorbar
% fscbh = 26;                     % font size colorbar header
% figs = (0.7/2);                 % figure size for return time surfaces

% CB_WIDTH = 0.035;               % how wide the color bar is
TICK_WIDTH = 1;                 % width of the tick strokes
TICK_LEN = [1 0.5]*0.04;        % length of the tick strokes
% MF_POS = [ 10 50 ...
%     dissFigSize(FORMAT)];     % position and size of the surface frame in the end
% ---------------------------- plot settings ---------------------------- %
MARKERSIZE = 10;
TICK_LEN = [0.015 0.008];
LCLINEW = 1;
LINEW = 2;

% font = 'LM Roman';
% font = 'Times New Roman';
% fs = 24;

lstyle = 'none';
c = [ [0 0 0];  % color het+IN
    [0 0 1];    % color hom+IN
    [1 0 0];    % color het-IN
    [0 1 0]];   % color hom-IN

% LCVAL = 15;                     % limit cycle value
% LCVALLOG = 5;

% CB_LAST_TICK = 'LC';

% jnode = 9;

% X_LABEL = '$\Delta\varphi$';
%% ============================= load data ============================== %

matName = 'fig-3-RtBsCase57+tk';
read_mat = false;
try 
    file = ['data/' matName '.mat'];
    L = load(file);
    tjgridHe        = L.tjgridHe;
    tjgridHel       = L.tjgridHel;
    tjgridHeSeq     = L.tjgridHeSeq;
    tjgridHelSeq    = L.tjgridHelSeq;
    gridfailHe      = L.gridfailHe;
    gridfailHel     = L.gridfailHel;
    gridfailHeSeq   = L.gridfailHeSeq;
    gridfailHelSeq  = L.gridfailHelSeq;
    tjgridHo        = L.tjgridHo;
    tjgridHol       = L.tjgridHol;
    tjgridHoSeq     = L.tjgridHoSeq;
    tjgridHolSeq    = L.tjgridHolSeq;
    gridfailHo      = L.gridfailHo;
    gridfailHol     = L.gridfailHol;
    gridfailHoSeq   = L.gridfailHoSeq;
    gridfailHolSeq  = L.gridfailHolSeq;
    
    tknodeHe        = L.tknodeHe;
    tknodeHel       = L.tknodeHel;
    tknodeHo        = L.tknodeHo;
    tknodeHol       = L.tknodeHol;
%     xg = L.xg;
%     yg = L.yg;
%     Yhom = L.Yhom;
    tblposs = L.tblposs;
    clear L;
    read_mat = true;
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be find.']);  
        otherwise 
            rethrow( ME ); 
    end 
end

mpc = loadcase( case57 );
mpc = runpf( mpc );
if( ~read_mat )
    disp('need to read from external');
    filen = 'rtsurfsCase57.mat';
    dpth = 'Dissertation/case57/';
    
% ========================== load graph coords ========================== %
%     gpth = [loadGraph('defpath') 'c57/'];
%     [xg,yg] = loadGraphXY( [gpth 'c57-PF.mat'] );
    
% ================================= Het+IN ============================== %
    fileHe = [dpth 'rrt03/' filen];
    load( fileHe, 'surfs', 'tjgrid', 'tknode', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHe = getAllMeanRT( tjgrid );
    tjgridHeSeq = glM2seqM( tjgridHe, mpc );
    tknodeHe = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHe = 1-bs(:,1);
    gridfailHeSeq = glM2seqM( gridfailHe, mpc );
    
% =============================== Het-IN ================================ %
    fileHel = [dpth 'rrt01/' filen];
    load( fileHel, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHel = getAllMeanRT( tjgrid );
    tjgridHelSeq = glM2seqM( tjgridHel, mpc );
    tknodeHel = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHel = 1-bs(:,1);
    gridfailHelSeq = glM2seqM( gridfailHel, mpc );
    
% =============================== Hom+IN ================================ %
    fileHo = [dpth 'rrt04/' filen];
    load( fileHo, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHo = getAllMeanRT( tjgrid );
    tjgridHoSeq = glM2seqM( tjgridHo, mpc );
    tknodeHo = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHo = 1-bs(:,1);
    gridfailHoSeq = glM2seqM( gridfailHo, mpc );
    
% =============================== Hom-IN ================================ %
    fileHol = [dpth 'rrt02/' filen];
    load( fileHol, 'surfs', 'tjgrid', 'tknode' );
    
% -------------------------- get return times  -------------------------- %
    tjgridHol = getAllMeanRT( tjgrid );
    tjgridHolSeq = glM2seqM( tjgridHol, mpc );
    tknodeHol = getAllMeanRT( tknode );
    
% ------------------------- get basin stability ------------------------- %
    bso.iseq = 1:Nn;
    bso.jseq = 1;
    bs = basinStab( surfs, bso );
    gridfailHol = 1-bs(:,1);
    gridfailHolSeq = glM2seqM( gridfailHol, mpc );
    
% ============================== save mat  ============================== %
    if( ~~save_mat )
        file = ['data/' matName '.mat'];
        save( file, ...
            'tjgridHe'  , 'tjgridHel'  , 'tjgridHeSeq'  , 'tjgridHelSeq'  , ...
            'gridfailHe', 'gridfailHel', 'gridfailHeSeq', 'gridfailHelSeq', ...
            'tjgridHo'  , 'tjgridHol'  , 'tjgridHoSeq'  , 'tjgridHolSeq'  , ...
            'gridfailHo', 'gridfailHol', 'gridfailHoSeq', 'gridfailHolSeq', ...
            'tknodeHe'  , 'tknodeHel'  , 'tknodeHo'  , 'tknodeHol'  ,...
            'tblposs' ...
...%             'xg', 'yg', 'Yhom',  ...
            );
    end
end
% %%

limche = logical( gridfailHe );
limchel = logical( gridfailHel );
limcho = logical( gridfailHo );
limchol = logical( gridfailHol );


%% graph plot

jnode = 1:Nn;
fig = figure;
set( fig, 'position',[123 56 figw figh]);    %

% ============================= graph plot  ============================= %

mpc = ext2int( mpc );

gpth = loadGraph('defpath');
[xn, yn] = loadGraphXY( [gpth 'c57-PF.mat'] );


gr57 = mpc2Graph( mpc );

% figure;
axgr = axes;
grpl57 = plot( gr57 );
set( grpl57, 'xdata', xn );
set( grpl57, 'ydata', yn );

set( axgr, 'xlim', [0 100], 'ylim', [0 100], ...
    ...'visible', 'off', ...
    'position', [0.5*XOFF+XGROFF YOFF*0.8 ...
    FIGSX*0.4 (FIGSY+YOFF*0.3)]);

clear nodeso;
% nodeso.genscale = 'power';
% nodeso.loadscale = 'power';
nodeso.tboxlabel = true;
nodeso.dualbus = 'both';    
nodeso.model = 'pf';
nodeso.numbering = 'gl';
hlret = highlightPGNodes( grpl57, mpc, nodeso );
% delete( hlret.bc );
for i = 1:Nn
    hlret.ntb(i).Position = tblposs(i,:)+[0 0 0 0];
end

highlightEdges( grpl57, gr57 );

xg = get( grpl57, 'xdata' );
yg = get( grpl57, 'ydata' );

set( axgr, 'visible', 'off' );

% %%
% -------------------------- plot bar t_j^grid -------------------------- %
axtg = axes;
p1 = plot(jnode(~limche), tjgridHe(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tjgridHo(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tjgridHel(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tjgridHol(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tjgridHe(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tjgridHo(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tjgridHel(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tjgridHol(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_j^{\,\,\rm grid}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
leg1 = legend([p1 p2]);
set( leg1, 'fontsize', LEG_FS,...
    'position', LEG_POS, ...
    'fontname', fontname, ...
    'edgecolor', 'none');

set( axtg, 'position', [XOFF+XSPACE+0.4*FIGSX (YOFF+FIGSY*0.5) (0.6*FIGSX) (FIGSY*0.5)],...
    'xlim', [0 Nn+1],...
    'ylim', [0 6],...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:10:60,...
    'xticklabels', {},...
    'ytick', 0:6, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtg.XAxis.MinorTickValues = 0:73;
axtg.YAxis.MinorTickValues = 1:12;

tbo.siz = dissFS('fr');
tba = dissAlabel( fig, '(a)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[0.01 -0.01]) 0.01 0.02] );


% -------------------------- plot bar t_k^node -------------------------- %
axtn = axes;
p1 = plot(jnode(~limche), tknodeHe(~limche) , 'o'); hold on;
set( p1, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(1,:),...
    'displayname', 'Het+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p2 = plot(jnode(~limcho), tknodeHo(~limcho) , 's');
set( p2, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(2,:),...
    'displayname', 'Hom+IN',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p3 = plot(jnode(~limchel), tknodeHel(~limchel) , 'd',...
    'linestyle', lstyle);
set( p3, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(3,:),...
    'displayname','Het-IN',...
    'linewidth', LINEW);
p4 = plot(jnode(~limchol), tknodeHol(~limchol) , 'v',...
    'linestyle', lstyle);
set( p4, 'markersize', MARKERSIZE, ...
    'markerfacecolor', 'none',...
    'color', c(4,:),...
    'displayname','Hom-IN',...
    'color', c(4,:),...
    'linewidth', LINEW);

p = plot(jnode(limche), tknodeHe(limche) , 'o'); hold on;
set( p, 'markersize', MARKERSIZE, ...
    'color', c(1,:), ...
    'markerfacecolor', c(1,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LCLINEW);
p = plot(jnode(limcho), tknodeHo(limcho) , 's');
set( p, 'markersize', MARKERSIZE, ...
    'color', c(2,:), ...
    'markerfacecolor', c(2,:),...
    'displayname', '',...
    'linestyle', lstyle,...
    'linewidth', LINEW);
p = plot(jnode(limchel), tknodeHel(limchel) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'color', c(3,:), ...
    'markerfacecolor', c(3,:),...
    'displayname','',...
    'linewidth', LCLINEW);
p = plot(jnode(limchol), tknodeHol(limchol) , 'd',...
    'linestyle', lstyle);
set( p, 'markersize', MARKERSIZE, ...
    'markerfacecolor', c(4,:),...
    'displayname','',...
    'color', c(4,:),...
    'linewidth', LINEW);

% ---------------------------- axis settings ---------------------------- %
ylgrid = ylabel('$\bar{t}_k^{\,\,\rm node}$', ...
    'fontsize', fontsiz, ...
    'interpreter', 'latex');
leg34 = legend([p3 p4]);
set( leg34, 'fontsize', LEG_FS,...
    'position', LEG_POS+[LEG_POS(3) 0 0 0], ...
    'fontname', fontname, ...
    'edgecolor', 'none');

set( axtn, 'position', [XOFF+XSPACE+0.4*FIGSX YOFF 0.6*FIGSX FIGSY*0.5],...
    'xlim', [0 Nn+1],...
    'ylim', [0 6],...
    'xgrid', 'on', 'ygrid', 'on',...
    'xminortick', 'on',...
    'yminortick', 'on' ,...
    'xminorgrid', 'on' ,...
    'yminorgrid', 'on' ,...
    'xtick', 0:10:60,...
    'ytick', 1:5, ...
    'fontsize', fontsiz,...
    'ticklength', TICK_LEN);
axtn.XAxis.MinorTickValues = 0:73;
axtn.YAxis.MinorTickValues = 1:12;

tbo.siz = dissFS('fr');
tba = dissAlabel( fig, '(b)', tbo );
tbapos = get( tba, 'position' );
set( tba, 'position', [(tbapos(1:2)+[0.01+0.375 -0.01]) 0.01 0.02] );

if( ~~save_fig )
    pth = 'figures/';
    saveas( fig, [pth 'case57RT.eps'], 'epsc' );
end

