
%% correlation of return times and gridfail vs mech power
% figure 3.17 of dissertation
%
% V1.0 mawolff@uos.de 10-Dec-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
% Path to pg_dyn_models 
% [Nishikawa and Motter, New J. Phys. 17, 015012 (2015) ]
% needed:
addpath('C:\Users\Anwender\Documents\MATLAB\pg_dyn_models');

save_mat = false;
% save_fig = true;
save_fig = false;

MARKERSIZE = 7;

FORMAT = 'hr';
fontsiz = dissFS(FORMAT);
fontname = 'Times New Roman';

ALABOFF = [0.01 -0.01 0 0];     % offset for adisslabel

ticklenx = [1 0.5]*0.04;    % tick length on x-axis for minor and major ticks
tickleny = [1 0.5]*0.025;    % tick length on y-axis for minor and major ticks

COLOR_RTS = vrggrad(9,10);
COLOR_C30 = vrggrad(1,10);
COLOR_C39 = vrggrad(3,10);
COLOR_C57 = vrggrad(5,11);
COLOR_C89 = vrggrad(7,10);

TGRID = '$\bar t_j^{\,\rm\scriptscriptstyle grid}$ [s]';

RTS_LEG = 'RTS-96';

X_LABEL = '$P^{\rm (m)}_j$ [100MW]';

%% ============================= load data ============================== %
mat_read = false;

try 
    matName = 'RTBSallCases';
    file = [ 'data/' matName '.mat'];
    L = load(file);
    tbjgrid = L.tbjgrid;
    gridfail = L.gridfail;
    
    mat_read = true;
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be find.']);  
        otherwise 
            rethrow( ME ); 
    end 
end

if( ~mat_read )
    [tbjgrid, gridfail] = loadResultsMPCases( );
    
% ============================== save mat  ============================== %
    if( ~~save_mat )
        file = ['data/' matName '.mat'];
        save( file, ...
            'tbjgrid'  , 'gridfail'  ...
            );
    end
    
end

%% ======================= get SM_model structs  ======================== %
mpc89 = loadcase( case89pegase );
mpc89 = ext2int( mpc89 );

mpc96 = load( 'data/RTS96' );
mpc96 = mpc96.mpc;
mpc96 = ext2int( mpc96 );

mpc30 = loadcase( case30 );
mpc30 = ext2int( mpc30 );

mpc39 = loadcase( case39 );
mpc39 = ext2int( mpc39 );

mpc57 = loadcase( case57 );
mpc57 = ext2int( mpc57 );

rso.dynmodel = 'sm';
rso.hom = false;
% ------------------------------- Het+IN  ------------------------------- %
rso.IN = true;
rso.redpass = false;

sm89He = raw2swing( mpc89, rso );
sm96He = raw2swing( mpc96, rso );
sm30He = raw2swing( mpc30, rso );
sm39He = raw2swing( mpc39, rso );
sm57He = raw2swing( mpc57, rso );

% ------------------------------- Het-IN  ------------------------------- %
rso.IN = false;

sm89Hel = raw2swing( mpc89, rso );
sm96Hel = raw2swing( mpc96, rso );
sm30Hel = raw2swing( mpc30, rso );
sm39Hel = raw2swing( mpc39, rso );
sm57Hel = raw2swing( mpc57, rso );

rso.hom = true;
% ------------------------------- Hom+IN  ------------------------------- %
rso.IN = true;

sm89Ho = raw2swing( mpc89, rso );
sm96Ho = raw2swing( mpc96, rso );
sm30Ho = raw2swing( mpc30, rso );
sm39Ho = raw2swing( mpc39, rso );
sm57Ho = raw2swing( mpc57, rso );

% ------------------------------- Hom-IN  ------------------------------- %
rso.IN = false;

sm89Hol = raw2swing( mpc89, rso );
sm96Hol = raw2swing( mpc96, rso );
sm30Hol = raw2swing( mpc30, rso );
sm39Hol = raw2swing( mpc39, rso );
sm57Hol = raw2swing( mpc57, rso );



%% gridfail vs power

X_LIM = [-10 15];

figHe = figure;
ax = axes; hold on;
plot( (sm89He.Pm), gridfail.c89He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C89, ...
    'color', 'none', ...
    'displayname', 'case 89');

plot( (sm57He.Pm), gridfail.c57He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C57, ...
    'color', 'none', ...
    'displayname', 'case 57');
plot( (sm39He.Pm), gridfail.c39He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C39, ...
    'color', 'none', ...
    'displayname', 'case 39');
plot( (sm30He.Pm), gridfail.c30He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C30, ...
    'color', 'none', ...
    'displayname', 'case 30');

plot( (sm96He.Pm), gridfail.rtsHe, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_RTS, ...
    'color', 'none', ...
    'displayname', RTS_LEG);

% ---------------------------- axis settings ---------------------------- %
set( ax, 'fontsize', 0.75*fontsiz, ...
    'fontname', fontname, ...
    'ticklength', tickleny, ...
    'xgrid', 'on', ...
    'xminorgrid', 'on', ...
    'ygrid', 'on');

xlh = dissXLabel(X_LABEL, FORMAT);
ylh= dissYLabel('$\mathcal{F}_j = (1-\mathcal{S}_j)$', FORMAT);
ax.XAxis.MinorTickValues = X_LIM(1):X_LIM(2);

alo.siz = fontsiz;
alh = dissAlabel( figHe, '(c)', alo );
apos = get( alh, 'position' );
set( alh, 'position', apos+ALABOFF);

tb = tbox( [0.55 0.9 0.1 0.1], 'Het+IN', 16 );
set( tb, 'fontname', fontname);

% ------------------------------- legend  ------------------------------- %
leg = legend( 'show' );
set( leg, 'location', 'north', ...
    'fontsize', 0.75*fontsiz, ...
    'position', [0.25387 0.67817 0.23036 0.3], ...
    'fontname', fontname);


% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    spth = 'figures/';
    saveas( figHe,  [spth 'gridfailVsPmHet+IN.eps'], 'epsc' );
end

%%
figHel = figure;
ax = axes; hold on;
plot( (sm89Hel.Pm), gridfail.c89Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C89, ...
    'color', 'none', ...
    'displayname', 'case 89');

plot( (sm57Hel.Pm), gridfail.c57Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C57, ...
    'color', 'none', ...
    'displayname', 'case 57');
plot( (sm39Hel.Pm), gridfail.c39Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C39, ...
    'color', 'none', ...
    'displayname', 'case 39');
plot( (sm30Hel.Pm), gridfail.c30Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C30, ...
    'color', 'none', ...
    'displayname', 'case 30');

plot( (sm96Hel.Pm), gridfail.rtsHel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', vrggrad(9,10), ...
    'color', 'none', ...
    'displayname', RTS_LEG);


% ---------------------------- axis settings ---------------------------- %
set( ax, 'fontsize', 0.75*fontsiz, ...
    'fontname', fontname, ...
    'ticklength', tickleny, ...
    'ytick', 0:0.1:0.3,...
    'xgrid', 'on', ...
    'xminorgrid', 'on', ...
    'ygrid', 'on');

xlh = dissXLabel(X_LABEL, FORMAT);
ylh = dissYLabel('$\mathcal{F}_j = (1-\mathcal{S}_j)$', FORMAT);
ax.XAxis.MinorTickValues = X_LIM(1):X_LIM(2);

alo.siz = fontsiz;
blh = dissAlabel( figHel, '(d)', alo );
bpos = get( blh, 'position' );
set( blh, 'position', bpos+ALABOFF);

tb = tbox( [0.45 0.9 0.1 0.1], 'Het-IN', 16 );
set( tb, 'fontname', fontname);

% ------------------------------- legend  ------------------------------- %
leg = legend( 'show' );
set( leg, 'location', 'north', ...
    'fontsize', 0.75*fontsiz, ...
    'position', [0.66637 0.66864 0.23036 0.3], ...
    'fontname', fontname);



% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    spth = 'figures/';
    saveas( figHel, [spth 'gridfailVsPmHet-IN.eps'], 'epsc' );
end

%% return time vs power
figHe = figure;
ax = axes; hold on;
plot( (sm89He.Pm), tbjgrid.c89He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C89, ...
    'color', 'none', ...
    'displayname', 'case 89');

plot( (sm57He.Pm), tbjgrid.c57He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C57, ...
    'color', 'none', ...
    'displayname', 'case 57');
plot( (sm39He.Pm), tbjgrid.c39He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C39, ...
    'color', 'none', ...
    'displayname', 'case 39');
plot( (sm30He.Pm), tbjgrid.c30He, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C30, ...
    'color', 'none', ...
    'displayname', 'case 30');

%   abs
plot( (sm96He.Pm), tbjgrid.rtsHe, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', vrggrad(9,10), ...
    'color', 'none', ...
    'displayname', RTS_LEG);


% ---------------------------- axis settings ---------------------------- %
set( ax, 'fontsize', 0.75*fontsiz, ...
    'fontname', fontname, ...
    'ticklength', tickleny, ...
    'xgrid', 'on', ...
    'xminorgrid', 'on', ...
    'ygrid', 'on');

xlh = dissXLabel(X_LABEL, FORMAT);
ylh= dissYLabel(TGRID, FORMAT);
ax.XAxis.MinorTickValues = X_LIM(1):X_LIM(2);

alo.siz = fontsiz;
clh = dissAlabel( figHe, '(a)', alo );
cpos = get( clh, 'position' );
set( clh, 'position', cpos+ALABOFF);

tb = tbox( [0.45 0.9 0.1 0.1], 'Het+IN', 16 );
set( tb, 'fontname', fontname);

% ------------------------------- legend  ------------------------------- %
leg = legend( 'show' );
set( leg, 'location', 'north', ...
    'fontsize', 0.75*fontsiz, ...
    'position', [0.16994 0.67103 0.23036 0.3], ...
    'fontname', fontname);


% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    spth = 'figures/';
    saveas( figHe,  [spth 'tgridVsPmHet+IN.eps'], 'epsc' );
end

% xlabel('P^{(m)}');
% ylabel('t^{grid}');
% 
% leg = legend( 'show' );
% set( leg, 'location', 'northwest' );

%%
figHel = figure;
ax = axes; hold on;
plot( (sm89Hel.Pm), tbjgrid.c89Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C89, ...
    'color', 'none', ...
    'displayname', 'case 89');

plot( (sm57Hel.Pm), tbjgrid.c57Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C57, ...
    'color', 'none', ...
    'displayname', 'case 57');
plot( (sm39Hel.Pm), tbjgrid.c39Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C39, ...
    'color', 'none', ...
    'displayname', 'case 39');
plot( (sm30Hel.Pm), tbjgrid.c30Hel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', COLOR_C30, ...
    'color', 'none', ...
    'displayname', 'case 30');

%   abs
plot( (sm96Hel.Pm), tbjgrid.rtsHel, 'o', ...
    'markersize', MARKERSIZE, 'linewidth', 2, ...
    'markerfacecolor', vrggrad(9,10), ...
    'color', 'none', ...
    'displayname', RTS_LEG);


% ---------------------------- axis settings ---------------------------- %
set( ax, 'fontsize', 0.75*fontsiz, ...
    'fontname', fontname, ...
    'ticklength', tickleny, ...
    'xgrid', 'on', ...
    'xminorgrid', 'on', ...
    'ygrid', 'on');

xlh = dissXLabel(X_LABEL, FORMAT);
ylh= dissYLabel(TGRID, FORMAT);
ax.XAxis.MinorTickValues = X_LIM(1):X_LIM(2);

alo.siz = fontsiz;
dlh = dissAlabel( figHel, '(b)', alo );
dpos = get( dlh, 'position' );
set( dlh, 'position', dpos+ALABOFF);

tb = tbox( [0.45 0.9 0.1 0.1], 'Het-IN', 16 );
set( tb, 'fontname', fontname);

% ------------------------------- legend  ------------------------------- %
leg = legend( 'show' );
set( leg, 'location', 'north', ...
    'fontsize', 0.75*fontsiz, ...
    'position', [0.16994 0.67103 0.23036 0.3], ...
    'fontname', fontname);


% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    spth = 'figures/';
    saveas( figHel,  [spth 'tgridVsPmHet-IN.eps'], 'epsc' );
end
