
%% figure 4.3 in dissertation
% shows a part of the fino data which is colored according to its mean wind
% velocity. Also separations of these one minute intervals are shown as
% vertical dashed lines
%
% V1.0 mawolff@uos.de 04-Aug-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

save_fig = false;

dashed_lines_h = false;
dashed_lines_v = true;

YLIM = [6 16];
TICK_LEN = [0.02 0.005];

FORMAT = 'hr';
fs = dissFS(FORMAT);                % fontsize
fontn = 'Times New Roman';  % font name

dpth = 'data/'; % path to data
spth = 'figures/';        % path to store figures

S = load( [dpth 'fig-4-3-windseries.mat'] );
wind = S.wind;
clear S;

partlen = 61;   % number of seconds per part
                % to have N seconds, N+1 data points (separators) are needed
exparts = 6;    % number of parts as example
t0 = 67056;
t1 = 67422;

xticks = 0:partlen:partlen*exparts;
xtickLabels = cell(numel(xticks),1);
for i = 1:numel(xticks)
    xtickLabels{i} = num2str( (i-1)*60 );
end

colors = [ vrggrad(2,10);
    vrggrad(4,10);
    vrggrad(9,10); ];

fig = figure;
set( fig, 'position', [542 559 dissFigSize(FORMAT).*[1.5 1]] );

for i = 1:exparts                   % for each interval/part of wind series that is plotted
    ta = t0 + (i-1)*partlen;        % first time of interval in the complete data set
    te = ta + partlen;              % last time of interval in the complete data set
    wpart = wind( ta:te );          % get the wind part from the complete data set
    tx = (i-1)*partlen:i*partlen;   % create values for the x-axis that start at 0
    
    me = mean( wpart);
    
    ci = 1;
    for c = 1:10
        if( 2*c < me && me < 2*(c+1) )
            ci = c;
        end
    end
    
    if( ~~dashed_lines_v )
        plot( [1 1]*tx(end), YLIM, 'k--' );
        hold on;
    end
    plot( tx, wpart, ...
        'color', vrggrad(ci,10), ...
        'linewidth', 1.5);
    hold on;
    plot( [tx(1) tx(end)], [1 1]*me,...
        'color', vrggrad(ci,10) );
end

if( ~~dashed_lines_h )
    for i = 4:7
        plot( [0 tx(end)], [1 1]*2*i, 'k--', ...
            'color', vrggrad(i,10) );
    end
end

set( gca, ...
    'fontsize', fs, ...
    'fontname', fontn, ...
    'xlim', [0 tx(end)+1], ...
    'ylim', YLIM, ...
    'xtick', xticks, ...
    'xticklabels', xtickLabels, ...
    'ygrid', 'off', ...
    'ticklength', TICK_LEN);

ylh = dissYLabel('$v$ [m/s]', FORMAT );
xlh = dissXLabel('$t$ [s]', FORMAT );

if( ~~save_fig )
    saveas( fig, [spth 'windparts.eps'], 'epsc' );
end




