
%% wind turbine power-curve
% figure 4.4 of dissertation
%
% V1.0 mawolff@uos.de 04-Aug-2020


global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

dpth = 'data/';
spth = 'figures/';
load( [dpth 'fig-4-4-powercurve.mat'] ); % provides todo

save_fig = false;

current_mean = false;


% %% fit the function

FORMAT = 'hs';
FS = dissFS(FORMAT);
FONTNAME = 'Times New Roman';

pcurvo = @(v,vr,pr) pr*min(v,vr).^3 /vr^3;
pcurv = @(v,vr,pr) pcurvo(v,vr,pr)/pr;

ve = 6;           % example mean for demonstration
vm = 8.25;          % mean wind speed over month / year

pr = max( pcy );    % rated power
vr = 12.5;          % rated speed

pcx2 = sort( [pcx; vr]);

fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT)] );
plot( pcx, pcy/pr, 'o', 'linewidth', 3, ...    % plot power curve measured
    'markersize', 3, 'color', 'k');         % from figure
hold on;
plot( pcx2, pcurv(pcx2,vr,pr), 'k', ...       % plot adjusted power curve
    'linewidth', 1.5);                      % from formula

R_LINE_WIDTH = 2.5;
plot( [1 1]*vm, [0 pcurv(vm,vr,pr)], 'r',   'linewidth', R_LINE_WIDTH );
plot( [0 vm], [1 1]*pcurv(vm,vr,pr), 'r',   'linewidth', R_LINE_WIDTH );

if( ~~current_mean )
    plot( [1 1]*ve, [0 pcurv(ve,vr,pr)], 'b:',  'linewidth', 2 );
    lot( [0 ve], [1 1]*pcurv(ve,vr,pr), 'b:',  'linewidth', 2 );
end

plot( [1 1]*vr, [0 pcurv(vr,vr,pr)], 'r--', 'linewidth', R_LINE_WIDTH );
plot( [0 vr], [1 1]*pcurv(vr,vr,pr), 'r--', 'linewidth', R_LINE_WIDTH );



ax = gca;   % get current axis
set( ax, 'fontsize', FS, ...
    'fontname', FONTNAME, ...
    'xlim', [0 16], ...
    'ylim', [0 1.1], ...
    'xminorgrid', 'on', ...
    'yminorgrid', 'on', ...
    'xminortick', 'on', ...
    'yminortick', 'on', ...
    'xgrid', 'on', ...
    'ygrid', 'on', ...
    'ticklen', [0.05 0.03], ...
    'ytick', 0:0.2:1);
ax.XAxis.MinorTickValues = 1:16;
ax.YAxis.MinorTickValues = 0:0.1:1.1;

% Y_TICK_LABELS = {'0'; '0.2'; '0.4'; '0.6'; ''; ''};
% set( ax, 'yticklabels', Y_TICK_LABELS );

LF = 1; % label factor
xlh = xlabel( '$v$ [m/s]', ...    % _{\rm{w}}
    'interpreter', 'latex', 'fontsize', LF*FS);
% set( xlh, 'position', [16.8458    0.0302   -1.0000]);
ylh = ylabel( '$P/P_{\rm{r}}$', ...
    'interpreter', 'latex', 'fontsize', LF*FS);
% set( ylh, 'position', [-1.1206    0.8138   -1.0000]);

TBF = 1; % textbox factor
tbvm = tbox( [0.52505 0.24528 0.1 0.07],...
    '$\langle v \rangle$', FS*TBF ); 
set( tbvm, 'interpreter', 'latex' );
% tbpm = tbox( [0.19551 0.34247 0.095192 0.069524],...
%     '$\langle P \rangle$', FS*TBF ); 
% set( tbpm, 'interpreter', 'latex' );

if( ~~current_mean )
    tbve = tbox( [0.40554 0.12671 0.095192 0.069524],...
        '$\bar{v}$', FS*TBF );
    set( tbve, 'interpreter', 'latex' );
    tbpe = tbox( [0.17588 0.1926 0.095192 0.069524],...
        '$\bar{P}$', FS*TBF );
    set( tbpe, 'interpreter', 'latex' );
end

tbvr = tbox( [0.72492 0.24528 0.095192 0.069524],...
    '$v_{\rm r}$', FS*TBF ); 
set( tbvr, 'interpreter', 'latex' );
% tbpr = tbox( [0.17588 0.78786 0.095192 0.069524],...
%     '$P_{\rm r}$', FS*TBF ); 
% set( tbpr, 'interpreter', 'latex' );

if( ~~save_fig )
    saveas( fig, [spth 'Powercurve.eps'], 'epsc');
end

