
%% figure 4.5 of dissertation
%
% V1.0 mawolff@uos.de 05-Aug-2020


global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================= preferences ============================= %
save_fig = false;        % save figure to figure folder and current dr5 figures

swcolor = true;

FORMAT = 'hs';
fs = dissFS(FORMAT);               % fontsize
font = 'Times New Roman';   % font style/name


TICK_LEN = [0.05 0.025]/2;
X_LIM = [-4 4];

if( ~~swcolor )
    GAUSS_COLOR = 0.75*[1 1 1];
else
    GAUSS_COLOR = 'g';
end


% ================================ load  ================================ %
dpth = 'data/';
readmat = true;
if( ~~readmat )
    L = load( [dpth 'fig-4-5-windbin8-10.mat'] );
    windBin0810 = L.windBin0810;
    clear L;
end

%% ======================== wind speed increments ======================== %
fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT)] );
axvelo = axes;
% ADDOFF = 0.025;
% set( axvelo, 'position', [XOFF+ADDOFF YOFF XWIDTH YWIDTH]);

binw = 0.2;
[y,x] = hist( diff(windBin0810), -5:binw:5 );
ynorm = y/((numel(windBin0810)-1)*binw);

semilogy( x,ynorm, 'ok', ...
    'linewidth', 2, 'markersize', 4) ;
[~,mu,sig] = pdfmom(x,ynorm );
hold on;
semilogy( x, mfggauss(x,mu,sig), 'color', GAUSS_COLOR, ...
    'linewidth', 1.5);
set( axvelo, 'xlim', [-1 1]*4, 'ylim', 10.^[-6 1], ...
    'ytick', 10.^(-5:2:1) );

set( axvelo, 'fontsize', fs, ...
    'ticklength', TICK_LEN, ...
    'xgrid', 'on'..., 'ygrid', 'on' ...
    ...'xtick', 0:3:9, ...
    ...'xminortick', 'on', ...
    ...'yminortick', 'on', ...
    ...'xlim', XLIM ...
    );

for i = -5:1
    semilogy( X_LIM, [1 1]*10.^i, 'color', [1 1 1]*0.8 );
end

xlh = xlabel('$\Delta v [m/s]$', ...
    'interpreter', 'latex', ...
    'fontsize', fs);
% set( xlh, 'position', [XLIM(2)*1.15    0.0133   -1.0000]);
ylh = ylabel('$\psi(\Delta v)$', ...
    'interpreter', 'latex', ...
    'fontsize', fs);
% rotYlabel( ylh );
% set( ylh, 'position', [-5.9  0.005  0]);
% YLIM = get( axsig, 'ylim' );
% set( ylh, 'position', [XLIM(2)*-0.3    delta(YLIM)*0.5         0]);

alo.siz = fs;
tbla = dissAlabel( fig, '(a)', alo );
% set( tbla );

% -------------------------- wind power curve  -------------------------- %
vm = 8.25;     % mean of wind velocity
vr = 12.5;
Pfac = @(Pm) Pm / vm.^3;
pcurve = @(v, Pm) Pfac(Pm) * min(v,vr).^3;

% Pm = 1.72;    % node 1,2, 11,12, 21,22
Pm = 2.4;       % node 13

%% ========================== power increments  ========================== %
figb = figure;
set( figb, 'position', [456 56 dissFigSize(FORMAT)] );
axpower = axes;
% set( axpower, 'position', [XOFF+XSPACE+XWIDTH YOFF XWIDTH YWIDTH]);

wpbin = pcurve( windBin0810, Pm );

binw = 0.2;
[y,x] = hist( diff(wpbin), -4.4:binw:4.4 );
ynorm = y/((numel(wpbin)-1)*binw);

semilogy( x,ynorm, 'ok', ...
    'linewidth', 2, 'markersize', 4) ;
[~,mu,sig] = pdfmom(x,ynorm );
hold on;
semilogy( x, mfggauss(x,mu,sig), 'color', GAUSS_COLOR, ...
    'linewidth', 1.5);

for i = -5:1
    semilogy( X_LIM, [1 1]*10.^i, 'color', [1 1 1]*0.8 );
end

set( axpower, 'xlim', [-1 1]*4, 'ylim', 10.^[-6 1], ...
    'ytick', 10.^(-5:2:1) );

xlh2 = xlabel( '$\Delta P^{\rm w}$', ...
    'fontsize', fs, ... %'fontname', font);
    'interpreter', 'latex' );
ylh2 = ylabel( '$\widetilde\psi(\Delta P^{\rm w})$', ...
    'fontsize', fs, ... %'fontname', font);
    'interpreter', 'latex' );

set( axpower, 'fontsize', fs, ...
    'xgrid', 'on', ... 'ygrid', 'on', ...
...%     'fontname', font, ...
    ...'xtick', 0:10:40, ...
    ...'xlim', [0 31], ...
    ...'ylim', YLIM_SIG, ...
    'ticklength', TICK_LEN);

alo.siz = fs;
tblb = dissAlabel( figb, '(b)', alo );


% ================================ save  ================================ %
if( ~~save_fig )
    
    if( ~~swcolor )
        saveas( fig, ['figures/' 'windIncrements-SW.eps'], 'epsc' );
        saveas( figb, ['figures/' 'powerIncrements-SW.eps'], 'epsc' );
    else
        saveas( fig, ['figures/' 'windIncrements.eps'], 'epsc' );
        saveas( figb, ['figures/' 'powerIncrements.eps'], 'epsc' );
    end
    
end
