
%% frequency histogram figure 4.6 of dissertation
% 
% V1.0 mawolff@uos.de 05-Aug-2020

close all;
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================= preferences ============================= %
save_fig = false;               % save figure to disk automatically

% ----------------------------- appearance  ----------------------------- %
FORMAT = 'hs';                  % figure format half page square type
fs = dissFS(FORMAT);       % fontsize of axis
tbfs = 0.9*fs;                  % textbox fontsize
ealpha = 0.1;                   % alpha of edges in surf plot
font = 'Times New Roman';       % set font style
TICK_LEN = [0.05 0.025];        % length of ticks and subticks in percent

FIG_SIZE = dissFigSize(FORMAT)*2/3; % three figures side by side instead of 2

Y_LIM = 10.^[0 7];               % ylim for frequency distribution

ao.siz = tbfs;                  % font size for subfigure labels

GREYCOLOR = [1 1 1]*0.5;
LGREYCOLOR = [1 1 1]*0.65;

% --------------------------- data selection  --------------------------- %
windlevel = 3;  %
nodej = 14;     % 13 in paper due to removal of synchronous condenser

% ============================= koordinaten ============================= %
Y_OFF = 0.18;
Y_SIZE_HALF = 0.35;
Y_SPACE = 0.055;

TB_X_OFF = 0.015;
TB_Y_OFF = -0.01;

% XOFF = 0.075;
% X_SPACE = 0.05;
% XWIDTH = 0.26;
YWIDTH = 0.72;

% YSPACER = 0.02; % vertical space between the graphs on the right

DINDEX = 0.02; % space between (left) border and (a) or (b)
YDINDEX = DINDEX*0.5; % space between (left) border and (a) or (b)
YWINDEX = 0.15; % width of the indice boxes (a) and (b)
YINDEX = Y_OFF+YWIDTH-YDINDEX-YWINDEX*0.3;

ylpoff  = [3 0 0];    % shift of y axes label to the left (sigma)
ylpoffa = [3 0 0];    % shift of y axes label to the left (alpha)

% ======================== load and process data ======================== %
% the histogram stored in figure04data is measured for node 13 (14, 1-33)
% and average wind speed 8-10m/s
S = load(['data/' 'fig-4-6-instLocFreqHist.mat' ]);
histhe = S.histhet;
histho = S.histhom;  %
muhe = S.muhet;
sighe = S.sighet;
muho = S.muhom;
sigho = S.sighom;
nghet = S.nghet;
nghom = S.nghom;
Nmeasure = S.Nmeasure;

S = load(['data/' 'fig-4-6-instLocFreqHistSL.mat' ]);
histheL = S.histhetL;
histhoL = S.histhomL;  %
muheL = S.muhetL;
sigheL = S.sighetL;
muhoL = S.muhomL;
sighoL = S.sighomL;
nghetL = S.nghetL;
nghomL = S.nghomL;
NmeasureL = S.Nmeasure;
histheS = S.histhet;
histhoS = S.histhom;  %
muheS = S.muhet;
sigheS = S.sighet;
muhoS = S.muhom;
sighoS = S.sighom;
nghetS = S.nghet;
nghomS = S.nghom;
NmeasureS = S.Nmeasure;

dpth = 'data/';
S = load( [dpth 'fig-4-7+8-statParamD0_054.mat'] );
muholS  = S.muhol;
sigholS = S.sighol;
ngholS  = S.nghol;
muhelS  = S.muhel;
sighelS = S.sighel;
nghelS  = S.nghel;

muhomall  = muholS;
sighomall = sigholS;
nghomall  = ngholS; 
muhetall  = muhelS;
sighetall = sighelS;
nghetall  = nghelS;

ngmhetall  = mean( nghetall, 2 );
ngmhomall  = mean( nghomall, 2 );
mumhetall  = mean( muhetall, 2 );
mumhomall  = mean( muhomall, 2 );
sigmhetall = mean( sighetall, 2 );
sigmhomall = mean( sighomall, 2 );

% originally figure06data but also needed here
% non gaussianity here % <x^4>/3<x^2> - 1 as finally in paper
S = load(['data/' 'fig-4-6-params100.mat' ]);
mushol  = S.mushol;
sigshol = S.sigshol;
ngshol  = S.ngshol;
mushel  = S.mushel;
sigshel = S.sigshel;
ngshel  = S.ngshel;

sighetallL = sigshel{windlevel};
nghetallL  = ngshel{windlevel};

nbin = 5000;
freqPerBin = 0.005;
maxfreq = nbin*freqPerBin;
binx = (-maxfreq/2+freqPerBin):freqPerBin:(maxfreq/2);

gauss = @(x,mu,sig ) freqPerBin*Nmeasure*1/sqrt( 2*pi*sig.^2 ) ...
    * exp( - (x-mu).^2 / (2*sig.^2) );

%% ============================== figure  =============================== %
% ------------------------------- Het-IN  ------------------------------- %
figa = figure;
set( figa, 'position', [123 56 FIG_SIZE] );

ax1 = axes;
% set( ax1, 'position', [XOFF YOFF XWIDTH YWIDTH+YSPACE]);

hstHeL = semilogy( binx/(2*pi), histheL, 'o', ...
    'markerfacecolor', GREYCOLOR );
set( hstHeL, 'linewidth', 2, ...
    'markersize', 4, ...
    'displayname', '$10\gamma_0$', ...
    'color', 'none');
hold on;
hstHeS = semilogy( binx/(2*pi), histheS, 'kx' );
set( hstHeS, 'linewidth', 2, ...
    'displayname', '$\gamma_0$', ...
    'markersize', 6 );

gss = semilogy( binx/(2*pi), gauss( binx, muheS, sigheS ), '-' );
set( gss, 'linewidth', 1.5, ...
    'markersize', 3, ...
    'color', 'g' );


set( ax1, 'xlim', [-1 1]*1.5/(2*pi),...
    'ylim', Y_LIM, ...
    'ytick', 10.^(0:2:10), ...
    'xminortick', 'on', ...
    'ticklength', TICK_LEN, ...
    'fontname', font, ...
    'fontsize', fs);

tba = dissAlabel( figa, '(a)', ao );
tbap = get( tba, 'position' );
set( tba, 'position', tbap + [TB_X_OFF TB_Y_OFF 0 0] );

xlha = xlabel( '$f$ [Hz]', ...
    'fontname', font, ...
    'fontsize', fs, ...
    'interpreter', 'latex');
ylha = ylabel( 'counts', ...
    'fontname', font, ...
    'fontsize', fs);

legA = legend([hstHeS hstHeL]);
set( legA, 'interpreter', 'latex', ...
    'location', 'northwest');


% ------------------------------- Hom-IN  ------------------------------- %
figb = figure;
set( figb, 'position', [123+FIG_SIZE(1) 56 FIG_SIZE] );

% ------------------------------ histogram ------------------------------ %
ax2 = axes;
% set( ax2, 'position', [XOFF+XWIDTH+XSPACE YOFF XWIDTH YWIDTH+YSPACE]);

hstHoL = semilogy( binx/(2*pi), histhoL, 'o', ...
    'markerfacecolor', GREYCOLOR );
set( hstHoL, 'linewidth', 2, ...
    'markersize', 4, ...
    'color', 'none', ...
    'displayname', '$10\gamma_0$' ...
    );
hold on;
hstHoS = semilogy( binx/(2*pi), histhoS, 'kx' );
set( hstHoS, 'linewidth', 2, ...
    'markersize', 6, ...
    'displayname', '$\gamma_0$');%, ...
%     'displayname', 'frequency node 11');

gss = semilogy( binx/(2*pi), gauss( binx, muhoS, sighoS ), 'g-' );
set( gss, 'linewidth', 1.5, ...
    'markersize', 3, ...
    'displayname', 'normal distribution');

set( ax2, 'xlim', [-1 1]*1.5/(2*pi), ...
    'ylim', Y_LIM , ...
    'ytick', 10.^(0:2:10), ...
    'xminortick', 'on', ...
    'ticklength', TICK_LEN, ...
    'fontname', font, ...
    'fontsize', fs);

tbb = dissAlabel( figb, '(b)', ao );
tbbp = get( tbb, 'position' );
set( tbb, 'position', tbbp + [TB_X_OFF TB_Y_OFF 0 0] );

% xlh = xlabel( '$\omega$ [Hz]', ...
xlhb = xlabel( '$f$ [Hz]', ...
    'fontname', font, ...
    'fontsize', fs, ...
    'interpreter', 'latex');
ylhb = ylabel( 'counts', ...
    'fontname', font, ...
    'fontsize', fs);

legB = legend([hstHoS hstHoL]);
set( legB, 'interpreter', 'latex', ...
    'location', 'northwest');


% leg = legend( 'show' );
% -------------------------------- sigma -------------------------------- %
figc = figure;
set( figc, 'position', [123+2*FIG_SIZE(1) 56 FIG_SIZE] );

axsig = axes;
axsigpos = get( axsig, 'position' );
set( axsig, 'position', ...
    [axsigpos(1) Y_OFF+Y_SIZE_HALF+Y_SPACE ...
    axsigpos(3) Y_SIZE_HALF] );
% [XOFF+2*(XSPACE+XWIDTH) ...
%     YOFF+YWIDTH/2+YSPACE XWIDTH YWIDTH/2-YSPACER/2]);
sigpl2 = plot( sighetall(nodej,:)/(2*pi), 'kx' );
set( sigpl2, 'linewidth', 2, ...
    'markersize', 6 );
hold on;
gss = plot( [1 100], [1 1]*sigmhetall(nodej)/(2*pi), 'g-' );
plot( sighetallL(nodej,:)/(2*pi) , 'o', ...
    'color', 'none', ...
    'markerfacecolor', LGREYCOLOR, ...
    'markersize', 5 );
set( gss, 'linewidth', 1.5, ...
    'markersize', 3 );

set( axsig, 'xlim', [1 100],...
    'ylim', [0 0.035], ...
    'xminortick', 'on', ...
    'ytick', (0:3)*1e-2, ...
    'ticklength', TICK_LEN, ...
    'fontname', font, ...
    'fontsize', fs, ...
    'xticklabel', {});
ax10l = axLabelExp( axsig, -2 );
set( ax10l, 'position', [0.07353 0.91181 0.069445 0.081146]);

ylhsig2 = dissYLabel('$\sigma$', FORMAT );
rotYlabel( ylhsig2 );
ylp = get( ylhsig2, 'position' );
set( ylhsig2, 'position', ylp-ylpoff );

tbc = dissAlabel( figc, '(c)', ao );
tbcp = get( tbc, 'position' );
set( tbc, 'position', tbcp + [TB_X_OFF TB_Y_OFF 0 0] );

% -------------------------------- alpha -------------------------------- %
axalpha = axes;
axapos = get( axalpha, 'position' );
set( axalpha, 'position', [axapos(1) Y_OFF axapos(3) Y_SIZE_HALF] );
alph2 = plot( nghetall(nodej,:), 'kx' );
set( alph2, 'linewidth', 2, ...
    'markersize', 6 );
hold on;
gss = plot( [1 100], [1 1]*ngmhetall(nodej), 'g-' ); %ngmhomall
set( gss, 'linewidth', 1.5, ...
    'markersize', 3 );

plot( nghetallL(nodej,:) , 'o', ...
    'color', 'none', ...
    'markerfacecolor', LGREYCOLOR, ...
    'markersize', 5 );

set( axalpha, 'xlim', [1 100],...
    'ylim', [0 6], ...
    'ytick', 0:2:6, ...
    'xminortick', 'on', ...
    'yminortick', 'on', ...
    'ticklength', TICK_LEN, ...
    'fontname', font, ...
    'fontsize', fs);

ylhng2 = dissYLabel('$\alpha$', FORMAT );
rotYlabel( ylhng2 );
ylp = get( ylhng2, 'position' );
set( ylhng2, 'position', ylp-ylpoffa );

dissXLabel('data set', FORMAT );

% ax10l = axLabelExp( axalpha2, -1 );
% set( ax10l, 'position', [0.69243 0.47496 0.070044 0.087404]);

tbd = dissAlabel( figc, '(d)', ao );
tbdp = get( tbd, 'position' );
set( tbd, 'position', tbdp+[TB_X_OFF -0.45 0 0] );


shiftYLabelVert(axsig);

% -------------------------------- save  -------------------------------- %
if( ~~save_fig )
    
    spth = 'figures/';
    saveas( figa, [spth 'instLocFreqHist-a.eps'], 'epsc' );
    saveas( figb, [spth 'instLocFreqHist-b.eps'], 'epsc' );
    saveas( figc, [spth 'instLocFreqHist-c.eps'], 'epsc' );
    
end



















