
%% Non gaussian parameter for all models and all nodes
% Figure 4.7 of dissertation
%
% compares the non gaussian parameter 
% alpha = < x^4 > / (3 < x^2 >^2 ) - 1          (1)
% for models (Het-IN, Hom-IN).
% Data from 05hel hol is used.
%
%
% V1.0 mawolff@uos.de 06-Aug-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

%% ============================ preferences ============================= %
save_fig = false;        % save figure to figure folder and current dr5 figures

% ----------------------------- appearance  ----------------------------- %
FORMAT = 'hr';
fs = dissFS(FORMAT);                % fontsize
font = 'Times New Roman'; % font style/name

TICK_LEN = [0.05 0.025];

% -------------------- data selection and processing -------------------- %
gens = 1:33;
gens([5 16 27]) = [];   % remove synchronous condensors

windlevel = 3;
YLIM_NG = [0 3];

%% =============================== load  ================================ %
% === new data ===
dpth = 'data/';

S = load( [dpth 'fig-4-7+8-statParamD0_54.mat'] );
muholL  = S.muhol;
sigholL = S.sighol;
ngholL  = S.nghol;
muhelL  = S.muhel;
sighelL = S.sighel;
nghelL  = S.nghel;

clear S;
S = load( [dpth 'fig-4-7+8-statParamD0_054.mat'] );
muholS  = S.muhol;
sigholS = S.sighol;
ngholS  = S.nghol;
muhelS  = S.muhel;
sighelS = S.sighel;
nghelS  = S.nghel;


% =============================== process =============================== %
ngmhelLM  = mean( nghelL, 2 );
ngmholLM  = mean( ngholL, 2 );
mumhelLM  = mean( muhelL, 2 );
mumholLM  = mean( muholL, 2 );
sigmhelLM = mean( sighelL, 2 );
sigmholLM = mean( sigholL, 2 );

ngmhelSM  = mean( nghelS, 2 );
ngmholSM  = mean( ngholS, 2 );
mumhelSM  = mean( muhelS, 2 );
mumholSM  = mean( muholS, 2 );
sigmhelSM = mean( sighelS, 2 );
sigmholSM = mean( sigholS, 2 );

% ============================= coordinates ============================= %
XOFF = 0.12;
YOFF = 0.18;
XSPACE = 0.17;
XWIDTH = 0.32*2+XSPACE-XOFF/6;
YWIDTH = 0.76;

TB_XPOS_A = 3;          % value positions of (a) and (b) labels
TB_YPOS_A = 4;         %
TB_XPOS_B = TB_XPOS_A;
TB_YPOS_B = 1;


% YLIM_NG = 10.^[0 1];     % axes limits
YLIM_NG = [0 4];

% YLIM_SIG = 10.^[-1.5 0];
YLIM_SIG = [0 0.5];
% ================================ plot  ================================ %
fig = figure;
set( fig, 'position', [123 56 541 353] );

% ----------------------- non-gaussian parameter  ----------------------- %
axng = axes;
set( axng, 'position', [XOFF YOFF XWIDTH YWIDTH]);
phes = plot( (ngmhelSM(gens)), ...
    'displayname', 'Het $\gamma_0$');
compareDotPlot(phes,1);
hold on;
phos = plot( (ngmholSM(gens)), 'o', ...
    'displayname', 'Hom $\gamma_0$');
set( phos, 'markersize', 6, 'linestyle', 'none', ...
    'linewidth', 0.1, 'markerfacecolor', unicolors('dr') );

phe = plot( (ngmhelLM(gens)), 'x', ...
    'displayname', 'Het $10\gamma_0$');
set( phe, 'markersize', 8, ...
    'linewidth', 2, ...
    'color', [0.6 0.6 1] );
% compareDotPlot(phe,1);
hold on;
pho = plot( (ngmholLM(gens)), 'o', ...
    'displayname', 'Hom $10\gamma_0$');
set( pho, 'markersize', 6, 'linestyle', 'none', ...
    'linewidth', 0.1, 'markerfacecolor', unicolors('do'), ...
    'color', 'none');

% set( gca, 'xscale' , 'log' );
set( gca, 'xscale' , 'linear' );
% set( gca, 'yscale' , 'log' );
set( gca, 'yscale' , 'linear' );

set( gca, 'ylim', YLIM_NG );
xlim( [0 31] );

xlh = xlabel( '$j$', ...
    'fontsize', 0.75*fs, ... %'fontname', font, ...
    'interpreter', 'latex');
ylh = ylabel( '$\bar{\alpha}_j$', ...
    'fontsize', 0.75*fs, ... %'fontname', font, ...
    'interpreter', 'latex');
rotYlabel( ylh, axng );
set( xlh, 'position', [14.8990    0.1635-0.5   -1.0000]);
% set( ylh, 'position', [-4.0719    2    0.0000]);

set( axng, 'fontsize', fs, 'fontname', font, ...
    'xtick', 0:10:40, ...
    'ytick', 0:3, ...
    'yminortick', 'on', ...
    'ticklength', TICK_LEN);

LEGPOS = ...
    [0.73134 0.694 0.25451 0.28754];

% axes( axcor );           % make non gaussian axis current axis again
leg = legend( [phes phos phe pho] ); % to have legend on top
set( leg, 'fontsize', fs*0.9, 'fontname', font, ...
    'position', LEGPOS, ...
    'edgecolor', 'none', ...
    'interpreter', 'latex');


% ================================ save  ================================ %
if( ~~save_fig )
    
    saveas( fig, ['figures/' 'nonGaussParam.eps'], 'epsc' );
    
end
