
%% statistical parameters in dependance of wind level
% figure 4.8 in Dissertation
%
%
% V2.0 mawolff@uos.de 06-Aug-2020


close all;
global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
% Path to pg_dyn_models 
% [Nishikawa and Motter, New J. Phys. 17, 015012 (2015) ]
% needed:
addpath('C:\Users\Anwender\Documents\MATLAB\pg_dyn_models');

%% ============================ preferences ============================= %
save_fig = false;        % save figure to figure folder and current dr5 figures

% ----------------------------- appearance  ----------------------------- %
FORMAT = 'hr';              %
fs = dissFS(FORMAT);        % fontsize
font = 'Times New Roman';   % font style/name
ao.siz = fs;                % font size of label (a) and (b)

TICK_LEN = [0.05 0.025];

YLIM_SIG = [0 0.4];

% -------------------- data selection and processing -------------------- %

% ============================= coordinates ============================= %
AXX = 0.15;
AXY = 0.2;
AXW = 0.8;
AXH = 0.7;

% YLIM_NG = 10.^[-1.5 2];     % axes limits
% YLIM_NG = [0 10];     % axes limits

% YLIM_SIG = 10.^[-2 0];

%% =============================== load  ================================ %
dpth = 'data/';
S = load( [dpth 'fig-4-8-SigmaWindPower.mat'] );
mushol  = S.mushol;
sigshol = S.sigshol;
ngshol  = S.ngshol;
mushel  = S.mushel;
sigshel = S.sigshel;
ngshel  = S.ngshel;

windlevel = 3;
% =============================== process =============================== %
muhol  = mushol{windlevel};
sighol = sigshol{windlevel};
nghol  = ngshol{windlevel}; 
muhel  = mushel{windlevel};
sighel = sigshel{windlevel};
nghel  = ngshel{windlevel};

ngmhel  = mean( nghel, 2 );
ngmhol  = mean( nghol, 2 );
mumhel  = mean( muhel, 2 );
mumhol  = mean( muhol, 2 );
sigmhel = mean( sighel, 2 );
sigmhol = mean( sighol, 2 );

% === new data ===
dpth = 'data/';

S = load( [dpth 'fig-4-7+8-statParamD0_54.mat'] );
muholL  = S.muhol;
sigholL = S.sighol;
ngholL  = S.nghol;
muhelL  = S.muhel;
sighelL = S.sighel;
nghelL  = S.nghel;

clear S;
S = load( [dpth 'fig-4-7+8-statParamD0_054.mat'] );
muholS  = S.muhol;
sigholS = S.sighol;
ngholS  = S.nghol;
muhelS  = S.muhel;
sighelS = S.sighel;
nghelS  = S.nghel;


% =============================== process =============================== %
ngmhelLM  = mean( nghelL, 2 );
ngmholLM  = mean( ngholL, 2 );
mumhelLM  = mean( muhelL, 2 );
mumholLM  = mean( muholL, 2 );
sigmhelLM = mean( sighelL, 2 );
sigmholLM = mean( sigholL, 2 );

ngmhelSM  = mean( nghelS, 2 );
ngmholSM  = mean( ngholS, 2 );
mumhelSM  = mean( muhelS, 2 );
mumholSM  = mean( muholS, 2 );
sigmhelSM = mean( sighelS, 2 );
sigmholSM = mean( sigholS, 2 );

%% =============================== plot  ================================ %
windmean = 5:2:17;

% --------------------------- standard deviation ------------------------ %
figa = figure;
set( figa, 'position', [123 56 dissFigSize(FORMAT)] );

ADDOFF = 0.025;
axcor = axes;
set( axcor, 'position', [AXX AXY AXW AXH] );

gens = 1:33;
gens([5 16 27]) = [];   % remove synchronous condensors
% gens(4) = [];           % remove reference node

sigmhelSM(4) = -1;
phe = plot( abs(sigmhelSM(gens)), ...
    'displayname', 'Het');
compareDotPlot(phe,1);
hold on;
pho = plot( abs(sigmholSM(gens)), ...
    'displayname', 'Hom');
compareDotPlot(pho,2);
set( gca, 'yscale' , 'linear' );

% phes = plot( (sigmhelSM(gens)), 'x', ...
%     'displayname', 'Het $\gamma_0$', ...
%     'color', [0.6 0.6 1]);
% % compareDotPlot(phes,1);
% hold on;
% phos = plot( (sigmholSM(gens)), 'o', ...
%     'displayname', 'Hom $\gamma_0$');
% set( phos, 'markersize', 6, 'linestyle', 'none', ...
%     'linewidth', 0.1, 'markerfacecolor', unicolors('do') );

xlha = dissXLabel( '$j$', FORMAT ); 
ylha = dissYLabel( '$\bar{\sigma}_j$', FORMAT );
rotYlabel( ylha, axcor );
set( xlha, 'position', [15.5000   -0.0368   -1.0000]);
set( ylha, 'position', [-4.3834    0.1750         0]);

set( axcor, 'fontsize', fs, ...
    'fontname', font, ...
    'xtick', 0:10:40, ...
    'xlim', [0 31], ...
    'ylim', YLIM_SIG, ...
    'ytick', 0:0.1:0.4, ...
    'ticklength', TICK_LEN);

LEGPOS = ...
    [0.17222 0.64856 0.16171 0.207];

axes( axcor );           % make non gaussian axis current axis again
leg = legend( [phe pho] ); % to have legend on top
set( leg, 'fontsize', fs, 'fontname', font, ...
    'position', LEGPOS, ...
    'edgecolor', 'none');

tba = dissAlabel( figa, '(a)', ao );
tbap = get( tba, 'position' );
set( tba, 'position', tbap+[0.005 -0.02 0 0] );



% ==================== power and standard dev correl ==================== %
figb = figure;
FIG_SIZE = dissFigSize(FORMAT);
set( figb, 'position', [123+FIG_SIZE(1) 56 FIG_SIZE] );

axsig = axes;
set( axsig, 'position', [AXX AXY AXW AXH] );

loaded = load( 'data/RTS96.mat', 'mpc' );
mpc = loaded.mpc;

rso.dynmodel = 'sm';
rso.IN = false;
rso.hom = false;
rso.splitP = true;
rso.hetpar = 1e-6;
rso.redpass = false;
rso.losses = false;

[sm, ~] = raw2swing( mpc, rso );

gens = 1:33;
gens([4 5 16 27]) = [];   % ref node (4) and remove synchronous condensors

% dends = [13 23];

% gnd = gens;
% gnd(dends) = [];

vmean = (2*windlevel+3);
Pwf = (vmean / 8.25)^3;         % factor between wind power and conventional 
                                % power for av. speed 5m/s
xp = sm.Pm(gens)*Pwf;           % power on x axis
ysig = abs( sigmhel(gens) );    % standard deviation on y axis

ysig2 = abs( sigmhelSM(gens) );

XLIM = [0 ceil(max(xp))];
XFIT = XLIM + [ 0 2 ];
% XFIT(2) = XFIT(2) + 2;

pf = polyfit( xp, ysig2, 1 );

phe = plot( xp, ysig2 , 'xb', ...
    'markersize', 8, ...
    'linewidth', 2, ...
    'displayname', '$\gamma_0$');
hold on;
plot( XFIT, polyval(pf, XFIT ), 'b', ...
    'linewidth', 1.5);

pheL = plot( xp, ysig , 'x', ...
    'markersize', 8, ...
    'linewidth', 2, ...
    'displayname', '$10\gamma_0$', ...
    'color', [0.6 0.6 1]);

set( axsig, 'fontsize', fs, ...
    'ticklength', TICK_LEN, ...
    'xtick', 0:3:9, ...
    'xminortick', 'on', ...
    'yminortick', 'on', ...
    'fontname', font, ...
    'xgrid', 'on', 'ygrid', 'on', ...
    'xminorgrid', 'on', 'yminorgrid', 'on', ...
    'xlim', XLIM ...
    );

xlhb = xlabel('$P^{\rm w}_j(v_{\rm c})$', ...
    'interpreter', 'latex', ...
    'fontsize', fs);
set( xlhb, 'position', [4.5000   -0.0305   -1.0000]);
ylhb = ylabel('$\bar{\sigma}_j$', ...
    'interpreter', 'latex', ...
    'fontsize', fs);
rotYlabel( ylhb );
YLIM = get( axsig, 'ylim' );
set( ylhb, 'position', [-1.2997    0.1500    0.0000]);

legB = legend([phe pheL]);
set( legB, 'interpreter', 'latex', ...
    'location', 'northwest');

tbb = dissAlabel( figb, '(b)', ao );
tbbp = get( tbb, 'position' );
set( tbb, 'position', tbbp+[0.005 -0.02 0 0] );


% ================================ save  ================================ %
if( ~~save_fig )
    
    saveas( figa, ['figures/' ...
        'stddevPowerCorrel-a.eps'], 'epsc' );
    saveas( figb, ['figures/' ...
        'stddevPowerCorrel-b.eps'], 'epsc' );
    
end

