
%% fig A.1 Dissertation illustrating graph theory
%
%

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));

%% ============================= settings  ============================== %
save_fig = false;
savemat = false;
FORMAT = 'hs';

labo.siz = dissFS( 'hs' );%*0.8;
FigSize = dissFigSize('hs')*2/3;
% fsIntLabels = labo.siz;         % font size of terminal node labels (inside black circles)
% nopt.bcScale = 20;

nopt.genscale = [];    
nopt.loadscale = [];
nopt.numbering = 'seq';
nopt.tboxlabel = true;
nopt.gencolor = [0.6902    0.0627    0.2078]*1.4; % gencolor etwas heller
nopt.redpass = false;
nopt.tbfs = labo.siz;

TB_POS = [0.05 0.940 0.025 0.025];
XLim = [-1.5 1];
YLim = [-1.5 1];
XLimRTS = [1250 1620];
YLimRTS = [800 1150];

%% ============================= load data ============================== %
mpc5 = loadcase( 'case5' );
% --------------------------- simplify graph  --------------------------- %
% simplify graph to make the Kron reduction more instructive
% gather generators and
% make nodes either generator or load nodes and never both
% mpc5 = gatherGen( mpc5 );                       % gather generators

mpc5.bus(4,3) = mpc5.bus(4,3) - mpc5.gen(3,2);  % make bus 4 effective load
mpc5.gen(3,:) = [];                             % and delete its generator
mpc5.gen(2,2) = mpc5.gen(2,2) - mpc5.bus(3,3);  % make bus 3 effective generator
mpc5.bus(3,3) = 0;                              % delete its load 


dpth = 'data/';
file = [dpth 'fig-A-1-GraphTheory.mat'];
load( file );

xgi = xgs(1:5);
xgt = xgs(6:10);
ygi = ygs(1:5);
ygt = ygs(6:10);

%% ============================== graph1  =============================== %
fig1 = figure;
set( fig1, 'position', [100 56 FigSize] );

gro.model = 'pf';
nopt.model = gro.model;
busGraph = mpc2Graph( mpc5, gro );

busgplot = plot( busGraph );

ax = gca;
set( ax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');
set( busgplot, 'xdata', xgt, 'ydata', ygt );

highlightEdges( busgplot, busGraph );
nopt.dualbus = 'effective';
tbl = highlightPGNodes( busgplot, mpc5, nopt );

for i = 1:5
    set( tbl.ntb(i), 'position', labelPosTerm(i,:) );
end


tb = dissAlabel( fig1, '(a)', labo );
% tbp = get( tb, 'position' );
% set( tb, 'position', tbp+[0.005 0 0 0] );
set( tb, 'position', TB_POS );


if( ~~ save_fig )
    spth = 'figures/';
    saveas( fig1, [spth 'graphCase5effBus.eps'], 'epsc' );
end

%% ============================== graph2  =============================== %
% demonstrate unsymmetrical transmission line operation using
% worst case phase shift in one line

loaded = load( 'data/simple5.mat' ); % use case 5 grid with exclusively gens and loads
mpc2 = loaded.mpc;
tapCol = 9; 
shiftCol = 10;
mpc2.branch(1,tapCol) = 1.05;       % create tap transformer at link 1->2
mpc2.branch(1,shiftCol) = pi/2;     % make worst case asymmetrical phase shift
% mpc2.branch = sortrows([mpc2.branch; mpc2.branch(1,[2 1 3:end])]);
mpc2.branch = sortrows( mpc2.branch, [1 2]);  % same sorting as matlab graph struct

y0 = makeYbus( mpc2 );
weights = abs(real(y0));

mpc2.branch = sortrows([mpc2.branch; mpc2.branch(1,[2 1 3:end])]);
mpc2.branch = sortrows( mpc2.branch, [1 2]);  % same sorting as matlab graph struct

ws = zeros(numel(mpc2.branch(:,1)) ,1);      % weight of single line is
for i = 1:numel(mpc2.branch(:,1))            % for every line
    ws(i) = weights(mpc2.branch(i,1),...     % the corresponding
        mpc2.branch(i,2));                   % matrix entry
end

gline = mpc2.branch(:,1:2);              % graph lines (gline) are the transmission lines
gnode = mpc2.bus(:,1);                   % nodes (gnode) are the buses

Nall = size(gnode,1);
nodec = 0.7*ones(Nall,1);


nodeName = cell(Nall,1);                  % nodeName must be cell
for i = 1:Nall                            % iterate through all nodes
    nodeName{i} = num2str(gnode(i,1));   % give them the corresponding number as name
end

lscale = @(x) x;
nodes = table(nodeName, gnode, nodec, ...
    'VariableNames', {'Name','NodeLabel','NodeColor'} );
edges = table([gline(:,1) gline(:,2)], lscale(ws),...
    'VariableNames',{'EndNodes', 'Weight'});

busGraph2 = digraph( edges, nodes);

%%

% gro.model = 'pf';
% nopt.model = gro.model;
% gro.symlines = false;
% busGraph2 = mpc2Graph( mpc2, gro );

fig2 = figure;
set( fig2, 'position', [123+FigSize(1) 56 FigSize] );

busgplot2 = plot( busGraph2 );

ax = gca;
set( ax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');
set( busgplot2, 'xdata', xgt, 'ydata', ygt );

% --------------------------- highlight edges --------------------------- %
% color the manipulated asymmetrical line
idx_color = [1 4];
col_symm = [1 1 1]*0.6;
col_asym = [0.6 0.6 1];
N = numel(mpc2.branch(:,1));
exo.dim = N;
colors = expand( col_symm, exo );
exo.dim = numel(idx_color);
mcol = expand( col_asym, exo );
colors( idx_color, : ) = mcol;

edgeo.arrowsize = 20;
edgeo.color = colors;
highlightEdges( busgplot2, busGraph2, edgeo );
nopt.dualbus = 'effective';
tbl = highlightPGNodes( busgplot2, mpc2, nopt );

for i = 1:5
    set( tbl.ntb(i), 'position', labelPosTerm(i,:) );
end


tb = dissAlabel( fig2, '(b)', labo );
% tbp = get( tb, 'position' );
% set( tb, 'position', tbp+[0.005 0 0 0] );
set( tb, 'position', TB_POS );

if( ~~ save_fig )
    spth = 'figures/';
    saveas( fig2, [spth 'graphCase5directed.eps'], 'epsc' );
end

%%

labelPosTerm = zeros(5,4);
for i = 1:5
    tbh = get( tbl.ntb(i) );
    labelPosTerm(i,:) = tbl.ntb(i).Position;
end

% savemat = true;
if( ~~savemat )
file = [dpth 'fig-A-1-GraphTheory.mat'];
    save( file, ...
        ...'par5', ...    % data tables
        'rtsTB_POS', ...         
        'xgs', 'ygs', ...               % node coordinates x, y
        'labelPosTerm');%, ...             % tbox label coordinates x,y,w,h
%         'labelPosInt', ...
%         'labelPosKron');                     
end







































