
%% Dissertation case 5 graph illustration
%
% V1.0 mawolff@uos.de 29-Oct-2020


global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));

%% ============================= settings  ============================== %
save_fig = false;
savemat = false;
FORMAT = 'hs';

labo.siz = dissFS( 'hs' );%*0.8;
FigSize = dissFigSize('hs')*2/3;

TB_POS = [0.05 0.940 0.025 0.025];
XLim = [-1.5 1];
YLim = [-1.5 1];

%% ============================= load data ============================== %

dpth = 'data/';
file = [dpth 'fig-A-6-case5Graph.mat'];
S = load( file );
labelPosGSM = S.labelPosGSM;
labelPosFSM = S.labelPosFSM;
xgsm = S.xgsm;
ygsm = S.ygsm;
xfsm = S.xfsm;
yfsm = S.yfsm;
clear S;

labelPosTerm = labelPosGSM(1:5,:);

xgt = xgsm(1:5);
ygt = ygsm(1:5);

%% ============================= PF graph  ============================== %
mpc5pf = loadcase('case5');
mpc5pf = gatherGen( mpc5pf );

gro.dir = false; % make directed graph
gro.model = 'pf';
pgGraphPF = mpc2Graph( mpc5pf, gro );    % 

figPF = figure;
set( figPF, 'position', [123 456 FigSize]);
pfax = axes;
set( pfax, 'position', [0.025 0.025 0.95 0.95] );

graphPF = plot( pgGraphPF );

set( graphPF, 'xdata', xgt, 'ydata', ygt );
set( pfax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');

%
clear nodeo;
nodeo.numbering = 'seq';
nodeo.tboxlabel = true;
nodeo.dualbus = 'both';
nodeo.tbshift = [-0.0165 0.0165];
% nodeo.dualbus = 'effective';
nodeo.model = 'pf';
nodeprop = highlightPGNodes( graphPF, mpc5pf, nodeo );

highlightEdges( graphPF, pgGraphPF );

for i = 1:5
    set( nodeprop.ntb(i), 'position', labelPosTerm(i,:), ...
        'fontsize', labo.siz);
end

tb = dissAlabel( figPF, '(a)', labo );
% tbp = get( tb, 'position' );
% set( tb, 'position', tbp+[0.005 0 0 0] );
set( tb, 'position', TB_POS );

pause(1);
if( ~~save_fig )
    spth = 'figures/';
    saveas( figPF, [spth 'case5-graph-PF.eps'], 'epsc');
end

%% ============================= gSM graph ============================== %
mpc5gsm = loadcase('case5');
mpc5gsm = gatherGen( mpc5gsm );

gro.dir = false; % make directed graph
gro.model = 'gsm';
pgGraphGSM = mpc2Graph( mpc5pf, gro );    % 

figGSM = figure;
set( figGSM, 'position', [123 456 FigSize]);
pfax = axes;
set( pfax, 'position', [0.025 0.025 0.95 0.95] );

graphGSM = plot( pgGraphGSM );

set( graphGSM, 'xdata', xgsm, 'ydata', ygsm );
set( pfax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');

%
clear nodeo;
nodeo.numbering = 'seq';
nodeo.tboxlabel = true;
nodeo.dualbus = 'both';
nodeo.tbshift = [-0.0165 0.0165];
% nodeo.dualbus = 'effective';
nodeo.model = 'gsm';
nodeprop = highlightPGNodes( graphGSM, mpc5pf, nodeo );

highlightEdges( graphGSM, pgGraphGSM );

for i = 1:10
    set( nodeprop.ntb(i), 'position', labelPosGSM(i,:), ...
        'fontsize', labo.siz);
end

tb = dissAlabel( figGSM, '(b)', labo );
% tbp = get( tb, 'position' );
% set( tb, 'position', tbp+[0.005 0 0 0] );
set( tb, 'position', TB_POS );

pause(1);
if( ~~save_fig )
    spth = 'figures/';
    saveas( figGSM, [spth 'case5-graph-GSM.eps'], 'epsc');
end

%% ============================= fSM graph ============================== %
mpc5fsm = loadcase('case5');

gro.dir = false; % make directed graph
gro.model = 'fsm';
pgGraphFSM = mpc2Graph( mpc5fsm, gro );    % 

figFSM = figure;
set( figFSM, 'position', [123 456 FigSize]);
pfax = axes;
set( pfax, 'position', [0.025 0.025 0.95 0.95] );

graphFSM = plot( pgGraphFSM );

set( graphFSM, 'xdata', xfsm, 'ydata', yfsm );
set( pfax, 'position', [0.01 0.01 0.98 0.98], ...
    'xlim', XLim, 'ylim', YLim, ...
    'Visible', 'off');

%
clear nodeo;
nodeo.numbering = 'seq';
nodeo.tboxlabel = true;
nodeo.dualbus = 'both';
nodeo.tbshift = [-0.0165 0.0165];
% nodeo.dualbus = 'effective';
nodeo.model = 'fsm';
nodeprop = highlightPGNodes( graphFSM, mpc5fsm, nodeo );

highlightEdges( graphFSM, pgGraphFSM );

for i = 1:13
    set( nodeprop.ntb(i), 'position', labelPosFSM(i,:), ...
        'fontsize', labo.siz);
end

tb = dissAlabel( figFSM, '(c)', labo );
% tbp = get( tb, 'position' );
% set( tb, 'position', tbp+[0.005 0 0 0] );
set( tb, 'position', TB_POS );

pause(1);
if( ~~save_fig )
    spth = 'figures/';
    saveas( figFSM, [spth 'case5-graph-FSM.eps'], 'epsc');
end


%%
% move_nodes = true;
move_nodes = false;
if( ~~move_nodes )
    % dpath =
    [xnn, ynn] = editGdots( xfsm, yfsm, graphFSM );
%     [xnn, ynn] = editGdots( xnn, ynn, graphPF );
    
%     saveGraphXY( xnn, ynn, [dpath 'fig-2-20-rts96PFVlevel.mat'] );

end


%%

labelPosTerm = zeros(5,4);
for i = 1:5
%     tbh = get( nodeprop.ntb(i) );
    labelPosTerm(i,:) = nodeprop.ntb(i).Position;
end

%%
labelPosGSM = zeros(10,4);
for i = 1:10
%     tbh = get( nodeprop.ntb(i) );
    labelPosGSM(i,:) = nodeprop.ntb(i).Position;
end

%%
labelPosFSM = zeros(13,4);
for i = 1:13
%     tbh = get( nodeprop.ntb(i) );
    labelPosFSM(i,:) = nodeprop.ntb(i).Position;
end

%%

if( ~~savemat )
file = [dpth 'fig-A-6-case5Graph.mat'];
    save( file, ...
        ...'par5', ...    % data tables
        'TB_POS', ...         
        'xfsm', 'yfsm', ...             % node coordinates x, y fSM model
        'xgsm', 'ygsm', ...             % node coordinates gSM model
        ...'labelPosTerm', ...             % tbox label coordinates x,y,w,h (temrinal nodes)
        'labelPosGSM', ...           % tbox label coords (terminal and internal gSM model)
        'labelPosFSM');                 % tbox label coords (terminal and internal fSM model)
end
