
%% dissertation figure A.08 swing ODE DAE
%
% V1.0 mawolff@uos.de 05-Dec-2020

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================== settings  ============================== %
save_fig = false;
save_mat = false;

FORMAT = 'fr';
fontsiz = dissFS(FORMAT);
fontname = 'Times New Roman';

PHASE   = '\varphi';
FREQ    = '\omega';
DAETEXT = '(DAE)';
ODETEXT = '(ODE)';
RATEXT  = '(DAE, $R_a > 0)$';

XOFF = 0.1;
XSPACE = 0.075;
XSIZE = [0.57 0.25];
YOFF = 0.15;
YSIZE = 0.8;

X_MAGN = [1 1.56];
Y_MAGN = [0.3 0.38];

%% ============================= read data ============================== %
pth = 'data/';
load( [pth 'solutionODE-DAE-DP01.mat'], ...
    'TDEQ', 'YDEQ', ...     % solutions of ode
    't', 'y', ...           % solutions of dae
    'tra', 'yra', ...       % solutions of dae with Ra > 0
    'yd', 'par', 'parODE' ...
    );

% %%
if( ~~save_mat )
    save( [pth 'solutionODE-DAE-DP01.mat'], ...
        'TDEQ', 'YDEQ', ...     % solutions of ode
        't', 'y', ...           % solutions of dae
        'tra', 'yra', ...
        'yd', 'par', 'parODE' ...
        );
end

%% compare solutions

daeSW = 3;
raDAE = [1:daeSW:250 251:200:32540 32541:daeSW:size(y,1)];

odeSW = 1;
% raODE = [1:100 101:odeSW:size(YDEQ,1)];
raODE = 1:odeSW:size(YDEQ,1);

fig = figure;
set( fig, 'position', [123 56 dissFigSize(FORMAT).*[1 0.75]]);

% ============================== main axis ============================== %
ax = axes; hold on;
set( ax, 'position', [XOFF YOFF XSIZE(1) YSIZE] );

% zoom box
xbox = [X_MAGN X_MAGN(2) X_MAGN(1) X_MAGN(1) ];
ybox = [Y_MAGN(1) Y_MAGN(1) Y_MAGN(2) Y_MAGN(2) Y_MAGN(1)];
zoomBox = plot(xbox,ybox, '--', ...
    'color', 'k', ...
    'linewidth', 1, ...
    'displayname', ' ');

ph1Ra = plot(tra,yra(:,1)-y(1,1), '-.', ...
    ...'markerfacecolor', vrggrad(2,10), ...
    'color', lightToDarkBlueGradient(2,10), ...
    'markersize', 5, ...
    'linewidth', 1.5, ...
    'displayname', ['$' PHASE '_1$' RATEXT]);
ph2Ra = plot(tra,yra(:,12)-y(1,12), '-.', ...
    ...'markerfacecolor', vrggrad(9,10), ...
    'color', lightToDarkBlueGradient(9,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' PHASE '_2$' RATEXT] );
fr1Ra = plot(tra,yra(:,2), ':', ...
    ...'markerfacecolor', vrggrad(4,10), ...
    'color', lightToDarkBlueGradient(4,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_1$' RATEXT]);
fr2Ra = plot(tra,yra(:,13), ':', ...
    ...'markerfacecolor', vrggrad(8,10), ...
    'color', lightToDarkBlueGradient(8,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_2$' RATEXT] );

ph1dae = plot(t(raDAE),y(raDAE,1)-y(1,1), 'o', ...
    'markerfacecolor', vrggrad(2,10), ...
    'color', 'none', ...
    'markersize', 5, ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_1$' DAETEXT]);
ph2dae = plot(t(raDAE),y(raDAE,12)-y(1,12), 'o', ...
    'markerfacecolor', vrggrad(9,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' PHASE '_1$' DAETEXT] );
fr1dae = plot(t(raDAE),y(raDAE,2), 'o', ...
    'markerfacecolor', vrggrad(4,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_1$' DAETEXT]);
fr2dae = plot(t(raDAE),y(raDAE,13), 'o', ...
    'markerfacecolor', vrggrad(8,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_2$' DAETEXT]);

ph1ode = plot(TDEQ(raODE),YDEQ(raODE,1)-YDEQ(1,1), '-', ...
    'color', vrggrad(2,10), ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_1$' ODETEXT]);
ph2ode = plot(TDEQ(raODE),YDEQ(raODE,2)-YDEQ(1,2), '-', ...
    'color', vrggrad(9,10), ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_2$' ODETEXT], ...
    'linestyle', '--');
fr1ode = plot(TDEQ(raODE),YDEQ(raODE,3), '-', ...
    'color', vrggrad(4,10), ...
    'linewidth', 2, ...
    'displayname', ['$' FREQ '_1$' ODETEXT]);
fr2ode = plot(TDEQ(raODE),YDEQ(raODE,4), '-', ...
    'color', vrggrad(8,10), ...
    'linewidth', 2, ...
    'displayname', ['$' FREQ '_2$' ODETEXT], ...
    'linestyle', '--'); 

set( ax, 'fontsize', fontsiz, ...
    'fontname', fontname, ...
    'xtick', 0:0.5:5, ...
    'xlim', [0 4],...
    'ylim', [-0.05 0.4],...
    'ytick', [-0.05 0:0.1:0.4],...
    'xgrid', 'on', 'ygrid', 'on' ...
    );

xl = dissXLabel( '$t$', FORMAT );
yl = dissYLabel( '$\omega, \varphi$', FORMAT );

LEG_POS = [0.50113 0.2681 0.1797 0.71231];
leg = legend([ph1ode ph2ode fr1ode fr2ode ...
    ph1dae ph2dae fr1dae fr2dae ...
    ph1Ra ph2Ra fr1Ra fr2Ra]);
set( leg, ...
    'fontsize', 0.9*fontsiz,...
    'fontname', fontname, ...
    'interpreter', 'latex');
set( leg, 'position', LEG_POS );

% ================================ zoom  ================================ %
ax = axes; hold on;
set( ax, 'position', [(XOFF+XSIZE(1)+XSPACE) YOFF XSIZE(2) YSIZE] );

% red dots for making steps better visibel
raDots = find( and(t>1.49, t<1.51) );
rdotsFr1dae = plot(t(raDots),y(raDots,2), 'x', ...
    'markerfacecolor', 'none', ...
    'color', 'r', ...
    'linewidth', 1.2, ...
    'markersize', 10, ...
    'displayname', ' ');

ph1Ra = plot(tra,yra(:,1)-y(1,1), '-.', ...
    ...'markerfacecolor', vrggrad(2,10), ...
    'color', lightToDarkBlueGradient(2,10), ...
    'markersize', 5, ...
    'linewidth', 1.5, ...
    'displayname', ['$' PHASE '_1$' RATEXT]);
ph2Ra = plot(tra,yra(:,12)-y(1,12), '-.', ...
    ...'markerfacecolor', vrggrad(9,10), ...
    'color', lightToDarkBlueGradient(9,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' PHASE '_2$' RATEXT] );
fr1Ra = plot(tra,yra(:,2), ':', ...
    ...'markerfacecolor', vrggrad(4,10), ...
    'color', lightToDarkBlueGradient(4,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_1$' RATEXT]);
fr2Ra = plot(tra,yra(:,13), ':', ...
    ...'markerfacecolor', vrggrad(8,10), ...
    'color', lightToDarkBlueGradient(8,10), ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_2$' RATEXT] );

ph1dae = plot(t(raDAE),y(raDAE,1)-y(1,1), 'o', ...
    'markerfacecolor', vrggrad(2,10), ...
    'color', 'none', ...
    'markersize', 5, ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_1$' DAETEXT]);
ph2dae = plot(t(raDAE),y(raDAE,12)-y(1,12), 'o', ...
    'markerfacecolor', vrggrad(9,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' PHASE '_1$' DAETEXT] );
fr1dae = plot(t(raDAE),y(raDAE,2), 'o', ...
    'markerfacecolor', vrggrad(4,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_1$' DAETEXT]);
fr2dae = plot(t(raDAE),y(raDAE,13), 'o', ...
    'markerfacecolor', vrggrad(8,10), ...
    'color', 'none', ...
    'linewidth', 2, ...
    'markersize', 5, ...
    'displayname', ['$' FREQ '_2$' DAETEXT]);

ph1ode = plot(TDEQ(raODE),YDEQ(raODE,1)-YDEQ(1,1), '-', ...
    'color', vrggrad(2,10), ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_1$' ODETEXT]);
ph2ode = plot(TDEQ(raODE),YDEQ(raODE,2)-YDEQ(1,2), '-', ...
    'color', vrggrad(9,10), ...
    'linewidth', 2, ...
    'displayname', ['$' PHASE '_2$' ODETEXT], ...
    'linestyle', '--');
fr1ode = plot(TDEQ(raODE),YDEQ(raODE,3), '-', ...
    'color', vrggrad(4,10), ...
    'linewidth', 2, ...
    'displayname', ['$' FREQ '_1$' ODETEXT]);
fr2ode = plot(TDEQ(raODE),YDEQ(raODE,4), '-', ...
    'color', vrggrad(8,10), ...
    'linewidth', 2, ...
    'displayname', ['$' FREQ '_2$' ODETEXT], ...
    'linestyle', '--'); 

set( ax, 'fontsize', fontsiz, ...
    'fontname', fontname, ...
    'xtick', 1:0.1:1.5, ...
    'xlim', X_MAGN,...
    'ylim', Y_MAGN,...
    'ytick', 0.3:0.02:0.36,...
    'xgrid', 'on', 'ygrid', 'on' ...
    );


ALABOFF = [-0.01 0 0 0];
BLABOFF = ALABOFF + [(XOFF+XSIZE(1)+XSPACE*0.25) 0 0 0];
alo.siz = fontsiz;
alh = dissAlabel( fig, '(a)', alo );
apos = get( alh, 'position' );
set( alh, 'position', apos+ALABOFF);

blh = dissAlabel( fig, '(b)', alo );
% bpos = get( blh, 'position' );
set( blh, 'position', apos+BLABOFF);

% ================================ save  ================================ %
if( ~~save_fig )
    spth = 'figures/';
    saveas(fig, [spth 'swingCompareODEDAE.eps'], 'epsc');
end
