
%% figure A.11 of Dissertation illustrating the 
%% interplay of power, frequency and primary control
% case 5 is used for demonstration
% for data generation see fig_A_11primaryControlData.m
%

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

%% ============================= settings  ============================== %
save_fig = false;

FORMAT = 'hs';
fontsize = dissFS( FORMAT );
axfs = 0.95;    % factor for ax labels
fontn = 'Times New Roman';

X_TICKS = 1:10;
tpert = [0.5 1.5];

dottedLineWidth = 1.5;

detra = 1:3300;     % range where the plot is shown in detail not as strait line
detrap = 1:3300;

dpth = 'data/';

% ----------------------------- coordinates ----------------------------- %
XOFF = 0.15;
YOFF = 0.15;
X_AXSIZE = 0.75;
Y_AXSIZE = 0.375;
Y_SPACE = 0.05;


%% ============================= load data ============================== %
S = load( [dpth 'fig-A-6-primControlFreqTemp.mat'] );
ttemp = S.T;
ftemp = S.Y;
clear S;
S = load( [dpth 'fig-A-6-primControlFreqPerm.mat'] );
tperm = S.T;
fperm = S.Y;
clear S;

Pm0 = 2.1;      % unperturbed power of node 1
Pmtemp = @(t) myheavisideArr(t-tpert(1)).*(...
    1-myheavisideArr(t-tpert(2)))*0.5*Pm0 + Pm0;
Pmperm = @(t) myheavisideArr(t-tpert(1)).*0.5*Pm0 + Pm0;
D = 50/(60*2*pi);   % damping constant

detra = [detra numel(ttemp)];
ttemp = ttemp(detra);
detrap = [detrap numel(tperm)];
tperm = tperm(detrap);
% igl =  [1 5 2 3 4];     % trafo GL -> SEQ

%% ================================ plot ================================ %
% ==================== figure temporary perturbation ==================== %
fig1 = figure;
set( fig1, 'position', [123 56 dissFigSize(FORMAT)] );

% ----------------------------- upper graph ----------------------------- %
Y_LIM1 = [-0.6 1.6]*Pm0;

yaxlabP = {'$-0.5P_1^0$', '$0$', '$0.5P_1^0$', '$P_1^0$', '$1.5P_1^0$'};
% with ticklabelinterpreter latex
% yaxlabP = {'-0.5P_1^0', '0', '0.5P_1^0', 'P_1^0', '1.5P_1^0'};


subplot(2,1,1); hold on;
ax1 = gca;
ppow = plot( ttemp, Pmtemp(ttemp'), 'b', ...
    'linewidth', 1.5, ...
    'displayname', '$P_1$');
% for i = 1:5
ppri = plot(ttemp,-D*sum(ftemp(detra,6:10),2), 'color', vrggrad(3,10), ...   %i-5+1,7
        'linewidth', 1.5, 'displayname', '$-D\sum_i \omega_i$');  % 'total primary control'
% end
set( ax1, 'xgrid', 'on', 'ygrid', 'on', ...
    'position', [XOFF YOFF+Y_AXSIZE+Y_SPACE X_AXSIZE Y_AXSIZE ], ...
    'ytick', -(Pm0/2):(Pm0/2):Pm0*1.5, ...
    'yticklabels', yaxlabP, ...
    'fontsize', axfs*fontsize, 'fontname', fontn, ...
    'ylim', Y_LIM1, ...
    'xtick', X_TICKS, ...
    'xticklabels', {} ,...
    'ticklabelinterpreter', 'latex');
plot( [1 1]*tpert(1), Y_LIM1, 'k:', ...
    'linewidth', dottedLineWidth);
plot( [1 1]*tpert(2), Y_LIM1, 'k:', ...
    'linewidth', dottedLineWidth);
leg1 = legend([ppow ppri]);
set( leg1, 'interpreter', 'latex' );
l1p = get( leg1, 'position' );
set( leg1, 'position', [l1p(1) 0.888 l1p(3:4)] );


% ----------------------------- lower graph ----------------------------- %
Y_LIM2 = [-0.2 2];
pl = zeros(5,1);    % plot handles

subplot(2,1,2); hold on;
ax2 = gca;
for i = 1:5
    pl(i) = plot(ttemp,ftemp(detra,i+5), 'color', vrggrad(i,5), ...
        'linewidth', 2, 'displayname', ['node ' num2str(i)]);
end
set( ax2, 'xgrid', 'on', 'ygrid', 'on', ...
    'xtick', X_TICKS, ...
    'ytick', -0.5:0.5:2,...
    'ylim', Y_LIM2, ...
    'fontsize', axfs*fontsize, 'fontname', fontn, ...
    'position', [XOFF YOFF X_AXSIZE Y_AXSIZE ] );
plot( [1 1]*tpert(1), Y_LIM2, 'k:', ...
    'linewidth', dottedLineWidth);
plot( [1 1]*tpert(2), Y_LIM2, 'k:', ...
    'linewidth', dottedLineWidth);

leg2 = legend(pl);
% set( leg2, 'position', [] );

dissXLabel('$t$ [s]', FORMAT);
dissYLabel('$\omega$ [Hz]', FORMAT);
tbo.siz = fontsize;
tba = dissAlabel( fig1, '(a)', tbo );
tbap = get( tba, 'position' );
set( tba, 'position', tbap+[0.005 -0.001 0 0]);

% ----------------------------- save figure ----------------------------- %
if( ~~save_fig )
    opth = cpath( 'adiss', 'figures' );
    saveas( fig1, [opth 'PrimaryControlA.eps'], 'epsc' );
end

%% =================== figure permanent perturbation ==================== %
fig2 = figure;
FIG_SIZE = dissFigSize(FORMAT);
set( fig2, 'position', [123+FIG_SIZE(1) 56 FIG_SIZE] );

% ----------------------------- upper graph ----------------------------- %
Y_LIM1 = [-0.6 1.6]*Pm0;

yaxlabP = {'$-0.5P_1^0$', '$0$', '$0.5P_1^0$', '$P_1^0$', '$1.5P_1^0$'};
% with ticklabelinterpreter latex
% yaxlabP = {'-0.5P_1^0', '0', '0.5P_1^0', 'P_1^0', '1.5P_1^0'};


subplot(2,1,1); hold on;
ax1 = gca;
ppow = plot( tperm, Pmperm(tperm'), 'b', ...
    'linewidth', 1.5, ...
    'displayname', '$P_1$');
% for i = 1:5
ppri = plot(tperm,-D*sum(fperm(detrap,6:10),2), 'color', vrggrad(3,10), ...   %i-5+1,7
        'linewidth', 1.5, 'displayname', '$-D\sum_i \omega_i$');  % 'total primary control'
% end
set( ax1, 'xgrid', 'on', 'ygrid', 'on', ...
    'position', [XOFF YOFF+Y_AXSIZE+Y_SPACE X_AXSIZE Y_AXSIZE ], ...
    'ytick', -(Pm0/2):(Pm0/2):Pm0*1.5, ...
    'yticklabels', yaxlabP, ...
    'fontsize', axfs*fontsize, 'fontname', fontn, ...
    'ylim', Y_LIM1, ...
    'xtick', X_TICKS, ...
    'xticklabels', {} ,...
    'ticklabelinterpreter', 'latex');
plot( [1 1]*tpert(1), Y_LIM1, 'k:', ...
    'linewidth', dottedLineWidth);
leg1 = legend([ppow ppri]);
set( leg1, 'interpreter', 'latex' );
l1p = get( leg1, 'position' );
set( leg1, 'position', [l1p(1) 0.75 l1p(3:4)] );


% ----------------------------- lower graph ----------------------------- %
Y_LIM2 = [-0.2 2];
pl = zeros(5,1);    % plot handles

subplot(2,1,2); hold on;
ax2 = gca;
for i = 1:5
    pl(i) = plot(tperm,fperm(detrap,i+5), 'color', vrggrad(i,5), ...
        'linewidth', 2, 'displayname', ['node ' num2str(i)]);
end
set( ax2, 'xgrid', 'on', 'ygrid', 'on', ...
    'xtick', X_TICKS, ...
    'ytick', -0.5:0.5:2,...
    'ylim', Y_LIM2, ...
    'fontsize', axfs*fontsize, 'fontname', fontn, ...
    'position', [XOFF YOFF X_AXSIZE Y_AXSIZE ] );
plot( [1 1]*tpert(1), Y_LIM2, 'k:', ...
    'linewidth', dottedLineWidth);

leg2 = legend(pl);
l2p = get( leg2, 'position' );
set( leg2, 'position', [l2p(1) 0.168 l2p(3:4)] );

dissXLabel('$t$ [s]', FORMAT);
dissYLabel('$\omega$ [Hz]', FORMAT);
tbo.siz = fontsize;
tba = dissAlabel( fig2, '(b)', tbo );
tbap = get( tba, 'position' );
set( tba, 'position', tbap+[0.005 -0.001 0 0]);

% ----------------------------- save figure ----------------------------- %
if( ~~save_fig )
    opth = cpath( 'adiss', 'figures' );
    saveas( fig2, [opth 'PrimaryControlB.eps'], 'epsc' );
end























