
%% Data preparation for figure A.11 of Dissertation illustrating the 
%% interplay of power, frequency and primary control
% case 5 is used for demonstration
%

global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods')
% Path to MATPOWER needed:
addpath(genpath('C:\Users\Anwender\Documents\MATLAB\matpower7.0'));
% Path to pg_dyn_models 
% [Nishikawa and Motter, New J. Phys. 17, 015012 (2015) ]
% needed:
addpath('C:\Users\Anwender\Documents\MATLAB\pg_dyn_models');

%% ============================= settings  ============================== %
save_dat = false;

dpth = 'data/';

%% ============================ case 5 data ============================= %
N = 5;
mpc5 = loadcase( 'case5' );
mpc5 = gatherGen(mpc5);
mpc5 = ext2int(mpc5);

rso.IN = false;
rso.dynmodel = 'sm';
rso.redpass = false;
[sm5, det5] = raw2swing( mpc5, rso );

% unperturbed
Va = sm5.ssa;
Vf = zeros(5,1);
y0 = [Va; Vf]';       % first voltage phases then magnitudes as in swing2nd
te = 10;        % end time

par.damping = sm5.D/(60*2*pi);
par.inertia = 2*sm5.H/(60*2*pi);
par.powerd = sm5.Pd;
par.Kangle = sm5.gamma;
par.Kmag   = sm5.K;

%% ---------------------- temporary perturbation  ----------------------- %
% power perturbation
Pmd = @(t) [myheaviside(t-0.5)*(1-myheaviside(t-1.5))*0.5*sm5.Pm(1) ...
    + sm5.Pm(1);
      sm5.Pm(2);
      sm5.Pm(3);
      sm5.Pm(4);
      sm5.Pm(5);];

par.powerm = Pmd;

options = odeset('RelTol',1e-5,'AbsTol', [ones(1,N) ones(1,N)*0.01]*1e-10); %[1e-2 1e-4 1e-2 1e-4]);
[T,Y] = ode45( @(t,y) swing2nd(t,y,par), ...     % solve swing equation
    [0 te], y0, options);  %#ok<ASGLU> % limits, initial conditions and options

if( ~~save_dat )
    save( [dpth 'fig-A-6-primControlFreqTemp.mat'], 'T', 'Y' );
end

%% ---------------------- permanent perturbation  ----------------------- %
Pmd = @(t) [myheaviside(t-0.5)*0.5*sm5.Pm(1) ...
    + sm5.Pm(1);
      sm5.Pm(2);
      sm5.Pm(3);
      sm5.Pm(4);
      sm5.Pm(5);];

par.powerm = Pmd;

options = odeset('RelTol',1e-5,'AbsTol', [ones(1,N) ones(1,N)*0.01]*1e-10); %[1e-2 1e-4 1e-2 1e-4]);
[T,Y] = ode45( @(t,y) swing2nd(t,y,par), ...     % solve swing equation
    [0 te], y0, options);  % limits, initial conditions and options

if( ~~save_dat )
    save( [dpth 'fig-A-6-primControlFreqPerm.mat'], 'T', 'Y' );
end

%% quick view
% igl = [1 3 4 5 2];
igl =  [1 5 2 3 4];
figure;
% axes; hold on;
subplot(2,1,1); hold on;
for i = 6:10
    plot(T,Y(:,igl(i-5)+5), 'color', vrggrad(i-5,5), ...   %i-5+1,7
        'linewidth', 2, 'displayname', ['node ' num2str(i-5)]);
    set( gca, 'xgrid', 'on', 'ygrid', 'on' );
end
subplot(2,1,2); hold on;
for i = 1:5
    plot(T,Y(:,igl(i)), 'color', vrggrad(i,5), ...
        'linewidth', 2, 'displayname', ['node ' num2str(i)]);
    set( gca, 'xgrid', 'on', 'ygrid', 'on' );
end

legend('show');

