function [ rootz, idx ] = aproxZeros( y, x, opt )
%% aproxZeros( opt )
% Determines the roots of a function only given as a series of numbers.
% * Parameters
%   * y : The array (time series) whose roots are requested.
%   * (x) : The corresponding x-values (Otherwise the index is returned)
%   * (opt) : option struct
%           opt.sing = true/false : If the program should try to detect
%               zeros caused by singularities. default: false
%           opt.sig = 10 (default) : How many sigma intervals are used to
%               determine whether the slope is a singularity
%           opt.sfix = 0 (default) : A fixed value that determines the
%               slope as singular. Set this to zero if you want to use the
%               determination by standard deviation.
%
%  * Returns
%   * roots : the found roots
%   * idx : the indices of the roots in the y-array
%
%
% V1.0 mawolff@uos.de 21-Jun-2017
 
%% pseudocode
 
 
%% input handling
noopt = (nargin < 3 && ~isstruct(x)) || nargin < 2;
if( nargin < 2)
    x = 1:numel(y);
end
if( noopt ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

if( opt.sfix == 0 )         % fixed threshold for singularity or slope dependant
    dy = y(2:end)-y(1:(end-1)); % calc slope
    stdev = sqrt(var(dy));
    opt.sfix = opt.sig*stdev;
end

rootz = zeros(numel(y),1);
idx = zeros(numel(y),1);
k = 1;
for i = 2:numel(y)
    
    if( opt.sing && abs(y(i)-y(i-1)) > opt.sfix )   % besser vorher die y's rausnehmen
        continue;
    end
    
    s1 = y(i-1) >= 0;
    s2 = y(i) >= 0;
    if( s2 ~= s1 )
        y1 = y(i-1);
        y2 = y(i);
        x1 = x(i-1);
        x2 = x(i);
        rootz(k) = x2 - y2*(x2-x1)/(y2-y1);
        idx(k) = i;
        k = k + 1;
    end
 
end
rootz = rootz(1:(k-1));
idx = idx(1:(k-1));


return;

end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'sing') )
    opt.sing = false;
 end
 if( ~isfield( opt, 'sig') )
    opt.sig = 10;
 end
 if( ~isfield( opt, 'sfix') )
    opt.sfix = 0;
 end
end
