function [ ] = compareDotPlot( p, i, opt )
%% compareDotPlot( opt )
%
% if comparing results of N outcomes this will set different markers for
% the plot depending of the number i = 1...N of the data set
% Only N = 8 different plots are predefined
%
% * Parameters
%   * p : plot handle
%   * i : number of plot to compare 
%   * opt : option struct
%       * opt.markerseq = {'x','o','d', 's', ...
%                           '^', 'v', '<', '>'};
%       * opt.lwidth = [2 3 2 1.5 ...
%                           2 2 2 2];
%       * opt.colorseq = {'b','r','g', 'k', ...
%                           'b', 'r', 'g', 'k'};
%
%  * Returns
%   * 
%
%
% V1.0 mawolff@uos.de 14-Aug-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
 set( p, 'linestyle', 'none',...
     'marker', opt.markerseq{i}, ...
     'color', opt.colorseq{i}, ...
     'linewidth', opt.lwidth(i), ...
     'markersize', opt.msize(i) );
 ax = get( p, 'parent' );
 set( ax, 'xgrid', 'on', ...
     'xminorgrid', 'on');
 ax.XAxis.MinorTickValues = get(p, 'XData' );
 return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'markerseq') )
    opt.markerseq = {'x','o','d', 's', ...
        '^', 'v', '<', '>'};
 end
 if( ~isfield( opt, 'lwidth') )
     opt.lwidth = [2 3 2 1.5 ...
         2 2 2 2];
 end
 if( ~isfield( opt, 'msize') )
     opt.msize = [8 3 8 8 ...
         6 6 6 6];
 end
 if(  ~isfield( opt, 'colorseq') )
     opt.colorseq = {'b','r','g', 'k', ...
         'b', 'r', 'g', 'k'};
 end
end
