function [ G, pg ] = createHeGraph( opt )
%% createHeGraph( opt )
%
% Creates a graph struct to plot based on the mpc for Het+IN model and
% SM_model. The only lines with weight larger than opt.thresh are shown.
%
% * Parameters
%   * opt : option struct
%       * opt.thresh = 0.1;
%       * opt.mpc = saveAndLoadMPC( 'RTS96' ); from which mpc file the
%               graph is created
%
%  * Returns
%   * G : Graph struct
%
%
% V1.0 mawolff@uos.de 15-Aug-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

mpche = opt.mpc;
sm = SM_model( mpche );
trio.diag = false;
trio.ud = 'urt';
Ktri = tri(sm.K, trio);
[x,y]= find( Ktri > opt.thresh);

xseq = igl2iseq(x, opt.mpc);
yseq = igl2iseq(y, opt.mpc);
G = graph(xseq,yseq);

Kseq = glM2seqM( sm.K, opt.mpc);
G = addWeight(Kseq,G);

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'thresh') )
    opt.thresh = 0.1;
 end
 if( ~isfield( opt, 'mpc') )
    opt.mpc = saveAndLoadMPC( 'RTS96');
 end
 
end
