function [ dV ] = delta( V, opt )
%% delta( V, opt )
% Calculates the difference of a number in a vector and its next number.
%   E.g. V = [1 4 5 9]. delta(V) = [3 1 4]. delta(V,2) = 1.
% * Parameters
%   * V : The vector whose difference should be calculated.
%   * opt/i : If a single number the difference at (i+1) - i is calculated
%       otherwise if its a struct, it is used as option struct
%       opt.idx = i : same as single number i.
%
%  * Returns
%   * dV : The differences of the vector or two elements
%
%
% V1.0 mawolff@uos.de 02-Jul-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.idx = 1:(numel(V)-1); 
end
if( nargin == 2)
    if( ~isstruct(opt) )
        ti = opt;
        clear opt;
        opt.idx = ti;
    end 
end
 
 opt = readOptions( opt );
 
 if( numel(opt.idx) >= numel(V) )
     opt.idx = opt.idx(1:(numel(V)-1));
 end
 
%% code

dV = V(opt.idx + 1) - V(opt.idx);
return;
 
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'idx') )
    opt.idx = 1:(numel(V)-1); 
 end
end
