function [ tb, tbg ] = dissAlabel( fig, str, opt )
%% dissAlabel( fig, txt, opt )
%
% creates a label in the top left corner of the given figure. This is meant
% to create labels for subfigures like '(a)'. It also creates a
% "ghost"-textbox in the bottom right corner. This results in the figure to
% be stored full size. Accordingly all subfigures should be of same size.
%
% * Parameters
%   * fig : the figure to plot the label into
%   * txt : the string of the label, usually '(a)', '(b)' etc.
%   * opt
%       * opt.siz = 12;
%       * opt.bordc = '-'; no border by default
%       * opt.bgc   = '-'; no background by default
%       * opt.fontname = 'Times New Roman';
%
%  * Returns
%   * tb  : handle of the textbox containing the label
%   * tbg : handle of the ghost textbox to force correct figure storage
%
%
% V1.0 mawolff@uos.de 25-Mar-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
figure(fig);

tb = tbox( [0.005 0.95 0.05 0.05], str, opt.siz, ...
    opt.bordc, opt.bgc );
set( tb, 'fontname', opt.fontname, 'fontweight', 'bold');
tbg = tbox( [0.99 0 0.01 0.01], ' ', 1, ...
    'none', 'none' );   % ghost text box to force matlab to store figure full screen

return;
 
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'bordc') )
    opt.bordc = '-';
 end
 if( ~isfield( opt, 'bgc') )
    opt.bgc = '-';
 end
 if( ~isfield( opt, 'siz') )
    opt.siz = 12;
 end
 if( ~isfield( opt, 'fontname' ) )
%      opt.fontname = 'LM Roman 12';
     opt.fontname = 'Times New Roman';
 end
end
