function [ M ] = expand( v, opt )
%% expand( v, opt )
%
% * Parameters
%   * v : the vector which should be expanded to a matrix
%   * opt : option struct:
%       * opt.d = 'h' : direction 'h' horizontal expanded e.g. 
%                1         1 1 1
%             v= 2  -> M=  2 2 2
%                3         3 3 3
%       * opt.d = 'v' vertical expanded
%            v= 1 2 3
%           
%               1 2 3
%         -> M= 1 2 3
%               1 2 3
%          If d is not given the orientiation of the vector decides.
%          (if v = v(N,1) 'h' is default otherwise 'v')
%       * opt.dim = []; if empty v is expanded to sqaure matrix otherwise
%               M has dimension dim x dim
%
%  * Returns
%   * M: the resulting matrix
%
%
% V1.1 mawolff@uos.de 13-Apr-2018

% V1.1 mawolff@uos.de 13-Apr-2018 ( dim added )

% V1.0 mawolff@uos.de 19-Jun-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 opt.vs = size(v);
 opt = readOptions( opt );
 
%% code
 
if( isempty( opt.dim ) )
    N = max(opt.vs);
    [dx,dy] = meshgrid(1:N, 1:N);
    if( strcmp(opt.d,'h'))
        M = v(dy);
    else
        M = v(dx);
    end
else
    if( strcmp(opt.d,'h'))
        [dx,~] = meshgrid(1:opt.vs(1), 1:opt.dim);
        M = v(dx');
    else
        [~,dy] = meshgrid(1:opt.dim, 1:opt.vs(2));
        M = v(dy');
    end
end

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'd') )
     if( opt.vs(1) > opt.vs(2))
         opt.d = 'h';
     else
         opt.d = 'v';
     end
 end
 if( ~isfield( opt, 'dim') )
     opt.dim = [];
 end
end
