function [idx, uidx, idxl, skipped] = findDlines( mpc, opt )
%% [idx, uidx, idxl] = findDlines( opt )
%
% Returns the indces of the lines that have the same starting and end node.
%
% * Parameters
%   * mpc : grid data in matpower case format (mpc.branch is relevant)
%
%  * Returns
%   * idx  : indices of the lines occuring multiple
%           (i.e. there are lines with higher indices that start at the
%            same node and end at the same node as this line)
%   * uidx : indices of the lines when multiple lines are removed
%           (assuming the line with the lowest index remains)
%   * idxl : list of indices referring to the multiple lines like:
%           {[1,5,6];[2,3]} when lines 1, 5 and 6 are identical and
%           lines 2 and 3 are identical
%
%
% V1.0 mawolff@uos.de 26-Jun-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 nodebeg = mpc.branch(:,1); % nodes where lines begin
 nodeend = mpc.branch(:,2); % nodes where lines end
 
%% code
N = numel(nodebeg); % number of lines
idx = nan(N,1);
uidx = nan(N,1);
idxl = cell(N,1);

dlines = nan(N,1);
udls = 0;       % no unique doubled lines yet
dls = 0;        % no doubled lines yet
skipped = 0;    % no skipped lines yet
for i=1:N 
    if( any(i==dlines) )    % if i was indicated as duplicated line already
        skipped = skipped + 1;  % note that is skipped and
        continue;           % dont consider it
    end
    j=find( nodebeg==nodebeg(i) );
    k0=find( nodeend(j)==nodeend(i) );
    if( numel( k0 ) > 1 )
        dl = j(k0(2:end));
        dlines( (dls+1):(dls+numel(dl)) ) = dl;
        dls = dls + numel(dl);
        udls = udls + 1;
        idx(udls) = i;
        uidx(udls) = i-skipped;
        idxl{udls} = j(k0);
    end
end

% idx  = idx( ~isnan(idx) );
% uidx = uidx( ~isnan(uidx) );
% idxl = idxl( ~isempty(idxl) );
idx = idx(1:udls);
uidx = uidx(1:udls);
idxl = idxl(1:udls);
return; 
 
end
 
 
function [ opt ] = readOptions( opt )
 % AUTO-GENERATED option handling
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
end
