function [tnode, inode, weight] = getInternalNodesFSM( mpc, xd,  opt )
%% [tnode, inode, weight] = getInternalNodesFSM( mpc, xd )
% [tnode, inode, weight] = getInternalNodesFSM( mpc )
%
% * Parameters
%   * mpc
%   * xd  : the (direct) internal reactances are calculated according to 
%           Nishikawas formula ( xd= 92.8 * P^(-1.3) ) when not given 
%           explicitly. The length of this vector must be equal to the 
%           number of internal nodes in the 'fsm' model.
%           It is equal to the number of generators plus the number
%           of loads (buses with load). (per bus: first gens then load)
%
%  * Returns
%   * tnode : the terminal nodes corresponding to the internal nodes 
%   * inode : the internal node numbers
%   * weight: the weight of the connections between internal and terminal
%               nodes (1/xd)
%
%
% V1.0 mawolff@uos.de 30-Jun-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
if( nargin < 2 )
    xd = [];
end
 
 opt = readOptions( opt );
 
%% code

xdf = @(P) 92.8 * abs(P).^(-1.3);
 
idxLoad = mpc.bus(:,3) ~= 0;
% idxGen = mpc.gen(:,1);
Nbus = size(mpc.bus,1);
Ng = size( mpc.gen, 1);
Nl = sum( idxLoad );

tnode = [ mpc.gen(:,1); mpc.bus(idxLoad,1) ];
inode = (1:(Ng+Nl))' + Nbus;

Pg = mpc.gen(:,2);
Pl = mpc.bus(idxLoad,3);

if( isempty( xd ) )
    weight = 1./ xdf( [Pg; Pl] );
else
    weight = 1./xd;
end

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 % AUTO-GENERATED option handling
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
end
