function [ maxT, limcy ] = getMaxRT( sur, opt )
%% getMaxRT( surf, opt )
% Determines the maximum value (return time) of the given surface
%
% * Parameters
%   * sur: The z-component of the return time surface
%   * opt: option struct
%       * opt.lim = inf : if there is a time to say this is limit cycle and
%                           no real return time
%   
%
%  * Returns
%   * maxT : The biggest return time in this surface
%   * limcy : if the limit cycle was reached (defined by opt.lim)
%
%
% V1.0 mawolff@uos.de 23-Oct-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

if( any(any( sur >= opt.lim)) )
    limcy = true;
    sur(sur >= opt.lim) = 0;
else
    limcy = false;
end
maxT = 0;
for i = 1:size(sur,1)
    for j = 1:size(sur,2)
        maxT = max(sur(i,j),maxT);
    end
end
 
return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'lim') )
    opt.lim = inf('double');
 end
end
