function [ gbus, lbus ] = getglbus( mpc, opt )
%% [ genbus, loadbus ] = getglbus( mpc, opt )
% [ genbus, loadbus ] = getglbus( mpc )
%
% for compatibility with old versions, still:
% [ genbus, loadbus ] = getglbus( mpc, '-idx' )  (deprecated)
%
%   get vectors with the ids of the generators busses and load busses
%   seperately. Works even if busses occur multiple and have not contineous
%   indices (e.g. 101, 101, 201, 202, 203...)
%
% * Parameters:
%   * mpc : [mpc format, see help of: caseformat.m or Matpower manual]
%   * opt : option struct
%       * opt.idx = true; return only the indices not the bus names
%
% * Returns
%   * gbus: The vector with all generator busses
%   * lbus: The vector with all load busses
%
% V1.1 mawolff@uos.de 30-Jun-2020

% * improved option handling

% V1.0 mawolff@uos.de 11-Jan-2017

%% code
%% input handling
if( nargin == 1)
    opt = struct();
else
    if( ~isstruct( opt ) )
        o2 = opt;
        clear opt;
        opt.idx = strcmpi( o2, '-idx' );
    end
end

opt = readOptions( opt );

%% calculations
gbus = unique( mpc.gen(:,1) );
gind = zeros( numel(gbus), 1);
lbus = mpc.bus(:,1);
lind = zeros( numel(lbus)-numel(gbus), 1 );
i = 1;
j = 1;
gj = 1; 
lj = 1;
while( i <= numel(lbus) )
    if( any(lbus(i)==gbus))
        lbus(i) = [];
        gind(gj) = j;
        j = j + 1;
        gj = gj + 1;
    else
        i = i + 1;
        lind(lj) = j;
        j = j + 1;
        lj = lj + 1;
    end
end

if( opt.idx )
    gbus = gind;
    lbus = lind;
end
return;

end

function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'idx') )
    opt.idx = true;
 end
 
end
