function [ seqM ] = glM2seqM( glM, mpc, opt )
%% glM2seqM( glM, mpc, opt )
% converts a matrix e.g. [1, 2, 3; 4, 5, 6; 7, 8, 9]_gl in sequentiell 
% format to a generator load indexing e.g. 
% [1, 3, 2; 7, 9, 8; 4, 6, 5]_seq
% if 1 and 3 are generators and 2 is load
%
%
% * Parameters
%   * glM : matrix in gl indexing
%   * mpc : the mpc-struct containing information which nodes are
%           generators and which are loads. If not provided 'RTS96' is
%           loaded
%   * opt : option struct
%           * opt.vdir = 'v'; the shaping direction if the in- and output
%                       is a vector
%
%  * Returns
%   * seqM : the corresponding matrix with seq indexing
%
%
% V1.01 mawolff@uos.de 24-May-2018

% V1.01 vector input added
 
%% pseudocode
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
if( nargin < 2 ) 
   mpc = saveAndLoadMPC('RTS96');
end
 
 opt = readOptions( opt );
 
%% code
 
% N = max(size(glM));
N = size(mpc.bus,1);
seq = iseq2igl(1:N,mpc);
if( numel(glM)== N )    % if its a vector not a matrix
    glM = shape(opt.vdir,glM);
    seqM = glM(seq,1);
elseif( numel(glM) == N*N )
    seqM = glM(seq,seq);
else
    if( size( glM,1 ) == N )
        seqM = glM(seq,:);
    else
        seqM = glM(:,seq);
    end
end
return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'vdir') )
    opt.vdir = 'v';
 end
end
