function [ ] = highlightEdges( p, G, opt )
%% highlightEdges( p, G, opt )
% The width of the edges is adjusted proportional to their weight obtained
% from G.Edges.Weight .
%
% * Parameters
%   * p : the plot handle of the graph G
%   * G : graph G
%   * opt: option struct
%       * opt.alpha = 1; the transparency of the edges (0 is complete
%                   transparent)
%       * opt.color = [0.6 0.6 0.6]; The color of the edges
%                       or N x 3 matrix where N is number of edges and the
%                       rows represent the colors in rgb
%       * opt.wfactor = 10; The proportional factor. The edge with highest
%                   weight has a width equal to the wfactor
%       * opt.arrowsize = 0.001; The size of the arrowhead indicating the 
%                   direction of the edge
%
%  * Returns
%   * 
%
%
% V1.0 mawolff@uos.de 17-May-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.alpha = 1; 
end

isdirected = strcmp( class(G), 'digraph');
 
 opt = readOptions( opt );
 
%% code
 
p.LineWidth = (G.Edges.Weight)/max(G.Edges.Weight)*opt.wfactor + opt.wmin; % + 2;
p.EdgeAlpha = opt.alpha;
p.EdgeColor = opt.color;

if( isdirected)
    p.ArrowSize = opt.arrowsize;
end
 
end
 
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'alpha') )
    opt.alpha = 1;
 end
 
 if( ~isfield( opt, 'color') )
    opt.color = [1 1 1]*0.6;
 end
 
 if( ~isfield( opt, 'wfactor') )
    opt.wfactor = 10;
 end
 if( ~isfield( opt, 'arrowsize') )
    opt.arrowsize = 0.001;
 end
 if( ~isfield( opt, 'wmin') )
    opt.wmin = 0;
 end
 
 
end
