function [ igl ] = iseq2igl( iseq, mpc, opt )
%% iseq2igl( iseq, mpc, opt )
% converts an index-series e.g. [1, 3, 2, 5]_gl in gl (generator load)
% format to a sequentiell index-series e.g. [1, 7, 2, 14]_seq
%
% here the following translation was used:
%      seq   gl
%      1     1
%      2     2
%      3    34
%      4    35
%      5    36
%      6    37
%      7     3
%       ...   ...
%
%
% * Parameters
%   * iseq : index-series in seq format
%   * mpc : the mpc-struct containing information which nodes are
%           generators and which are loads. If not provided 'RTS96' is
%           loaded
%   * opt : option struct (nothing yet)
%
%  * Returns
%   * igl : the corresponding index series in gl format
%
%
% V1.0 mawolff@uos.de 23-May-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
if( nargin < 2 ) 
   mpc = saveAndLoadMPC('RTS96');
end
 
 opt = readOptions( opt );
 
%% code
 
[gb, lb] = getglbus(mpc,'-idx');

gl = [gb;lb];
N = numel(gl);

[~,iglsort] = sortrows([(1:N)' gl],2);

igl = iglsort(iseq);
return;
 
end
 
 
function [ opt ] = readOptions( opt )
 % AUTO-GENERATED option handling
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
end
