function [ xg, yg ] = loadGraphXY( file, opt )
%% [xg, yg] = loadGraphXY( file, opt )
% [xg, yg] = loadGraphXY( file )
% [xg, yg] = loadGraphXY( 'ui' )
% [xg, yg] = loadGraphXY( opt )
% 
% 
% * Parameters
%   * file : the file to read the coordinates of the graph from
%       * if file == 'ui', a UI window will open to choose a file
%   * opt  : option struct
%       * opt.ui = false; if a file selection via UI is requested
%
%  * Returns
%   * xg : the x-values of the grid node coordinates
%   * yg : the y-values of the grid node coordinates
%
%
% V1.0 mawolff@uos.de 22-Aug-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
    if( isstruct( file ) )
        opt = file;
    else
        opt.ui = false;
        if( strcmp( file, 'ui') )
            opt.ui = true;
        end
    end
end
 
 opt = readOptions( opt );
 
%% code

if( opt.ui )
    [fil, pth ] = uigetfile( {'*.mat', '.mat'; ...
        '*.*', 'all files'} );
    file = [pth fil];
end

try
    
    S = load( file );

    if( isfield( S, 'tab' ) )
        xg = S.tab(:,1);
        yg = S.tab(:,2);
    elseif( isfield( S, 'xg' ) )
        xg = S.xg;
        yg = S.yg;
    end
    
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be found.']);  
        otherwise 
            rethrow( ME ); 
    end 
end 

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'ui') )
    opt.ui = false;
 end
end
