function [ ] = plotTicks( ax, z, len, opt )
%% plotTicks( ax )
% plotTicks( ax, z )
% plotTicks( ax, opt )
% plotTicks( ax, z, len )
% plotTicks( ax, z, len, opt )
%
% Plots black lines on the ticks specified for the given axes.
%
% * Parameters
%   * ax  : axes to re-plot the ticks
%   * z   : height of the ticks (default: intmax('int32') )
%   * len : length of the ticks (default: 3% of axes limits)
%           len(1) = length of x-ticks, len(2) = length of y-ticks
%   * opt
%       * opt.color = 'k'; color of the ticks
%       * opt.linewidth = 1.5; linewidth of the ticks
%       * opt.xtick = get( ax, 'xtick' );
%       * opt.ytick = get( ax, 'ytick' );
%
%  * Returns
%   * nothing yet
%
%
% V1.0 mawolff@uos.de 14-Jul-2020
 
%% pseudocode
 
 
%% input handling
YLIM = get( ax, 'ylim' );
XLIM = get( ax, 'xlim' );

if( nargin < 4 )
    if( nargin == 2 )
        if( isstruct( z ) )
            opt = z;
            clear z;
            z = double(intmax('int32'));
        end
    else
        opt = struct();
    end
end
if( nargin < 3 )
    len = [0.03*diff(YLIM) 0.03*diff(XLIM)];
end
if( nargin < 2 )
    z = double(intmax('int32'));
end

opt = readOptions( opt, ax );

if( strcmpi( opt.ypos, 'left' ) )
    ytickx = [0 len(2)]+XLIM(1);
elseif( strcmpi( opt.ypos, 'right' ) )
    ytickx = XLIM(2)-[len(2) 0];
else
    error( '''ypos'' has to be ''left'' or ''right''.' );
end

%% code
% set( ax, 'Layer', 'top'); % ticks over surface plot
% grid off

for i = 1:numel( opt.xtick )
    plot3( [1 1]*opt.xtick(i), [0 len(1)]+YLIM(1), [1 1]*z, ...
        'color', opt.color, ...
        'linewidth', opt.linewidth );
end
for i = 1:numel( opt.ytick )
    plot3( ytickx, [1 1]*opt.ytick(i), [1 1]*z,...
        'color', opt.color, ...
        'linewidth', opt.linewidth );
end
 
end
 
 
function [ opt ] = readOptions( opt, ax )
 
 if( ~isfield( opt, 'color') )
    opt.color = 'k';
 end
 if( ~isfield( opt, 'linewidth') )
    opt.linewidth = 1.5;
 end
 if( ~isfield( opt, 'xtick') )
    opt.xtick = get( ax, 'xtick' );
 end
 if( ~isfield( opt, 'ytick') )
    opt.yticks = get( ax, 'ytick' );
 end
 if( ~isfield( opt, 'ypos') )
    opt.ypos = 'left';
 end

end
