function [ ] = rotYlabel( ylh, ax, opt )
%% rotYlabel( opt )
%
% rotates the y label for 2D plots such that it is upright/vertical 
%
% * Parameters
%   * ylh : the handle of the y label
%   * (ax): the axis corresponding to the plot. If not given the current
%           axis is chosen
%
%  * Returns
%   * nothing
%
%
% V1.0 mawolff@uos.de 14-Jan-2019
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
if( nargin < 2 ) 
   ax = gca;
end
 
 opt = readOptions( opt );
 
%% code
 
xb = get( ax, 'xlim' );
yb = get( ax, 'ylim' );
xsc = get( ax, 'xscale' );
ysc = get( ax, 'yscale' );
if( strcmpi( xsc, 'linear' ) )
%     hint: delta is implemented in MATLAB versions newer than 2017 afaik
    xpos = min(xb) - 0.1* delta( xb ); 
elseif( strcmpi( xsc, 'log' ) )
    xpos = 10^( min(log10(xb)) - 0.1* delta( log10(xb) ) ); 
else
    error( 'The scale of the x axis could not be determined' );
end
if( strcmpi( ysc, 'linear' ) )
    ypos = mean( yb );
elseif( strcmpi( ysc, 'log' ) )
    yb( yb == 0) = 1;
    ypos = 10^(mean( log10(yb) ));
else
    error( 'The scale of the y axis could not be determined' );
end

set(ylh, 'Rotation', 0);
set(ylh, 'Position', [xpos ypos 0]);

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'yscale') )
    opt.yscale = 'lin';
 end
end
