function [  ] = saveGraphXY( xg, yg, file, opt ) %#ok<INUSL>
%% [xg, yg] = saveGraphXY( file, opt )
% [xg, yg] = saveGraphXY( xg, yg, file )
% [xg, yg] = saveGraphXY( xg, yg, 'ui' )
% [xg, yg] = saveGraphXY( xg, yg, opt )
% 
% 
% * Parameters
%   * xg   : x-coordinates of graph
%   * yg   : y-coordinates of graph
%   * file : the file to save the coordinates of the graph to
%       * if file == 'ui', a UI window will open to choose a file
%   * opt  : option struct
%       * opt.ui = false; if a file selection via UI is requested
%
%  * Returns
%   * 
%
%
% V1.0 mawolff@uos.de 20-Mar-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 4 ) 
    if( isstruct( file ) )
        opt = file;
    else
        opt.ui = false;
        if( strcmp( file, 'ui') )
            opt.ui = true;
        end
    end
end
 
 opt = readOptions( opt );
 
%% code

if( opt.ui )
    [fil, pth ] = uiputfile( {'*.mat', '.mat'; ...
        '*.*', 'all files'}, 'Save graph coordinates', opt.defpath );
    file = [pth fil];
end

try
    
    save( file, 'xg', 'yg' );

%     if( isfield( S, 'tab' ) )
%         xg = S.tab(:,1);
%         yg = S.tab(:,2);
%     elseif( isfield( S, 'xg' ) )
%         xg = S.xg;
%         yg = S.yg;
%     end
    
catch ME 
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be found.']);  
        otherwise 
            rethrow( ME ); 
    end 
end 

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'ui') )
    opt.ui = false;
 end
 if( ~isfield( opt, 'defpath') )
    opt.defpath = cpath('func', 'GraphTools\GraphCoord');
 end
end
