function [ tbox ] = textbox( coords, str, siz, bordc, bgc, opt )
%% textbox( coords, string, size, bordc, bgc, opt)
% textbox( coords, string, size, bordc, bgc)
% textbox( coords, string, size, opt)
% textbox( coords, string, size)
%
% creates a textbox in the current figure
% * Parameters:
%   * coords : coords of the box x1 y1 w h
%   * string : the string of the box
%   * size : the fontsize
%   * (bordc) : color of the border in [rgb] (or string => 'none') (optional)
%   * (bgc) : backgroundcolor in [rgb] (or string => 'none') (optional)
%   * (opt) : optionstruct
%       * opt.
%
% * Returns:
%   * tarrow : the handle of the textbox
%
% V1.1 mawolff@uos.de 22-Nov-2017

%% code

if( nargin == 4)
    if( isstruct( bgc) )
        opt = bgc;
        clear bgc;
        bgc = '-';
        bordc = '-';
    end
end
if ( nargin < 5 )
    bgc = '-';
end
if ( nargin < 4 )
    bordc = '-';
end
if( nargin < 6 )
    opt.f = 0;
end
    opt = readOpt( opt );

x1 = coords(1);
y1 = coords(2);
w = coords(3);
h = coords(4);
tbox = annotation('textbox',[x1 y1 w h],'String', str);
set(tbox,'fontsize',siz);
if( ischar(bordc) )
    set(tbox,'edgecolor', 'none');
else
    set(tbox,'edgecolor', bordc(1:3));
end
if( ischar(bgc) )
    set(tbox,'backgroundcolor', 'none');
else
    set(tbox,'backgroundcolor', bgc(1:3));
end
set( tbox, 'HorizontalAlignment', opt.alignx,...
        'VerticalAlignment', opt.aligny);

return;
end

function [ opt ] = readOpt( opt)

    if( ~isfield( opt, 'alignx'))
        opt.alignx = 'center';
    end
    
    if( ~isfield( opt, 'aligny'))
        opt.aligny = 'middle';
    end
end

