function [ Mt ] = tri( M, opt )
%% tri( M, opt )
% Deletes entries such that the Matrix becomes an upper right triangle or
% lower left.
% * Parameters
%   * M : The matrix which entries should be deleted
%   * opt: options:
%       * opt.ud = 'urt' (upper right triangle, default)
%       * opt.ud = 'llt' (lower left triangle)
%       * opt.diag = true (default)/false: if the diagonal elements are
%               kept (only possible for quadratic matrices).
%
%  * Returns
%   * Mt: matrix in triangle form
%
%
% V1.0 mawolff@uos.de 09-Jun-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
[mx,my] = meshgrid(1:size(M,1),1:size(M,2));
if( strcmp( opt.ud, 'urt' ) )
    urt = mx > my;        % upper right triangle of the matrix
    if(opt.diag)
        urt = urt+ eye(size(M,1));
    end
    Mt = M.*urt;
else
    llt = mx < my;
    if(opt.diag)
        llt = llt + eye(size(M,1));
    end
    Mt = M.*llt;
end

return;
    
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'ud') )
    opt.ud = 'urt';
 end
 if( ~isfield( opt, 'diag') )
    opt.diag = true;
 end
end
