function [ rgb] = unicolors( which, opt )
%% unicolors( which, o )
% unicolors( which )
% unicolors( which, '-1' )
%
% returns the fitting rgb tripple of the wanted color in rage of 0 to 255
% or 0 to 1
%
% * Parameters
%   * which: char defining color, possibilities:
%       'darkred'       or 'dr'
%       'red'           or 'r'
%       'darkorange'    or 'do'
%       'orange'        or 'o'
%       'lightorange'   or 'lo'
%   * opt: option struct
%       * opt.crange = '-1'; returning colors in range [0 1]
%                      '255' or ''; returning colors in range [0 255]
%   
%
%  * Returns
%   * rgb : 3x1 vector
%
%
% V1.1 mawolff@uos.de 15-Jul-2020

% * added default option and updated option handling

%% input handling
if( nargin < 2 ) 
   opt = struct(); 
end
if( nargin == 2 )
    if( ischar( opt ) )
        cr = opt;
        clear opt;
        opt.crange = cr;
    end
end
 
opt = readOptions( opt );
 
%% code
if( regex(opt.crange, '-1') )
    f = 255;
else
    f = 1;
end

switch lower(which)
    case {'darkred', 'dr'}
        rgb = [176 16 53]/f;
        return;
    case {'red', 'r'}
        rgb = [255 163 143]/f;
        return;
    case {'darkorange','do'}
        rgb = [255 179 21]/f;
        return;
    case {'orange','o'}
        rgb = [255 220 94]/f;
        return;
    case {'lightorange','lo'}
        rgb = [255 232 184]/f;
        return;
end

warning('color not found, returning white.');
rgb = [1 1 1]*255/f;
return;
end

function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'crange') )
    opt.crange = '-1';
 end
 
end