function [ xn, yn ] = xyin0100( x, y, opt )
%% [xn, yn] = xyin0100( x, y, opt )
% [xn, yn] = xyin0100( x, y )
%
% maps a set of xy-values into the interval [b/2 100-b/2], where b is an 
% adjustable border
%
%
% * Parameters
%   * x  : the x values 
%   * y  : the y values
%   * opt: 
%       * opt.b = 0.05; by default 5% border
%       * opt.max = 100; the range of the interval xyn in [b/2 opt.max-b/2]
%
%  * Returns
%   * xn : new x values mapped  into the interval [b/2 100-b/2]
%   * yn : new y values mapped  into the interval [b/2 100-b/2]
%
%
% V1.0 mawolff@uos.de 26-Jun-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 3 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
hb = opt.b /2;      % half border abbreviation
xn = x - min(x);    % x values starting at 0
yn = y - min(y);    % y values starting at 0
xn = hb + (1-opt.b)*(xn/max(xn));
yn = hb + (1-opt.b)*(yn/max(yn));
xn = opt.max(1) *xn;
yn = opt.max(2) *yn;

return;
 
end
 
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'b') )
    opt.b = 0.05;
 end
 if( ~isfield( opt, 'max') )
    opt.max = [100 100];
 end
 if( numel(opt.max) == 1 )
     opt.max = [1 1]*opt.max;
 end
 
end
