function [ G ] = ybus2Graph( y0, opt )
%% ybus2Graph( y0, opt )
%
% * Parameters
%   * y0 : admittance matrix of the network
%   * opt : option struct
%       * opt.xg : nodes x-coordinates
%       * opt.yg : nodes y-coordinates
%       * opt.lscale = @(x) x : weight scale function for lines
%       * opt.nodec : node color vector
%
%  * Returns
%   * G : matlab Graph struct
%
%
% V1.0 mawolff@uos.de 24-Jan-2018
 
%% pseudocode
 
 
%% input handling

if( nargin < 2 )
    opt.f = 0;
end

%  opt.N = size( mpc.bus,1);                   % get number of nodes 
 
%% code

lopt.diag = false;              % no self loops
lns = tri( y0, lopt );          % get the line matrix
[fb,tb] = find( abs( lns ) > 0 );   % get from and to nodes/busses
Nl = numel(fb);                 % number of lines

 weights = zeros(size(y0,1),size(y0,2));     % predefine weights-matrix
 for i = 1:size(weights,1)                   % iterate through its upper
     for j = (i+1):size(weights,1)           % triangle to make it symmetric
         weights(i,j) = abs(y0(i,j)+y0(j,i))/2; % define weight as mean of the
         weights(j,i) = weights(i,j);        % from-to and to-from admittance
     end
 end
 ws = zeros( Nl ,1);                         % weight of single line is
 for i = 1:Nl                                % for every line
     ws(i) = weights(fb(i), tb(i));          % the corresponding matrix entry
 end

Nlabel = unique([fb;tb]);
opt.N = numel( Nlabel );
opt = readOptions( opt );
nodeName = cell(opt.N,1);                  % nodeName must be cell
 for i = 1:opt.N                           % iterate through all nodes
     nodeName{i} = num2str(Nlabel(i));   % give them the corresponding number as name
 end
 
 if( numel(opt.xg) < opt.N )
     nodes = table(nodeName, Nlabel,opt.nodec, ...
         'VariableNames', {'Name','NodeLabel','NodeColor'} );
 else
     nodes = table(nodeName, opt.xg, opt.yg, Nlabel,opt.nodec, ...
         'VariableNames', {'Name','XCoord','YCoord','NodeLabel','NodeColor'} );
 end
 edges = table([fb tb], opt.lscale(ws), ...
     'VariableNames',{'EndNodes', 'Weight'});
 
%  if( opt.dir )
%      G = digraph( edges, nodes);
%  else
G = graph( edges, nodes);
%  end
 return;
 
end
 
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'xg') )
    opt.xg = 0;
 end
 if( ~isfield( opt, 'yg') )
    opt.yg = 0;
 end
 if( ~isfield( opt, 'lscale') )
    opt.lscale = @(x) x;
 end
 if( ~isfield( opt, 'nodec') )
    opt.nodec = 0.7*ones(opt.N,1);
 end
 if( ~isfield( opt, 'dir') )
    opt.dir = false;
 end
end
